"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.BatchScheduleImplementation = void 0;
const config_1 = require("../config/config");
const mssql_service_1 = require("../db/mssql.service");
const redis_service_1 = require("../db/redis.service");
const schedule_model_1 = require("../models/schedule.model");
const interaction_model_1 = require("../models/interaction.model");
const interaction_service_1 = require("../services/interaction.service");
const linguistic_service_1 = require("../services/linguistic.service");
const process_service_1 = require("../services/process.service");
const promises_shared_1 = require("../shared/promises.shared");
const utilities_1 = require("../shared/utilities");
const log4js = require("log4js");
const moment = require("moment");
const billing_implementation_1 = require("./billing.implementation");
const batch_schedule_service_1 = require("../services/batch-schedule.service");
const study_service_1 = require("../services/study.service");
const arrays_shared_1 = require("../shared/arrays.shared");
const evaluation_plan_service_1 = require("../services/evaluation-plan.service");
let logger = log4js.getLogger('Task');
const CURRENT_SYSTEM_SERVICE = 'SCHEDULE';
const CURRENT_IMPLEMENTATION = 'BacthScheduleImplementation';
class BatchScheduleImplementation {
    Execute(executionCode, rateRecursive) {
        let executeIndex = 1;
        let ratio = config_1.RATIO_BATCH_SCHEDULE;
        let executeResult = true;
        return promises_shared_1.SleepPromise(config_1.SLEEP_BATCH_SCHEDULE)
            .then(_ => {
            return redis_service_1.Redis.LRange('BatchScheduleList', 0, ratio - 1);
        })
            .then((result) => {
            let promiseArray = [];
            ratio = result && result.length <= ratio ? result.length : 0;
            while (ratio >= executeIndex) {
                promiseArray.push(this.BacthScheduleExecute(executionCode));
                executeIndex++;
            }
            return Promise.all(promiseArray);
        })
            .then((results) => {
            if (results.filter(item => item === false).length > 0) {
                executeResult = false;
            }
            return redis_service_1.Redis.LRange('BatchScheduleList', 0, ratio - 1);
        })
            .then((result) => {
            if (result && result.length && rateRecursive >= 1)
                return this.Execute(executionCode, rateRecursive - 1);
            return Promise.resolve(executeResult);
        })
            .then(result => {
            let currentResult = executeResult && result ? true : false;
            return Promise.resolve(currentResult);
        })
            .catch((error) => {
            let errorMessage = typeof error === 'string' ? error : (error.message || error.description);
            logger.error('[BatchScheduleImplementation::Execute] executionCode:', executionCode, 'getting error:', errorMessage);
            return Promise.resolve(false);
        });
    }
    BacthScheduleExecute(executionCode) {
        let currentQueueId = null;
        let currentBatchScheduleData = null;
        let vcc = null;
        let batchScheduleId = null;
        let batchtype = null;
        let batchScheduleExecutionId = null;
        return redis_service_1.Redis.RPopLPush('BatchScheduleList', 'BatchScheduleListExecute')
            .then((queueId) => {
            if (!queueId)
                throw 'SERVER_ERROR_NO_QUEUE_ID_FOUND_ON_BATCH_SCHEDULE_LIST';
            logger.info('[BatchScheduleImplementation::BacthScheduleExecute] executionCode:', executionCode, 'launching queueId:', queueId);
            currentQueueId = queueId;
            let queueData = currentQueueId.split(':');
            vcc = queueData[0];
            batchScheduleId = queueData[1];
            batchtype = queueData[2];
            batchScheduleExecutionId = queueData[3];
            return redis_service_1.Redis.GetBatchSchedule(currentQueueId);
        })
            .then(result => {
            currentBatchScheduleData = result;
            if (currentBatchScheduleData.execution.status === 'PAUSED_BY_INSUFFICIENT_REQUESTS_BALANCE')
                currentBatchScheduleData.execution.status = 'BATCH_CREATED';
            if (currentBatchScheduleData?.execution?.status
                &&
                    ['REGISTERED_ON_REDIS', 'READY_TO_EXECUTE'].includes(currentBatchScheduleData.execution.status))
                return this.CreateNewScheduledBatchExecutions(currentQueueId, executionCode, currentBatchScheduleData);
            else if (currentBatchScheduleData?.execution?.status === 'BATCH_CREATED'
                &&
                    currentBatchScheduleData?.execution?.processId)
                return this.StartScheduleBatchExecutions(currentQueueId, executionCode, currentBatchScheduleData);
            else
                logger.info('[BatchScheduleImplementation::BacthScheduleExecute] executionCode:', executionCode, 'ommiting queueId:', currentQueueId);
            return Promise.resolve(true);
        })
            .then(result => {
            if (currentBatchScheduleData)
                return redis_service_1.Redis.SaveBatchSchedule(currentQueueId, currentBatchScheduleData);
            return Promise.resolve(true);
        })
            .then(_ => {
            let promiseArray = [];
            if (currentBatchScheduleData?.execution?.status === 'BATCH_STARTED') {
                logger.info('[BatchScheduleImplementation::BacthScheduleExecute] executionCode:', executionCode, 'getting queueId:', currentQueueId, 'BATCH_STARTED');
                promiseArray.push(redis_service_1.Redis.DeleteQueue('BatchScheduleListExecute', currentQueueId));
                promiseArray.push(redis_service_1.Redis.DeleteBatchSchedule(currentQueueId));
            }
            else if (['CANCELLED_BY_FAILED_PROCESS', 'CANCELLED_BY_NO_INTERACTIONS_FOUND'].includes(currentBatchScheduleData?.execution?.status)) {
                logger.info('[BatchScheduleImplementation::BacthScheduleExecute] executionCode:', executionCode, 'getting queueId:', currentQueueId, currentBatchScheduleData.execution.status);
                promiseArray.push(redis_service_1.Redis.DeleteQueue('BatchScheduleListExecute', currentQueueId));
                promiseArray.push(redis_service_1.Redis.DeleteBatchSchedule(currentQueueId));
            }
            else if (currentBatchScheduleData?.execution?.retry >= config_1.RETRY_BATCH_SCHEDULE) {
                logger.warn('[BatchScheduleImplementation::BacthScheduleExecute] executionCode:', executionCode, 'getting queueId:', currentQueueId, 'REMOVED_BY_RETRIES', currentBatchScheduleData.execution.errorMessage);
                promiseArray.push(redis_service_1.Redis.DeleteQueue('BatchScheduleListExecute', currentQueueId));
                promiseArray.push(redis_service_1.Redis.DeleteBatchSchedule(currentQueueId));
            }
            else if (currentBatchScheduleData?.execution?.errorMessage) {
                logger.warn('[BatchScheduleImplementation::BacthScheduleExecute] executionCode:', executionCode, 'getting queueId:', currentQueueId, 'ERROR', currentBatchScheduleData.execution.errorMessage);
                promiseArray.push(mssql_service_1.MSSql.SaveBatchError(vcc, CURRENT_SYSTEM_SERVICE, batchScheduleId, batchtype, CURRENT_IMPLEMENTATION, batchScheduleExecutionId, null, currentBatchScheduleData.execution.errorFunction, currentBatchScheduleData.execution.errorMessage, currentBatchScheduleData.execution.errorDetail));
            }
            else if (currentQueueId && !currentBatchScheduleData) {
                logger.warn('[BatchScheduleImplementation::BacthScheduleExecute] executionCode:', executionCode, 'getting queueId:', currentQueueId, 'DATA_DOES_NOT_EXIST');
                promiseArray.push(redis_service_1.Redis.DeleteQueue('BatchScheduleListExecute', currentQueueId));
            }
            else {
                logger.info('[BatchScheduleImplementation::BacthScheduleExecute] executionCode:', executionCode, 'getting queueId:', currentQueueId, 'QUEUE_WAS_NOT_REMOVED');
            }
            return Promise.all(promiseArray);
        })
            .then(_ => {
            let promiseArray = [];
            if (currentBatchScheduleData?.execution?.retry >= config_1.RETRY_BATCH_SCHEDULE) {
                promiseArray.push(redis_service_1.Redis.DeleteBatchSchedule(currentQueueId));
                promiseArray.push(mssql_service_1.MSSql.SaveBatchError(vcc, CURRENT_SYSTEM_SERVICE, batchScheduleId, batchtype, CURRENT_IMPLEMENTATION, batchScheduleExecutionId, null, currentBatchScheduleData.errorFunction, currentBatchScheduleData.errorMessage, currentBatchScheduleData.errorDetail));
            }
            return Promise.all(promiseArray);
        })
            .then(_ => {
            return Promise.resolve(true);
        })
            .catch((err) => {
            let errorMessage = typeof err === 'string' ? err : (err.message || err.description);
            logger.error('[BatchScheduleImplementation::BacthScheduleExecute] executionCode:', executionCode, 'getting queueId:', currentQueueId, 'getting errorMessage:', errorMessage);
            return Promise.resolve(false);
        });
    }
    GenerateNewScheduleBatchExecution(executionCode, batchscheduleId) {
        let batchSchedules = [];
        return promises_shared_1.DummyPromise()
            .then(_ => {
            return mssql_service_1.MSSql.GetBatchSchedulesByCriteria(null, 'STATUS', 'CREATED,RUNNING', true);
        })
            .then(result => {
            if (!result)
                throw 'SERVER_ERROR_SEARCH_BATCH_SCHEDULES_FAILED';
            if (result && !result.length)
                throw 'SERVER_WARNING_BATCH_SCHEDULES_NOT_FOUND';
            batchSchedules = result.filter(r => ['TRANSCRIPTION', 'EVALUATION_PLAN'].includes(r.batchType));
            if (batchscheduleId)
                batchSchedules = batchSchedules.filter(bs => bs.id === batchscheduleId);
            return mssql_service_1.MSSql.GetBatchScheduleExecutionsByCriteria(null, 'LAST_CREATED_EXECUTIONS', batchSchedules.map(bs => bs.id).toString());
        })
            .then((result) => {
            let executionList = [];
            batchSchedules.forEach(bs => {
                let lastBatchScheduleExecution = result.find(r => r.batchScheduleId === bs.id);
                bs.lastExecutionDate = lastBatchScheduleExecution ? lastBatchScheduleExecution.executionDate : bs.firstExecutionDate;
                let newBatchScheduleExecutions = batch_schedule_service_1.GenerateNewBatcScheduleExecutions(bs, 7, lastBatchScheduleExecution ? lastBatchScheduleExecution.order : 0);
                executionList = executionList.concat(newBatchScheduleExecutions);
            });
            let promiseArray = [];
            executionList.forEach(el => promiseArray.push(mssql_service_1.MSSql.SaveBatchScheduleExecution(el.vcc, el, el.createdByUserId)));
            return Promise.all(promiseArray);
        })
            .then(_ => {
            return Promise.resolve(true);
        })
            .catch((err) => {
            let errorMessage = typeof err === 'string' ? err : (err.message || err.description);
            logger.error('[BacthScheduleImplementation::GenerateNewScheduleBatchExecution] executionCode:', executionCode, 'getting errorMessage:', errorMessage);
            return Promise.resolve(false);
        });
    }
    RemoveScheduleBatchExecutions(vcc, scheduleData, executionCode) {
        return promises_shared_1.DummyPromise()
            .then(_ => {
            return mssql_service_1.MSSql.GetBatchScheduleExecutionsByCriteria(vcc, 'NO_EXECUTED_BY_SCHEDULE_ID', scheduleData.id);
        })
            .then(result => {
            let promiseArray = [];
            result.map(r => {
                promiseArray.push(mssql_service_1.MSSql.DeleteBatchScheduleExecution(vcc, r.id));
                if (r.status === 'REGISTERED_ON_REDIS') {
                    let queueId = `${vcc}:${scheduleData.id}:${scheduleData.batchType}:${r.id}`;
                    promiseArray.push(redis_service_1.Redis.DeleteQueue('BatchScheduleListExecute', queueId));
                    promiseArray.push(redis_service_1.Redis.DeleteBatchSchedule(queueId));
                }
            });
            return Promise.all(promiseArray);
        })
            .then(_ => {
            return Promise.resolve(true);
        })
            .catch((err) => {
            let errorMessage = typeof err === 'string' ? err : (err.message || err.description);
            logger.error('[BacthScheduleImplementation::RemoveScheduleBatchExecutions] executionCode:', executionCode, 'getting errorMessage:', errorMessage);
            return Promise.resolve(false);
        });
    }
    GetNewScheduledBatchExecutions(executionCode, batchscheduleId) {
        let batchSchedules = [];
        return promises_shared_1.DummyPromise()
            .then(_ => {
            return mssql_service_1.MSSql.GetBatchSchedulesByCriteria(null, 'STATUS', 'CREATED,RUNNING', true);
        })
            .then(result => {
            if (!result)
                throw 'SERVER_ERROR_SEARCH_BATCH_SCHEDULES_FAILED';
            if (result && !result.length)
                throw 'SERVER_WARNING_BATCH_SCHEDULES_NOT_FOUND';
            batchSchedules = result;
            if (batchscheduleId)
                batchSchedules = batchSchedules.filter(bs => bs.id === batchscheduleId);
            return mssql_service_1.MSSql.GetBatchScheduleExecutionsByCriteria(null, 'NOT_CREATED_BY_BS_ID', batchSchedules.map(bs => bs.id).toString());
        })
            .then((result) => {
            batchSchedules.forEach(bs => {
                bs.executionList = result.filter(r => r.batchScheduleId === bs.id);
            });
            return Promise.resolve(true);
        })
            .then(_ => {
            let promiseArray = [];
            batchSchedules.forEach(bs => {
                let pendingBatchSchedules = bs.executionList.filter(el => el.status === 'PENDING');
                pendingBatchSchedules = pendingBatchSchedules.filter(pbs => moment(pbs.executionDate).isBefore(moment().utc().add(1, 'days').utc()));
                if (pendingBatchSchedules.length) {
                    promiseArray.push(this.SaveNewScheduledExecutions(executionCode, bs, pendingBatchSchedules));
                }
                else {
                    logger.warn('[BacthScheduleImplementation::GetNewScheduledBatchExecutions] executionCode:', executionCode, 'no execution to register on redis for ', bs.id);
                }
            });
            return Promise.all(promiseArray);
        })
            .then(_ => {
            return Promise.resolve(true);
        })
            .catch((err) => {
            let errorMessage = typeof err === 'string' ? err : (err.message || err.description);
            logger.error('[BacthScheduleImplementation::GetNewScheduledBatchExecutions] executionCode:', executionCode, 'getting errorMessage:', errorMessage);
            return Promise.resolve(false);
        });
    }
    SaveNewScheduledExecutions(executionCode, bs, pendingBatchSchedules) {
        return promises_shared_1.DummyPromise()
            .then(_ => {
            let promiseArray = [];
            pendingBatchSchedules.map(pbs => {
                pbs.status = 'REGISTERED_ON_REDIS';
                promiseArray.push(this.SaveNewScheduledExecution(executionCode, bs, pbs));
            });
            return Promise.all(promiseArray);
        })
            .then(result => {
            logger.info('[BacthScheduleImplementation::SaveNewScheduledExecutions] executionCode:', executionCode, 'updating db for:', bs.id);
            return mssql_service_1.MSSql.SaveBatchSchedule(bs.vcc, bs, 'system');
        })
            .then(_ => {
            return Promise.resolve(true);
        })
            .catch((err) => {
            let errorMessage = typeof err === 'string' ? err : (err.message || err.description);
            logger.error('[BacthScheduleImplementation::SaveNewScheduledExecutions] executionCode:', executionCode, 'getting errorMessage:', errorMessage);
            return Promise.resolve(false);
        });
    }
    SaveNewScheduledExecution(executionCode, bs, bse) {
        let queueId = `${bs.vcc}:${bs.id}:${bs.batchType}:${bse.id}`;
        return promises_shared_1.DummyPromise()
            .then(_ => {
            return redis_service_1.Redis.GetBatchSchedule(queueId);
        })
            .then(result => {
            if (result || (result && result.status === 'REGISTERED_ON_REDIS')) {
                logger.warn('[BacthScheduleImplementation::SaveNewScheduledExecution] executionCode:', executionCode, 'the scheduleExecution is already registered:', bs.id, '=>', bse.id);
                throw 'SERVER_WARNING_SCHEDULE_EXECUTION_IS_ALREADY_REGISTERED';
            }
            let startDate = null;
            let endDate = null;
            switch (bs.frequency) {
                case 'daily':
                    startDate = moment(bse.executionDate).utc().add(-1, 'days').utc();
                    endDate = moment(bse.executionDate).utc().add(-1, 'milliseconds').utc();
                    break;
                case 'weekly':
                    startDate = moment(bse.executionDate).utc().add(-1, 'weeks').utc();
                    endDate = moment(bse.executionDate).utc().add(-1, 'milliseconds').utc();
                    break;
                case 'monthly':
                    startDate = moment(bse.executionDate).utc().add(-1, 'month').set('date', bs.monthDay).utc();
                    endDate = moment(bse.executionDate).utc().add(-1, 'milliseconds').utc();
                    break;
                case 'custom_per_days':
                    startDate = moment(bse.executionDate).utc().add(-1 * bs.daysInterval, 'days').utc();
                    endDate = moment(bse.executionDate).utc().add(-1, 'milliseconds').utc();
                    break;
                case 'custom_per_hours':
                    startDate = moment(bse.executionDate).utc().add(-1 * bs.hoursInterval, 'hours').utc();
                    endDate = moment(bse.executionDate).utc().add(-1, 'milliseconds').utc();
                    break;
            }
            switch (bs.batchType) {
                case 'EVALUATION_PLAN':
                    bs.batchData.evaluationPlan.startDate = startDate.toDate();
                    bs.batchData.evaluationPlan.endDate = endDate.toDate();
                    bs.batchData.evaluationPlan.dateFin = moment(bse.executionDate).utc().add(bs.daysToExpire, 'days').utc().toDate();
                    break;
                case 'TEMPLATE': break;
                case 'TRANSCRIPTION':
                    bs.batchData.startDate = startDate.toDate();
                    bs.batchData.endDate = endDate.toDate();
                    break;
            }
            let promiseArray = [];
            promiseArray.push(redis_service_1.Redis.CreateQueueL('BatchScheduleList', queueId));
            promiseArray.push(redis_service_1.Redis.SaveBatchSchedule(queueId, {
                execution: bse,
                batchData: bs.batchData,
                processFiltersId: bs.processFiltersId,
                retry: 0
            }));
            return Promise.all(promiseArray);
        })
            .then(_ => {
            return mssql_service_1.MSSql.SaveBatchScheduleExecution(bse.vcc, bse, 'system');
        })
            .then(_ => {
            return Promise.resolve(true);
        })
            .catch((err) => {
            let errorMessage = typeof err === 'string' ? err : (err.message || err.description);
            logger.error('[BacthScheduleImplementation::SaveNewScheduledExecution] executionCode:', executionCode, 'getting errorMessage:', errorMessage);
            return Promise.resolve(false);
        });
    }
    CreateNewScheduledBatchExecutions(queueId, executionCode, scheduleBatchData) {
        let queueData = queueId.split(':');
        let vcc = queueData[0];
        let batchScheduleId = queueData[1];
        let batchtype = queueData[2];
        let batchScheduleExecutionId = queueData[3];
        return promises_shared_1.DummyPromise()
            .then(result => {
            if (scheduleBatchData.execution && scheduleBatchData.execution.status && scheduleBatchData.execution.status === 'REGISTERED_ON_REDIS') {
                if (moment(scheduleBatchData.execution.executionDate).add(scheduleBatchData.batchData.executionDelay ? scheduleBatchData.batchData.executionDelay : 0, 'hours').isBefore(moment().utc())) {
                    scheduleBatchData.execution.status = 'READY_TO_EXECUTE';
                }
            }
            if (scheduleBatchData.execution?.status === 'READY_TO_EXECUTE') {
                switch (batchtype) {
                    case 'TRANSCRIPTION': return this.CreateProcessSchedule(scheduleBatchData, queueId, executionCode);
                    case 'TEMPLATE': return this.CreateTemplateSchedule(scheduleBatchData, queueId, executionCode);
                    case 'EVALUATION_PLAN': return this.CreateEvaluationPlanSchedule(scheduleBatchData, queueId, executionCode);
                    default: throw 'SERVER_ERROR_BATCH_SCHEDULE_WITH_UNKNOWN_BATCH_TYPE';
                }
            }
            else
                throw 'SERVER_ERROR_BATCH_SCHEDULE_WITH_NOT_ALLOWED_STATUS';
        })
            .then(result => {
            let scheduleExecution = new schedule_model_1.ScheduleExecution();
            scheduleExecution.vcc = vcc;
            scheduleExecution.id = batchScheduleExecutionId;
            scheduleExecution.batchScheduleId = batchScheduleId;
            scheduleExecution.batchId = scheduleBatchData.execution.batchId;
            scheduleExecution.status = scheduleBatchData.execution.status;
            return mssql_service_1.MSSql.SaveBatchScheduleExecution(vcc, scheduleExecution, 'system');
        })
            .then(_ => {
            return Promise.resolve(true);
        })
            .catch((err) => {
            let errorMessage = typeof err === 'string' ? err : (err.message || err.description);
            logger.error('[BacthScheduleImplementation::CreateNewScheduledBatchExecutions] executionCode:', executionCode, 'getting queueId:', queueId, 'getting errorMessage:', errorMessage);
            return Promise.resolve(false);
        });
    }
    CreateProcessSchedule(scheduleBatchData, queueId, executionCode) {
        let queueData = queueId.split(':');
        let vcc = queueData[0];
        let batchScheduleId = queueData[1];
        let batchtype = queueData[2];
        let batchScheduleExecutionId = queueData[3];
        let interactionSearch = new interaction_model_1.InteractionSearch();
        let process = {};
        process.start = false;
        process.clientId = scheduleBatchData.batchData.clientId;
        process.description = scheduleBatchData.batchData.description;
        process.environment = scheduleBatchData.batchData.environment;
        process.environmentType = scheduleBatchData.batchData.environmentType;
        process.language = scheduleBatchData.batchData.language;
        process.dictionaryId = scheduleBatchData.batchData.dictionaryId;
        process.percentage = scheduleBatchData.batchData.samplePercentage;
        process.type = scheduleBatchData.batchData.type;
        process.batchScheduleId = batchScheduleId;
        return promises_shared_1.DummyPromise()
            .then(_ => {
            return mssql_service_1.MSSql.GetProcessFilters(vcc, scheduleBatchData.processFiltersId);
        })
            .then(result => {
            interactionSearch = interaction_service_1.GetInteractionSearchFromProcessFilter(result);
            interactionSearch.searchType = 'TO_SEARCH';
            interactionSearch.dateIni = moment(scheduleBatchData.batchData.startDate).toDate();
            interactionSearch.dateFin = moment(scheduleBatchData.batchData.endDate).toDate();
            return Promise.resolve(true);
        })
            .then(_ => {
            let promiseArray = [];
            promiseArray = interaction_service_1.GetChannelNewInteractionSummaryPromises(vcc, scheduleBatchData.batchData.vccType, true, 'SUMMARIZED', interactionSearch, null);
            return Promise.all(promiseArray);
        })
            .then(result => {
            let totalInteractions = 0;
            result.forEach(r => totalInteractions += r.totalInteractions);
            if (totalInteractions) {
                logger.info('[BacthScheduleImplementation::CreateProcessScheduleBatch] executionCode:', executionCode, 'creating process for', queueId, 'interactions:', totalInteractions);
                interactionSearch.searchType = 'TO_OPERATE';
                process.name = batchScheduleId;
                if (process.name.length > 40)
                    process.name = process.name.substr(0, 40);
                process.name += '-' + ('000' + scheduleBatchData.execution.order).slice(-3);
                process.name += '-' + utilities_1.guid().slice(-5);
                scheduleBatchData.execution.batchId = process.name;
                scheduleBatchData.execution.processId = process.name;
                return process_service_1.CreateProcess(vcc, scheduleBatchData.batchData.vccType, process, interactionSearch, scheduleBatchData.execution.createdByUserId);
            }
            else {
                logger.info('[BacthScheduleImplementation::CreateProcessScheduleBatch] executionCode:', executionCode, 'no interactions found for', queueId);
                scheduleBatchData.execution.status = 'CANCELLED_BY_NO_INTERACTIONS_FOUND';
                throw 'SERVER_ERROR_NO_INTERACTIONS_FOUND_FOR_EXECUTION';
            }
        })
            .then(result => {
            if (result && result.status) {
                scheduleBatchData.execution.status = 'BATCH_CREATED';
                scheduleBatchData.retry = 0;
                scheduleBatchData.errorMessage = null;
                scheduleBatchData.errorDetail = null;
            }
            else {
                scheduleBatchData.retry = scheduleBatchData.retry ? scheduleBatchData.retry + 1 : 0;
                scheduleBatchData.errorFunction = 'CreateProcessScheduleBatch';
                scheduleBatchData.errorMessage = result ? result.saveResult : 'CREATE_BATCH_SCHEDULE_FAILED';
                scheduleBatchData.errorDetail = result ? result.saveResult : 'CREATE_BATCH_SCHEDULE_FAILED';
            }
            return Promise.resolve(true);
        })
            .catch((err) => {
            let errorMessage = typeof err === 'string' ? err : (err.message || err.description);
            logger.error('[BacthScheduleImplementation::CreateProcessScheduleBatch] executionCode:', executionCode, 'getting errorMessage:', errorMessage);
            scheduleBatchData.retry = scheduleBatchData.retry ? scheduleBatchData.retry + 1 : 0;
            scheduleBatchData.errorFunction = 'CreateProcessScheduleBatch';
            scheduleBatchData.errorMessage = errorMessage;
            scheduleBatchData.errorDetail = JSON.stringify(err);
            if (scheduleBatchData.execution.status === 'CANCELLED_BY_NO_INTERACTIONS_FOUND')
                scheduleBatchData.retry = config_1.RETRY_BATCH_SCHEDULE;
            return Promise.resolve(false);
        });
    }
    CreateTemplateSchedule(scheduleBatchData, queueId, executionCode) {
        let queueData = queueId.split(':');
        let vcc = queueData[0];
        let batchScheduleId = queueData[1];
        let batchtype = queueData[2];
        let batchScheduleExecutionId = queueData[3];
        let process = {};
        let study = {};
        study.description = 'Scheduled template';
        study.processId = scheduleBatchData.execution.processId;
        study.status = 0.01;
        study.templateId = scheduleBatchData.processFiltersId;
        study.batchScheduleId = batchScheduleId;
        study.type = 'TEMPLATE';
        return promises_shared_1.DummyPromise()
            .then(_ => {
            return mssql_service_1.MSSql.GetProcessTask(vcc, study.processId);
        })
            .then(result => {
            process = result;
            if (process && process.status !== 'FAILED') {
                logger.info('[BacthScheduleImplementation::CreateTemplateSchedule] executionCode:', executionCode, 'creating process for', queueId, 'process:', process.name);
                study.name = batchScheduleId;
                if (study.name.length > 40)
                    study.name = study.name.substr(0, 40);
                study.name += '-' + utilities_1.guid().slice(-9);
                scheduleBatchData.execution.batchId = study.name;
                return study_service_1.CreateStudy(vcc, scheduleBatchData.batchData.vccType, study, scheduleBatchData.execution.createdByUserId);
            }
            else {
                logger.info('[BacthScheduleImplementation::CreateTemplateSchedule] executionCode:', executionCode, 'failed process for', queueId);
                scheduleBatchData.execution.status = 'CANCELLED_BY_FAILED_PROCESS';
                throw 'SERVER_ERROR_FAILED_PROCESS_FOR_EXECUTION';
            }
        })
            .then(result => {
            if (result && result.resultSaveStudy) {
                scheduleBatchData.execution.status = 'BATCH_CREATED';
                scheduleBatchData.retry = 0;
                scheduleBatchData.errorMessage = null;
                scheduleBatchData.errorDetail = null;
            }
            else {
                scheduleBatchData.retry = scheduleBatchData.retry ? scheduleBatchData.retry + 1 : 0;
                scheduleBatchData.errorFunction = 'CreateTemplateSchedule';
                scheduleBatchData.errorMessage = result ? result.saveResult : 'CREATE_BATCH_SCHEDULE_FAILED';
                scheduleBatchData.errorDetail = result ? result.saveResult : 'CREATE_BATCH_SCHEDULE_FAILED';
            }
            return Promise.resolve(true);
        })
            .catch((err) => {
            let errorMessage = typeof err === 'string' ? err : (err.message || err.description);
            logger.error('[BacthScheduleImplementation::CreateTemplateSchedule] executionCode:', executionCode, 'getting errorMessage:', errorMessage);
            scheduleBatchData.retry = scheduleBatchData.retry ? scheduleBatchData.retry + 1 : 0;
            scheduleBatchData.errorFunction = 'CreateTemplateSchedule';
            scheduleBatchData.errorMessage = errorMessage;
            scheduleBatchData.errorDetail = JSON.stringify(err);
            if (scheduleBatchData.execution.status === 'CANCELLED_BY_FAILED_PROCESS')
                scheduleBatchData.retry = config_1.RETRY_BATCH_SCHEDULE;
            return Promise.resolve(false);
        });
    }
    CreateEvaluationPlanSchedule(scheduleBatchData, queueId, executionCode) {
        let queueData = queueId.split(':');
        let vcc = queueData[0];
        let batchScheduleId = queueData[1];
        let batchtype = queueData[2];
        let batchScheduleExecutionId = queueData[3];
        let interactionSearch = new interaction_model_1.InteractionSearch();
        let evaluationPlan = {};
        evaluationPlan.status = 1;
        evaluationPlan.description = scheduleBatchData.batchData.evaluationPlan.description;
        evaluationPlan.environmentId = scheduleBatchData.batchData.evaluationPlan.environmentId;
        evaluationPlan.transcriptionLanguage = scheduleBatchData.batchData.evaluationPlan.transcriptionLanguage;
        evaluationPlan.modality = scheduleBatchData.batchData.evaluationPlan.modality;
        evaluationPlan.dateFin = scheduleBatchData.batchData.evaluationPlan.dateFin;
        evaluationPlan.idEvaluationFormat = scheduleBatchData.batchData.evaluationPlan.idEvaluationFormat;
        evaluationPlan.templateId = scheduleBatchData.batchData.evaluationPlan.templateId;
        evaluationPlan.enabled = scheduleBatchData.batchData.evaluationPlan.enabled;
        evaluationPlan.evaluation_mode = scheduleBatchData.batchData.evaluationPlan.evaluation_mode;
        evaluationPlan.filterByContentPercentage = scheduleBatchData.batchData.evaluationPlan.filterByContentPercentage;
        evaluationPlan.filterByContentQueryName = scheduleBatchData.batchData.evaluationPlan.filterByContentQueryName;
        evaluationPlan.filterByContentQuery = scheduleBatchData.batchData.evaluationPlan.filterByContentQuery;
        evaluationPlan.evaluatePercentage = scheduleBatchData.batchData.evaluationPlan.evaluatePercentage;
        evaluationPlan.auditors = scheduleBatchData.batchData.evaluationPlan.auditors.map(a => {
            let aep = {};
            aep.idAuditor = a.auditorId;
            aep.percentage = a.percentage;
            return aep;
        });
        evaluationPlan.evaluationStarted = true;
        evaluationPlan.batchScheduleId = batchScheduleId;
        let process = {};
        if (scheduleBatchData.batchData.process) {
            process.start = false;
            process.clientId = scheduleBatchData.batchData.process.clientId;
            process.description = scheduleBatchData.batchData.process.description;
            process.environment = scheduleBatchData.batchData.process.environment;
            process.environmentType = scheduleBatchData.batchData.process.environmentType;
            process.language = scheduleBatchData.batchData.process.language;
            process.percentage = scheduleBatchData.batchData.process.samplePercentage;
            process.type = scheduleBatchData.batchData.process.type;
            process.batchScheduleId = batchScheduleId;
            evaluationPlan.processId = process.name;
        }
        return promises_shared_1.DummyPromise()
            .then(_ => {
            return mssql_service_1.MSSql.GetProcessFilters(vcc, scheduleBatchData.processFiltersId);
        })
            .then(result => {
            interactionSearch = interaction_service_1.GetInteractionSearchFromProcessFilter(result);
            interactionSearch.searchType = 'TO_SEARCH';
            interactionSearch.storageId = utilities_1.guid();
            interactionSearch.dateIni = moment(scheduleBatchData.batchData.evaluationPlan.startDate).toDate();
            interactionSearch.dateFin = moment(scheduleBatchData.batchData.evaluationPlan.endDate).toDate();
            logger.info('[BacthScheduleImplementation::CreateEvaluationPlanSchedule] executionCode:', executionCode, 'filter for schedule', queueId, interactionSearch);
            return Promise.resolve(true);
        })
            .then(_ => {
            let promiseArray = [];
            promiseArray = interaction_service_1.GetChannelNewInteractionSummaryPromises(vcc, scheduleBatchData.batchData.evaluationPlan.vccType, true, 'SUMMARIZED', interactionSearch, null);
            return Promise.all(promiseArray);
        })
            .then(result => {
            let totalInteractions = 0;
            result.forEach(r => totalInteractions += r ? r.totalInteractions : 0);
            if (totalInteractions) {
                logger.info('[BacthScheduleImplementation::CreateEvaluationPlanSchedule] executionCode:', executionCode, 'creating process for', queueId, 'interactions:', totalInteractions);
                interactionSearch.searchType = 'TO_OPERATE';
                if (scheduleBatchData.batchData.process) {
                    process.name = utilities_1.guid();
                    evaluationPlan.processId = process.name;
                }
                evaluationPlan.id = batchScheduleId.toLowerCase();
                if (evaluationPlan.id.length > 40)
                    evaluationPlan.id = evaluationPlan.id.substr(0, 40);
                evaluationPlan.id += '-' + ('000' + scheduleBatchData.execution.order).slice(-3);
                evaluationPlan.id += '-' + utilities_1.guid().slice(-5);
                evaluationPlan.description = evaluationPlan.id;
                scheduleBatchData.execution.batchId = evaluationPlan.id;
                evaluation_plan_service_1.SetAuditorInteractionsNumbersFromPercentage(evaluationPlan.auditors, evaluationPlan.modality, totalInteractions);
                return evaluation_plan_service_1.CreateEvaluationPlan(vcc, scheduleBatchData.batchData.evaluationPlan.vccType, evaluationPlan, process, process?.type ? true : false, interactionSearch, scheduleBatchData.execution.createdByUserId);
            }
            else {
                logger.info('[BacthScheduleImplementation::CreateEvaluationPlanSchedule] executionCode:', executionCode, 'no interactions found for', queueId);
                scheduleBatchData.execution.status = 'CANCELLED_BY_NO_INTERACTIONS_FOUND';
                throw 'SERVER_ERROR_NO_INTERACTIONS_FOUND_FOR_EXECUTION';
            }
        })
            .then(result => {
            return Promise.resolve(result);
        })
            .then(result => {
            if (result) {
                if (scheduleBatchData.batchData.process)
                    scheduleBatchData.batchData.process.name = process.name;
                scheduleBatchData.batchData.evaluationPlan.id = evaluationPlan.id;
                scheduleBatchData.batchData.evaluationPlan.description = evaluationPlan.description;
                scheduleBatchData.batchData.evaluationPlan.processId = evaluationPlan.processId;
                scheduleBatchData.execution.status = evaluationPlan.evaluationStarted ? 'BATCH_STARTED' : 'BATCH_CREATED';
                scheduleBatchData.retry = 0;
                scheduleBatchData.errorMessage = null;
                scheduleBatchData.errorDetail = null;
            }
            else {
                scheduleBatchData.retry = scheduleBatchData.retry ? scheduleBatchData.retry + 1 : 0;
                scheduleBatchData.errorFunction = 'CreateEvaluationPlanSchedule';
                scheduleBatchData.errorMessage = 'CREATE_BATCH_SCHEDULE_FAILED';
                scheduleBatchData.errorDetail = 'CREATE_BATCH_SCHEDULE_FAILED';
            }
            return Promise.resolve(true);
        })
            .catch((err) => {
            let errorMessage = typeof err === 'string' ? err : (err.message || err.description);
            logger.error('[BacthScheduleImplementation::CreateEvaluationPlanSchedule] executionCode:', executionCode, 'getting errorMessage:', errorMessage);
            scheduleBatchData.retry = scheduleBatchData.retry ? scheduleBatchData.retry + 1 : 0;
            scheduleBatchData.errorFunction = 'CreateEvaluationPlanSchedule';
            scheduleBatchData.errorMessage = errorMessage;
            scheduleBatchData.errorDetail = JSON.stringify(err);
            if (scheduleBatchData.execution.status === 'CANCELLED_BY_NO_INTERACTIONS_FOUND')
                scheduleBatchData.retry = config_1.RETRY_BATCH_SCHEDULE;
            return Promise.resolve(false);
        });
    }
    StartScheduleBatchExecutions(queueId, executionCode, scheduleBatchData) {
        let queueData = queueId.split(':');
        let vcc = queueData[0];
        let batchScheduleId = queueData[1];
        let batchtype = queueData[2];
        let batchScheduleExecutionId = queueData[3];
        return promises_shared_1.DummyPromise()
            .then(_ => {
            let promiseArray = [];
            let billing = new billing_implementation_1.BillingImplementation();
            promiseArray.push(billing.GetVCCRates(vcc, scheduleBatchData.execution.processId));
            promiseArray.push(billing.GetVCCBalance(vcc));
            return Promise.all(promiseArray);
        })
            .then(result => {
            let billingData = {
                rates: result[0],
                balance: result[1]
            };
            switch (batchtype) {
                case 'TRANSCRIPTION': return this.StartScheduledProcess(scheduleBatchData, billingData, queueId, executionCode);
                case 'TEMPLATE': return this.StartScheduledTemplate(scheduleBatchData, billingData, queueId, executionCode);
                case 'EVALUATION_PLAN': return this.StartScheduledEvaluationPlan(scheduleBatchData, billingData, queueId, executionCode);
                default: throw 'SERVER_ERROR_BATCH_SCHEDULE_WITH_UNKNOWN_BATCH_TYPE';
            }
        })
            .then(_ => {
            let scheduleExecution = new schedule_model_1.ScheduleExecution();
            scheduleExecution.vcc = vcc;
            scheduleExecution.id = batchScheduleExecutionId;
            scheduleExecution.batchScheduleId = batchScheduleId;
            scheduleExecution.batchId = scheduleBatchData.execution.batchId;
            scheduleExecution.status = scheduleBatchData.execution.status;
            return mssql_service_1.MSSql.SaveBatchScheduleExecution(vcc, scheduleExecution, 'system');
        })
            .then(_ => {
            return Promise.resolve(true);
        })
            .catch((err) => {
            let errorMessage = typeof err === 'string' ? err : (err.message || err.description);
            logger.error('[BacthScheduleImplementation::StartScheduleBatchExecutions] executionCode:', executionCode, 'getting errorMessage:', errorMessage);
            scheduleBatchData.retry = scheduleBatchData.retry ? scheduleBatchData.retry + 1 : 0;
            scheduleBatchData.errorFunction = 'StartScheduleBatchExecutions';
            scheduleBatchData.errorMessage = errorMessage;
            scheduleBatchData.errorDetail = JSON.stringify(err);
            if (scheduleBatchData.execution.status === 'CANCELLED_BY_NO_INTERACTIONS_FOUND')
                scheduleBatchData.retry = config_1.RETRY_BATCH_SCHEDULE;
            return Promise.resolve(false);
        });
    }
    StartScheduledProcess(scheduleBatchData, billingData, queueId, executionCode) {
        let queueData = queueId.split(':');
        let vcc = queueData[0];
        let batchScheduleId = queueData[1];
        let batchtype = queueData[2];
        let batchScheduleExecutionId = queueData[3];
        let started = true;
        return promises_shared_1.DummyPromise()
            .then(_ => {
            return process_service_1.GetProcessChannelInteractions(vcc, scheduleBatchData.execution.batchId);
        })
            .then(result => {
            if (!result)
                throw 'SERVER_ERROR_CAN_NOT_GET_PROCESS_INTERACTIONS';
            let callResult = result.find(r => r.interactionType === 'CALL');
            if (callResult) {
                let transcriptionBalance = billingData.balance.find(b => b.systemService === 'TRANSCRIPTION');
                let ratedTime = billingData.rates.transcriptionRatePerSecond * callResult.notTranscribedDurationTime;
                let transcriptionAvailableBalance = transcriptionBalance.availableBalance - (transcriptionBalance.exceeded ? transcriptionBalance.difference : 0);
                if (ratedTime <= transcriptionAvailableBalance) {
                    return linguistic_service_1.StartProcess(vcc, scheduleBatchData.execution.batchId, scheduleBatchData.execution.createdByUserId);
                }
                else {
                    started = false;
                    scheduleBatchData.execution.status === 'PAUSED_BY_INSUFFICIENT_TRANSCRIPTION_BALANCE';
                }
            }
            else {
                return linguistic_service_1.StartProcess(vcc, scheduleBatchData.execution.batchId, scheduleBatchData.execution.createdByUserId);
            }
            return Promise.resolve(true);
        })
            .then(result => {
            if (started) {
                if (result)
                    scheduleBatchData.execution.status = 'BATCH_STARTED';
                else
                    scheduleBatchData.execution.status = 'BATCH_COULD_NOT_BE_STARTED';
            }
            scheduleBatchData.retry = 0;
            scheduleBatchData.errorFunction = null;
            scheduleBatchData.errorMessage = null;
            scheduleBatchData.errorDetail = null;
            return Promise.resolve(true);
        })
            .catch((err) => {
            let errorMessage = typeof err === 'string' ? err : (err.message || err.description);
            logger.error('[BacthScheduleImplementation::StartScheduleProcess] executionCode:', executionCode, 'getting errorMessage:', errorMessage);
            scheduleBatchData.retry = scheduleBatchData.retry ? scheduleBatchData.retry + 1 : 0;
            scheduleBatchData.errorFunction = 'StartScheduleProcess';
            scheduleBatchData.errorMessage = errorMessage;
            scheduleBatchData.errorDetail = JSON.stringify(err);
            if (scheduleBatchData.execution.status === 'CANCELLED_BY_NO_INTERACTIONS_FOUND')
                scheduleBatchData.retry = config_1.RETRY_BATCH_SCHEDULE;
            return Promise.resolve(false);
        });
    }
    StartScheduledTemplate(scheduleBatchData, billingData, queueId, executionCode) {
        let queueData = queueId.split(':');
        let vcc = queueData[0];
        let batchScheduleId = queueData[1];
        let batchtype = queueData[2];
        let batchScheduleExecutionId = queueData[3];
        let started = true;
        let studyId = scheduleBatchData.execution.batchId;
        let templateId = scheduleBatchData.processFiltersId;
        let processId = scheduleBatchData.execution.processId;
        return promises_shared_1.DummyPromise()
            .then(_ => {
            let promiseArray = [];
            promiseArray.push(mssql_service_1.MSSql.GetTemplateThemeItems(vcc, templateId, null));
            promiseArray.push(mssql_service_1.MSSql.GetTemplateThemeModels(vcc, templateId, null));
            promiseArray.push(mssql_service_1.MSSql.GetProcessInteractionsHistory(vcc, processId, billingData.rates.cognitiveAnalysisAverageSecondsPerRequest));
            return Promise.all(promiseArray);
        })
            .then(result => {
            let semanticAnalysisBalance = billingData.balance.find(b => b.systemService === 'SEMANTIC_ANALYSIS');
            let cognitiveAnalysisBalance = billingData.balance.find(b => b.systemService === 'COGNITIVE_ANALYSIS');
            let totalQueries = result[0].length;
            let totalModels = 0;
            let modelIntents = result[1].map(mi => { return mi.id; });
            let models = modelIntents.map(i => i.split('|')[0]);
            let nluModels = arrays_shared_1.GetUniqueFromArray(models);
            totalModels = nluModels.length;
            let historicalInteractionsData = result[2];
            let semanticAnalysisAvailableBalance = semanticAnalysisBalance.availableBalance - (semanticAnalysisBalance.exceeded ? semanticAnalysisBalance.difference : 0);
            let cognitiveAnalysisAvailableBalance = cognitiveAnalysisBalance.availableBalance - (cognitiveAnalysisBalance.exceeded ? cognitiveAnalysisBalance.difference : 0);
            let ratedQueries = billingData.rates.semanticAnalysisRatePerTransaction * totalQueries;
            let ratedModels = 0;
            ratedModels = Math.ceil(totalModels
                *
                    billingData.rates.cognitiveAnalysisRatePerTransaction
                * ((billingData.rates.processCallPhrases * (1 + billingData.rates.cognitiveAnalysisBillingSecureRate))
                    +
                        (billingData.rates.processTextPhrases * (1 + billingData.rates.cognitiveAnalysisTextBillingSecureRate))));
            if (ratedQueries <= semanticAnalysisAvailableBalance
                &&
                    ratedModels <= cognitiveAnalysisAvailableBalance) {
                let studyData = {
                    studyId: studyId,
                    processId: processId
                };
                return study_service_1.BeginStudy(vcc, studyData, scheduleBatchData.execution.createdByUserId);
            }
            else {
                started = false;
                scheduleBatchData.execution.status = 'PAUSED_BY_INSUFFICIENT_REQUESTS_BALANCE';
                logger.warn('[BacthScheduleImplementation::StartScheduledTemplate] executionCode:', executionCode, 'RatedQueries:', ratedQueries, 'AvailableBalance:', semanticAnalysisAvailableBalance);
                logger.warn('[BacthScheduleImplementation::StartScheduledTemplate] executionCode:', executionCode, 'RatedModels:', ratedModels, 'AvailableBalance:', cognitiveAnalysisAvailableBalance);
            }
            return Promise.resolve(true);
        })
            .then(result => {
            if (started) {
                if (result?.status)
                    scheduleBatchData.execution.status = 'BATCH_STARTED';
                else
                    scheduleBatchData.execution.status = 'BATCH_COULD_NOT_BE_STARTED';
            }
            scheduleBatchData.retry = 0;
            scheduleBatchData.errorFunction = null;
            scheduleBatchData.errorMessage = null;
            scheduleBatchData.errorDetail = null;
            return Promise.resolve(true);
        })
            .catch((err) => {
            let errorMessage = typeof err === 'string' ? err : (err.message || err.description);
            logger.error('[BacthScheduleImplementation::StartScheduledTemplate] executionCode:', executionCode, 'getting errorMessage:', errorMessage);
            scheduleBatchData.retry = scheduleBatchData.retry ? scheduleBatchData.retry + 1 : 0;
            scheduleBatchData.errorFunction = 'StartScheduledTemplate';
            scheduleBatchData.errorMessage = errorMessage;
            scheduleBatchData.errorDetail = JSON.stringify(err);
            return Promise.resolve(false);
        });
    }
    StartScheduledEvaluationPlan(scheduleBatchData, billingData, queueId, executionCode) {
        let queueData = queueId.split(':');
        let vcc = queueData[0];
        let batchScheduleId = queueData[1];
        let batchtype = queueData[2];
        let batchScheduleExecutionId = queueData[3];
        let started = true;
        return promises_shared_1.DummyPromise()
            .then(_ => {
            let promiseArray = [];
            promiseArray.push(process_service_1.GetProcessChannelInteractions(vcc, scheduleBatchData.batchData.evaluationPlan.processId));
            promiseArray.push(mssql_service_1.MSSql.GetTemplateThemeItems(vcc, scheduleBatchData.batchData.evaluationPlan.templateId, null));
            promiseArray.push(mssql_service_1.MSSql.GetTemplateThemeModels(vcc, scheduleBatchData.batchData.evaluationPlan.templateId, null));
            promiseArray.push(mssql_service_1.MSSql.GetProcessInteractionsHistory(vcc, scheduleBatchData.batchData.evaluationPlan.processId, billingData.rates.cognitiveAnalysisAverageSecondsPerRequest));
            return Promise.all(promiseArray);
        })
            .then(result => {
            let transcriptionBalance = billingData.balance.find(b => b.systemService === 'TRANSCRIPTION');
            let semanticAnalysisBalance = billingData.balance.find(b => b.systemService === 'SEMANTIC_ANALYSIS');
            let cognitiveAnalysisBalance = billingData.balance.find(b => b.systemService === 'COGNITIVE_ANALYSIS');
            let callResult = result[0].find(r => r.interactionType === 'CALL');
            let totalQueries = result[1].length;
            let totalModels = 0;
            let ratedTime = billingData.rates.transcriptionRatePerSecond * callResult.notTranscribedDurationTime;
            let modelIntents = result[2].map(mi => { return mi.id; });
            let models = modelIntents.map(i => i.split('|')[0]);
            let nluModels = arrays_shared_1.GetUniqueFromArray(models);
            totalModels = nluModels.length;
            let historicalInteractionsData = result[3];
            let transcriptionAvailableBalance = transcriptionBalance.availableBalance - (transcriptionBalance.exceeded ? transcriptionBalance.difference : 0);
            let semanticAnalysisAvailableBalance = semanticAnalysisBalance.availableBalance - (semanticAnalysisBalance.exceeded ? semanticAnalysisBalance.difference : 0);
            let cognitiveAnalysisAvailableBalance = cognitiveAnalysisBalance.availableBalance - (cognitiveAnalysisBalance.exceeded ? cognitiveAnalysisBalance.difference : 0);
            let ratedQueries = billingData.rates.semanticAnalysisRatePerTransaction * totalQueries;
            let ratedModels = 0;
            ratedModels = Math.ceil(totalModels
                *
                    billingData.rates.cognitiveAnalysisRatePerTransaction
                * ((billingData.rates.processCallPhrases * (1 + billingData.rates.cognitiveAnalysisBillingSecureRate))
                    +
                        (billingData.rates.processTextPhrases * (1 + billingData.rates.cognitiveAnalysisTextBillingSecureRate))));
            if (ratedTime <= transcriptionAvailableBalance
                &&
                    ratedQueries <= semanticAnalysisAvailableBalance
                &&
                    ratedModels <= cognitiveAnalysisAvailableBalance) {
                logger.info('[BacthScheduleImplementation::StartScheduledEvaluationPlan] executionCode:', executionCode, 'Starting plan :', scheduleBatchData.batchData.evaluationPlan);
                return evaluation_plan_service_1.StartEvaluationPlan(vcc, scheduleBatchData.batchData.evaluationPlan);
            }
            else {
                started = false;
                scheduleBatchData.execution.status = 'PAUSED_BY_INSUFFICIENT_BALANCE';
                logger.warn('[BacthScheduleImplementation::StartScheduledEvaluationPlan] executionCode:', executionCode, 'RatedTime:', ratedTime, 'AvailableBalance:', transcriptionAvailableBalance);
                logger.warn('[BacthScheduleImplementation::StartScheduledEvaluationPlan] executionCode:', executionCode, 'RatedQueries:', ratedQueries, 'AvailableBalance:', semanticAnalysisAvailableBalance);
                logger.warn('[BacthScheduleImplementation::StartScheduledEvaluationPlans] executionCode:', executionCode, 'RatedModels:', ratedModels, 'AvailableBalance:', cognitiveAnalysisAvailableBalance);
            }
            return Promise.resolve(true);
        })
            .then(result => {
            if (started) {
                if (result)
                    scheduleBatchData.execution.status = 'BATCH_STARTED';
                else
                    scheduleBatchData.execution.status = 'BATCH_COULD_NOT_BE_STARTED';
            }
            scheduleBatchData.retry = 0;
            scheduleBatchData.errorFunction = null;
            scheduleBatchData.errorMessage = null;
            scheduleBatchData.errorDetail = null;
            return Promise.resolve(true);
        })
            .catch((err) => {
            let errorMessage = typeof err === 'string' ? err : (err.message || err.description);
            logger.error('[BacthScheduleImplementation::StartScheduledEvaluationPlan] executionCode:', executionCode, 'getting errorMessage:', errorMessage);
            scheduleBatchData.retry = scheduleBatchData.retry ? scheduleBatchData.retry + 1 : 0;
            scheduleBatchData.errorFunction = 'StartScheduledEvaluationPlan';
            scheduleBatchData.errorMessage = errorMessage;
            scheduleBatchData.errorDetail = JSON.stringify(err);
            return Promise.resolve(false);
        });
    }
    UpdateScheduleBatch(queueId, executionCode, scheduleBatchData) {
        return promises_shared_1.DummyPromise()
            .then(_ => {
            return Promise.resolve(true);
        })
            .catch((err) => {
            let errorMessage = typeof err === 'string' ? err : (err.message || err.description);
            logger.error('[BacthScheduleImplementation::UpdateScheduleBatch] executionCode:', executionCode, 'getting errorMessage:', errorMessage);
            scheduleBatchData.retry = scheduleBatchData.retry ? scheduleBatchData.retry + 1 : 0;
            scheduleBatchData.errorFunction = 'UpdateScheduleBatch';
            scheduleBatchData.errorMessage = errorMessage;
            scheduleBatchData.errorDetail = JSON.stringify(err);
            if (scheduleBatchData.execution.status === 'CANCELLED_BY_NO_INTERACTIONS_FOUND')
                scheduleBatchData.retry = config_1.RETRY_BATCH_SCHEDULE;
            return Promise.resolve(false);
        });
    }
}
exports.BatchScheduleImplementation = BatchScheduleImplementation;
