"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.DeepgramServiceImplementation = void 0;
const moment = require("moment");
const log4js = require("log4js");
const redis_service_1 = require("../db/redis.service");
const promises_shared_1 = require("../shared/promises.shared");
const arrays_shared_1 = require("../shared/arrays.shared");
const transcriptions_download_implementation_1 = require("./transcriptions-download.implementation");
const task_config_1 = require("../config/task.config");
const sdk_1 = require("@deepgram/sdk");
const server_config_1 = require("../config/server.config");
const transcriptions_upload_implementation_1 = require("./transcriptions-upload.implementation");
const deepgram = sdk_1.createClient(task_config_1.DEEPGRAM_KEY);
let logger = log4js.getLogger('Task');
const CURRENT_SYSTEM_SERVICE = 'TRANSCRIPTION_SERVICE';
const CURRENT_IMPLEMENTATION = 'DeepgramServiceImplementation';
class DeepgramServiceImplementation {
    constructor() {
        this.executionCode = null;
        this.currentQueueId = null;
        this.availableServers = task_config_1.DEEPGRAM_SERVERS;
    }
    GetEngineFreeStorage(executionCode) {
        let logLabel = `[${CURRENT_IMPLEMENTATION}::GetEngineFreeStorage] `;
        return promises_shared_1.DummyPromise()
            .then(_ => {
            return Promise.resolve(true);
        });
    }
    UploadAudio(interaction, stream, dictionaryContent, currentQueueId) {
        let logLabel = `[${CURRENT_IMPLEMENTATION}::UploadAudio] [vcc::id => ${interaction.vcc}::${interaction.id}]: `;
        let isSync = interaction.retry ? true : false;
        return promises_shared_1.DummyPromise()
            .then(_ => {
            return Promise.resolve(stream);
        })
            .then((file) => {
            let config = {
                model: 'nova-2',
                language: interaction.language,
                smart_format: true,
                punctuate: true,
                multichannel: true
            };
            if (interaction.channels === 'mono') {
                config.diarize = true;
                interaction.isDiariced = true;
            }
            if (dictionaryContent.length) {
                let replaces = [];
                let keywords = [];
                dictionaryContent.forEach(dc => {
                    if (typeof dc === 'object') {
                        dc.sounds_like.forEach(sl => replaces.push(`${dc.content}:${sl}`));
                        keywords.push(`${dc.content}:`);
                    }
                    else
                        keywords.push(`${dc}:`);
                });
                if (replaces.length)
                    config.replace = replaces;
                if (keywords.length)
                    config.keywords = keywords;
            }
            if (isSync) {
                const promiseDeepgram = async () => {
                    return (await deepgram.listen.prerecorded.transcribeFile(file, config));
                };
                return promiseDeepgram()
                    .then(result => {
                    if (result) {
                        let transcriptionsDownload = new transcriptions_download_implementation_1.TranscriptionsDownloadImplementation();
                        return transcriptionsDownload.DownloadTranscription(interaction, result.result, this.currentQueueId, this.executionCode);
                    }
                    else
                        throw 'TRANSCRIPTION_FAILED';
                });
            }
            else {
                let transcriptionsUpload = new transcriptions_upload_implementation_1.TranscriptionsUploadImplementation();
                let url = `${server_config_1.WEBHOOK_URL}/deepgram/webhook/`;
                url += `?token=${transcriptionsUpload.GetSystemUserToken()}`;
                url += `&speech=${server_config_1.EXTERNAL_URL}/api/public/task/update_transcription_queue/${currentQueueId}/`;
                const promiseDeepgram = async () => {
                    return (await deepgram.listen.prerecorded.transcribeFileCallback(file, new sdk_1.CallbackUrl(url), config));
                };
                return promiseDeepgram()
                    .then(result => {
                    console.log('ResultX:', result);
                    if (result) {
                        logger.info(`${logLabel}executionCode: ${this.executionCode} getting queueId: ${this.currentQueueId} TRANSCRIBBED`);
                        interaction['status'] = 3;
                        interaction['statusDescription'] = 'pendingTranscription';
                        interaction['retry'] = 0;
                        interaction['uploadedDate'] = moment().format('x');
                        interaction['errorFunction'] = null;
                        interaction['errorMessage'] = null;
                        interaction['errorDetail'] = null;
                        return Promise.resolve(interaction);
                    }
                    else
                        throw 'TRANSCRIPTION_FAILED';
                });
            }
        })
            .then(result => {
            if (!result)
                throw 'SAVE_CONTENT_FAILED';
            return Promise.resolve(interaction);
        })
            .catch((err) => {
            let errorMessage = typeof err === 'string' ? err : 'ERROR';
            logger.error(`${logLabel}executionCode: ${this.executionCode} getting queueId: ${this.currentQueueId} ${errorMessage}`);
            if (typeof errorMessage === 'string'
                &&
                    errorMessage.includes('Too Many Requests'))
                interaction['retry'] = 0;
            else
                interaction['retry'] = interaction['retry'] ? interaction['retry'] + 1 : 1;
            interaction['status'] = -3;
            interaction['errorFunction'] = 'UploadAudio';
            interaction['errorMessage'] = errorMessage;
            interaction['errorDetail'] = errorMessage;
            return Promise.resolve(interaction);
        });
    }
    DownloadTranscription(interaction, currentQueueId) {
        return promises_shared_1.DummyPromise();
    }
    StandardizeResponse(rawResponse, isDiariced) {
        let processedResponse = {
            format: '2.8',
            job: {
                created_at: rawResponse.metadata.created,
                data_name: '',
                duration: rawResponse.metadata.duration
            },
            metadata: {
                created_at: moment().utc().format('YYYY-MM-DDTHH:mm:ss.000Z'),
                language_pack_info: {
                    adapted: false,
                    itn: true,
                    language_description: 'Spanish',
                    word_delimiter: ' ',
                    writing_direction: 'left-to-right',
                },
                transcription_config: {
                    channel_diarization_labels: ['Agent', 'Caller'],
                    diarization: 'channel',
                    language: 'es'
                },
                type: 'transcription'
            },
            results: []
        };
        if (rawResponse.results?.channels) {
            let channel0 = rawResponse.results.channels[0];
            if (channel0?.alternatives[0]) {
                let clientItems = channel0.alternatives[0];
                clientItems.words.forEach(w => {
                    if (isDiariced) {
                        if (w.speaker === 0)
                            w.channel = 'Caller';
                        else
                            w.channel = 'Agent';
                    }
                    else
                        w.channel = 'Caller';
                    let items = this.StringSplitter(w.punctuated_word);
                    items.forEach((item, i) => {
                        w.type = item.toLowerCase() === w.word ? 'word' : 'punctuation';
                        w.content = item;
                        processedResponse.results.push(this.GetItemObject(w));
                    });
                });
            }
            let channel1 = rawResponse.results.channels[1];
            if (channel1?.alternatives[0]) {
                let agentItems = channel1.alternatives[0];
                agentItems.words.forEach(w => {
                    w.channel = 'Agent';
                    let items = this.StringSplitter(w.punctuated_word);
                    items.forEach((item, i) => {
                        w.type = item.toLowerCase() === w.word ? 'word' : 'punctuation';
                        w.content = item;
                        processedResponse.results.push(this.GetItemObject(w));
                    });
                });
            }
        }
        processedResponse.results = arrays_shared_1.SortObjectArrayByValue(processedResponse.results, 'start_time', 'A');
        return processedResponse;
    }
    StringSplitter(string) {
        const regex = /([a-zA-ZáéíóúüñÁÉÍÓÚÜÑ]+|[.,;!?])/g;
        const result = string.match(regex);
        return result || [];
    }
    GetItemObject(data) {
        let word = {
            alternatives: [
                {
                    confidence: data.confidence,
                    content: data.content,
                    language: 'es'
                }
            ],
            channel: data.channel,
            end_time: data.end,
            start_time: data.start,
            type: data.type
        };
        return word;
    }
    GetTranscriptionApi(index) {
        let apiIndex = index % this.availableServers.length;
        let server = this.availableServers[apiIndex];
        return server;
    }
    ReinsertQueue(interaction, currentQueueId) {
        let logLabel = `[${CURRENT_IMPLEMENTATION}::ReinsertQueue] `;
        return promises_shared_1.DummyPromise()
            .then(_ => {
            interaction.statusDescription = 'pendingUpload';
            interaction.errorDetail = 'CALLBACK_FAILED';
            interaction.errorMessage = 'CALLBACK_FAILED';
            interaction.retry = 1;
            return redis_service_1.Redis.SaveProcessInteraction(`${interaction.vcc}:${interaction.processId}`, currentQueueId, interaction);
        })
            .then(_ => {
            return redis_service_1.Redis.DeleteInsertOnList('TranscriptionsStandbyExecute', 'TranscriptionsUpload', currentQueueId);
        })
            .catch((err) => {
            let errorMessage = typeof err === 'string' ? err : (err.message || err.description);
            logger.error(`${logLabel}executionCode:`, this.executionCode, 'getting queueId:', currentQueueId, 'getting error:', errorMessage);
            return Promise.resolve(false);
        });
    }
}
exports.DeepgramServiceImplementation = DeepgramServiceImplementation;
