"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.GenerativeAnalysisImplementation = void 0;
const tiktoken_1 = require("tiktoken");
const log4js = require("log4js");
const mssql_service_1 = require("../db/mssql.service");
const billing_batch_model_1 = require("../models/billing-batch.model");
const promises_shared_1 = require("../shared/promises.shared");
const redis_service_1 = require("../db/redis.service");
const config_1 = require("../config/config");
const file_manager_1 = require("../shared/file-manager");
const semantic_analysis_download_implementation_1 = require("./semantic-analysis-download.implementation");
const moment = require("moment");
const billing_implementation_1 = require("./billing.implementation");
const generative_service_implementation_1 = require("./generative-service.implementation");
const integration_config_1 = require("../config/integration.config");
let logger = log4js.getLogger('Task');
const CURRENT_SYSTEM_SERVICE = 'GENERATIVE_ANALYSIS';
const CURRENT_IMPLEMENTATION = 'GenerativeAnalysisImplementation';
class GenerativeAnalysisImplementation {
    constructor() {
        this.errorData = new billing_batch_model_1.BatchError();
        this.queueBase = 'GenerativeAnalysis';
    }
    Execute(executionCode, rateRecursive, availableBatch, cancelledBatch) {
        let logLabel = `[${CURRENT_IMPLEMENTATION}::${this.Execute.name}] [executionCode => ${executionCode}]: `;
        let currentBlockPromise = 'BLOCK_BEFORE_PROMISE';
        let executeIndex = 1;
        let ratio = config_1.RATIO_GENERATIVE_ANALYSIS;
        let alowedRatio = 0;
        let notAlowedRatio = 0;
        let executeResult = true;
        return promises_shared_1.DummyPromise()
            .then(_ => {
            currentBlockPromise = 'BLOCK_01_GET_FROM_REDIS';
            return redis_service_1.Redis.GetLengthFromList(this.queueBase);
        })
            .then(result => {
            currentBlockPromise = 'BLOCK_02_GET_QUEUE_ITEMS';
            if (!result)
                throw 'SERVER_INFO_REDIS_KEYS_FOR_GENERATIVE_ANALYSIS_NOT_FOUND';
            return redis_service_1.Redis.LRange(this.queueBase, 0, ratio - 1);
        })
            .then((result) => {
            currentBlockPromise = 'BLOCK_03_LAUNCHING_EXECUTIONS';
            let promiseArray = [];
            ratio = result?.length <= ratio ? result.length : 0;
            logger.info(`${logLabel}generativeAnalysis lrange result: ${ratio}`);
            result.forEach(r => {
                let arrQueue = r.split(':');
                if (availableBatch.find(b => b.VCC === arrQueue[0]
                    &&
                        b.BatchTypeId.toLowerCase() === arrQueue[7].toLowerCase()))
                    alowedRatio++;
                else
                    notAlowedRatio++;
            });
            while (ratio >= executeIndex) {
                promiseArray.push(this.GenerativeAnalysisExecute(executionCode, availableBatch, cancelledBatch));
                executeIndex++;
            }
            return Promise.all(promiseArray);
        })
            .then((results) => {
            currentBlockPromise = 'BLOCK_04_RECHECKING_QUEUE';
            if (results.filter(item => item === false).length > notAlowedRatio)
                executeResult = false;
            return redis_service_1.Redis.LRange(this.queueBase, 0, ratio - 1);
        })
            .then((result) => {
            currentBlockPromise = 'BLOCK_05_RELAUNCHING_EXECUTE_IF_NEEDED';
            if (result?.length && rateRecursive >= 1)
                return this.Execute(executionCode, rateRecursive - 1, availableBatch, cancelledBatch);
            return this.SaveGenerativeResultFile();
        })
            .then(_ => {
            return Promise.resolve(executeResult);
        })
            .then(result => {
            currentBlockPromise = 'BLOCK_06_RESOLVING_IMPL';
            let currentResult = executeResult && result ? true : false;
            return Promise.resolve(currentResult);
        })
            .catch((err) => {
            let errorMessage = typeof err === 'string' ? err : (err.message || err.description);
            logLabel += `Block ${currentBlockPromise} `;
            logger.error(`${logLabel}getting error: ${errorMessage}`);
            return Promise.resolve(false);
        });
    }
    GenerativeAnalysisExecute(executionCode, availableBatch, cancelledBatch) {
        let logLabel = `[${CURRENT_IMPLEMENTATION}::${this.GenerativeAnalysisExecute.name}] [executionCode => ${executionCode}]: `;
        let currentBlockPromise = 'BLOCK_BEFORE_PROMISE';
        let vcc = '';
        let processId = '';
        let interactionType = '';
        let batchType = '';
        let batchTypeId = '';
        let currentQueueId = '';
        let interactionModel = null;
        let queueProgressStorage = '';
        let filePath = '';
        return redis_service_1.Redis.RemoveQueueL(this.queueBase)
            .then((result) => {
            currentBlockPromise = 'BLOCK_01_VALIDATING_QUEUE';
            let arrQueue = result.split(':');
            let promiseArray = [];
            if (availableBatch.find(b => b.VCC === arrQueue[0] && b.BatchTypeId.toLowerCase() === arrQueue[8].toLowerCase())) {
                promiseArray.push(Promise.resolve('ALLOWED'));
                promiseArray.push(redis_service_1.Redis.CreateQueueL(`${this.queueBase}Execute`, result));
                promiseArray.push(Promise.resolve(result));
            }
            else {
                promiseArray.push(Promise.resolve('NOT_ALLOWED'));
                promiseArray.push(redis_service_1.Redis.CreateQueueL(this.queueBase, result));
                promiseArray.push(mssql_service_1.MSSql.SaveBatchStatus(arrQueue[0], arrQueue[8], 'GENERATIVE_ANALYSIS', 'PAUSED'));
            }
            return Promise.all(promiseArray);
        })
            .then((result) => {
            currentBlockPromise = 'BLOCK_02_GETTING_QUEUE_DATA_FROM_REDIS';
            if (result && result[0] === 'NOT_ALLOWED')
                throw 'SERVER_ERROR_NOT_ALLOWED_QUEUE';
            let queueId = result[2];
            if (queueId?.length > 0) {
                currentQueueId = queueId;
                logLabel += `[queueId => ${currentQueueId}] `;
                logger.info(`${logLabel}START`);
                let item = currentQueueId.split(':');
                if (item?.length) {
                    vcc = item[0];
                    processId = item[1];
                    interactionType = item[2];
                    batchType = item[7] ? item[7] : null;
                    batchTypeId = item[8] ? item[8] : null;
                    if (batchType) {
                        queueProgressStorage = `${vcc}:${processId}:${batchType}`;
                        if (batchTypeId)
                            queueProgressStorage += `:${batchTypeId}`;
                    }
                    logger.info(`${logLabel}storage: ${queueProgressStorage}`);
                    return redis_service_1.Redis.GetModelInteraction(currentQueueId);
                }
                else
                    logger.error(`${logLabel}QUEUE_FORMAT_ERROR`);
            }
            return Promise.resolve(null);
        })
            .then((result) => {
            currentBlockPromise = 'BLOCK_03_ASSIGNING_EVALUATION_DATA';
            if (!result)
                logger.warn(`${logLabel}getting error: NOT_FOUND_INTERACTION_MODEL`);
            if (cancelledBatch.find(b => b.VCC === result.vcc && b.BatchTypeId.toLowerCase() === (result.studyId ? result.studyId : result.evaluationPlanId).toLowerCase())) {
                result.status = -2;
                result.statusDescription = 'REMOVED_BY_CANCELLED';
                result.retry = config_1.RETRY_GENERATIVE_ANALYSIS;
                result.errorFunction = 'CognitiveAnalysisExecute';
                result.errorMessage = 'REMOVED_BY_CANCELLED';
                result.errorDetail = 'REMOVED_BY_CANCELLED';
            }
            interactionModel = result;
            return file_manager_1.CreateFolder(`${config_1.GENERATIVE_ANALYSIS_FOLDER}/${vcc}`);
        })
            .then((result) => {
            currentBlockPromise = 'BLOCK_03_ASSIGNING_EVALUATION_DATA';
            if (!result)
                throw 'ERROR_CREATE_COGNITIVE_ANALYSIS_FOLDER';
            if (interactionModel?.statusDescription === 'pendingPhrases') {
                logger.info(`${logLabel}pendingPhrases`);
                return this.GetPhrasesFromIdathaJson(interactionModel, currentQueueId, executionCode);
            }
            else if (interactionModel?.statusDescription === 'pendingNLU') {
                logger.info(`${logLabel}pendingNLU`);
                return this.GetEvaluation(interactionModel, currentQueueId, executionCode);
            }
            return Promise.resolve(null);
        })
            .then((resultInteraction) => {
            currentBlockPromise = 'BLOCK_04_UPDATTING_EVALUATION_DATA';
            if (interactionModel)
                return redis_service_1.Redis.SaveModelInteraction(currentQueueId, interactionModel);
            return Promise.resolve(null);
        })
            .then((result) => {
            let promiseArray = [];
            if (result
                &&
                    interactionModel?.status === 6
                &&
                    interactionModel?.statusDescription === 'ready') {
                logger.info(`${logLabel}getting queueId: ${currentQueueId} READY`);
                this.CloseModelAnalysis(currentQueueId, true, 'SUCCED');
                promiseArray.push(redis_service_1.Redis.DeleteQueue(`${this.queueBase}Execute`, currentQueueId));
                if (batchType === 'tipification') {
                    interactionModel.statusDescription = 'pendingAnalysis';
                    promiseArray.push(redis_service_1.Redis.CreateQueueL('TipificationAnalysis', currentQueueId));
                    promiseArray.push(redis_service_1.Redis.SaveTipificationInteraction(currentQueueId, interactionModel));
                }
            }
            else if (result
                &&
                    interactionModel?.status === 6
                &&
                    interactionModel?.statusDescription === 'pendingNLU'
                &&
                    interactionModel.retry < config_1.RETRY_GENERATIVE_ANALYSIS) {
                promiseArray.push(redis_service_1.Redis.CreateQueueL(this.queueBase, currentQueueId));
                promiseArray.push(redis_service_1.Redis.DeleteQueue(`${this.queueBase}Execute`, currentQueueId));
            }
            else if (result
                &&
                    interactionModel?.status === -2
                &&
                    interactionModel.statusDescription === 'REMOVED_BY_CANCELLED') {
                logger.error(`${logLabel}getting queueId: ${currentQueueId} REMOVED_BY_CANCELLED`);
                this.CloseModelAnalysis(currentQueueId, false, 'REMOVED_BY_CANCELLED');
                promiseArray.push(redis_service_1.Redis.DeleteQueue(`${this.queueBase}Execute`, currentQueueId));
            }
            else if (result
                &&
                    interactionModel?.status === -6
                &&
                    interactionModel.statusDescription === 'ERROR_FILE_JSON') {
                logger.error(`${logLabel}getting queueId: ${currentQueueId} ERROR_FILE_JSON`);
                this.CloseModelAnalysis(currentQueueId, false, 'JSON_PROBLEM');
                promiseArray.push(redis_service_1.Redis.DeleteQueue(`${this.queueBase}Execute`, currentQueueId));
            }
            else if (interactionModel?.retry >= config_1.RETRY_GENERATIVE_ANALYSIS) {
                logger.error(`${logLabel}getting queueId: ${currentQueueId} REMOVED_BY_RETRIES`);
                this.CloseModelAnalysis(currentQueueId, false, 'REMOVED_BY_RETRIES');
                promiseArray.push(redis_service_1.Redis.DeleteQueue(`${this.queueBase}Execute`, currentQueueId));
            }
            else if (currentQueueId && !interactionModel) {
                logger.error(`${logLabel}getting queueId: ${currentQueueId} INTERACTION_DO_NOT_EXIST`);
                this.CloseModelAnalysis(currentQueueId, false, 'INTERACTION_DO_NOT_EXIST');
                promiseArray.push(redis_service_1.Redis.DeleteQueue(`${this.queueBase}Execute`, currentQueueId));
            }
            else
                logger.error(`${logLabel}getting queueId: ${currentQueueId} QUEUE_WAS_NOT_REMOVED`);
            return Promise.all(promiseArray);
        })
            .then(_ => {
            let promiseArray = [];
            if (interactionModel?.status === 6
                &&
                    interactionModel?.statusDescription === 'ready') {
                promiseArray.push(redis_service_1.Redis.UpdateProgress(queueProgressStorage));
                promiseArray.push(redis_service_1.Redis.DeleteModelInteraction(currentQueueId));
                switch (batchType) {
                    case 'study':
                    case 'template':
                        logger.debug(`${logLabel}getting queueId: ${currentQueueId} Saving cognitive result for study:`, interactionModel);
                        promiseArray.push(redis_service_1.Redis.SaveReadyModelInteraction(currentQueueId, interactionModel, executionCode));
                        break;
                    case 'ability':
                    case 'tipification':
                        logger.debug(`${logLabel}getting queueId: ${currentQueueId} Saving cognitive result for evaluationplan:`, interactionModel);
                        promiseArray.push(redis_service_1.Redis.SaveReadyModelAbility(currentQueueId, interactionModel, executionCode));
                        break;
                    default:
                        logger.error(`${logLabel}etting queueId:`, currentQueueId, 'TYPE_IS_NOT_FOUND', batchType);
                }
            }
            else if (interactionModel?.retry >= config_1.RETRY_GENERATIVE_ANALYSIS) {
                promiseArray.push(redis_service_1.Redis.UpdateProgress(queueProgressStorage));
                promiseArray.push(redis_service_1.Redis.UpdateErrorProgress(queueProgressStorage));
                promiseArray.push(redis_service_1.Redis.DeleteModelInteraction(currentQueueId));
                switch (batchType) {
                    case 'study':
                    case 'template':
                        logger.error(`${logLabel}getting queueId: ${currentQueueId} Saving cognitive result for study:`, interactionModel);
                        promiseArray.push(redis_service_1.Redis.SaveReadyModelInteraction(currentQueueId, interactionModel, executionCode));
                        break;
                    case 'ability':
                    case 'tipification':
                        logger.error(`${logLabel}getting queueId: ${currentQueueId} Saving cognitive result for evaluationplan:`, interactionModel);
                        promiseArray.push(redis_service_1.Redis.SaveReadyModelAbility(currentQueueId, interactionModel, executionCode));
                        break;
                    default:
                        logger.error(`${logLabel}getting queueId: ${currentQueueId} TYPE_IS_NOT_FOUND`, batchType);
                        break;
                }
                promiseArray.push(mssql_service_1.MSSql.SaveBatchError(vcc, CURRENT_SYSTEM_SERVICE, batchTypeId, ['study', 'template'].includes(batchType) ? batchTypeId : null, CURRENT_IMPLEMENTATION, interactionModel.id, interactionModel.modelId, this.errorData.errorFunction, this.errorData.errorMessage, this.errorData.errorDetail));
            }
            return Promise.all(promiseArray);
        })
            .then(_ => {
            return Promise.resolve(true);
        })
            .catch((err) => {
            let errorMessage = typeof err === 'string' ? err : (err.message || err.description);
            logLabel += `Block ${currentBlockPromise} `;
            logger.error(`${logLabel}getting error: ${errorMessage}`);
            return Promise.resolve(false);
        });
    }
    GetPhrasesFromIdathaJson(interaction, currentQueueId, executionCode) {
        let logLabel = `[${CURRENT_IMPLEMENTATION}::${this.GetPhrasesFromIdathaJson.name}] [executionCode::currentQueueId => ${executionCode}::${currentQueueId}]: `;
        let currentBlockPromise = 'BLOCK_BEFORE_PROMISE';
        let existsDocument = false;
        let totalPhrases = 0;
        let semanticAnalyisDownloadImpl = new semantic_analysis_download_implementation_1.SemanticAnalysisDownloadImplementation();
        return promises_shared_1.DummyPromise()
            .then(_ => {
            return semanticAnalyisDownloadImpl.GetJsonForModelAnalysis(interaction, 'GENERATIVE');
        })
            .then((result) => {
            if (!result)
                throw 'ERROR_NOT_FOUND_FILE_JSON';
            if (result?.existsDocument) {
                existsDocument = result.existsDocument;
                totalPhrases = result.phrases.length;
                delete (result.existsDocument);
                return Promise.resolve(null);
            }
            return semanticAnalyisDownloadImpl.SaveJsonForModelAnalysis(interaction, result, 'GENERATIVE');
        })
            .then(result => {
            if (result?.status || existsDocument) {
                interaction['status'] = 6;
                interaction['totalPhrases'] = interaction['totalPhrases'] ? interaction['totalPhrases'] : totalPhrases;
                interaction['statusDescription'] = 'pendingNLU';
                interaction['retry'] = 0;
                interaction['uploadedDate'] = moment().format('x');
                interaction['errorMessage'] = null;
                this.errorData.CleanErrorData();
            }
            else {
                logger.warn(`${logLabel}executionCode:`, executionCode, 'getting queueId:', currentQueueId, 'getting error:', 'NOT_FOUND_PHASES');
                interaction['status'] = -6;
                interaction['retry'] = interaction['retry'] ? interaction['retry'] + 1 : 1;
                interaction['uploadedDate'] = moment().format('x');
                interaction['errorFunction'] = this.GetPhrasesFromIdathaJson.name;
                interaction['errorMessage'] = 'NOT_FOUND_PHASES';
                this.errorData.SetErrorData(this.GetPhrasesFromIdathaJson.name, interaction['errorMessage'], interaction['errorMessage']);
            }
            return Promise.resolve(interaction);
        })
            .catch((err) => {
            let errorMessage = typeof err === 'string' ? err : (err.message || err.description);
            logger.error(`${logLabel}executionCode:`, executionCode, 'getting queueId:', currentQueueId, 'getting error:', errorMessage);
            interaction['status'] = -6;
            interaction['retry'] = interaction['retry'] ? interaction['retry'] + 1 : 1;
            interaction['uploadedDate'] = moment().format('x');
            interaction['errorMessage'] = errorMessage;
            this.errorData.SetErrorData(this.GetPhrasesFromIdathaJson.name, errorMessage, err);
            return Promise.resolve(interaction);
        });
    }
    GetEvaluation(interaction, currentQueueId, executionCode) {
        let logLabel = `[${CURRENT_IMPLEMENTATION}::${this.GetEvaluation.name}] [executionCode::currentQueueId => ${executionCode}::${currentQueueId}]: `;
        let currentBlockPromise = 'BLOCK_BEFORE_PROMISE';
        let item = currentQueueId.split(':');
        let type = item[7] ? item[7] : null;
        let path = '';
        let isLast = false;
        return promises_shared_1.DummyPromise()
            .then(_ => {
            currentBlockPromise = 'BLOCK_01_FIND_INTERACTION_JSON';
            let regex = /\.(mp3|wav|ogg|json)$/;
            let jsonFileName = interaction.fileName.replace(regex, '.json');
            path = `${config_1.GENERATIVE_ANALYSIS_FOLDER}/${interaction.vcc}/${jsonFileName}`;
            return this.GetModelResults(interaction, path, currentQueueId, executionCode);
        })
            .then(result => {
            currentBlockPromise = 'BLOCK_02_EVALUATE';
            if (!result || !result.status)
                throw 'ERROR_SAVE_DOCUMENT';
            isLast = result.isLast;
            if (isLast && type !== 'tipification')
                return this.Evaluate(interaction, path, executionCode, currentQueueId);
            return Promise.resolve(null);
        })
            .then((result) => {
            currentBlockPromise = 'BLOCK_03_RESOLVING_RESULT';
            if (isLast) {
                interaction.succedItems = result ? result : [];
                if (!result)
                    logger.warn(`${logLabel}getting error:`, 'NOT_FOUND_SCORE');
            }
            return Promise.resolve(true);
        })
            .then(result => {
            if (result) {
                interaction['status'] = 6;
                interaction['statusDescription'] = isLast ? 'ready' : 'pendingNLU';
                interaction['totalReadPhrases'] = interaction['totalReadPhrases'] ? interaction['totalReadPhrases'] + 1 : 1;
                interaction['retry'] = 0;
                interaction['uploadedDate'] = moment().format('x');
                interaction['errorMessage'] = null;
                this.errorData.CleanErrorData();
            }
            return Promise.resolve(interaction);
        })
            .catch((err) => {
            let errorMessage = typeof err === 'string' ? err : (err.message || err.description);
            if (errorMessage === 'NOT_FOUND_PHRASE') {
                logger.warn(`${logLabel}getting error:`, errorMessage);
                interaction['status'] = 6;
                interaction['statusDescription'] = 'ready';
                interaction['totalReadPhrases'] = interaction['totalReadPhrases'] ? interaction['totalReadPhrases'] + 1 : 1;
                interaction['retry'] = 0;
            }
            else if (errorMessage === 'OUT_OF_VOLUME_QUOTA') {
                logger.error(`${logLabel}getting error:`, errorMessage);
                interaction['status'] = -6;
                interaction['retry'] = config_1.RETRY_GENERATIVE_ANALYSIS;
            }
            else {
                logger.error(`${logLabel}getting error:`, errorMessage);
                interaction['status'] = -6;
                interaction['retry'] = interaction['retry'] ? interaction['retry'] + 1 : 1;
            }
            interaction['uploadedDate'] = moment().format('x');
            interaction['errorMessage'] = errorMessage;
            this.errorData.SetErrorData(this.GetEvaluation.name, errorMessage, err);
            return Promise.resolve(interaction);
        });
    }
    GetModelResults(interaction, path, currentQueueId, executionCode) {
        let index = 0;
        let isLast = false;
        let alreadyEvaluated = false;
        return promises_shared_1.DummyPromise()
            .then(_ => {
            return file_manager_1.ReadFileContent(path, 'utf-8', true);
        })
            .then(result => {
            let jsonContent;
            try {
                jsonContent = typeof result === 'string' ? JSON.parse(result) : result;
            }
            catch (ex) {
                jsonContent = null;
                this.errorData.SetErrorData(this.GetModelResults.name, 'ERROR_FILE_JSON', ex);
                throw 'ERROR_FILE_JSON';
            }
            return Promise.resolve(jsonContent);
        })
            .then(result => {
            if (!result)
                throw 'ERROR_NOT_FOUND_FILE_JSON';
            if (result.phrases)
                alreadyEvaluated = !result.phrases.map(p => p[interaction.modelId] ? true : false).includes(false) ? true : false;
            index = interaction.totalReadPhrases ? interaction.totalReadPhrases : 0;
            isLast = interaction.totalPhrases - 1 <= index ? true : false;
            return this.GetResultsFromGenerativeEngine(interaction, result, currentQueueId, executionCode);
        })
            .then(result => {
            return Promise.resolve({ status: result ? true : false, isLast });
        })
            .catch(err => {
            let errorMessage = typeof err === 'string' ? err : (err.message || err.description);
            logger.info(`[CognitiveAnalysisImplementation::GetNluResults] executionCode: ${executionCode} getting queueId: ${currentQueueId} getting error: ${errorMessage}`);
            return Promise.reject(err);
        });
    }
    GetResultsFromGenerativeEngine(interaction, content, currentQueueId, executionCode) {
        let logLabel = `[${CURRENT_IMPLEMENTATION}::${this.GetResultsFromGenerativeEngine.name}] [interactionId::modelName => ${interaction.id}::${interaction.modelId}]: `;
        let phrase = '';
        let index = 0;
        let billing = new billing_implementation_1.BillingImplementation();
        return promises_shared_1.DummyPromise()
            .then(_ => {
            index = interaction.totalReadPhrases ? interaction.totalReadPhrases : 0;
            phrase = content.phrases[index]?.content ? content.phrases[index].content : null;
            logger.info(`${logLabel}evaluating index ${index}: ${phrase}`);
            if (!phrase)
                throw 'NOT_FOUND_PHRASE';
            return Promise.resolve(true);
        })
            .then(_ => {
            billing.AddUpdateModelAnalysisDetailToMainFile(currentQueueId, 'GENERATIVE', 'GenaiApi', integration_config_1.GENERATIVE_API_URL);
            billing.AddUpdateModelAnalysisDetailToMainFile(currentQueueId, 'GENERATIVE', 'TotalTokens', this.CalculateTokens(phrase));
            let generativeServiceImpl = new generative_service_implementation_1.GenerativeServiceImplementation(interaction.vcc);
            return generativeServiceImpl.Query(interaction.appId, phrase);
        })
            .then(result => {
            delete (result.query);
            delete (result.topScoringIntent);
            billing.AddUpdateModelAnalysisDetailToMainFile(currentQueueId, 'GENERATIVE', 'TotalValidRequest', 1);
            if (result.intents?.length)
                result.intents.forEach(i => i.intent = i.intent.replace(/[\[\]]/g, ''));
            return redis_service_1.Redis.SaveGenerativeResult(interaction, index, result);
        })
            .then(result => {
            if (!result)
                throw 'SERVER_ERROR_GENERATIVE_REQUEST_RESULT_NOT_FOUND';
            return Promise.resolve(true);
        })
            .catch(err => {
            let errorMessage = typeof err === 'string' ? err : (err.message || err.description);
            logger.error(`${logLabel}getting error: ${errorMessage}`);
            billing.AddUpdateModelAnalysisDetailToMainFile(currentQueueId, 'GENERATIVE', 'TotalInvalidRequest', 1);
            return Promise.reject(err);
        });
    }
    SaveGenerativeResultFile() {
        let logLabel = `[${CURRENT_IMPLEMENTATION}::${this.SaveGenerativeResultFile.name}] `;
        let storages = [];
        return promises_shared_1.DummyPromise()
            .then(_ => {
            return redis_service_1.Redis.GetAllGenerativeResultStorages();
        })
            .then((result) => {
            if (!result.length)
                throw 'SERVER_ERROR_GENERATIVE_RESULTS_TO_SAVE_NOT_FOUND';
            logger.info(`${logLabel}found ${result.length} storages`);
            let promiseArray = [];
            result.forEach(s => {
                storages.push({ storage: s });
                promiseArray.push(redis_service_1.Redis.GetGenerativeResult(s));
            });
            return Promise.all(promiseArray);
        })
            .then((result) => {
            result.forEach((r, i) => storages[i].items = r);
            return Promise.resolve(true);
        })
            .then(_ => {
            logger.info(`${logLabel}updating ${storages.length} files`);
            let promiseArray = [];
            storages.forEach(s => promiseArray.push(this.UpdateGenerativeFile(s)));
            return Promise.all(promiseArray);
        })
            .then((result) => {
            let promiseArray = [];
            result.forEach((r, i) => {
                if (r)
                    promiseArray.push(redis_service_1.Redis.DeleteGenerativeResult(storages[i].storage));
            });
        })
            .then(_ => {
            return Promise.resolve(true);
        })
            .catch(err => {
            let errorMessage = typeof err === 'string' ? err : (err.message || err.description);
            logger.error(`${logLabel}getting error:`, errorMessage);
            return Promise.resolve(false);
        });
    }
    UpdateGenerativeFile(storage) {
        let logLabel = `[${CURRENT_IMPLEMENTATION}::${this.UpdateGenerativeFile.name}] `;
        let path = `${config_1.GENERATIVE_ANALYSIS_FOLDER}/${storage.storage.replace(':', '/')}.json`;
        let jsonContent = null;
        return promises_shared_1.DummyPromise()
            .then(_ => {
            return file_manager_1.ReadFileContent(path, 'utf-8', true);
        })
            .then(result => {
            try {
                jsonContent = typeof result === 'string' ? JSON.parse(result) : result;
            }
            catch (ex) {
                jsonContent = null;
                throw 'ERROR_FILE_JSON';
            }
            return Promise.resolve(true);
        })
            .then(_ => {
            storage.items.forEach(i => {
                jsonContent.phrases[i.index][i.model] = i.results;
            });
            return file_manager_1.WriteFileContent(path, JSON.stringify(jsonContent), 'utf-8');
        })
            .catch(err => {
            let errorMessage = typeof err === 'string' ? err : (err.message || err.description);
            logger.error(`${logLabel}getting error:`, errorMessage);
            return Promise.resolve(false);
        });
    }
    Evaluate(interaction, path, executionCode, currentQueueId) {
        let logLabel = `[${CURRENT_IMPLEMENTATION}::${this.Evaluate.name}] [executionCode::currentQueueId => ${executionCode}::${currentQueueId}] `;
        let studyTemplateThemes = [];
        let evaluationPlanAbilities = [];
        return promises_shared_1.DummyPromise()
            .then(_ => {
            if (['STUDY', 'TEMPLATE'].includes(interaction.batchTypeName))
                return redis_service_1.Redis.GetStudyTemplateThemes(interaction.vcc, interaction.studyId);
            else
                return redis_service_1.Redis.GetEvaluationPlanAbilities(interaction.vcc, interaction.evaluationPlanId);
        })
            .then(result => {
            if (['STUDY', 'TEMPLATE'].includes(interaction.batchTypeName))
                studyTemplateThemes = result;
            else
                evaluationPlanAbilities = result;
            return file_manager_1.ReadFileContent(path, 'utf-8', true);
        })
            .then((resultFile) => {
            let jsonContent = null;
            try {
                if (resultFile) {
                    jsonContent = JSON.parse(resultFile);
                }
            }
            catch (ex) {
                logger.error(`${logLabel}getting error:`, ex);
                this.errorData.SetErrorData(this.Evaluate.name, 'ERROR_FILE_JSON', ex);
                throw 'ERROR_FILE_JSON';
            }
            return jsonContent;
        })
            .then((jsonContent) => {
            if (!jsonContent)
                throw 'ERROR_FILE_JSON';
            let phrases = jsonContent['phrases'];
            let succedItems = [];
            if (phrases?.length) {
                phrases.forEach((phrase, i) => {
                    let currentPhrase = phrase[interaction.modelId];
                    console.log(currentPhrase);
                    if (currentPhrase?.intents?.length) {
                        if (['STUDY', 'TEMPLATE'].includes(interaction.batchTypeName)) {
                            if (!studyTemplateThemes.length)
                                logger.warn(`${logLabel}}studyTemplateThemes is empty, need items to get succedItems`);
                            let templateThemes = interaction.templateThemes.split(',');
                            let succedTemplateThemeItems = [];
                            templateThemes.map(tt => {
                                let currentTemplateTheme = studyTemplateThemes.find(stt => stt.id === tt);
                                if (currentTemplateTheme) {
                                    let succedIntents = [];
                                    currentTemplateTheme.items.map(tti => {
                                        let intent = currentPhrase.intents.find(i => i.intent === tti.id.split('|')[1]);
                                        if (!intent)
                                            intent = currentPhrase.intents.find(i => i.intent === tti.query.split('|')[1]);
                                        if (intent) {
                                            if (intent.score >= tti.acceptancePercentage) {
                                                if (tti.entities && tti.entities.length > 0) {
                                                    let succeedEntities = [];
                                                    tti.entities.map(e => {
                                                        let entities = currentPhrase.entities.filter(en => en.type === e.entityName && en.score >= e.acceptancePercentage);
                                                        if (entities && entities.length) {
                                                            succeedEntities.push(entities[0]);
                                                        }
                                                    });
                                                    if (succeedEntities.length > 0) {
                                                        intent.entities = succeedEntities;
                                                        succedIntents.push(intent);
                                                        succedTemplateThemeItems.push(tti.id);
                                                    }
                                                }
                                                else {
                                                    intent.entities = [];
                                                    succedIntents.push(intent);
                                                    succedTemplateThemeItems.push(tti.id);
                                                }
                                            }
                                        }
                                    });
                                    logger.info(`${logLabel}succedIntents result:`, succedIntents);
                                    if (succedIntents.length > 0) {
                                        if (succedItems.find(stt => stt.id === tt)) {
                                            let succeedTemplateTheme = succedItems.find(stt => stt.id === tt);
                                            succeedTemplateTheme.intents.map(i => {
                                                let succeedIntent = succedIntents.find(si => si.intent === i.id.split('|')[1]);
                                                if (succeedIntent.score > i.score) {
                                                    i = succeedIntent;
                                                }
                                            });
                                        }
                                        else {
                                            succedItems.push({
                                                templateThemeId: tt,
                                                templateThemeItems: succedTemplateThemeItems,
                                                intents: succedIntents
                                            });
                                        }
                                    }
                                }
                            });
                        }
                        else {
                            if (!evaluationPlanAbilities.length)
                                logger.warn(`${logLabel}evaluationPlanAbilities is empty, need items to get succedItems`);
                            let abilities = interaction.abilities.split(',');
                            abilities.forEach(a => {
                                let currentAbility = evaluationPlanAbilities.find(epa => epa.idAbility === a);
                                if (currentAbility) {
                                    let succedIntents = [];
                                    currentAbility.queries.forEach(q => {
                                        let intent = currentPhrase.intents.find(i => i.intent === q.queryContent.split('|')[1]);
                                        if (!intent)
                                            intent = currentPhrase.intents.find(i => i.intent === q.queryId);
                                        if (intent) {
                                            if (intent.score >= q.acceptancePercentage) {
                                                succedIntents.push(intent);
                                            }
                                        }
                                    });
                                    logger.info(`${logLabel}phrase #${i} succedIntents for ability ${currentAbility.titleAbility} result:`, succedIntents);
                                    if (succedIntents.length > 0) {
                                        if (succedItems.find(sa => sa.abilityId === a)) {
                                            let succeedAbility = succedItems.find(sa => sa.abilityId === a);
                                            succeedAbility.intents.map(i => {
                                                let succeedIntent = succedIntents.find(si => si.intent === i.intent);
                                                if (succeedIntent.score > i.score) {
                                                    i = succeedIntent;
                                                }
                                            });
                                        }
                                        else {
                                            succedItems.push({
                                                abilityId: a,
                                                intents: succedIntents
                                            });
                                        }
                                    }
                                    else {
                                    }
                                }
                            });
                        }
                    }
                });
            }
            logger.info(`${logLabel}succedItems result:`, succedItems);
            this.errorData.CleanErrorData();
            return Promise.resolve(succedItems);
        })
            .catch((error) => {
            let errorMessage = typeof error === 'string' ? error : (error.message || error.description);
            logger.error(`${logLabel}getting error:`, errorMessage);
            this.errorData.SetErrorData(this.Evaluate.name, errorMessage, error);
            return Promise.resolve(null);
        });
    }
    CloseModelAnalysis(queueId, resultCode, resultDescription) {
        let billing = new billing_implementation_1.BillingImplementation();
        let myBatchName = '';
        let myQueueIdArray = queueId.split(':');
        switch (myQueueIdArray[7]) {
            case 'study':
                myBatchName = 'STUDY';
                break;
            case 'template':
                myBatchName = 'TEMPLATE';
                break;
            case 'ability':
                myBatchName = 'EVALUATION_PLAN';
                break;
            case 'tipification':
                myBatchName = 'EVALUATION_PLAN';
                break;
        }
        billing.AddCloseModelAnalysisDetailToMainFile(myQueueIdArray[0], myQueueIdArray[1], myQueueIdArray[6], myBatchName, myQueueIdArray[8], resultCode ? 1 : 0, resultDescription);
    }
    SaveUsageOnRedis(vcc, evaluationPlanId, interactionId, appId, tokens) {
        let logLabel = `[${CURRENT_IMPLEMENTATION}::${this.SaveUsageOnRedis.name}] [vcc::evaluationPlan::interactionId::appId => ${vcc}::${evaluationPlanId}::${interactionId}::${appId}]: `;
        logger.info(`${logLabel}Registering +${tokens} tokens`);
        let currentUsage = 0;
        return promises_shared_1.DummyPromise()
            .then(_ => {
            return redis_service_1.Redis.GetGenerativeUsage(vcc, evaluationPlanId, interactionId, appId);
        })
            .then(result => {
            if (result)
                currentUsage = result;
            return redis_service_1.Redis.SaveGenerativeUsage(vcc, evaluationPlanId, interactionId, appId, currentUsage + tokens);
        })
            .catch((error) => {
            let errorMessage = typeof error === 'string' ? error : (error.message || error.description);
            logger.error(`${logLabel} getting error: ${errorMessage}`);
            return Promise.resolve(false);
        });
    }
    CalculateTokens(phrase) {
        let logLabel = `[${CURRENT_IMPLEMENTATION}::${this.CalculateTokens.name}] [phrase => ${phrase}]: `;
        const enc = tiktoken_1.encoding_for_model("gpt-4-turbo-preview");
        const result = enc.encode(phrase);
        console.log(phrase + ':', result);
        logger.info(`${logLabel}Found tokens: ${result.length}`);
        return result.length;
    }
}
exports.GenerativeAnalysisImplementation = GenerativeAnalysisImplementation;
