"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.GenerativeServiceImplementation = void 0;
const log4js = require("log4js");
const mssql_service_1 = require("../db/mssql.service");
const redis_service_1 = require("../db/redis.service");
const integration_config_1 = require("../config/integration.config");
const billing_batch_model_1 = require("../models/billing-batch.model");
const generative_model_1 = require("../models/generative.model");
const promises_shared_1 = require("../shared/promises.shared");
const request_shared_1 = require("../shared/request.shared");
let logger = log4js.getLogger('Task');
const CURRENT_SYSTEM_SERVICE = 'GENERATIVE_SERVICE';
const CURRENT_IMPLEMENTATION = 'GenerativeServiceImplementation';
class GenerativeServiceImplementation {
    constructor(vcc) {
        this.vcc = null;
        this.urlBase = `${integration_config_1.GENERATIVE_API_PROTOCOL}://${integration_config_1.GENERATIVE_API_URL}`;
        this.url01 = `${this.urlBase}/rasa/api/qna/dialoguerouter`;
        this.url02 = `${this.urlBase}/qna/query/dialoguerouter`;
        this.apikey = null;
        this.retry = 0;
        this.maxRetries = 2;
        this.errorData = new billing_batch_model_1.BatchError();
        this.vcc = vcc;
    }
    AddApp(model) {
        let logLabel = `[${CURRENT_IMPLEMENTATION}::AddApp] [vcc::app => ${this.vcc}::${model.name}]: `;
        let requestId = null;
        return promises_shared_1.DummyPromise()
            .then(_ => {
            if (!this.apikey)
                return this.GetGenerativeEngineToken();
            else
                return Promise.resolve(true);
        })
            .then(_ => {
            let postOptions = {
                uri: this.url01,
                headers: {
                    'Content-Type': 'application/json',
                    'apikey': this.apikey
                },
                body: new generative_model_1.GenerativeCreate(model),
                validateErrorResponse: true,
                retryProtocol: false
            };
            return request_shared_1.SimplePostRequest(postOptions);
        })
            .then(result => {
            if (!result)
                throw 'SERVER_ERROR_CREATED_MODEL_FAILED';
            logger.info(`${logLabel} Request ID for model: ${result.id}`);
            requestId = result.id;
            return promises_shared_1.SleepPromise(2.5);
        })
            .then(_ => {
            return Promise.resolve(requestId);
        })
            .catch((err) => {
            let errorMessage = typeof err === 'string' ? err : 'ERROR';
            logger.error(`${logLabel} Error: `, err);
            if (typeof errorMessage === 'string'
                &&
                    errorMessage.includes('Name duplicated'))
                errorMessage = 'SERVER_ERROR_APP_NAME_ALREADY_EXISTS_ON_GENERATIVE_ENGINE';
            else if (err?.code === 'ERR_BAD_REQUEST')
                errorMessage = 'SERVER_ERROR_FAILED_FROM_GENERATIVE_SERVER';
            return Promise.reject(errorMessage);
        });
    }
    SaveApp(model) {
        let logLabel = `[${CURRENT_IMPLEMENTATION}::SaveApp] [vcc::app => ${this.vcc}::${model.name}]: `;
        let requestId = null;
        return promises_shared_1.DummyPromise()
            .then(_ => {
            if (!this.apikey)
                return this.GetGenerativeEngineToken();
            else
                return Promise.resolve(true);
        })
            .then(_ => {
            let putOptions = {
                uri: `${this.url01}/${model.id}/version`,
                headers: {
                    'Content-Type': 'application/json',
                    'apikey': this.apikey
                },
                body: new generative_model_1.GenerativeSave(model),
                retryProtocol: true
            };
            return request_shared_1.SimplePutRequest(putOptions);
        })
            .then(result => {
            if (!result)
                throw 'SERVER_ERROR_CREATED_MODEL_FAILED';
            logger.info(`${logLabel} Request ID for model: ${result.id}`);
            requestId = result.id;
            return promises_shared_1.SleepPromise(2.5);
        })
            .then(_ => {
            return Promise.resolve(requestId);
        })
            .catch((err) => {
            let errorMessage = typeof err === 'string' ? err : 'ERROR';
            logger.error(`${logLabel} Error: `, err);
            if (err?.code === 'ERR_BAD_REQUEST')
                errorMessage = 'SERVER_ERROR_FAILED_FROM_GENERATIVE_SERVER';
            return Promise.reject(errorMessage);
        });
    }
    GetAppById(id) {
        let logLabel = `[${CURRENT_IMPLEMENTATION}::GetAppById] [vcc::appId => ${this.vcc}::${id}]: `;
        return promises_shared_1.DummyPromise()
            .then(_ => {
            if (!this.apikey)
                return this.GetGenerativeEngineToken();
            else
                return Promise.resolve(true);
        })
            .then(_ => {
            let getOptions = {
                uri: `${this.url01}/${id}`,
                headers: {
                    'Content-Type': 'application/json',
                    'apikey': this.apikey
                },
                retryProtocol: true
            };
            return request_shared_1.SimpleGetRequest(getOptions);
        })
            .then(result => {
            if (!result)
                throw 'SERVER_ERROR_GET_APP_FAILED';
            let item = new generative_model_1.GenerativeGetAppById();
            item = Object.assign(item, result);
            return Promise.resolve(item.ConvertToGenerative());
        })
            .catch((err) => {
            let errorMessage = typeof err === 'string' ? err : 'ERROR';
            logger.error(`${logLabel} Error: `, err);
            if (err?.code === 'ERR_BAD_REQUEST')
                errorMessage = 'SERVER_ERROR_FAILED_FROM_GENERATIVE_SERVER';
            return Promise.reject(errorMessage);
        });
    }
    DeleteAppById(id) {
        let logLabel = `[${CURRENT_IMPLEMENTATION}::DeleteAppById] [vcc::appId => ${this.vcc}::${id}]: `;
        return promises_shared_1.DummyPromise()
            .then(_ => {
            if (!this.apikey)
                return this.GetGenerativeEngineToken();
            else
                return Promise.resolve(true);
        })
            .then(_ => {
            let deleteOptions = {
                uri: `${this.url01}/${id}`,
                headers: {
                    'Content-Type': 'application/json',
                    'apikey': this.apikey
                },
                retryProtocol: true
            };
            return request_shared_1.SimpleDeleteRequest(deleteOptions);
        })
            .then(result => {
            if (!result)
                throw 'SERVER_ERROR_GET_APP_FAILED';
            return Promise.resolve(true);
        })
            .catch((err) => {
            let errorMessage = typeof err === 'string' ? err : 'ERROR';
            logger.error(`${logLabel} Error: `, err);
            if (err?.code === 'ERR_BAD_REQUEST')
                errorMessage = 'SERVER_ERROR_FAILED_FROM_GENERATIVE_SERVER';
            return Promise.resolve(false);
        });
    }
    GetRequestById(id) {
        let logLabel = `[${CURRENT_IMPLEMENTATION}::GetRequestById] [vcc::request => ${this.vcc}::${id}]: `;
        return promises_shared_1.DummyPromise()
            .then(_ => {
            if (!this.apikey)
                return this.GetGenerativeEngineToken();
            else
                return Promise.resolve(true);
        })
            .then(_ => {
            let getOptions = {
                uri: `${this.url01}/request/${id}`,
                headers: {
                    'Content-Type': 'application/json',
                    'apikey': this.apikey
                },
                retryProtocol: true
            };
            return request_shared_1.SimpleGetRequest(getOptions);
        })
            .then(result => {
            if (!result)
                throw 'SERVER_ERROR_GET_REQUEST_FAILED';
            logger.info(`${logLabel} Generated ID for model: ${result.dialoguerouter}`);
            return Promise.resolve(result.dialoguerouter);
        })
            .catch((err) => {
            let errorMessage = typeof err === 'string' ? err : 'ERROR';
            logger.error(`${logLabel} Error: `, err);
            if (err?.code === 'ERR_BAD_REQUEST')
                errorMessage = 'SERVER_ERROR_FAILED_FROM_GENERATIVE_SERVER';
            return Promise.reject(errorMessage);
        });
    }
    GetAppsList(firstPage, defaultRowsNumber) {
        let logLabel = `[${CURRENT_IMPLEMENTATION}::GetAppsList] [vcc => ${this.vcc}]: `;
        let allGeneratives = [];
        let totalGeneratives = 0;
        firstPage ?? (firstPage = 0);
        defaultRowsNumber ?? (defaultRowsNumber = 15);
        let searchModels = (page, rows) => {
            let getOptions = {
                uri: `${this.url01}/?page=${page}&size=${rows}`,
                headers: {
                    'Content-Type': 'application/json',
                    'apikey': this.apikey
                },
                retryProtocol: true
            };
            return request_shared_1.SimpleGetRequest(getOptions);
        };
        let getModel = (data) => {
            let item = new generative_model_1.GenerativeGetAllService();
            item = Object.assign(item, data);
            return item.ConvertToGenerative();
        };
        return promises_shared_1.DummyPromise()
            .then(_ => {
            if (!this.apikey)
                return this.GetGenerativeEngineToken();
            else
                return Promise.resolve(true);
        })
            .then(_ => {
            logger.info(`${logLabel} Requesting first page`);
            return searchModels(firstPage, defaultRowsNumber);
        })
            .then(result => {
            if (!result || !result.data)
                throw 'SERVER_ERROR_GET_ALL_GENERATIVE_MODELS_FAILED';
            totalGeneratives = result.count;
            allGeneratives = result.data.map(d => getModel(d));
            if (defaultRowsNumber >= totalGeneratives) {
                return Promise.resolve([]);
            }
            else {
                let promiseArray = [];
                let totalPages = Math.ceil(totalGeneratives / defaultRowsNumber);
                logger.info(`${logLabel} Requesting ${totalPages} pages of generative models`);
                for (let page = 1; page <= totalPages; page++) {
                    promiseArray.push(searchModels(page, defaultRowsNumber));
                }
                return Promise.all(promiseArray);
            }
        })
            .then(results => {
            if (results.filter(r => !r.status).length)
                throw 'SERVER_ERROR_AT_LEAST_ONE_OF_GET_APPS_LIST_REQUEST_FAILED';
            results.forEach(r => allGeneratives = allGeneratives.concat(r.data.map(d => getModel(d))));
            logger.info(`${logLabel} Returning ${allGeneratives.length} generative models`);
            return Promise.resolve(allGeneratives);
        })
            .catch((err) => {
            let errorMessage = typeof err === 'string' ? err : 'ERROR';
            logger.error(`${logLabel} Error: `, err);
            if (err?.code === 'ERR_BAD_REQUEST')
                errorMessage = 'SERVER_ERROR_FAILED_FROM_GENERATIVE_SERVER';
            return Promise.reject(errorMessage);
        });
    }
    PublishVersion(appId) {
        let logLabel = `[${CURRENT_IMPLEMENTATION}::PublishVersion] [vcc::appId => ${this.vcc}::${appId}]: `;
        return promises_shared_1.DummyPromise()
            .then(_ => {
            if (!this.apikey)
                return this.GetGenerativeEngineToken();
            else
                return Promise.resolve(true);
        })
            .then(_ => {
            let putOptions = {
                uri: `${this.url01}/${appId}/active`,
                headers: {
                    'Content-Type': 'application/json',
                    'apikey': this.apikey
                },
                retryProtocol: true
            };
            return request_shared_1.SimplePutRequest(putOptions);
        })
            .then(result => {
            if (!result)
                throw 'SERVER_ERROR_GET_APP_FAILED';
            return Promise.resolve(result.data[0].published);
        })
            .catch((err) => {
            let errorMessage = typeof err === 'string' ? err : 'ERROR';
            logger.error(`${logLabel} Error: `, err);
            if (err?.code === 'ERR_BAD_REQUEST')
                errorMessage = 'SERVER_ERROR_FAILED_FROM_GENERATIVE_SERVER';
            return Promise.reject(errorMessage);
        });
    }
    Query(appId, text) {
        let logLabel = `[${CURRENT_IMPLEMENTATION}::${this.Query.name}] [vcc::appId => ${this.vcc}::${appId}]: `;
        return promises_shared_1.DummyPromise()
            .then(_ => {
            if (!this.apikey)
                return this.GetGenerativeEngineToken();
            else
                return Promise.resolve(true);
        })
            .then(_ => {
            let postOptions = {
                uri: `${this.url02}/${appId}?maxResults=3`,
                headers: {
                    'Content-Type': 'application/json',
                    'apikey': this.apikey
                },
                body: {
                    query: text
                },
                retryProtocol: true
            };
            return request_shared_1.SimplePostRequest(postOptions);
        })
            .then(result => {
            if (!result)
                throw 'SERVER_ERROR_QUERY_FAILED';
            if (!result.topScoring)
                throw 'SERVER_ERROR_NO_RESULTS_FOR_REQUEST';
            let intents = [];
            result.intents.map(i => {
                intents.push({
                    intent: i.intent,
                    score: i.score
                });
            });
            let parseResult = {
                query: result.text ? result.text : null,
                topScoringIntent: {
                    intent: result?.topScoring?.intent ? result.topScoring.intent : null,
                    score: result?.topScoring?.score ? result.topScoring.score : 0
                },
                intents: intents
            };
            logger.info(`${logLabel} FormattedResult: ` + JSON.stringify(parseResult));
            return Promise.resolve(parseResult);
        })
            .catch((err) => {
            let errorMessage = typeof err === 'string' ? err : 'ERROR';
            logger.error(`${logLabel} Error: `, err);
            if (err?.code === 'ERR_BAD_REQUEST')
                errorMessage = 'SERVER_ERROR_FAILED_FROM_GENERATIVE_SERVER';
            return Promise.reject(errorMessage);
        });
    }
    GetGenerativeEngineToken() {
        let logLabel = `[${CURRENT_IMPLEMENTATION}::GetGenerativeEngineToken] [vcc => ${this.vcc}]: `;
        let cacheUpdateNeeded = false;
        return redis_service_1.Redis.GetGenerativeToken(this.vcc)
            .then(result => {
            if (result) {
                this.apikey = result;
                return Promise.resolve(result);
            }
            else {
                cacheUpdateNeeded = true;
                return mssql_service_1.MSSql.GetGenerativeToken(this.vcc);
            }
        })
            .then(result => {
            if (result) {
                this.apikey ?? (this.apikey = result.token);
                if (cacheUpdateNeeded)
                    redis_service_1.Redis.SaveGenerativeToken(this.vcc, this.apikey);
                return Promise.resolve(true);
            }
            else
                throw 'SERVER_ERROR_GENERATIVE_TOKEN_NOT_FOUND';
        })
            .catch((err) => {
            let errorMsg = (typeof err === 'string' ? err : err.message || err.description || 'SERVER_ERROR_UNKNOWN_MESSAGE');
            logger.error(`${logLabel}getting errorMsg:`, errorMsg);
            return Promise.reject(errorMsg);
        });
    }
    ErroHandler(err, callback) {
        this.retry += 1;
        if (this.retry <= this.maxRetries) {
            console.log(`[${CURRENT_IMPLEMENTATION}::ErroHandler] Retry ${this.retry}/${this.maxRetries}...`);
            return new Promise((resolve, reject) => {
                setTimeout(() => {
                    return resolve(callback);
                }, 10000);
            });
        }
        else {
            console.log(`[${CURRENT_IMPLEMENTATION}::ErroHandler] Retry ${this.retry}`, err);
            return Promise.reject(err);
        }
    }
}
exports.GenerativeServiceImplementation = GenerativeServiceImplementation;
