"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.LUISServiceImplementation = void 0;
const log4js = require("log4js");
const config_1 = require("../config/config");
const promises_shared_1 = require("../shared/promises.shared");
const request_shared_1 = require("../shared/request.shared");
let logger = log4js.getLogger('Task');
class LUISServiceImplementation {
    GetCognitiveEngineName() {
        return 'LUIS';
    }
    GetCognitiveEngineCharsPerPhrase() {
        return config_1.COGNITIVE_SERVICES_LUIS_CHARS_PER_PHRASE;
    }
    LoadAzureAccounts(subscriptionRegion, subscriptionKey, data) {
        let getOptions = {
            uri: `https://${subscriptionRegion}.${config_1.LUIS_URL_ACCOUNTS}`,
            headers: {
                'Content-Type': 'application/x-www-form-urlencoded',
                'Ocp-Apim-Subscription-Key': subscriptionKey,
                'Authorization': data.token_type + ' ' + data.access_token
            },
            retryProtocol: false,
            timeout: 60000
        };
        logger.info('[LUISServiceImplementation::LoadAzureAccounts] getting data: ' + JSON.stringify(data));
        return request_shared_1.SimpleGetRequest(getOptions)
            .then((response) => {
            logger.info('[LUISServiceImplementation::LoadAzureAccounts] getting response: ' + JSON.stringify(response));
            return Promise.resolve(response);
        })
            .catch((err) => {
            let errorMessage = typeof err === 'string' ? err : (err.message || err.description);
            logger.error('[LUISServiceImplementation::LoadAzureAccounts] getting error:', errorMessage);
            return Promise.reject(err);
        });
    }
    ModelQuery(subscriptionRegion, subscriptionKey, appId, query, processAudios, nluName, resolve, task) {
        let getOptions = {
            uri: `https://${subscriptionRegion}.${config_1.LUIS_URL_STUDY}/${appId}?subscription-key=${subscriptionKey}&verbose=true&timezoneOffset=-300&q=` + encodeURIComponent(query),
            retryProtocol: false,
            timeout: 60000
        };
        logger.info('[LUISServiceImplementation::ModelQuery] getting getOptions:', getOptions, 'subscriptionRegion:', subscriptionRegion, 'subscriptionKey:', subscriptionKey, 'appId:', appId, 'query:', query, 'processAudios:', processAudios, 'nluName:', nluName);
        return request_shared_1.SimpleGetRequest(getOptions)
            .then((response) => {
            return Promise.resolve(response);
        }).catch((err) => {
            let errorMessage = typeof err === 'string' ? err : (err.message || err.description);
            logger.error('[LUISServiceImplementation::ModelQuery] getting error:', errorMessage, 'subscriptionRegion:', subscriptionRegion, 'subscriptionKey:', subscriptionKey, 'appId:', appId, 'query:', query, 'processAudios:', processAudios, 'nluName:', nluName);
            return Promise.reject(err);
        });
    }
    DeleteAppIntent(subscriptionRegion, subscriptionKey, appId, intentId) {
        let deleteOptions = {
            uri: `https://${subscriptionRegion}.${config_1.LUIS_URL}/${appId}/versions/0.1/intents/${intentId}/?deleteUtterances=false`,
            headers: {
                'Content-Type': 'application/json',
                'Ocp-Apim-Subscription-Key': subscriptionKey
            },
            timeout: 60000
        };
        logger.info('[LUISServiceImplementation::DeleteAppIntent] getting deleteOptions:', deleteOptions, 'subscriptionRegion:', subscriptionRegion, 'subscriptionKey:', subscriptionKey, 'appId:', appId, 'intentId:', intentId);
        return request_shared_1.SimpleDeleteRequest(deleteOptions)
            .then((response) => {
            logger.info('[LUISServiceImplementation::DeleteAppIntent] getting response successfully subscriptionRegion:', subscriptionRegion, 'subscriptionKey:', subscriptionKey, 'appId:', appId, 'intentId:', intentId);
            return response;
        })
            .catch((err) => {
            let errorMessage = typeof err === 'string' ? err : (err.message || err.description);
            logger.error('[LUISServiceImplementation::DeleteAppIntent] getting error:', errorMessage, 'subscriptionRegion:', subscriptionRegion, 'subscriptionKey:', subscriptionKey, 'appId:', appId, 'intentId:', intentId);
            return Promise.reject(err);
        });
    }
    DeleteAppUtterance(subscriptionRegion, subscriptionKey, appId, intentId, utteranceId) {
        let deleteOptions = {
            uri: `https://${subscriptionRegion}.${config_1.LUIS_URL}/${appId}/versions/0.1/examples/${utteranceId}`,
            headers: {
                'Content-Type': 'application/json',
                'Ocp-Apim-Subscription-Key': subscriptionKey
            },
            timeout: 60000
        };
        logger.info('[LUISServiceImplementation::DeleteAppUtterance] getting deleteOptions:', deleteOptions, 'subscriptionRegion:', subscriptionRegion, 'subscriptionKey:', subscriptionKey, 'appId:', appId, 'utteranceId:', utteranceId);
        return request_shared_1.SimpleDeleteRequest(deleteOptions)
            .then((response) => {
            logger.info('[LUISServiceImplementation::DeleteAppUtterance] getting response successfully subscriptionRegion:', subscriptionRegion, 'subscriptionKey:', subscriptionKey, 'appId:', appId, 'utteranceId:', utteranceId);
            return response;
        })
            .catch((err) => {
            let errorMessage = typeof err === 'string' ? err : (err.message || err.description);
            logger.error('[LUISServiceImplementation::DeleteAppUtterance] getting error:', errorMessage, 'subscriptionRegion:', subscriptionRegion, 'subscriptionKey:', subscriptionKey, 'appId:', appId, 'utteranceId:', utteranceId);
            return Promise.reject(err);
        });
    }
    TrainApp(subscriptionRegion, subscriptionKey, appId) {
        let data = {};
        let postOptions = {
            uri: `https://${subscriptionRegion}.${config_1.LUIS_URL}/${appId}/versions/0.1/train`,
            body: data,
            headers: {
                'Content-Type': 'application/json',
                'Ocp-Apim-Subscription-Key': subscriptionKey
            },
            retryProtocol: false,
            timeout: 300000
        };
        logger.info('[LUISServiceImplementation::TrainApp] getting postOptions:', postOptions, 'subscriptionRegion:', subscriptionRegion, 'subscriptionKey:', subscriptionKey, 'appId:', appId);
        return request_shared_1.SimplePostRequest(postOptions)
            .then((response) => {
            logger.info('[LUISServiceImplementation::TrainApp] getting response successfully subscriptionRegion:', subscriptionRegion, 'subscriptionKey:', subscriptionKey, 'appId:', appId);
            return response;
        })
            .catch((err) => {
            let errorMessage = typeof err === 'string' ? err : (err.message || err.description);
            logger.error('[LUISServiceImplementation::TrainApp] getting error:', errorMessage, 'subscriptionRegion:', subscriptionRegion, 'subscriptionKey:', subscriptionKey, 'appId:', appId);
            return Promise.reject(err);
        });
    }
    GetTrainApp(subscriptionRegion, subscriptionKey, appId) {
        let data = {};
        let getOptions = {
            uri: `https://${subscriptionRegion}.${config_1.LUIS_URL}/${appId}/versions/0.1/train`,
            body: data,
            headers: {
                'Content-Type': 'application/json',
                'Ocp-Apim-Subscription-Key': subscriptionKey
            },
            retryProtocol: false,
            timeout: 60000
        };
        logger.info('[LUISServiceImp<lementation::GetTrainApp] getting getOptions:', getOptions, 'subscriptionRegion:', subscriptionRegion, 'subscriptionKey:', subscriptionKey, 'appId:', appId);
        return request_shared_1.SimpleGetRequest(getOptions)
            .then((response) => {
            logger.info('[LUISServiceImplementation::GetTrainApp] getting response successfully subscriptionRegion:', subscriptionRegion, 'subscriptionKey:', subscriptionKey, 'appId:', appId);
            return response;
        })
            .catch((err) => {
            let errorMessage = typeof err === 'string' ? err : (err.message || err.description);
            logger.error('[LUISServiceImplementation::GetTrainApp] getting error:', errorMessage, 'subscriptionRegion:', subscriptionRegion, 'subscriptionKey:', subscriptionKey, 'appId:', appId);
            return Promise.reject(err);
        });
    }
    PublishApp(subscriptionRegion, subscriptionKey, appId, isStaging) {
        let data = { versionId: '0.1', isStaging: isStaging, directVersionPublish: false };
        let postOptions = {
            uri: `https://${subscriptionRegion}.${config_1.LUIS_URL}/${appId}/publish`,
            body: data,
            headers: {
                'Content-Type': 'application/json',
                'Ocp-Apim-Subscription-Key': subscriptionKey
            },
            retryProtocol: false,
            timeout: 300000
        };
        logger.info('[LUISServiceImplementation::PublishApp] getting postOptions:', postOptions, 'subscriptionRegion:', subscriptionRegion, 'subscriptionKey:', subscriptionKey, 'appId:', appId);
        return request_shared_1.SimplePostRequest(postOptions)
            .then((response) => {
            logger.info('[LUISServiceImplementation::PublishApp] getting response successfully subscriptionRegion:', subscriptionRegion, 'subscriptionKey:', subscriptionKey, 'appId:', appId);
            return response;
        })
            .catch((err) => {
            let errorMessage = typeof err === 'string' ? err : (err.message || err.description);
            logger.error('[LUISServiceImplementation::PublishApp] getting error:', errorMessage, 'subscriptionRegion:', subscriptionRegion, 'subscriptionKey:', subscriptionKey, 'appId:', appId);
            return Promise.reject(err);
        });
    }
    GetManagerToken(info) {
        let data = "grant_type=client_credentials"
            + "&client_id=" + info.client_id
            + "&client_secret=" + encodeURIComponent(info.client_secret)
            + "&resource=" + encodeURIComponent(info.resource);
        let postOptions = {
            uri: 'https://login.microsoftonline.com/' + info.tenant_id + '/oauth2/token',
            headers: {
                'Content-Type': 'application/x-www-form-urlencoded'
            },
            body: data,
            retryProtocol: false,
            timeout: 60000
        };
        logger.info('[LUISServiceImplementation::GetManagerToken] getting data: ' + JSON.stringify(info));
        return request_shared_1.SimplePostRequest(postOptions)
            .then((response) => {
            logger.info('[LUISServiceImplementation::GetManagerToken] getting response: ' + JSON.stringify(response));
            return Promise.resolve(response);
        })
            .catch((err) => {
            let errorMessage = typeof err === 'string' ? err : (err.message || err.description);
            logger.error('[LUISServiceImplementation::GetManagerToken] getting error:', errorMessage);
            return Promise.reject(err);
        });
    }
    UnAssignSubscriptionKeyOfVCC(subscriptionRegion, subscriptionKey, appId, data) {
        logger.info('[LUISServiceImplementation::UnAssignSubscriptionKeyOfVCC] getting subscriptionRegion: ' + subscriptionRegion);
        logger.info('[LUISServiceImplementation::UnAssignSubscriptionKeyOfVCC] getting appId: ' + appId);
        let body = "azureSubscriptionId=" + data.azureSubscriptionId
            + "&resourceGroup=" + data.resourceGroup
            + "&accountName=" + data.accountName;
        let deleteOptions = {
            uri: `https://${subscriptionRegion}.${config_1.LUIS_URL}/${appId}/azureaccounts`,
            headers: {
                'Content-Type': 'application/x-www-form-urlencoded',
                'Ocp-Apim-Subscription-Key': subscriptionKey,
                'Authorization': data.token_type + ' ' + data.access_token
            },
            body: body,
            timeout: 60000
        };
        logger.info('[LUISServiceImplementation::UnAssignSubscriptionKeyOfVCC] getting data: ' + JSON.stringify(data));
        return request_shared_1.SimpleDeleteRequest(deleteOptions)
            .then((response) => {
            logger.info('[LUISServiceImplementation::UnAssignSubscriptionKeyOfVCC] getting response: ' + JSON.stringify(response));
            return Promise.resolve(response);
        })
            .catch((err) => {
            let errorMessage = typeof err === 'string' ? err : (err.message || err.description);
            logger.error('[LUISServiceImplementation::UnAssignSubscriptionKeyOfVCC] getting error:', errorMessage);
            return Promise.reject(err);
        });
    }
    AssignSubscriptionKeyOfVCC(subscriptionRegion, subscriptionKey, appId, data) {
        logger.info('[LUISServiceImplementation::AssignSubscriptionKeyOfVCC] getting subscriptionRegion: ' + subscriptionRegion);
        logger.info('[LUISServiceImplementation::AssignSubscriptionKeyOfVCC] getting appId: ' + appId);
        let body = "azureSubscriptionId=" + data.azureSubscriptionId
            + "&resourceGroup=" + data.resourceGroup
            + "&accountName=" + data.accountName;
        let postOptions = {
            uri: `https://${subscriptionRegion}.${config_1.LUIS_URL}/${appId}/azureaccounts`,
            headers: {
                'Content-Type': 'application/x-www-form-urlencoded',
                'Ocp-Apim-Subscription-Key': subscriptionKey,
                'Authorization': data.token_type + ' ' + data.access_token
            },
            body: body,
            retryProtocol: false,
            timeout: 60000
        };
        logger.info('[LUISServiceImplementation::AssignSubscriptionKeyOfVCC] getting data: ' + JSON.stringify(data));
        return request_shared_1.SimplePostRequest(postOptions)
            .then((response) => {
            logger.info('[LUISServiceImplementation::AssignSubscriptionKeyOfVCC] getting response: ' + JSON.stringify(response));
            return Promise.resolve(response);
        })
            .catch((err) => {
            let errorMessage = typeof err === 'string' ? err : (err.message || err.description);
            logger.error('[LUISServiceImplementation::AssignSubscriptionKeyOfVCC] getting error:', errorMessage);
            return Promise.reject(err);
        });
    }
    GetCultures(subscriptionRegion, subscriptionKey) {
        let getOptions = {
            uri: `https://${subscriptionRegion}.${config_1.LUIS_URL}/cultures`,
            headers: {
                'Content-Type': 'application/json',
                'Ocp-Apim-Subscription-Key': subscriptionKey
            },
            retryProtocol: false,
            timeout: 60000
        };
        logger.info('[LUISServiceImplementation::GetCultures] getting getOptions:', getOptions, 'subscriptionRegion:', subscriptionRegion, 'subscriptionKey:', subscriptionKey);
        return request_shared_1.SimpleGetRequest(getOptions)
            .then((response) => {
            logger.info('[LUISServiceImplementation::GetCultures] getting response successfully subscriptionRegion:', subscriptionRegion, 'subscriptionKey:', subscriptionKey);
            return Promise.resolve(response);
        })
            .catch((err) => {
            let errorMessage = typeof err === 'string' ? err : (err.message || err.description);
            logger.error('[LUISServiceImplementation::GetCultures] getting error:', errorMessage, 'subscriptionRegion:', subscriptionRegion, 'subscriptionKey:', subscriptionKey);
            return Promise.reject(err);
        });
    }
    AddApp(subscriptionRegion, subscriptionKey, nlu) {
        let data = { name: nlu.name, description: nlu.description, culture: nlu.culture };
        let postOptions = {
            uri: `https://${subscriptionRegion}.${config_1.LUIS_URL}/`,
            body: data,
            headers: {
                'Content-Type': 'application/json',
                'Ocp-Apim-Subscription-Key': subscriptionKey
            },
            retryProtocol: false,
            timeout: 60000
        };
        logger.info('[LUISServiceImplementation::AddApp] getting postOptions:', postOptions, 'subscriptionRegion:', subscriptionRegion, 'subscriptionKey:', subscriptionKey, 'nlu:', nlu);
        return request_shared_1.SimplePostRequest(postOptions)
            .then((response) => {
            logger.info('[LUISServiceImplementation::AddApp] getting response successfully subscriptionRegion:', subscriptionRegion, 'subscriptionKey:', subscriptionKey, 'nlu:', nlu);
            return response;
        })
            .catch((err) => {
            let errorMessage = typeof err === 'string' ? err : (err.message || err.description);
            logger.error('[LUISServiceImplementation::AddApp] getting error:', errorMessage, 'subscriptionRegion:', subscriptionRegion, 'subscriptionKey:', subscriptionKey, 'nlu:', nlu);
            return Promise.reject(err);
        });
    }
    DeleteApp(subscriptionRegion, subscriptionKey, nlu) {
        let deleteOptions = {
            uri: `https://${subscriptionRegion}.${config_1.LUIS_URL}/${nlu.appId}?force=true`,
            headers: {
                'Content-Type': 'application/json',
                'Ocp-Apim-Subscription-Key': subscriptionKey
            },
            timeout: 60000
        };
        logger.info('[LUISServiceImplementation::DeleteApp] getting deleteOptions:', deleteOptions, 'subscriptionRegion:', subscriptionRegion, 'subscriptionKey:', subscriptionKey, 'appId:', nlu.appId);
        return request_shared_1.SimpleDeleteRequest(deleteOptions)
            .then((response) => {
            logger.info('[LUISServiceImplementation::DeleteApp] getting response successfully subscriptionRegion:', subscriptionRegion, 'subscriptionKey:', subscriptionKey, 'appId:', nlu.appId);
            return response;
        })
            .catch((err) => {
            let errorMessage = typeof err === 'string' ? err : (err.message || err.description);
            logger.error('[LUISServiceImplementation::DeleteApp] getting error:', errorMessage, 'subscriptionRegion:', subscriptionRegion, 'subscriptionKey:', subscriptionKey, 'appId:', nlu.appId);
            return Promise.reject(err);
        });
    }
    GetAppsList(subscriptionRegion, subscriptionKey) {
        logger.info('[LUISServiceImplementation::GetAppsList] getting subscriptionRegion:', subscriptionRegion, 'subscriptionKey:', subscriptionKey);
        return this.GetResultsFromPaginatedRequest(subscriptionRegion, subscriptionKey, `https://${subscriptionRegion}.${config_1.LUIS_URL}/`, 0, 500)
            .then((response) => {
            logger.info('[LUISServiceImplementation::GetAppsList] getting response successfully subscriptionRegion:', subscriptionRegion, 'subscriptionKey:', subscriptionKey);
            return Promise.resolve(response);
        })
            .catch((err) => {
            let errorMessage = typeof err === 'string' ? err : (err.message || err.description);
            logger.error('[LUISServiceImplementation::GetAppsList] getting error:', errorMessage, 'subscriptionRegion:', subscriptionRegion, 'subscriptionKey:', subscriptionKey);
            return Promise.reject(err);
        });
    }
    GetAppById(appId, subscriptionRegion, subscriptionKey) {
        return null;
    }
    AddIntent(subscriptionRegion, subscriptionKey, appId, name) {
        let data = { name: name };
        let postOptions = {
            uri: `https://${subscriptionRegion}.${config_1.LUIS_URL}/${appId}/versions/0.1/intents`,
            body: data,
            headers: {
                'Content-Type': 'application/json',
                'Ocp-Apim-Subscription-Key': subscriptionKey
            },
            retryProtocol: false,
            timeout: 60000
        };
        logger.info('[LUISServiceImplementation::AddIntent] getting postOptions:', postOptions, 'subscriptionRegion:', subscriptionRegion, 'subscriptionKey:', subscriptionKey, 'appId:', appId, 'name:', name);
        return request_shared_1.SimplePostRequest(postOptions)
            .then((response) => {
            logger.info('[LUISServiceImplementation::AddIntent] getting response successfully subscriptionRegion:', subscriptionRegion, 'subscriptionKey:', subscriptionKey, 'appId:', appId, 'name:', name);
            return response;
        })
            .catch((err) => {
            let errorMessage = typeof err === 'string' ? err : (err.message || err.description);
            logger.error('[LUISServiceImplementation::AddIntent] getting error:', errorMessage, 'subscriptionRegion:', subscriptionRegion, 'subscriptionKey:', subscriptionKey, 'appId:', appId, 'name:', name);
            return Promise.reject(err);
        });
    }
    AddUtterance(subscriptionRegion, subscriptionKey, appId, utterance) {
        let entityLabels = [];
        if (utterance && utterance.entityLabels && utterance.entityLabels.length > 0) {
            utterance.entityLabels.map(el => {
                entityLabels.push({
                    entityName: el.entityName,
                    startCharIndex: el.start,
                    endCharIndex: el.end
                });
            });
        }
        let data = {
            text: utterance.name,
            intentName: utterance.intent,
            entityLabels: entityLabels
        };
        let postOptions = {
            uri: `https://${subscriptionRegion}.${config_1.LUIS_URL}/${appId}/versions/0.1/example`,
            body: data,
            headers: {
                'Content-Type': 'application/json',
                'Ocp-Apim-Subscription-Key': subscriptionKey
            },
            retryProtocol: false,
            timeout: 60000
        };
        logger.info('[LUISServiceImplementation::AddUtterance] getting postOptions:', postOptions, 'subscriptionRegion:', subscriptionRegion, 'subscriptionKey:', subscriptionKey, 'appId:', appId, 'utterance:', utterance);
        return request_shared_1.SimplePostRequest(postOptions)
            .then((response) => {
            logger.info('[LUISServiceImplementation::AddUtterance] getting response successfully subscriptionRegion:', subscriptionRegion, 'subscriptionKey:', subscriptionKey, 'appId:', appId, 'utterance:', utterance);
            return response;
        })
            .catch((err) => {
            let errorMessage = typeof err === 'string' ? err : (err.message || err.description);
            logger.error('[LUISServiceImplementation::AddUtterance] getting error:', errorMessage, 'subscriptionRegion:', subscriptionRegion, 'subscriptionKey:', subscriptionKey, 'appId:', appId, 'utterance:', utterance);
            return Promise.reject(err);
        });
    }
    GetAppVersion(subscriptionRegion, subscriptionKey, appId) {
        let getOptions = {
            uri: `https://${subscriptionRegion}.${config_1.LUIS_URL}/${appId}/versions/0.1/`,
            headers: {
                'Content-Type': 'application/json',
                'Ocp-Apim-Subscription-Key': subscriptionKey
            },
            retryProtocol: false,
            timeout: 60000
        };
        logger.info('[LUISServiceImplementation::GetAppVersion] getting getOptions:', getOptions, 'subscriptionRegion:', subscriptionRegion, 'subscriptionKey:', subscriptionKey, 'appId:', appId);
        return request_shared_1.SimpleGetRequest(getOptions)
            .then((response) => {
            logger.info('[LUISServiceImplementation::GetAppVersion] getting response successfully subscriptionRegion:', subscriptionRegion, 'subscriptionKey:', subscriptionKey, 'appId:', appId);
            return response;
        })
            .catch((err) => {
            let errorMessage = typeof err === 'string' ? err : (err.message || err.description);
            logger.error('[LUISServiceImplementation::GetAppVersion] getting error:', errorMessage, 'subscriptionRegion:', subscriptionRegion, 'subscriptionKey:', subscriptionKey, 'appId:', appId);
            return Promise.reject(err);
        });
    }
    GetIntentsList(subscriptionRegion, subscriptionKey, appId) {
        logger.info('[LUISServiceImplementation::GetIntentsList] getting subscriptionRegion:', subscriptionRegion, 'subscriptionKey:', subscriptionKey, 'appId:', appId);
        return this.GetResultsFromPaginatedRequest(subscriptionRegion, subscriptionKey, `https://${subscriptionRegion}.${config_1.LUIS_URL}/${appId}/versions/0.1/intents/`, 0, 500)
            .then((response) => {
            logger.info('[LUISServiceImplementation::GetIntentsList] getting response successfully subscriptionRegion:', subscriptionRegion, 'subscriptionKey:', subscriptionKey, 'appId:', appId);
            return Promise.resolve(response);
        })
            .catch((err) => {
            let errorMessage = typeof err === 'string' ? err : (err.message || err.description);
            logger.error('[LUISServiceImplementation::GetIntentsList] getting error:', errorMessage, 'subscriptionRegion:', subscriptionRegion, 'subscriptionKey:', subscriptionKey, 'appId:', appId);
            return Promise.reject(err);
        });
    }
    GetIntentsListDemo(subscriptionRegion, subscriptionKey, appId, resolve, reject) {
        logger.info('[LUISServiceImplementation::GetIntentsListDemo] getting subscriptionRegion:', subscriptionRegion, 'subscriptionKey:', subscriptionKey, 'appId:', appId);
        return this.GetResultsFromPaginatedRequest(subscriptionRegion, subscriptionKey, `https://${subscriptionRegion}.${config_1.LUIS_URL}/${appId}/versions/0.1/intents/`, 0, 500)
            .then((response) => {
            return resolve ? resolve(response) : Promise.resolve(response);
        })
            .catch((err) => {
            let errorMessage = typeof err === 'string' ? err : (err.message || err.description);
            logger.error('[LUISServiceImplementation::GetIntentsListDemo] getting error:', errorMessage, 'subscriptionRegion:', subscriptionRegion, 'subscriptionKey:', subscriptionKey, 'appId:', appId);
            return Promise.resolve(false);
        });
    }
    GetUtterancesList(subscriptionRegion, subscriptionKey, appId) {
        logger.info('[LUISServiceImplementation::GetUtterancesList] subscriptionRegion:', subscriptionRegion, 'subscriptionKey:', subscriptionKey, 'appId:', appId);
        return this.GetResultsFromPaginatedRequest(subscriptionRegion, subscriptionKey, `https://${subscriptionRegion}.${config_1.LUIS_URL}/${appId}/versions/0.1/examples/`, 0, 500)
            .then((response) => {
            logger.info('[LUISServiceImplementation::GetUtterancesList] getting response successfully subscriptionRegion:', subscriptionRegion, 'subscriptionKey:', subscriptionKey, 'appId:', appId);
            return Promise.resolve(response);
        })
            .catch((err) => {
            let errorMessage = typeof err === 'string' ? err : (err.message || err.description);
            logger.error('[LUISServiceImplementation::GetUtterancesList] getting error:', errorMessage, 'subscriptionRegion:', subscriptionRegion, 'subscriptionKey:', subscriptionKey, 'appId:', appId);
            return Promise.reject(err);
        });
    }
    GetUtterancesFromMD(appId) {
        return promises_shared_1.DummyPromise()
            .then(result => {
            return Promise.resolve([]);
        });
    }
    AddSimpleEntity(subscriptionRegion, subscriptionKey, appId, entity) {
        let data = { name: entity.name };
        let postOptions = {
            uri: `https://${subscriptionRegion}.${config_1.LUIS_URL}/${appId}/versions/0.1/entities`,
            body: data,
            headers: {
                'Content-Type': 'application/json',
                'Ocp-Apim-Subscription-Key': subscriptionKey
            },
            retryProtocol: false,
            timeout: 60000
        };
        logger.info('[LUISServiceImplementation::AddEntitySimple] getting postOptions:', postOptions, 'subscriptionRegion:', subscriptionRegion, 'subscriptionKey:', subscriptionKey, 'appId:', appId, 'utterance:', entity);
        return request_shared_1.SimplePostRequest(postOptions)
            .then((response) => {
            logger.info('[LUISServiceImplementation::AddEntitySimple] getting response successfully subscriptionRegion:', subscriptionRegion, 'subscriptionKey:', subscriptionKey, 'appId:', appId, 'utterance:', entity);
            return response;
        })
            .catch((err) => {
            let errorMessage = typeof err === 'string' ? err : (err.message || err.description);
            logger.error('[LUISServiceImplementation::AddEntitySimple] getting error:', errorMessage, 'subscriptionRegion:', subscriptionRegion, 'subscriptionKey:', subscriptionKey, 'appId:', appId, 'utterance:', entity);
            return Promise.reject(err);
        });
    }
    RenameEntity(subscriptionRegion, subscriptionKey, appId, entityId, entityName) {
        let data = { name: entityName };
        let putOptions = {
            uri: `https://${subscriptionRegion}.${config_1.LUIS_URL}/${appId}/versions/0.1/entities/${entityId}`,
            method: 'PUT',
            body: data,
            headers: {
                'Content-Type': 'application/json',
                'Ocp-Apim-Subscription-Key': subscriptionKey
            },
            timeout: 60000
        };
        logger.info('[LUISServiceImplementation::RenameEntity] getting postOptions:', putOptions, 'subscriptionRegion:', subscriptionRegion, 'subscriptionKey:', subscriptionKey, 'appId:', appId, 'utterance:', entityId);
        return request_shared_1.SimplePutRequest(putOptions)
            .then((response) => {
            logger.info('[LUISServiceImplementation::RenameEntity] getting response successfully subscriptionRegion:', subscriptionRegion, 'subscriptionKey:', subscriptionKey, 'appId:', appId, 'utterance:', entityId);
            return response;
        })
            .catch((err) => {
            let errorMessage = typeof err === 'string' ? err : (err.message || err.description);
            logger.error('[LUISServiceImplementation::RenameEntity] getting error:', errorMessage, 'subscriptionRegion:', subscriptionRegion, 'subscriptionKey:', subscriptionKey, 'appId:', appId, 'utterance:', entityId);
            return Promise.reject(err);
        });
    }
    GetEntitiesList(subscriptionRegion, subscriptionKey, appId) {
        let getOptions = {
            uri: `https://${subscriptionRegion}.${config_1.LUIS_URL}/${appId}/versions/0.1/entities`,
            headers: {
                'Content-Type': 'application/json',
                'Ocp-Apim-Subscription-Key': subscriptionKey
            },
            retryProtocol: false,
            timeout: 60000
        };
        logger.info('[LUISServiceImplementation::GetEntities] getting getOptions:', getOptions, 'subscriptionRegion:', subscriptionRegion, 'subscriptionKey:', subscriptionKey, 'appId:', appId);
        return request_shared_1.SimpleGetRequest(getOptions)
            .then((response) => {
            logger.info('[LUISServiceImplementation::GetEntities] getting response successfully subscriptionRegion:', subscriptionRegion, 'subscriptionKey:', subscriptionKey, 'appId:', appId);
            return Promise.resolve(response);
        })
            .catch((err) => {
            let errorMessage = typeof err === 'string' ? err : (err.message || err.description);
            logger.error('[LUISServiceImplementation::GetEntities] getting error:', errorMessage, 'subscriptionRegion:', subscriptionRegion, 'subscriptionKey:', subscriptionKey, 'appId:', appId);
            return Promise.reject(err);
        });
    }
    GetEntity(subscriptionRegion, subscriptionKey, appId, entityId) {
        let getOptions = {
            uri: `https://${subscriptionRegion}.${config_1.LUIS_URL}/${appId}/versions/0.1/entities/${entityId}`,
            headers: {
                'Content-Type': 'application/json',
                'Ocp-Apim-Subscription-Key': subscriptionKey
            },
            retryProtocol: false,
            timeout: 60000
        };
        logger.info('[LUISServiceImplementation::GetEntity] getting getOptions:', getOptions, 'subscriptionRegion:', subscriptionRegion, 'subscriptionKey:', subscriptionKey, 'appId:', appId);
        return request_shared_1.SimpleGetRequest(getOptions)
            .then((response) => {
            logger.info('[LUISServiceImplementation::GetEntity] getting response successfully subscriptionRegion:', subscriptionRegion, 'subscriptionKey:', subscriptionKey, 'appId:', appId);
            return Promise.resolve(response);
        })
            .catch((err) => {
            let errorMessage = typeof err === 'string' ? err : (err.message || err.description);
            logger.error('[LUISServiceImplementation::GetEntity] getting error:', errorMessage, 'subscriptionRegion:', subscriptionRegion, 'subscriptionKey:', subscriptionKey, 'appId:', appId);
            return Promise.reject(err);
        });
    }
    DeleteAppEntity(subscriptionRegion, subscriptionKey, appId, entityId) {
        let deleteOptions = {
            uri: `https://${subscriptionRegion}.${config_1.LUIS_URL}/${appId}/versions/0.1/entities/${entityId}`,
            headers: {
                'Content-Type': 'application/json',
                'Ocp-Apim-Subscription-Key': subscriptionKey
            },
            timeout: 60000
        };
        logger.info('[LUISServiceImplementation::DeleteEntity] getting deleteOptions:', deleteOptions, 'subscriptionRegion:', subscriptionRegion, 'subscriptionKey:', subscriptionKey, 'appId:', appId, 'utteranceId:', entityId);
        return request_shared_1.SimpleDeleteRequest(deleteOptions)
            .then((response) => {
            logger.info('[LUISServiceImplementation::DeleteEntity] getting response successfully subscriptionRegion:', subscriptionRegion, 'subscriptionKey:', subscriptionKey, 'appId:', appId, 'utteranceId:', entityId);
            return response;
        })
            .catch((err) => {
            let errorMessage = typeof err === 'string' ? err : (err.message || err.description);
            logger.error('[LUISServiceImplementation::DeleteEntity] getting error:', errorMessage, 'subscriptionRegion:', subscriptionRegion, 'subscriptionKey:', subscriptionKey, 'appId:', appId, 'utteranceId:', entityId);
            return Promise.reject(err);
        });
    }
    GetAllIntentsFromRASAContent(content) {
        return null;
    }
    GetAllUtterancesFromRASAContent(content) {
        return null;
    }
    ExportModel(nlu, res) {
        return null;
    }
    GetResultsFromPaginatedRequest(subscriptionRegion, subscriptionKey, url, skip, take) {
        skip = skip ? skip : 0;
        take = take ? take : 500;
        let allResults = [];
        let getOptions = {
            uri: `${url}?skip=${skip}&take=${take}`,
            headers: {
                'Content-Type': 'application/json',
                'Ocp-Apim-Subscription-Key': subscriptionKey
            },
            retryProtocol: false,
            timeout: 60000
        };
        return request_shared_1.SimpleGetRequest(getOptions)
            .then((result) => {
            if (!result)
                throw 'SERVER_ERROR_GET_LUIS_ITEMS_SERVICE_FAILED';
            if (result.length && result.length >= take) {
                allResults = result;
                return this.GetResultsFromPaginatedRequest(subscriptionRegion, subscriptionKey, url, skip + take, take);
            }
            else {
                return Promise.resolve(result);
            }
        })
            .then(result => {
            allResults = allResults.concat(result);
            return Promise.resolve(allResults);
        })
            .catch((err) => {
            let errorMessage = typeof err === 'string' ? err : (err.message || err.description);
            logger.error('[LUISServiceImplementation::GetResultsFromPaginatedRequest] getting error:', errorMessage, 'subscriptionRegion:', subscriptionRegion, 'subscriptionKey:', subscriptionKey);
            return Promise.reject(err);
        });
    }
    RunApp(appId, task) {
        return null;
    }
    StopApp(appId, port, task) {
        return null;
    }
}
exports.LUISServiceImplementation = LUISServiceImplementation;
