"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.RASAServiceImplementation = void 0;
const moment = require("moment");
const child_process = require("child_process");
const log4js = require("log4js");
const { NodeSSH } = require('node-ssh');
const utterance_model_1 = require("../models/utterance.model");
const config_1 = require("../config/config");
const rasa_config_1 = require("../config/rasa.config");
const mssql_service_1 = require("../db/mssql.service");
const redis_service_1 = require("../db/redis.service");
const promises_shared_1 = require("../shared/promises.shared");
const file_manager_1 = require("../shared/file-manager");
const platform_service_1 = require("../services/platform.service");
const utilities_1 = require("../shared/utilities");
const utterance_service_1 = require("../services/utterance.service");
const arrays_shared_1 = require("../shared/arrays.shared");
const request_shared_1 = require("../shared/request.shared");
let exec = child_process.exec;
let logger = log4js.getLogger('Task');
class RASAServiceImplementation {
    constructor() {
        this.mySsh = new NodeSSH();
    }
    GetCognitiveEngineName() {
        return 'RASA';
    }
    GetCognitiveEngineCharsPerPhrase() {
        return config_1.COGNITIVE_SERVICES_RASA_CHARS_PER_PHRASE;
    }
    AddApp(subscriptionRegion, subscriptionKey, nlu) {
        let installationId = '';
        let vcc = nlu.vcc;
        let nluName = nlu.name;
        let appId = '';
        let rootFolder = config_1.RASA_ROOT_FOLDER;
        let nluLocalFolder = config_1.PATH_FOLDER_TEMP_NLU + '/' + vcc;
        let nluRemoteFolder = null;
        return promises_shared_1.DummyPromise()
            .then(result => {
            return platform_service_1.GetPlatformVersion();
        }).then(result => {
            logger.info('[RASAServiceImplementation::AddApp] Getting result:', result, 'after GetVersion');
            if (result) {
                installationId = result.installationId;
                let cmd = 'ls -d *' + installationId + '::*';
                return this.ExecuteRequest({ cmd: cmd, cwd: rootFolder });
            }
            else {
                throw 'SERVER_ERROR_NO_VERSION_DATA';
            }
        })
            .then(result => {
            if (result === null)
                throw 'SERVER_ERROR_NO_RESULT_FROM_RASA_SERVER';
            let arrContent = result.stdout.split(/\r?\n/);
            if (arrContent.includes(vcc + '::' + nlu.name))
                throw 'MODEL_NLU_ALREADY_EXIST';
            let cmd = 'mkdir -p ';
            appId = installationId + '::' + vcc + '::' + nluName.toLocaleLowerCase();
            cmd += appId;
            return this.ExecuteRequest({ cmd: cmd, cwd: rootFolder });
        }).then(result => {
            if (result && result.stderr && result.stderr !== '') {
                if (result.stderr.includes('No space left on device'))
                    throw 'SERVER_ERROR_NO_SPACE_ON_DEVICE';
            }
            logger.debug('[RASAServiceImplementation::AddApp] Create nlu folder: ' + JSON.stringify(result));
            return file_manager_1.CreateFolder(nluLocalFolder);
        }).then(result => {
            logger.debug('[RASAServiceImplementation::AddApp] Create folder: ' + JSON.stringify(result));
            let regex = new RegExp(`(?<=^language:\\s).*$`, 'm');
            let nluConfigContent = rasa_config_1.RASA_NLU_CONFIG_CONTENT;
            let newNluConfigTextContent = nluConfigContent.replace(regex, nlu.culture);
            let tempModelConfigPath = nluLocalFolder + '/' + nluName + '-config.yml';
            let promiseArray = [];
            promiseArray.push(file_manager_1.WriteFileContent(tempModelConfigPath, newNluConfigTextContent, 'utf-8'));
            nluRemoteFolder = rootFolder + '/' + appId;
            promiseArray.push(this.ExecuteRequest({ cmd: 'mkdir -p data', cwd: nluRemoteFolder }));
            promiseArray.push(this.ExecuteRequest({ cmd: 'mkdir -p models', cwd: nluRemoteFolder }));
            return Promise.all(promiseArray);
        }).then(result => {
            logger.debug('[RASAServiceImplementation::AddApp] Create nlu: ' + JSON.stringify(result));
            if (nlu.importNLU) {
                switch (nlu.importNLUFrom) {
                    case 'RASA': return this.ImportNLUFromRASA(nlu);
                    default: throw 'SERVER_ERROR_NOT_FOUND_COGNITIVE_ENGINE_SOURCE';
                }
            }
            else {
                return Promise.resolve(true);
            }
        }).then(result => {
            let promiseArray = [];
            promiseArray.push(this.ExecuteRequest({ type: 'putfile', originPath: nluLocalFolder + '/' + nluName + '-config.yml', targetPath: nluRemoteFolder + '/config.yml' }));
            promiseArray.push(this.ExecuteRequest({ type: 'putfile', originPath: config_1.STATIC_PY_FOLDER_PATH + '/nlu-train-data-generator.py', targetPath: nluRemoteFolder + '/trainDataGenerator.py' }));
            if (nlu.importNLU) {
                promiseArray.push(this.ExecuteRequest({ type: 'putfile', originPath: nluLocalFolder + '/' + nluName + '-nlu.md', targetPath: nluRemoteFolder + '/data/nlu.md' }));
            }
            else {
                promiseArray.push(this.ExecuteRequest({ cmd: 'touch nlu.md', cwd: nluRemoteFolder + '/data' }));
            }
            return Promise.all(promiseArray);
        }).then(result => {
            let cmd = 'ls ';
            let modelsFolder = nluRemoteFolder + '/models';
            return this.ExecuteRequest({ cmd: cmd, cwd: modelsFolder });
        }).then(result => {
            logger.debug('[RASAServiceImplementation::AddApp] Then: ' + JSON.stringify(result));
            return Promise.resolve(vcc + '::' + nluName.toLowerCase());
        }).catch((error) => {
            let errorMessage = typeof error === 'string' ? error : 'ERROR';
            logger.error('[RASAServiceImplementation::AddApp] Error: ', error);
            if (error.code && error.code === 'ETIMEDOUT')
                errorMessage = 'SERVER_ERROR_TIMEOUT_FROM_RASA_SERVER';
            if (error.code && error.code === 'ECONNREFUSED')
                errorMessage = 'SERVER_ERROR_CONNECTION_REFUSED_FROM_RASA_SERVER';
            return Promise.reject(errorMessage);
        });
    }
    DeleteApp(subscriptionRegion, subscriptionKey, nlu) {
        let installationId = '';
        let vcc = nlu.vcc;
        let nluName = nlu.name;
        let appId = '';
        let rootFolder = config_1.RASA_ROOT_FOLDER;
        let nluFolder = null;
        return promises_shared_1.DummyPromise()
            .then(result => {
            return platform_service_1.GetPlatformVersion();
        }).then(result => {
            logger.info('[RASAServiceImplementation::AddApp] Getting result:', result, 'after GetVersion');
            if (result) {
                installationId = result.installationId;
                let cmd = 'ls -d *' + installationId + '::*';
                return this.ExecuteRequest({ cmd: cmd, cwd: rootFolder });
            }
            else {
                throw 'SERVER_ERROR_NO_VERSION_DATA';
            }
        })
            .then(result => {
            if (result === null)
                throw 'SERVER_ERROR_NO_RESULT_FROM_RASA_SERVER';
            let arrContent = result.stdout.split(/\r?\n/);
            appId = installationId + '::' + vcc + '::' + nluName.toLowerCase();
            if (!arrContent.includes(appId))
                throw 'MODEL_NLU_NOT_FOUND';
            let cmd = 'rm -rf ';
            cmd += appId;
            return this.ExecuteRequest({ cmd: cmd, cwd: rootFolder });
        }).then(result => {
            let cmd = 'ls -d *' + installationId + '::*';
            return this.ExecuteRequest({ cmd: cmd, cwd: rootFolder });
        }).then(result => {
            if (result === null)
                throw 'SERVER_ERROR_NO_RESULT_FROM_RASA_SERVER';
            let arrContent = result.stdout.split(/\r?\n/);
            appId = installationId + '::' + vcc + '::' + nluName.toLowerCase();
            if (!arrContent.includes(appId))
                return Promise.resolve(true);
            else
                throw 'SERVER_ERROR_COULDNOT_DELETE_MODEL';
        }).catch((error) => {
            let errorMessage = typeof error === 'string' ? error : 'ERROR';
            logger.error('[RASAServiceImplementation::DeleteApp] Error: ', error);
            if (error.code && error.code === 'ETIMEDOUT')
                errorMessage = 'SERVER_ERROR_TIMEOUT_FROM_RASA_SERVER';
            if (error.code && error.code === 'ECONNREFUSED')
                errorMessage = 'SERVER_ERROR_CONNECTION_REFUSED_FROM_RASA_SERVER';
            return Promise.resolve(errorMessage);
        });
    }
    GetAppVersion(subscriptionRegion, subscriptionKey, appId) {
        let installationId = '';
        let vcc = appId.split('::')[0];
        let model = appId.split('::')[1];
        return promises_shared_1.DummyPromise()
            .then(result => {
            return mssql_service_1.MSSql.GetNLUByAppId(vcc, appId);
        }).then(result => {
            return Promise.resolve(result);
        }).catch((error) => {
            let errorMessage = typeof error === 'string' ? error : 'ERROR';
            logger.error('[RASAServiceImplementation::GetAppVersion] Error: ', error);
            return Promise.reject(errorMessage);
        });
    }
    GetAppsList(subscriptionRegion, subscriptionKey) {
        let installationId = '';
        let rootFolder = config_1.RASA_ROOT_FOLDER;
        return promises_shared_1.DummyPromise()
            .then(_ => {
            return platform_service_1.GetPlatformVersion();
        }).then(result => {
            logger.info('[RASAServiceImplementation::GetAppsList] Getting result:', result, 'after GetVersion');
            if (result) {
                installationId = result.installationId;
                let cmd = 'ls -d *' + installationId + '::*';
                return this.ExecuteRequest({ cmd: cmd, cwd: rootFolder });
            }
            else {
                throw 'ERROR_NO_VERSION_DATA';
            }
        })
            .then(result => {
            if (result === null)
                throw 'SERVER_ERROR_NO_RESULT_FROM_RASA_SERVER';
            let arrContent = result.stdout.split(/\r?\n/);
            let res = [];
            arrContent.filter(ac => ac.trim().length > 0).map(i => res.push({ id: i.split('::')[1] + '::' + i.split('::')[2] }));
            return Promise.resolve(res);
        }).catch((error) => {
            let errorMessage = typeof error === 'string' ? error : 'ERROR';
            logger.error('[RASAServiceImplementation::GetAppsList] Error: ', error);
            if (error.code && error.code === 'ETIMEDOUT')
                errorMessage = 'SERVER_ERROR_TIMEOUT_FROM_RASA_SERVER';
            if (error.code && error.code === 'ECONNREFUSED')
                errorMessage = 'SERVER_ERROR_CONNECTION_REFUSED_FROM_RASA_SERVER';
            return Promise.reject(errorMessage);
        });
    }
    GetAppById(appId, subscriptionRegion, subscriptionKey) {
        return null;
    }
    GetCultures(subscriptionRegion, subscriptionKey) {
        let cultures = [];
        cultures.push({ name: "Chinese", code: "zh" });
        cultures.push({ name: "Dutch", code: "nl" });
        cultures.push({ name: "English", code: "en" });
        cultures.push({ name: "French", code: "fr" });
        cultures.push({ name: "German", code: "de" });
        cultures.push({ name: "Italian", code: "it" });
        cultures.push({ name: "Portuguese", code: "pt" });
        cultures.push({ name: "Spanish", code: "es" });
        return Promise.resolve(cultures.filter(c => config_1.COGNITIVE_SERVICES_RASA_AVAILABLE_LANGUAGES.includes(c.code)));
    }
    AddIntent(subscriptionRegion, subscriptionKey, appId, name) {
        return promises_shared_1.DummyPromise()
            .then(result => {
            return Promise.resolve(utilities_1.guid());
        });
    }
    DeleteAppIntent(subscriptionRegion, subscriptionKey, appId, intentId) {
        let installationId = '';
        let vcc = appId.split('::')[0];
        let model = appId.split('::')[1];
        let rootFolder = config_1.RASA_ROOT_FOLDER;
        let modelFolder = '';
        let nluLocalFolder = config_1.PATH_FOLDER_TEMP_NLU + '/' + vcc;
        let oldNluTextContent = '';
        let newNluTextContent = '';
        return promises_shared_1.DummyPromise()
            .then(result => {
            return platform_service_1.GetPlatformVersion();
        }).then(result => {
            logger.info('[RASAServiceImplementation::DeleteAppIntent] Getting result:', result, 'after GetVersion');
            return mssql_service_1.MSSql.DeleteIntent(vcc, appId, intentId);
        })
            .then(result => {
            return utterance_service_1.GetUtterances(vcc, appId, null, null, null, true);
        }).catch((error) => {
            let errorMessage = typeof error === 'string' ? error : 'ERROR';
            logger.error('[RASAServiceImplementation::DeleteAppIntent] Error: ', error);
            if (error.code && error.code === 'ETIMEDOUT')
                errorMessage = 'SERVER_ERROR_TIMEOUT_FROM_RASA_SERVER';
            if (error.code && error.code === 'ECONNREFUSED')
                errorMessage = 'SERVER_ERROR_CONNECTION_REFUSED_FROM_RASA_SERVER';
            return Promise.reject(errorMessage);
        });
    }
    GetIntentsList(subscriptionRegion, subscriptionKey, appId) {
        let vcc = appId.split('::')[0];
        return promises_shared_1.DummyPromise()
            .then(result => {
            return mssql_service_1.MSSql.GetIntents(vcc, appId, null, null, null);
        })
            .then(result => {
            let res = result ? result : [];
            let arrRes = [];
            res.map(r => {
                arrRes.push({
                    id: r.intentId,
                    name: r.name
                });
            });
            return Promise.resolve(arrRes);
        })
            .catch((error) => {
            let errorMessage = typeof error === 'string' ? error : 'ERROR';
            logger.error('[RASAServiceImplementation::GetIntentsList] Error: ', error);
            if (error.code && error.code === 'ETIMEDOUT')
                errorMessage = 'SERVER_ERROR_TIMEOUT_FROM_RASA_SERVER';
            if (error.code && error.code === 'ECONNREFUSED')
                errorMessage = 'SERVER_ERROR_CONNECTION_REFUSED_FROM_RASA_SERVER';
            return Promise.reject(errorMessage);
        });
    }
    AddUtterance(subscriptionRegion, subscriptionKey, appId, utterance) {
        let vcc = utterance.appId.split('::')[0];
        let intentName = utterance.intent;
        let text = utterance.name;
        let entityLabels = utterance.entityLabels ? utterance.entityLabels : [];
        let savedUtterance = null;
        let savedUtteranceEntities = [];
        return promises_shared_1.DummyPromise()
            .then(result => {
            return mssql_service_1.MSSql.GetUtterance(vcc, appId, intentName, text);
        })
            .then(result => {
            savedUtterance = result;
            if (savedUtterance) {
                utterance.id = savedUtterance.id;
            }
            else {
                utterance.id = utilities_1.guid();
            }
        })
            .then(result => {
            if (savedUtterance) {
                return mssql_service_1.MSSql.GetUtteranceEntities(vcc, utterance.id);
            }
            else {
                return Promise.resolve([]);
            }
        })
            .then(result => {
            if (result)
                savedUtteranceEntities = result;
            let promiseArray = [];
            entityLabels.map(el => {
                if (!savedUtteranceEntities.find(sue => sue.utteranceId === utterance.id && sue.entityName === el.entityName && sue.start === el.start && sue.end === el.end)) {
                    promiseArray.push(mssql_service_1.MSSql.SaveUtteranceEntity(vcc, utterance.id, el.entityName, el.start, el.end, el.startTokenIndex, el.endTokenIndex));
                }
            });
            savedUtteranceEntities.map(sue => {
                if (!entityLabels.find(el => el.utteranceId === utterance.id && el.entityName === sue.entityName && el.start === sue.start && el.end === sue.end)) {
                    promiseArray.push(mssql_service_1.MSSql.DeleteUtteranceEntity(vcc, utterance.id, sue.entityName, sue.start, sue.end, sue.startTokenIndex, sue.endTokenIndex));
                }
            });
            return Promise.all(promiseArray);
        })
            .then(result => {
            return Promise.resolve({
                UtteranceText: utterance.name,
                ExampleId: savedUtterance ? savedUtterance.id : utilities_1.guid()
            });
        })
            .catch((error) => {
            let errorMessage = typeof error === 'string' ? error : 'ERROR';
            logger.error('[RASAServiceImplementation::AddUtterance] Error: ', error);
            if (error.code && error.code === 'ETIMEDOUT')
                errorMessage = 'SERVER_ERROR_TIMEOUT_FROM_RASA_SERVER';
            if (error.code && error.code === 'ECONNREFUSED')
                errorMessage = 'SERVER_ERROR_CONNECTION_REFUSED_FROM_RASA_SERVER';
            return Promise.reject(errorMessage);
        });
    }
    DeleteAppUtterance(subscriptionRegion, subscriptionKey, appId, intentId, utteranceId) {
        let installationId = '';
        let vcc = appId.split('::')[0];
        let model = appId.split('::')[1];
        let rootFolder = config_1.RASA_ROOT_FOLDER;
        let modelFolder = '';
        let nluLocalFolder = config_1.PATH_FOLDER_TEMP_NLU + '/' + vcc;
        let oldNluTextContent = '';
        let newNluTextContent = '';
        return promises_shared_1.DummyPromise()
            .then(result => {
            return platform_service_1.GetPlatformVersion();
        }).then(result => {
            logger.info('[RASAServiceImplementation::AddIntent] Getting result:', result, 'after GetVersion');
            return mssql_service_1.MSSql.DeleteUtterance(vcc, appId, intentId, utteranceId);
        })
            .then(result => {
            return utterance_service_1.GetUtterances(vcc, appId, null, null, null, true);
        }).catch((error) => {
            let errorMessage = typeof error === 'string' ? error : 'ERROR';
            logger.error('[RASAServiceImplementation::DeleteUtterance] Error: ', error);
            if (error.code && error.code === 'ETIMEDOUT')
                errorMessage = 'SERVER_ERROR_TIMEOUT_FROM_RASA_SERVER';
            if (error.code && error.code === 'ECONNREFUSED')
                errorMessage = 'SERVER_ERROR_CONNECTION_REFUSED_FROM_RASA_SERVER';
            return Promise.reject(errorMessage);
        });
    }
    GetUtterancesList(subscriptionRegion, subscriptionKey, appId) {
        let vcc = appId.split('::')[0];
        let model = appId.split('::')[1];
        let installationId = '';
        let rootFolder = config_1.RASA_ROOT_FOLDER;
        let utterances = [];
        return promises_shared_1.DummyPromise()
            .then(result => {
            return utterance_service_1.GetUtterances(vcc, appId, null, null, null, true);
        })
            .then(result => {
            if (result)
                utterances = result;
            return Promise.resolve(utterances);
        })
            .catch((error) => {
            let errorMessage = typeof error === 'string' ? error : 'ERROR';
            logger.error('[RASAServiceImplementation::GetUtterancesList] Error: ', error);
            if (error.code && error.code === 'ETIMEDOUT')
                errorMessage = 'SERVER_ERROR_TIMEOUT_FROM_RASA_SERVER';
            if (error.code && error.code === 'ECONNREFUSED')
                errorMessage = 'SERVER_ERROR_CONNECTION_REFUSED_FROM_RASA_SERVER';
            return Promise.reject(errorMessage);
        });
    }
    GetUtterancesFromMD(appId) {
        let installationId = '';
        let rootFolder = config_1.RASA_ROOT_FOLDER;
        let utterances = [];
        return promises_shared_1.DummyPromise()
            .then(result => {
            return platform_service_1.GetPlatformVersion();
        })
            .then(result => {
            if (result) {
                installationId = result.installationId;
                let cmd = 'cat nlu.md';
                let modelFolder = installationId + '::' + appId.toLowerCase();
                return this.ExecuteRequest({ cmd: cmd, cwd: rootFolder + '/' + modelFolder + '/data' });
            }
            else {
                throw 'ERROR_NO_VERSION_DATA';
            }
        })
            .then(result => {
            if (result === null)
                throw 'SERVER_ERROR_NO_RESULT_FROM_RASA_SERVER';
            let nluTextContent = result.stdout;
            utterances = this.GetAllUtterancesFromRASAContent(nluTextContent);
            return Promise.resolve(utterances);
        })
            .catch((error) => {
            let errorMessage = typeof error === 'string' ? error : 'ERROR';
            logger.error('[RASAServiceImplementation::GetUtterancesFromMD] Error: ', error);
            if (error.code && error.code === 'ETIMEDOUT')
                errorMessage = 'SERVER_ERROR_TIMEOUT_FROM_RASA_SERVER';
            if (error.code && error.code === 'ECONNREFUSED')
                errorMessage = 'SERVER_ERROR_CONNECTION_REFUSED_FROM_RASA_SERVER';
            return Promise.reject(errorMessage);
        });
    }
    AddSimpleEntity(subscriptionRegion, subscriptionKey, appId, entity) {
        return promises_shared_1.DummyPromise()
            .then(result => {
            return Promise.resolve(utilities_1.guid());
        });
    }
    RenameEntity(subscriptionRegion, subscriptionKey, appId, entityId, entityName) {
        return null;
    }
    GetEntitiesList(subscriptionRegion, subscriptionKey, appId) {
        return promises_shared_1.DummyPromise()
            .then(result => {
            return mssql_service_1.MSSql.GetEntities(appId.split('::')[0], appId, '', '', '');
        });
    }
    GetEntity(subscriptionRegion, subscriptionKey, appId, entityId) {
        return null;
    }
    DeleteAppEntity(subscriptionRegion, subscriptionKey, appId, entityId) {
        return null;
    }
    TrainAppManual(subscriptionRegion, subscriptionKey, appId) {
        let installationId = '';
        let vcc = appId.split('::')[0];
        let model = appId.split('::')[1];
        let rootFolder = config_1.RASA_ROOT_FOLDER;
        let modelFolder = '';
        return promises_shared_1.DummyPromise()
            .then(result => {
            return platform_service_1.GetPlatformVersion();
        }).then(result => {
            logger.info('[RASAServiceImplementation::AddApp] Getting result:', result, 'after GetVersion');
            if (result) {
                installationId = result.installationId;
                let cmd = 'rasa train nlu -c config.yml --out models';
                modelFolder = rootFolder + '/' + installationId + '::' + appId.toLowerCase();
                return this.ExecuteRequest({ cmd: cmd, cwd: modelFolder });
            }
            else {
                throw 'SERVER_ERROR_NO_VERSION_DATA';
            }
        })
            .then(result => {
            if (!result)
                throw 'SERVER_ERROR_COULDNOT_TRAIN_MODEL';
            if (result && result.stdout) {
                let matchRegex = new RegExp(`Your Rasa model is trained and saved `, 'm');
                let searchOnContent = result.stdout.match(matchRegex);
                if (searchOnContent) {
                    return Promise.resolve(true);
                }
                else
                    throw 'SERVER_ERROR_MODEL_TRAIN_FAILED';
            }
            else {
                throw 'SERVER_ERROR_MODEL_TRAIN_FAILED';
            }
        }).catch((error) => {
            let errorMessage = typeof error === 'string' ? error : 'ERROR';
            logger.error('[RASAServiceImplementation::TrainApp] Error: ', error);
            if (error.code && error.code === 'ETIMEDOUT')
                errorMessage = 'SERVER_ERROR_TIMEOUT_FROM_RASA_SERVER';
            if (error.code && error.code === 'ECONNREFUSED')
                errorMessage = 'SERVER_ERROR_CONNECTION_REFUSED_FROM_RASA_SERVER';
            return Promise.reject(errorMessage);
        });
    }
    TrainApp(subscriptionRegion, subscriptionKey, appId) {
        let installationId = '';
        let vcc = appId.split('::')[0];
        let model = appId.split('::')[1];
        let rootFolder = config_1.RASA_ROOT_FOLDER;
        let modelFolderPath = '';
        let modelPort = null;
        let modelTrainFile = null;
        return promises_shared_1.DummyPromise()
            .then(result => {
            return this.GenerateNluMd(appId);
        })
            .then(result => {
            return platform_service_1.GetPlatformVersion();
        })
            .then(result => {
            logger.info('[RASAServiceImplementation::TrainApp] Getting result:', result, 'after GetVersion');
            if (result) {
                installationId = result.installationId;
                modelFolderPath = config_1.RASA_ROOT_FOLDER + '/' + installationId + '::' + appId.toLowerCase();
                let cmd = 'python3 trainDataGenerator.py';
                return this.ExecuteRequest({ cmd: cmd, cwd: modelFolderPath });
            }
            else {
                throw 'SERVER_ERROR_NO_VERSION_DATA';
            }
        })
            .then(result => {
            if (!result)
                throw 'SERVER_ERROR_CANNOT_GENERATE_TRAIN_DATA_FILE';
            return this.RunApp(installationId + '::' + appId.toLowerCase(), 'TRAIN');
        }).then(result => {
            if (!result)
                throw 'SERVER_ERROR_CANNOT_RUN_APP';
            modelPort = result;
            let getOptions = {
                uri: `http://${config_1.RASA_IP}:${modelPort}/status`,
                timeout: 300000
            };
            return request_shared_1.SimpleGetRequest(getOptions)
                .then((response) => {
                return Promise.resolve(JSON.stringify(response));
            })
                .catch((err) => {
                logger.debug('[RASAServiceImplementation::TraingApp] RP RESULT: ', err.message);
                return Promise.resolve(err.message);
            });
        }).then(result => {
            logger.debug('[RASAServiceImplementation::TrainApp] Status RESULT: ', result);
            if (result
                &&
                    (result.includes('refused')
                        ||
                            result.includes('ECONNREFUSED'))) {
                redis_service_1.Redis.DeleteRunningRasaModel(installationId + '::' + appId.toLowerCase());
                throw 'SERVER_ERROR_CANNOT_RUN_APP';
            }
            modelTrainFile = 'nlu-' + moment.utc().format('YYYYMMDD-HHmmss') + '.tar.gz';
            let cmd = `curl -X POST http://${config_1.RASA_IP}:${modelPort}/model/train -H 'Accept-Encoding: gzip, deflate' -H 'Content-Type: application/json' -d @${modelFolderPath}/trainData.json --output ${modelFolderPath}/models/${modelTrainFile}`;
            return this.ExecuteRequest({ cmd: cmd, cwd: modelFolderPath, executor: 'ssh' });
        }).then(result => {
            let cmd = 'ls ';
            return this.ExecuteRequest({ cmd: cmd, cwd: modelFolderPath + '/models' });
        }).then(result => {
            let arrNluFiles = result && result.stdout ? result.stdout.split('\n') : [];
            if (!arrNluFiles.includes(modelTrainFile))
                throw 'SERVER_ERROR_TRAIN_MODEL_FAILED';
            let cmd = `find . ! -name '${modelTrainFile}' -type f -exec rm -f {} +`;
            return this.ExecuteRequest({ cmd: cmd, cwd: modelFolderPath + '/models' });
        })
            .then(result => {
            return this.StopApp(appId, modelPort, 'TRAIN');
        }).then(result => {
            return Promise.resolve({ status: 'UpToDate', statusId: 2 });
        }).catch((error) => {
            let errorMessage = typeof error === 'string' ? error : 'ERROR';
            logger.error('[RASAServiceImplementation::TrainApp] Error: ', error);
            if (error === 'SERVER_ERROR_MISSING_UTTERANCES_TO_TRAIN')
                return Promise.resolve({ status: 'Queued', statusId: 9 });
            if (error.code && error.code === 'ETIMEDOUT')
                errorMessage = 'SERVER_ERROR_TIMEOUT_FROM_RASA_SERVER';
            if (error.code && error.code === 'ECONNREFUSED')
                errorMessage = 'SERVER_ERROR_CONNECTION_REFUSED_FROM_RASA_SERVER';
            return Promise.reject(errorMessage);
        });
    }
    ModelQuery(subscriptionRegion, subscriptionKey, appId, query, processAudios, nluName, resolve, task) {
        let installationId = '';
        let vcc = appId.split('::')[0];
        let model = appId.split('::')[1];
        let rootFolder = config_1.RASA_ROOT_FOLDER;
        let modelFolder = '';
        let modelPort = null;
        return promises_shared_1.DummyPromise()
            .then(result => {
            return platform_service_1.GetPlatformVersion();
        }).then(result => {
            if (result) {
                installationId = result.installationId;
                modelFolder = rootFolder + '/' + installationId + '::' + appId.toLowerCase();
                if (task === 'TEST') {
                    return redis_service_1.Redis.GetRunningRasaModelFromTest(installationId + '::' + appId.toLowerCase());
                }
                else
                    return redis_service_1.Redis.GetRunningRasaModel(installationId + '::' + appId.toLowerCase());
            }
            else {
                throw 'SERVER_ERROR_NO_VERSION_DATA';
            }
        })
            .then(result => {
            if (!result)
                throw 'SERVER_ERROR_CANNOT_GET_MODEL_PORT';
            modelPort = result.port;
            let getOptions = {
                uri: `http://${config_1.RASA_IP}:${modelPort}/status`,
                retryProtocol: false,
                timeout: 60000
            };
            return request_shared_1.SimpleGetRequest(getOptions)
                .then((response) => {
                return Promise.resolve(JSON.stringify(response));
            })
                .catch((err) => {
                logger.debug('[RASAServiceImplementation::TraingApp] RP RESULT: ', err.message);
                return Promise.resolve(err.message);
            });
        }).then(result => {
            logger.debug('[RASAServiceImplementation::ModelQuery] Status RESULT: ', result);
            if (result
                &&
                    (result.includes('refused')
                        ||
                            result.includes('ECONNREFUSED'))) {
                redis_service_1.Redis.DeleteRunningRasaModel(installationId + '::' + appId.toLowerCase());
                throw 'SERVER_ERROR_CANNOT_RUN_APP';
            }
            let postOptions = {
                uri: `http://${config_1.RASA_IP}:${modelPort}/model/parse`,
                body: {
                    text: query.replace('"', '\"')
                },
                retryProtocol: true,
                timeout: 60000
            };
            logger.info('[RASAServiceImplementation::ModelQuery] getting getOptions:', postOptions, 'appId:', appId, 'query:', query, 'processAudios:', processAudios, 'nluName:', nluName);
            return request_shared_1.SimplePostRequest(postOptions);
        }).then(result => {
            if (!result)
                throw 'SERVER_ERROR_PARSE_REQUEST_FAILED';
            if (!result.intent_ranking)
                throw 'SERVER_ERROR_NO_RESULTS_FOR_REQUEST';
            logger.info('[RASAServiceImplementation::ModelQuery] Result: ' + JSON.stringify(result));
            let intents = [];
            result.intent_ranking.map(ir => {
                intents.push({
                    intent: ir.name,
                    score: ir.confidence
                });
            });
            let uniqueEntityNames = [];
            uniqueEntityNames = arrays_shared_1.GetUniqueFromObjectArray(result.entities, 'entity');
            let uniqueEntities = [];
            uniqueEntityNames.map(uen => {
                let currentEntities = result.entities.filter(e => e.entity === uen.entity);
                currentEntities = arrays_shared_1.SortObjectArrayByValue(currentEntities, 'confidence_entity', 'D');
                uniqueEntities.push(currentEntities[0]);
            });
            let entities = [];
            uniqueEntities.map(e => {
                entities.push({
                    entity: e.value,
                    type: e.entity,
                    startIndex: e.start,
                    endIndex: e.end,
                    score: e.confidence_entity
                });
            });
            let parseResult = {
                query: result.text ? result.text : null,
                topScoringIntent: {
                    intent: result.intent && result.intent.name ? result.intent.name : null,
                    score: result.intent && result.intent.confidence ? result.intent.confidence : 0
                },
                intents: intents,
                entities: entities
            };
            logger.info('[RASAServiceImplementation::ModelQuery] FormattedResult: ' + JSON.stringify(parseResult));
            return Promise.resolve(parseResult);
        }).catch((error) => {
            let errorMessage = typeof error === 'string' ? error : 'ERROR';
            logger.error('[RASAServiceImplementation::ModelQuery] Error: ', error);
            if (error.code && error.code === 'ETIMEDOUT')
                errorMessage = 'SERVER_ERROR_TIMEOUT_FROM_RASA_SERVER';
            if (error.code && error.code === 'ECONNREFUSED') {
                redis_service_1.Redis.DeleteRunningRasaModel(installationId + '::' + appId.toLowerCase());
                errorMessage = 'SERVER_ERROR_CONNECTION_REFUSED_FROM_RASA_SERVER';
            }
            return Promise.reject(errorMessage);
        });
    }
    PublishApp(subscriptionRegion, subscriptionKey, appId, isStaging) {
        if (isStaging)
            return this.RunApp(appId, 'PARSE');
        else
            return this.StopApp(appId, 0, 'PARSE');
    }
    ExportModel(nlu, res) {
        let installationId = '';
        let rootFolder = config_1.RASA_ROOT_FOLDER;
        let modelFolder = null;
        let modelToExport = {};
        let intentsToExport = [];
        let entitiesToExport = [];
        let utteranceEntitiesToExport = [];
        return promises_shared_1.DummyPromise()
            .then(result => {
            return mssql_service_1.MSSql.GetNLUByAppId(nlu.vcc, nlu.appId);
        })
            .then(result => {
            if (!result)
                throw 'SERVER_ERROR_NLU_NOT_FOUND';
            modelToExport = {
                name: result.name,
                culture: result.culture,
                description: result.description,
                cognitiveEngine: this.GetCognitiveEngineName()
            };
            let promiseArray = [];
            promiseArray.push(mssql_service_1.MSSql.GetIntents(nlu.vcc, nlu.appId, null, null, null));
            promiseArray.push(mssql_service_1.MSSql.GetUtterances(nlu.vcc, nlu.appId, null));
            promiseArray.push(mssql_service_1.MSSql.GetEntities(nlu.vcc, nlu.appId, '', '', null));
            return Promise.all(promiseArray);
        })
            .then(result => {
            if (!result[0])
                throw 'SERVER_ERROR_NLU_INTENTS_NOT_FOUND';
            if (!result[1])
                throw 'SERVER_ERROR_NLU_UTTERANCES_NOT_FOUND';
            if (!result[2])
                throw 'SERVER_ERROR_NLU_ENTITIES_NOT_FOUND';
            let promiseArray = [];
            result[0].map(i => {
                let intentUtterances = [];
                result[1].filter(r => r.intentId === i.intentId).map(u => {
                    intentUtterances.push({
                        id: u.id,
                        text: u.text
                    });
                    promiseArray.push(mssql_service_1.MSSql.GetUtteranceEntities(nlu.vcc, u.id));
                });
                intentsToExport.push({
                    id: i.intentId,
                    name: i.name,
                    color: i.color,
                    utterances: intentUtterances
                });
            });
            result[2].map(e => {
                entitiesToExport.push({
                    id: utilities_1.guid(),
                    name: e.name,
                    type: e.type
                });
            });
            modelToExport['intents'] = intentsToExport;
            modelToExport['entities'] = entitiesToExport;
            return Promise.all(promiseArray);
        })
            .then(result => {
            result.map(r => {
                r.map(ue => {
                    utteranceEntitiesToExport.push({
                        utteranceId: ue.utteranceId,
                        entityName: ue.entityName,
                        start: ue.start,
                        end: ue.end,
                        startTokenIndex: ue.startTokenIndex,
                        endTokenIndex: ue.endTokenIndex
                    });
                });
            });
            modelToExport['utteranceEntities'] = utteranceEntitiesToExport;
            return Promise.resolve(true);
        })
            .then(result => {
            modelToExport.intents.map(i => {
                i.utterances.map(u => {
                    let newId = utilities_1.guid();
                    let oldId = u.id;
                    u.id = newId;
                    modelToExport.utteranceEntities.filter(ue => ue.utteranceId === oldId).map(ue => { ue.utteranceId = newId; });
                });
            });
            return Promise.resolve(modelToExport);
        })
            .catch((error) => {
            let errorMessage = typeof error === 'string' ? error : 'ERROR';
            logger.error('[RASAServiceImplementation::ExportModel] Error: ', error);
            if (error.code && error.code === 'ETIMEDOUT')
                errorMessage = 'SERVER_ERROR_TIMEOUT_FROM_RASA_SERVER';
            if (error.code && error.code === 'ECONNREFUSED')
                errorMessage = 'SERVER_ERROR_CONNECTION_REFUSED_FROM_RASA_SERVER';
            return Promise.reject(errorMessage);
        });
    }
    GetIntentsListDemo(subscriptionRegion, subscriptionKey, appId, resolve, reject) {
        let vcc = appId.split('::')[0];
        return promises_shared_1.DummyPromise()
            .then(result => {
            return mssql_service_1.MSSql.GetIntents(vcc, appId, null, null, null);
        })
            .then(result => {
            let res = result ? result : [];
            let arrRes = [];
            res.map(r => {
                arrRes.push({
                    id: r.intentId,
                    name: r.name
                });
            });
            return Promise.resolve(arrRes);
        })
            .catch((error) => {
            let errorMessage = typeof error === 'string' ? error : 'ERROR';
            logger.error('[RASAServiceImplementation::GetIntentsListDemo] Error: ', error);
            if (error.code && error.code === 'ETIMEDOUT')
                errorMessage = 'SERVER_ERROR_TIMEOUT_FROM_RASA_SERVER';
            if (error.code && error.code === 'ECONNREFUSED')
                errorMessage = 'SERVER_ERROR_CONNECTION_REFUSED_FROM_RASA_SERVER';
            return Promise.reject(errorMessage);
        });
    }
    GetTrainApp(subscriptionRegion, subscriptionKey, appId) {
        let installationId = '';
        let rootFolder = config_1.RASA_ROOT_FOLDER;
        let ssh = null;
        return promises_shared_1.DummyPromise()
            .then(result => {
            return platform_service_1.GetPlatformVersion();
        }).then(result => {
            logger.info('[RASAServiceImplementation::GetIntentsList] Getting result:', result, 'after GetVersion');
            if (result) {
                installationId = result.installationId;
                let cmd = 'cat nlu.md';
                let modelFolder = installationId + '::' + appId.toLowerCase();
                return this.ExecuteRequest({ cmd: cmd, cwd: rootFolder + '/' + modelFolder + '/data' });
            }
            else {
                throw 'ERROR_NO_VERSION_DATA';
            }
        })
            .then(result => {
            if (result === null)
                throw 'SERVER_ERROR_NO_RESULT_FROM_RASA_SERVER';
            let arrIntentItems = this.GetAllIntentsFromRASAContent(result.stdout);
            let arrUtteranceItems = this.GetAllUtterancesFromRASAContent(result.stdout);
            let arrRes = [];
            arrIntentItems.map(aii => {
                let intentDetails = {
                    exampleCount: 0,
                    failureReason: arrUtteranceItems.filter(aui => aui.intentLabel === aii.id).length === 0 ? 'FewLabels' : 'Aborted',
                    status: 'Fail',
                    statusId: 1
                };
                arrRes.push({ details: intentDetails, modelId: aii.id });
            });
            return Promise.resolve(arrRes);
        }).catch((error) => {
            let errorMessage = typeof error === 'string' ? error : 'ERROR';
            logger.error('[RASAServiceImplementation::GetIntentsList] Error: ', error);
            if (error.code && error.code === 'ETIMEDOUT')
                errorMessage = 'SERVER_ERROR_TIMEOUT_FROM_RASA_SERVER';
            if (error.code && error.code === 'ECONNREFUSED')
                errorMessage = 'SERVER_ERROR_CONNECTION_REFUSED_FROM_RASA_SERVER';
            return Promise.reject(errorMessage);
        });
    }
    GetManagerToken(info) {
        return null;
    }
    LoadAzureAccounts(subscriptionRegion, subscriptionKey, data) {
        return null;
    }
    UnAssignSubscriptionKeyOfVCC(subscriptionRegion, subscriptionKey, appId, data) {
        return null;
    }
    AssignSubscriptionKeyOfVCC(subscriptionRegion, subscriptionKey, appId, data) {
        return null;
    }
    GetAllIntentsFromRASAContent(content) {
        let arrIntentItems = content.split(/\r?\n/);
        let arrIntents = arrIntentItems.filter(aii => aii.includes('## intent:'));
        let res = arrIntents ? arrIntents.map(ai => ai.replace('## intent:', '')) : [];
        let arrRes = [];
        res.map(r => {
            if (r.trim() !== 'None') {
                arrRes.push({
                    id: r.trim(),
                    name: r.trim()
                });
            }
        });
        return arrRes;
    }
    GetAllUtterancesFromRASAContent(content) {
        let arrIntentBlock = content.split(/##\sintent:/);
        let arrUterances = [];
        arrIntentBlock.map(aib => {
            if (aib !== '') {
                let arrIntentItems = aib.split(/\r?\n/);
                let intent = arrIntentItems.shift();
                arrIntentItems.map(aii => {
                    if (aii !== '' && intent.trim() !== 'None') {
                        aii = aii.replace('- ', '');
                        let utterance = new utterance_model_1.UtteranceObject();
                        utterance.entityLabels = [];
                        utterance.entityPredictions = [];
                        utterance.id = utilities_1.guid(),
                            utterance.intentLabel = intent.trim(),
                            utterance.intentPredictions = [
                                {
                                    name: intent.trim(),
                                    score: null
                                }
                            ],
                            utterance.multiIntentPredictions = null,
                            utterance.text = aii.trim(),
                            utterance.tokenizedText = aii.split(/\s/);
                        arrUterances.push(utterance);
                    }
                });
            }
        });
        return arrUterances;
    }
    RunApp(appId, task) {
        logger.info('[RASAServiceImplementation::RunApp] Launching: ' + appId + ', task: ' + task);
        let modelFolder = config_1.RASA_ROOT_FOLDER + '/';
        let selectedPort = null;
        let runModelCommand = null;
        let resultMessage = null;
        return promises_shared_1.DummyPromise()
            .then(result => {
            if (appId.split('::').length === 2) {
                return platform_service_1.GetPlatformVersion();
            }
            else {
                return Promise.resolve(true);
            }
        }).then(result => {
            logger.info('[RASAServiceImplementation::RunApp] Getting result:', result, 'after GetVersion');
            if (appId.split('::').length === 2) {
                appId = result.installationId + '::' + appId.toLowerCase();
            }
            modelFolder += appId;
            if (task === 'TRAIN') {
                logger.debug('[RASAServiceImplementation::RunApp] Close other training instances');
                return this.StopApp(appId, null, task);
            }
            else {
                return Promise.resolve(true);
            }
        })
            .then(result => {
            let cmd = `for i in {${config_1.RASA_AVAILABLE_PORTS}}; do (exec 2>&-; echo > /dev/tcp/localhost/$i && echo $i is open); done`;
            return this.ExecuteRequest({ cmd: cmd, cwd: modelFolder, executor: 'ssh' });
        }).then(result => {
            if (!result)
                throw 'SERVER_ERROR_CANNOT_GET_PORTS_LIST';
            let freePorts = [];
            let rangeAvailablePorts = config_1.RASA_AVAILABLE_PORTS.split('..');
            for (let _i = Number(rangeAvailablePorts[0]); _i <= Number(rangeAvailablePorts[1]); _i++) {
                if (result.stdout === '' || !result.stdout.includes(_i + ' is open'))
                    freePorts.push(_i);
            }
            if (freePorts.length === 0)
                throw 'SERVER_ERROR_AVAILABLE_PORTS_NOT_FOUND';
            selectedPort = freePorts[Math.floor(Math.random() * freePorts.length)];
            runModelCommand = `nohup /usr/local/bin/rasa run --model ${modelFolder}/models --enable-api --log-file ${modelFolder}/${task}.log -p ${selectedPort} < /dev/null > /dev/null 2>&1 &`;
            logger.info('[RASAServiceImplementation::RunApp] Launching: ' + runModelCommand);
            this.ExecuteRequest({ cmd: 'sudo su -', stdin: runModelCommand, cwd: modelFolder, executor: 'ssh' });
            return promises_shared_1.SleepPromise(10);
        }).then(result => {
            logger.info('[RASAServiceImplementation::RunApp] Calling CheckRunningApp from RunApp');
            return this.CheckRunningApp(appId, selectedPort, task);
        }).then(result => {
            if (task === 'TRAIN') {
                return Promise.resolve(true);
            }
            else if (task === 'PARSE') {
                let runningModel = {
                    id: appId,
                    port: selectedPort,
                    created: moment.utc().format('YYYY-MM-DD HH:mm:ss')
                };
                return redis_service_1.Redis.SaveRunningRasaModel(appId, runningModel);
            }
            else if (task === 'TEST') {
                let runningModelFromTest = {
                    id: appId,
                    port: selectedPort,
                    created: moment.utc().format('YYYY-MM-DD HH:mm:ss')
                };
                return redis_service_1.Redis.SaveRunningRasaModelFromTest(appId, runningModelFromTest);
            }
        }).then(result => {
            return Promise.resolve(selectedPort);
        }).catch((error) => {
            let errorMessage = typeof error === 'string' ? error : 'ERROR';
            logger.error('[RASAServiceImplementation::RunApp] Error: ', error, '=>', resultMessage);
            if (error.code && error.code === 'ETIMEDOUT')
                errorMessage = 'SERVER_ERROR_TIMEOUT_FROM_RASA_SERVER';
            if (error.code && error.code === 'ECONNREFUSED')
                errorMessage = 'SERVER_ERROR_CONNECTION_REFUSED_FROM_RASA_SERVER';
            if (error?.code === 'ECONNRESET')
                errorMessage = 'SERVER_ERROR_CONNECTION_RESETED_FROM_RASA_SERVER';
            return Promise.reject(errorMessage);
        });
    }
    CheckRunningApp(appId, port, action, counter) {
        counter = counter ? counter : 0;
        let modelFolder = config_1.RASA_ROOT_FOLDER + '/' + appId;
        return promises_shared_1.SleepPromise(5)
            .then(result => {
            if (counter >= 10) {
                redis_service_1.Redis.DeleteRunningRasaModel(appId);
                throw 'SERVER_ERROR_RETRY_LIMIT_EXCEEDED';
            }
            let cmd = 'ls ';
            return this.ExecuteRequest({ cmd: cmd, cwd: modelFolder });
        }).then(result => {
            if (result.stdout.split('\n').includes(action + '.log')) {
                return Promise.resolve(true);
            }
            else {
                return this.CheckRunningApp(appId, port, action, counter + 1);
            }
        }).then(result => {
            let getOptions = {
                uri: `http://${config_1.RASA_IP}:${port}/status`,
                timeout: 60000
            };
            return request_shared_1.SimpleGetRequest(getOptions)
                .then((response) => {
                return Promise.resolve(JSON.stringify(response));
            })
                .catch((err) => {
                logger.debug(`[RASAServiceImplementation::CheckRunningApp] ${appId} => RP RESULT: ${err.message}`);
                return Promise.resolve(err.message);
            });
        })
            .then(result => {
            logger.debug(`[RASAServiceImplementation::CheckRunningApp] ${appId} => Status RESULT: ${result}`);
            if (result
                &&
                    (result.includes('refused')
                        ||
                            result.includes('ECONNREFUSED')
                        ||
                            result.includes('ECONNRESET'))) {
                return this.CheckRunningApp(appId, port, action, counter + 1);
            }
            else {
                return Promise.resolve(true);
            }
        }).catch((error) => {
            let errorMessage = typeof error === 'string' ? error : 'ERROR';
            logger.error('[RASAServiceImplementation::CheckRunningApp] Error: ', error);
            if (error.code && error.code === 'ETIMEDOUT')
                errorMessage = 'SERVER_ERROR_TIMEOUT_FROM_RASA_SERVER';
            if (error.code && error.code === 'ECONNREFUSED')
                errorMessage = 'SERVER_ERROR_CONNECTION_REFUSED_FROM_RASA_SERVER';
            if (error?.code === 'ECONNRESET')
                errorMessage = 'SERVER_ERROR_CONNECTION_RESETED_FROM_RASA_SERVER';
            return Promise.reject(errorMessage);
        });
    }
    StopApp(appId, port, task) {
        let modelFolder = config_1.RASA_ROOT_FOLDER + '/';
        logger.info('[RASAServiceImplementation::StopApp] Stopping: ' + appId + ', port: ' + port + ', task: ' + task);
        return promises_shared_1.DummyPromise()
            .then(result => {
            if (appId.split('::').length === 2) {
                return platform_service_1.GetPlatformVersion();
            }
            else {
                return true;
            }
        }).then(result => {
            logger.info('[RASAServiceImplementation::StopApp] Getting result:', result, 'after GetVersion');
            if (appId.split('::').length === 2) {
                appId = result.installationId + '::' + appId.toLowerCase();
            }
            modelFolder += appId;
            let cmd = 'ps -ax | grep ' + appId;
            return this.ExecuteRequest({ cmd: cmd, cwd: modelFolder });
        })
            .then(result => {
            if (!result)
                throw 'SERVER_ERROR_CANNOT_FOUND_RUNNING_PROCESSES';
            let arrAllRunningProcesses = result.stdout.split('\n');
            let arrAppRunningProcesses = arrAllRunningProcesses.filter(aarp => aarp.includes(appId));
            let arrKillPid = [];
            arrAppRunningProcesses.map(aarp => {
                if (aarp.includes(task + '.log'))
                    arrKillPid.push(Number(aarp.trim().split(' ')[0]));
            });
            if (arrKillPid.length > 0) {
                let cmd = 'pkill -f "' + modelFolder + '/' + task + '.log';
                if (port)
                    cmd += ' -p ' + port;
                cmd += '" ';
                logger.info('[RASAServiceImplementation::StopApp] Stopping: ' + cmd);
                this.ExecuteRequest({ cmd: 'sudo su -', stdin: cmd, cwd: modelFolder, executor: 'ssh' });
                return promises_shared_1.SleepPromise(5);
            }
            else {
                return Promise.resolve(true);
            }
        }).then(result => {
            if (task === 'TEST')
                return Promise.resolve(redis_service_1.Redis.DeleteRunningRasaModelFromTest(appId));
            else
                return Promise.resolve(redis_service_1.Redis.DeleteRunningRasaModel(appId));
        }).catch((error) => {
            let errorMessage = typeof error === 'string' ? error : 'ERROR';
            logger.error('[RASAServiceImplementation::StopApp] Error: ', error);
            if (error.code && error.code === 'ETIMEDOUT')
                errorMessage = 'SERVER_ERROR_TIMEOUT_FROM_RASA_SERVER';
            if (error.code && error.code === 'ECONNREFUSED')
                errorMessage = 'SERVER_ERROR_CONNECTION_REFUSED_FROM_RASA_SERVER';
            return Promise.reject(errorMessage);
        });
    }
    ImportNLUFromRASA(nlu) {
        let nluContent = null;
        let nluFileRedisKey = 'NLU_FILE:' + nlu.createdByUserId;
        let tempModelNLUPath = config_1.PATH_FOLDER_TEMP_NLU + '/' + nlu.vcc + '/' + nlu.name + '-nlu.md';
        return promises_shared_1.DummyPromise().then(result => {
            return redis_service_1.Redis.GetFileNLU(nlu.vcc, (nluFileRedisKey));
        }).then(result => {
            if (!result)
                throw 'SERVER_ERROR_NLU_REDIS_DATA_NOT_FOUND';
            if (result.data && !result.data.includes('## intent:None'))
                result.data += '\n## intent:None\n- xxxxx\n\n';
            nluContent = result.data;
            return file_manager_1.WriteFileContent(tempModelNLUPath, nluContent, 'utf-8');
        }).then(result => {
            if (!result)
                throw 'SERVER_ERROR_CANNOT_CREATE_NLU_FILE';
            return Promise.resolve(nluContent);
        }).catch((error) => {
            let errorMessage = typeof error === 'string' ? error : 'ERROR';
            logger.error('[RASAServiceImplementation::ImportNLUFromRASA] Error: ', error);
            return Promise.reject(errorMessage);
        });
    }
    GenerateNluMd(appId) {
        let installationId = '';
        let vcc = appId.split('::')[0];
        let model = appId.split('::')[1];
        let rootFolder = config_1.RASA_ROOT_FOLDER;
        let modelFolder = '';
        let nluLocalFolder = config_1.PATH_FOLDER_TEMP_NLU + '/' + vcc;
        let newNluTextContent = '';
        return promises_shared_1.DummyPromise()
            .then(result => {
            return platform_service_1.GetPlatformVersion();
        })
            .then(result => {
            if (result) {
                installationId = result.installationId;
                modelFolder = rootFolder + '/' + installationId + '::' + appId.toLowerCase();
                return file_manager_1.CreateFolder(nluLocalFolder);
            }
            else {
                throw 'ERROR_NO_VERSION_DATA';
            }
        })
            .then(result => {
            if (!result)
                throw 'SERVER_ERROR_CANNOT_CREATE_NLU_TEMPORARY_FOLDER';
            return this.GetNluContentFromDatabase(appId);
        })
            .then(result => {
            if (!result)
                throw 'SERVER_ERROR_CANNOT_GENERATE_NLU_CONTENT';
            newNluTextContent = result;
            let tempModelPath = nluLocalFolder + '/' + model + '.md';
            return file_manager_1.WriteFileContent(tempModelPath, newNluTextContent, 'utf-8');
        })
            .then(result => {
            if (!result)
                throw 'SERVER_ERROR_CREATE_NLU_MD_FAILED';
            return Promise.resolve(this.ExecuteRequest({ type: 'putfile', originPath: nluLocalFolder + '/' + model + '.md', targetPath: modelFolder + '/data/nlu.md' }));
        })
            .then(result => {
            if (result === null)
                throw 'SERVER_ERROR_NO_RESULT_FROM_RASA_SERVER';
            return Promise.resolve(true);
        })
            .catch((error) => {
            let errorMessage = typeof error === 'string' ? error : 'ERROR';
            logger.error('[RASAServiceImplementation::GenerateNluMd] Error: ', error);
            if (error.code && error.code === 'ETIMEDOUT')
                errorMessage = 'SERVER_ERROR_TIMEOUT_FROM_RASA_SERVER';
            if (error.code && error.code === 'ECONNREFUSED')
                errorMessage = 'SERVER_ERROR_CONNECTION_REFUSED_FROM_RASA_SERVER';
            return Promise.reject(errorMessage);
        });
    }
    GetNluContentFromDatabase(appId) {
        let vcc = appId.split('::')[0];
        let model = appId.split('::')[1];
        let nluIntents = [];
        let nluUtteranceObjects = [];
        let nluMdContent = null;
        return promises_shared_1.DummyPromise()
            .then(result => {
            return mssql_service_1.MSSql.GetIntents(vcc, appId, null, null, null);
        })
            .then(result => {
            if (!result)
                throw 'SERVER_ERROR_MISSING_INTENTS_TO_TRAIN';
            nluIntents = result;
            return utterance_service_1.GetUtterances(vcc, appId, null, null, null, true);
        })
            .then(result => {
            if (!result)
                throw 'SERVER_ERROR_MISSING_UTTERANCES_TO_TRAIN';
            nluUtteranceObjects = result;
            nluMdContent = this.GenerateNluMdContent(nluIntents, nluUtteranceObjects);
            return Promise.resolve(nluMdContent);
        })
            .catch((error) => {
            let errorMessage = typeof error === 'string' ? error : 'ERROR';
            logger.error('[RASAServiceImplementation::GetContentFromDatabase] Error: ', error);
            if (error.code && error.code === 'ETIMEDOUT')
                errorMessage = 'SERVER_ERROR_TIMEOUT_FROM_RASA_SERVER';
            if (error.code && error.code === 'ECONNREFUSED')
                errorMessage = 'SERVER_ERROR_CONNECTION_REFUSED_FROM_RASA_SERVER';
            return Promise.reject(errorMessage);
        });
    }
    GenerateNluMdContent(intents, utterances) {
        let nluContent = '';
        intents.map(i => {
            let intentBlockText = `## intent:${i.intentId}\n`;
            utterances.filter(u => u.intentId === i.intentId).map(u => {
                let labeledUtterance = null;
                if (u.entityLabels && u.entityLabels.length) {
                    labeledUtterance = this.GenerateLabeledUtterance(u.text, u.entityLabels);
                }
                else {
                    labeledUtterance = u.text;
                }
                intentBlockText += `- ${labeledUtterance}\n`;
            });
            intentBlockText += '\n';
            nluContent += intentBlockText;
        });
        nluContent += '## intent:None\n- xxxxx';
        return nluContent;
    }
    GenerateLabeledUtterance(utteranceText, entityLabels) {
        let labeledUtterance = utteranceText;
        let addedText = '';
        entityLabels.map(el => {
            let splittedText = labeledUtterance.split('');
            labeledUtterance = '';
            splittedText.map((st, i) => {
                if (i === el.start + addedText.length) {
                    labeledUtterance += '[' + st;
                }
                else if (i === el.end + addedText.length) {
                    labeledUtterance += st + `](${el.entityName})`;
                }
                else {
                    labeledUtterance += st;
                }
            });
            addedText += `[](${el.entityName})`;
        });
        return labeledUtterance;
    }
    GetStringToRegex(text) {
        let specialCharacters = ['\\', '[', ']', '^', '$', '.', '|', '¿', '?', '¡', '!', '*', '+', '(', ')'];
        specialCharacters.map(sc => {
            text = text.split(sc).join('\\' + sc);
        });
        return text;
    }
    GetSsh() {
        return promises_shared_1.DummyPromise()
            .then(result => {
            if (this.mySsh && this.mySsh.connection) {
                return Promise.resolve(this.mySsh);
            }
            else {
                this.mySsh = null;
                this.mySsh = new NodeSSH();
                return this.mySsh.connect({
                    host: config_1.RASA_IP,
                    username: config_1.RASA_USER,
                    password: config_1.RASA_PASSWORD,
                    port: config_1.RASA_PORT,
                    readyTimeout: 300000,
                    keepaliveCountMax: 10
                });
            }
        }).then(result => {
            logger.info('[RASAServiceImplementation::GetSsh] Getting result:', (result && result.connection ? true : false), 'after Connect');
            return Promise.resolve(this.mySsh);
        }).catch((error) => {
            let errorMessage = typeof error === 'string' ? error : 'ERROR';
            logger.error('[RASAServiceImplementation::GetSsh] Error: ', error);
            if (error.code && error.code === 'ETIMEDOUT')
                errorMessage = 'SERVER_ERROR_TIMEOUT_FROM_RASA_SERVER';
            if (error.code && error.code === 'ECONNREFUSED')
                errorMessage = 'SERVER_ERROR_CONNECTION_REFUSED_FROM_RASA_SERVER';
            return Promise.reject(errorMessage);
        });
    }
    ExecuteRequest(args) {
        args.type = args.type ? args.type : 'exec';
        if (args.executor) {
            if (args.executor === 'ssh') {
                logger.debug('[RASAServiceImplementation::ExecuteRequest] From: SSH');
                return this.SShExecute(args);
            }
            else {
                logger.debug('[RASAServiceImplementation::ExecuteRequest] From: Subsystem');
                return this.LocaleExecute(args);
            }
        }
        else {
            if (config_1.RASA_IP === '127.0.0.1'
                &&
                    process?.platform
                &&
                    process.platform != 'win32') {
                logger.debug('[RASAServiceImplementation::ExecuteRequest] From: Subsystem');
                return this.LocaleExecute(args);
            }
            else {
                logger.debug('[RASAServiceImplementation::ExecuteRequest] From: SSH');
                return this.SShExecute(args);
            }
        }
    }
    SShExecute(args) {
        return promises_shared_1.DummyPromise()
            .then(result => {
            return this.GetSsh();
        })
            .then(result => {
            switch (args.type) {
                case 'exec':
                    logger.debug('[RASAServiceImplementation::SShExecute] CMD: ', args.cmd);
                    if (args.cmd && args.cmd.includes('sudo')) {
                        return this.mySsh.execCommand('sudo su -', { cwd: args.cwd, stdin: args.stdin + '\n', stdout: 'DONE', execOptions: { pty: true } });
                    }
                    else {
                        return this.mySsh.execCommand(args.cmd, { cwd: args.cwd });
                    }
                case 'putfile':
                    return this.mySsh.putFile(args.originPath, args.targetPath);
            }
        })
            .then(result => {
            logger.debug('[RASAServiceImplementation::SShExecute] RES: ', result);
            return Promise.resolve(result);
        })
            .catch((error) => {
            let errorMessage = typeof error === 'string' ? error : 'ERROR';
            logger.error('[RASAServiceImplementation::SShExecute] Error: ', error);
            if (error.code && error.code === 'ETIMEDOUT')
                errorMessage = 'SERVER_ERROR_TIMEOUT_FROM_RASA_SERVER';
            if (error.code && error.code === 'ECONNREFUSED')
                errorMessage = 'SERVER_ERROR_CONNECTION_REFUSED_FROM_RASA_SERVER';
            return Promise.reject(errorMessage);
        });
    }
    LocaleExecute(args) {
        return promises_shared_1.DummyPromise()
            .then(result => {
            switch (args.type) {
                case 'exec':
                    break;
                case 'putfile':
                    args.cmd = `sudo cp ${args.originPath} ${args.targetPath} && sudo chown ${config_1.RASA_USER}:${config_1.RASA_USER} ${args.targetPath}`;
                    break;
            }
            logger.debug('[RASAServiceImplementation::LocaleExecute] DEBUG-CMD: ', args.cmd);
            return this.ExecuteBashCommand(args);
        })
            .then(result => {
            logger.debug('[RASAServiceImplementation::LocaleExecute] DEBUG-RES: ', result);
            return Promise.resolve({ stdout: result.stdout, stderr: result.stderr });
        })
            .catch((error) => {
            let errorMessage = typeof error === 'string' ? error : 'ERROR';
            logger.error('[RASAServiceImplementation::LocaleExecute] Error: ', error);
            if (error.code && error.code === 'ETIMEDOUT')
                errorMessage = 'SERVER_ERROR_TIMEOUT_FROM_RASA_SERVER';
            if (error.code && error.code === 'ECONNREFUSED')
                errorMessage = 'SERVER_ERROR_CONNECTION_REFUSED_FROM_RASA_SERVER';
            return Promise.resolve({ stderr: error });
        });
    }
    ExecuteBashCommand(args) {
        return new Promise((resolve, reject) => {
            try {
                let cmd = null;
                if (args.cmd
                    &&
                        (args.cmd.includes('ls ')
                            ||
                                args.cmd.includes('cat ')
                            ||
                                args.cmd.includes('ps -ax'))) {
                    cmd = args.cmd;
                }
                else {
                    cmd = `runuser -l ${config_1.RASA_USER} -c '${(args.cwd ? ('cd ' + args.cwd + ';') : '')}${args.cmd}'`;
                }
                child_process.exec(cmd, {
                    cwd: args.cwd
                }, (err, stdout, stderr) => {
                    let objectResult = { err: err, stdout: stdout, stderr: stderr };
                    if (err) {
                        if (err && err.message.includes('Command failed:')) {
                            if (err.cmd && err.cmd.includes('ls')) {
                                resolve(objectResult);
                            }
                            else {
                                reject(objectResult);
                            }
                        }
                        else {
                            reject(objectResult);
                        }
                    }
                    else {
                        resolve(objectResult);
                    }
                });
            }
            catch (e) {
                reject(e);
            }
        });
    }
}
exports.RASAServiceImplementation = RASAServiceImplementation;
