"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.SemanticAnalysisConvertAudioImplementation = void 0;
const config_1 = require("../config/config");
const mssql_service_1 = require("../db/mssql.service");
const redis_service_1 = require("../db/redis.service");
const promises_shared_1 = require("../shared/promises.shared");
const log4js = require("log4js");
const moment = require("moment");
const billing_batch_model_1 = require("../models/billing-batch.model");
const file_manager_1 = require("../shared/file-manager");
const child_process_1 = require("child_process");
const billing_implementation_1 = require("./billing.implementation");
const task_service_1 = require("../services/task.service");
let logger = log4js.getLogger('Task');
const CURRENT_SYSTEM_SERVICE = 'TRANSCRIPTION';
const CURRENT_IMPLEMENTATION = 'SemanticAnalysisConvertAudioImplementation';
class SemanticAnalysisConvertAudioImplementation {
    constructor() {
        this.errorData = new billing_batch_model_1.BatchError();
        this.queueBase = 'SemanticAnalysisConvertAudio';
    }
    Execute(executionCode, rateRecursive) {
        logger.info(`[SemanticAnalysisConvertAudioImplementation::Execute] executionCode: ${executionCode} executing wave #${rateRecursive}`);
        let startTime = moment();
        let executeIndex = 1;
        let ratio = config_1.RATIO_SEMANTIC_ANALYSIS;
        let executeResult = true;
        let currentBlockPromise = null;
        return promises_shared_1.SleepPromise(config_1.SLEEP_SEMANTIC_ANALYSIS)
            .then(_ => {
            currentBlockPromise = 'BLOCK_01_GET_FROM_REDIS';
            return redis_service_1.Redis.GetLengthFromList(this.queueBase);
        })
            .then(result => {
            currentBlockPromise = 'BLOCK_02_GET_QUEUE_ITEMS';
            if (!result)
                throw 'SERVER_INFO_REDIS_KEYS_FOR_SEMANTIC_ANALYSIS_CONVERT_AUDIO_NOT_FOUND';
            return redis_service_1.Redis.LRange(this.queueBase, 0, ratio - 1);
        })
            .then((result) => {
            currentBlockPromise = 'BLOCK_03_SEMANTIC_ANALYSIS_CONVERT_AUDIO_IMPL';
            ratio = result && result.length <= ratio ? result.length : 0;
            let promiseArray = [];
            while (ratio >= executeIndex) {
                promiseArray.push(this.SemanticAnalysisConvertAudioExecute(executionCode));
                executeIndex++;
            }
            if (!promiseArray.length)
                promiseArray.push(Promise.resolve(true));
            return Promise.all(promiseArray);
        })
            .then((results) => {
            currentBlockPromise = 'BLOCK_04_GETTING_KEYS_FOR_RECURSIVITY';
            if (results.filter(item => item === false).length > 0) {
                executeResult = false;
            }
            return redis_service_1.Redis.LRange(this.queueBase, 0, ratio - 1);
        })
            .then((result) => {
            currentBlockPromise = 'BLOCK_05_RELAUNCH_SEMANTIC_ANALYSIS_UPLOAD_IMP';
            if (result?.length && rateRecursive >= 1)
                return this.Execute(executionCode, rateRecursive - 1);
            return Promise.resolve(executeResult);
        })
            .then(result => {
            currentBlockPromise = 'BLOCK_06_RESOLVING_IMPL';
            if (rateRecursive === config_1.RATE_RECURSIVE_SEMANTIC_ANALYSIS)
                logger.info(`[SemanticAnalysisConvertAudioImplementation::Execute] executionCode: ${executionCode} Finishing execution wave #${rateRecursive} after ${moment().diff(startTime, 'minutes')} minutes`);
            let currentResult = executeResult && result ? true : false;
            return Promise.resolve(currentResult);
        })
            .catch((error) => {
            let errorMessage = typeof error === 'string' ? error : (error.message || error.description);
            if (errorMessage.startsWith('SERVER_INFO_'))
                logger.info(`[SemanticAnalysisConvertAudioImplementation::Execute] executionCode: ${executionCode}, Block: ${currentBlockPromise}, getting info:`, errorMessage);
            else
                logger.error(`[SemanticAnalysisConvertAudioImplementation::Execute] executionCode: ${executionCode}, Block: ${currentBlockPromise}, getting error:`, errorMessage);
            return Promise.resolve(false);
        });
    }
    SemanticAnalysisConvertAudioExecute(executionCode) {
        let storage = '';
        let currentQueueId = '';
        let processInteraction = null;
        let queueItems = null;
        return redis_service_1.Redis.RPopLPush(this.queueBase, `${this.queueBase}Execute`)
            .then((queueId) => {
            if (queueId?.length > 0) {
                currentQueueId = queueId;
                logger.info('[SemanticAnalysisConvertAudioImplementation::SemanticAnalysisExecute] executionCode:', executionCode, 'getting queueId:', currentQueueId, 'START');
                queueItems = task_service_1.GetItemFromQueueId(currentQueueId);
                storage = `${queueItems.vcc}:${queueItems.processId}`;
                logger.info('[SemanticAnalysisConvertAudioImplementation::SemanticAnalysisConvertAudioExecute] executionCode:', executionCode, 'getting queueId:', currentQueueId, 'storage :', storage);
                return redis_service_1.Redis.GetProcessInteraction(storage, currentQueueId);
            }
            return Promise.resolve(null);
        })
            .then((processInteractionResult) => {
            if (processInteractionResult) {
                processInteraction = processInteractionResult;
                logger.info('[SemanticAnalysisConvertAudioImplementation::SemanticAnalysisConvertAudioExecute] executionCode:', executionCode, 'getting queueId:', currentQueueId, 'getting GetProcessInteraction: 1, status:', processInteraction.statusDescription);
                if (processInteraction.statusDescription === 'pendingConvertAudio') {
                    if (config_1.KEEP_AUDIO_FILE_AFTER_TRANSCRIPTION)
                        return this.ConvertAudio(processInteraction, currentQueueId, executionCode);
                    else
                        return this.DeleteAudio(processInteraction, currentQueueId, executionCode);
                }
            }
            if (!processInteraction && currentQueueId) {
                logger.warn('[SemanticAnalysisConvertAudioImplementation::SemanticAnalysisConvertAudioExecute] executionCode:', executionCode, 'getting queueId:', currentQueueId, 'getting GetProcessInteraction: 0');
                if (!processInteraction) {
                    logger.warn('[SemanticAnalysisConvertAudioImplementation::SemanticAnalysisConvertAudioExecute] executionCode:', executionCode, 'getting queueId:', currentQueueId, 'removing queue for lost data');
                    redis_service_1.Redis.DeleteQueue(`${this.queueBase}Execute`, currentQueueId);
                    throw 'SERVER_ERROR_INTERACTION_DATA_NOT_FOUND_ON_STORAGE';
                }
            }
            return Promise.resolve(null);
        })
            .then((result) => {
            if (result)
                processInteraction = result;
            if (processInteraction)
                return redis_service_1.Redis.SaveProcessInteraction(storage, currentQueueId, processInteraction);
            return Promise.resolve(null);
        })
            .then((result) => {
            let promiseArray = [];
            let billing = new billing_implementation_1.BillingImplementation();
            if (result
                &&
                    processInteraction?.status === 6
                &&
                    processInteraction?.statusDescription === 'ready') {
                logger.info('[SemanticAnalysisConvertAudioImplementation::SemanticAnalysisConvertAudioExecute] executionCode:', executionCode, 'getting queueId:', currentQueueId, 'QUEUE_FINISHED');
                billing.AddUpdateTranscriptionDetailToMainFile(processInteraction, 'ValidChar', processInteraction.batchTypeName, processInteraction.batchTypeId);
                billing.AddUpdateTranscriptionDetailToMainFile(processInteraction, 'ValidWords', processInteraction.batchTypeName, processInteraction.batchTypeId);
                billing.AddUpdateTranscriptionDetailToMainFile(processInteraction, 'TotalPhrases', processInteraction.batchTypeName, processInteraction.batchTypeId);
                billing.AddUpdateTranscriptionDetailToMainFile(processInteraction, 'CharsPerPhrase', processInteraction.batchTypeName, processInteraction.batchTypeId);
                billing.AddUpdateTranscriptionDetailToMainFile(processInteraction, 'IdathaApi', processInteraction.batchTypeName, processInteraction.batchTypeId);
                billing.AddCloseTranscriptionDetailToMainFile(processInteraction, processInteraction.batchTypeName, processInteraction.batchTypeId, 1, 'SUCCED');
                promiseArray.push(redis_service_1.Redis.DeleteQueue('SemanticAnalysisConvertAudioExecute', currentQueueId));
            }
            else if (processInteraction?.retry >= config_1.RETRY_SEMANTIC_ANALYSIS) {
                logger.error('[SemanticAnalysisConvertAudioImplementation::SemanticAnalysisConvertAudioExecute] executionCode:', executionCode, 'getting queueId:', currentQueueId, 'REMOVED_BY_RETRIES');
                billing.AddCloseTranscriptionDetailToMainFile(processInteraction, processInteraction.batchTypeName, processInteraction.batchTypeId, 0, 'REMOVED_BY_RETRIES');
                promiseArray.push(redis_service_1.Redis.DeleteQueue('SemanticAnalysisConvertAudioExecute', currentQueueId));
            }
            else if (currentQueueId && !processInteraction) {
                logger.error('[SemanticAnalysisConvertAudioImplementation::SemanticAnalysisConvertAudioExecute] executionCode:', executionCode, 'getting queueId:', currentQueueId, 'INTERACTION_DO_NOT_EXIST');
                let interactionData = {
                    vcc: queueItems.vcc,
                    processId: queueItems.processId,
                    id: queueItems.interactionId,
                    section: queueItems.interactionSection,
                    part: queueItems.interactionPart
                };
                billing.AddCloseTranscriptionDetailToMainFile(interactionData, processInteraction.batchTypeName, processInteraction.batchTypeId, 0, 'INTERACTION_DO_NOT_EXIST');
                promiseArray.push(redis_service_1.Redis.DeleteQueue('SemanticAnalysisConvertAudioExecute', currentQueueId));
            }
            else {
                logger.warn('[SemanticAnalysisConvertAudioImplementation::SemanticAnalysisConvertAudioExecute] executionCode:', executionCode, 'getting queueId:', currentQueueId, 'QUEUE_WAS_NOT_REMOVED');
            }
            if (!promiseArray.length)
                promiseArray.push(Promise.resolve(true));
            return Promise.all(promiseArray);
        })
            .then((result) => {
            let promiseArray = [];
            if (processInteraction?.status === 6
                &&
                    processInteraction?.statusDescription === 'ready') {
                promiseArray.push(redis_service_1.Redis.UpdateProgress(storage));
                promiseArray.push(redis_service_1.Redis.DeleteProcessInteraction(storage, currentQueueId));
                promiseArray.push(redis_service_1.Redis.SaveReadyProcessInteraction(executionCode, currentQueueId, processInteraction));
            }
            else if (processInteraction?.retry >= config_1.RETRY_SEMANTIC_ANALYSIS) {
                processInteraction.isCancel = true;
                promiseArray.push(redis_service_1.Redis.UpdateProgress(storage));
                promiseArray.push(redis_service_1.Redis.UpdateErrorProgress(storage));
                promiseArray.push(redis_service_1.Redis.DeleteProcessInteraction(storage, currentQueueId));
                promiseArray.push(redis_service_1.Redis.SaveReadyProcessInteraction(executionCode, currentQueueId, processInteraction));
                promiseArray.push(mssql_service_1.MSSql.SaveBatchError(queueItems.vcc, CURRENT_SYSTEM_SERVICE, processInteraction.batchTypeId, processInteraction.batchTypeId, CURRENT_IMPLEMENTATION, processInteraction.id, null, processInteraction.errorFunction, processInteraction.errorMessage, processInteraction.errorDetail));
            }
            if (!promiseArray.length)
                promiseArray.push(Promise.resolve(true));
            return Promise.all(promiseArray);
        })
            .then(_ => {
            return Promise.resolve(true);
        })
            .catch((err) => {
            let errorMessage = typeof err === 'string' ? err : (err.message || err.description);
            logger.error('[SemanticAnalysisConvertAudioImplementation::SemanticAnalysisConvertAudioExecute] executionCode:', executionCode, 'getting queueId:', currentQueueId, 'Catching error: ' + errorMessage);
            if (processInteraction)
                logger.error('[SemanticAnalysisConvertAudioImplementation::SemanticAnalysisConvertAudioExecute] executionCode:', executionCode, 'getting queueId:', currentQueueId, 'getting error:', errorMessage, ' vcc', processInteraction.vcc, 'processId:', processInteraction.processId, 'id:', processInteraction.id, 'section:', processInteraction.section, 'part:', processInteraction.part);
            return Promise.resolve(false);
        });
    }
    ConvertAudio(processInteraction, currentQueueId, executionCode) {
        let newFormat = 'mp3';
        let newOldFileNamePath = '';
        let currentFolder = ['allegro', 'i6', 'i-ftp', 'i-sftp', 'i-s3'].includes(processInteraction?.environmentType) ? config_1.PATH_FOLDER_TEMP_AUDIO : config_1.PATH_FOLDER_PROCESS_AUDIO;
        let oldFilePath = `${currentFolder}/${processInteraction.vcc}/${processInteraction.fileName}`;
        let newFilePath = `${currentFolder}/${processInteraction.vcc}/${processInteraction.fileName.replace('.wav', `.${newFormat}`).replace('.ogg', `.${newFormat}`)}`;
        let forceConvertion = false;
        if (oldFilePath === newFilePath) {
            forceConvertion = true;
            newOldFileNamePath = oldFilePath.replace('.mp3', '_changed.mp3');
        }
        return promises_shared_1.DummyPromise()
            .then(_ => {
            if (forceConvertion)
                return file_manager_1.RenameFile(oldFilePath, newOldFileNamePath);
            return Promise.resolve(true);
        })
            .then(_ => {
            if (forceConvertion)
                oldFilePath = newOldFileNamePath;
            return Promise.resolve(true);
        })
            .then(_ => {
            let channel = 1;
            if (processInteraction.channels !== 'mono')
                channel = 2;
            let cmd = `${config_1.FFMPEG_FOLDER_PATH}ffmpeg -y -i "${oldFilePath}" -vn -ar 22050 -ac ${channel} -b:a 16k "${newFilePath}"`;
            logger.debug('[SemanticAnalysisConvertAudioImplementation::ConvertAudio] executionCode:', executionCode, 'getting queueId:', currentQueueId, 'getting cmd:', cmd);
            return new Promise((resolve, reject) => {
                try {
                    child_process_1.exec(cmd, (err, stdout, stderr) => {
                        if (err) {
                            if (err.message?.includes('No such file or directory')
                                ||
                                    err.message?.includes('Command failed')) {
                                logger.error('[SemanticAnalysisConvertAudioImplementation::ConvertAudio] executionCode:', executionCode, 'getting queueId:', currentQueueId, 'the original file was not found.');
                                return resolve(true);
                            }
                            else {
                                return reject(err);
                            }
                        }
                        else {
                            processInteraction.fileName = processInteraction.fileName.replace('.wav', `.${newFormat}`).replace('.ogg', `.${newFormat}`);
                            return resolve(stdout.media);
                        }
                    });
                }
                catch (e) {
                    if (e.message?.includes('No such file or directory')
                        ||
                            e.message?.includes('Command failed')) {
                        logger.error('[SemanticAnalysisConvertAudioImplementation::ConvertAudio] executionCode:', executionCode, 'getting queueId:', currentQueueId, 'the original file was not found.');
                        processInteraction.fileName = null;
                        return resolve(true);
                    }
                    else {
                        return reject(e);
                    }
                }
            });
        })
            .then(_ => {
            return file_manager_1.DeleteFile(oldFilePath, true);
        })
            .then(_ => {
            processInteraction['status'] = 6;
            processInteraction['statusDescription'] = 'ready';
            processInteraction['format'] = newFormat;
            processInteraction['fileName'] = processInteraction.fileName;
            processInteraction['retry'] = 0;
            return Promise.resolve(processInteraction);
        })
            .catch((err) => {
            let errorMessage = typeof err === 'string' ? err : (err.message || err.description);
            logger.error('[SemanticAnalysisConvertAudioImplementation::ConvertAudio] executionCode:', executionCode, 'getting queueId:', currentQueueId, 'getting error:', errorMessage);
            processInteraction['status'] = -6;
            processInteraction['retry'] = processInteraction['retry'] ? processInteraction['retry'] + 1 : 1;
            processInteraction['uploadedDate'] = moment().format('x');
            processInteraction['errorMessage'] = errorMessage;
            this.errorData.SetErrorData('ConvertAudio', errorMessage, err);
            if (processInteraction['retry'] === config_1.RETRY_SEMANTIC_ANALYSIS) {
                processInteraction['status'] = 6;
                processInteraction['statusDescription'] = 'ready';
            }
            return Promise.resolve(processInteraction);
        });
    }
    DeleteAudio(processInteraction, currentQueueId, executionCode) {
        let currentFolder = ['allegro', 'i6', 'i-ftp', 'i-sftp', 'i-s3'].includes(processInteraction?.environmentType) ? config_1.PATH_FOLDER_TEMP_AUDIO : config_1.PATH_FOLDER_PROCESS_AUDIO;
        let oldFilePath = `${currentFolder}/${processInteraction.vcc}/${processInteraction.fileName}`;
        return promises_shared_1.DummyPromise()
            .then(_ => {
            return file_manager_1.DeleteFile(oldFilePath, true);
        })
            .then(_ => {
            processInteraction['status'] = 6;
            processInteraction['statusDescription'] = 'ready';
            processInteraction['retry'] = 0;
            return Promise.resolve(processInteraction);
        })
            .catch((err) => {
            let errorMessage = typeof err === 'string' ? err : (err.message || err.description);
            logger.error('[SemanticAnalysisConvertAudioImplementation::DeleteAudio] executionCode:', executionCode, 'getting queueId:', currentQueueId, 'getting error:', errorMessage);
            processInteraction['status'] = -6;
            processInteraction['retry'] = processInteraction['retry'] ? processInteraction['retry'] + 1 : 1;
            processInteraction['uploadedDate'] = moment().format('x');
            processInteraction['errorMessage'] = errorMessage;
            this.errorData.SetErrorData('DeleteAudio', errorMessage, err);
            if (processInteraction['retry'] === config_1.RETRY_SEMANTIC_ANALYSIS) {
                processInteraction['status'] = 6;
                processInteraction['statusDescription'] = 'ready';
            }
            return Promise.resolve(processInteraction);
        });
    }
}
exports.SemanticAnalysisConvertAudioImplementation = SemanticAnalysisConvertAudioImplementation;
