"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.SemanticAnalysisDownloadImplementation = void 0;
const config_1 = require("../config/config");
const mssql_service_1 = require("../db/mssql.service");
const redis_service_1 = require("../db/redis.service");
const promises_shared_1 = require("../shared/promises.shared");
const log4js = require("log4js");
const moment = require("moment");
const billing_batch_model_1 = require("../models/billing-batch.model");
const billing_implementation_1 = require("./billing.implementation");
const file_manager_1 = require("../shared/file-manager");
const idatha_service_1 = require("../services/idatha.service");
const cognitive_analysis_implementation_1 = require("./cognitive-analysis.implementation");
const idatha_model_1 = require("../models/idatha.model");
const request_shared_1 = require("../shared/request.shared");
const generic_result_1 = require("../models/generic-result");
let logger = log4js.getLogger('Task');
const CURRENT_SYSTEM_SERVICE = 'TRANSCRIPTION';
const CURRENT_IMPLEMENTATION = 'SemanticAnalysisDownloadImplementation';
class SemanticAnalysisDownloadImplementation {
    constructor() {
        this.errorData = new billing_batch_model_1.BatchError();
        this.queueBase = 'SemanticAnalysisDownload';
        this.previousQueue = 'SemanticAnalysisUpload';
        this.nextQueue = 'SemanticAnalysisConvertAudio';
    }
    Execute(executionCode, rateRecursive) {
        logger.info(`[SemanticAnalysisDownloadImplementation::Execute] executionCode: ${executionCode} executing wave #${rateRecursive}`);
        let startTime = moment();
        let executeIndex = 1;
        let ratio = config_1.RATIO_SEMANTIC_ANALYSIS_DOWNLOAD;
        let executeResult = true;
        let currentBlockPromise = null;
        return promises_shared_1.SleepPromise(config_1.SLEEP_SEMANTIC_ANALYSIS)
            .then(_ => {
            currentBlockPromise = 'BLOCK_01_GET_FROM_REDIS';
            return redis_service_1.Redis.GetLengthFromList(this.queueBase);
        })
            .then(result => {
            currentBlockPromise = 'BLOCK_02_GET_QUEUE_ITEMS';
            if (!result)
                throw 'SERVER_INFO_REDIS_KEYS_FOR_SEMANTIC_ANALYSIS_DOWNLOAD_NOT_FOUND';
            return redis_service_1.Redis.LRange(this.queueBase, 0, ratio - 1);
        })
            .then((result) => {
            currentBlockPromise = 'BLOCK_03_SEMANTIC_ANALYSIS_DOWNLOAD_IMPL';
            ratio = result && result.length <= ratio ? result.length : 0;
            let promiseArray = [];
            while (ratio >= executeIndex) {
                promiseArray.push(this.SemanticAnalysisDownloadExecute(executionCode));
                executeIndex++;
            }
            if (!promiseArray.length)
                promiseArray.push(Promise.resolve(true));
            return Promise.all(promiseArray);
        })
            .then((results) => {
            currentBlockPromise = 'BLOCK_04_GETTING_KEYS_FOR_RECURSIVITY';
            if (results.filter(item => item === false).length > 0) {
                executeResult = false;
            }
            return redis_service_1.Redis.LRange(this.queueBase, 0, ratio - 1);
        })
            .then((result) => {
            currentBlockPromise = 'BLOCK_05_RELAUNCH_SEMANTIC_ANALYSIS_UPLOAD_IMP';
            if (result?.length && rateRecursive >= 1)
                return this.Execute(executionCode, rateRecursive - 1);
            return Promise.resolve(executeResult);
        })
            .then(result => {
            currentBlockPromise = 'BLOCK_06_RESOLVING_IMPL';
            if (rateRecursive === config_1.RATE_RECURSIVE_SEMANTIC_ANALYSIS_DOWNLOAD)
                logger.info(`[SemanticAnalysisDownloadImplementation::Execute] executionCode: ${executionCode} Finishing execution wave #${rateRecursive} after ${moment().diff(startTime, 'minutes')} minutes`);
            let currentResult = executeResult && result ? true : false;
            return Promise.resolve(currentResult);
        })
            .catch((error) => {
            let errorMessage = typeof error === 'string' ? error : (error.message || error.description);
            if (errorMessage.startsWith('SERVER_INFO_'))
                logger.info(`[SemanticAnalysisDownloadImplementation::Execute] executionCode: ${executionCode}, Block: ${currentBlockPromise}, getting info:`, errorMessage);
            else
                logger.error(`[SemanticAnalysisDownloadImplementation::Execute] executionCode: ${executionCode}, Block: ${currentBlockPromise}, getting error:`, errorMessage);
            return Promise.resolve(false);
        });
    }
    SemanticAnalysisDownloadExecute(executionCode) {
        let storage = '';
        let currentQueueId = '';
        let processInteraction = null;
        let vcc = '';
        let processId = '';
        let interactionType = '';
        let interactionId = '';
        let interactionSection = 0;
        let interactionPart = 0;
        return redis_service_1.Redis.RPopLPush(this.queueBase, `${this.queueBase}Execute`)
            .then((queueId) => {
            if (queueId?.length > 0) {
                currentQueueId = queueId;
                logger.info('[SemanticAnalysisDownloadImplementation::SemanticAnalysisDownloadExecute] executionCode:', executionCode, 'getting queueId:', currentQueueId, 'START');
                let item = queueId.split(':');
                if (item?.length) {
                    vcc = item[0];
                    processId = item[1];
                    interactionType = item[2];
                    interactionId = item[3];
                    interactionSection = Number(item[4]);
                    interactionPart = Number(item[5]);
                    storage = vcc + ':' + processId;
                    logger.info('[SemanticAnalysisDownloadImplementation::SemanticAnalysisDownloadExecute] executionCode:', executionCode, 'getting queueId:', currentQueueId, 'storage :', storage);
                    return redis_service_1.Redis.GetProcessInteraction(storage, currentQueueId);
                }
                else {
                    logger.error('[SemanticAnalysisDownloadImplementation::SemanticAnalysisDownloadExecute] executionCode:', executionCode, 'getting queueId:', currentQueueId, 'QUEUE_FORMAT_ERROR');
                }
            }
            return Promise.resolve(null);
        })
            .then((processInteractionResult) => {
            if (processInteractionResult) {
                processInteraction = processInteractionResult;
                logger.info('[SemanticAnalysisDownloadImplementation::SemanticAnalysisDownloadExecute] executionCode:', executionCode, 'getting queueId:', currentQueueId, 'getting GetProcessInteraction: 1, status:', processInteraction.statusDescription);
                if (processInteraction.statusDescription === 'pendingDownloadJSON') {
                    let validTime = Number(moment().format('x'));
                    let nextExecutionTime = Number(processInteraction.uploadedDate);
                    nextExecutionTime = nextExecutionTime + config_1.NEXT_EXECUTION_TIME_SEMANTIC_ANALYSIS;
                    if (validTime > nextExecutionTime) {
                        return this.DownloadJSON(processInteraction, currentQueueId, executionCode);
                    }
                    else {
                        logger.info('[SemanticAnalysisDownloadImplementation::SemanticAnalysisDownloadExecute] executionCode:', executionCode, 'getting queueId:', currentQueueId, 'NEXT_EXECUTION', nextExecutionTime, 'CURRENT_TIME', validTime);
                    }
                }
            }
            if (!processInteraction && currentQueueId) {
                logger.warn('[SemanticAnalysisDownloadImplementation::SemanticAnalysisDownloadExecute] executionCode:', executionCode, 'getting queueId:', currentQueueId, 'getting GetProcessInteraction: 0');
                if (!processInteraction) {
                    logger.warn('[SemanticAnalysisDownloadImplementation::SemanticAnalysisDownloadExecute] executionCode:', executionCode, 'getting queueId:', currentQueueId, 'removing queue for lost data');
                    redis_service_1.Redis.DeleteQueue(`${this.queueBase}Execute`, currentQueueId);
                    throw 'SERVER_ERROR_INTERACTION_DATA_NOT_FOUND_ON_STORAGE';
                }
            }
            return Promise.resolve(null);
        })
            .then((result) => {
            if (result)
                processInteraction = result;
            if (processInteraction)
                return redis_service_1.Redis.SaveProcessInteraction(storage, currentQueueId, processInteraction);
            return Promise.resolve(null);
        })
            .then((result) => {
            let promiseArray = [];
            let billing = new billing_implementation_1.BillingImplementation();
            if (result
                &&
                    processInteraction?.status === 6
                &&
                    processInteraction?.statusDescription === 'ready') {
                logger.info('[SemanticAnalysisDownloadImplementation::SemanticAnalysisDownloadExecute] executionCode:', executionCode, 'getting queueId:', currentQueueId, 'QUEUE_FINISHED');
                billing.AddUpdateTranscriptionDetailToMainFile(processInteraction, 'ValidChar', processInteraction.batchTypeName, processInteraction.batchTypeId);
                billing.AddUpdateTranscriptionDetailToMainFile(processInteraction, 'ValidWords', processInteraction.batchTypeName, processInteraction.batchTypeId);
                billing.AddUpdateTranscriptionDetailToMainFile(processInteraction, 'TotalPhrases', processInteraction.batchTypeName, processInteraction.batchTypeId);
                billing.AddUpdateTranscriptionDetailToMainFile(processInteraction, 'CharsPerPhrase', processInteraction.batchTypeName, processInteraction.batchTypeId);
                billing.AddUpdateTranscriptionDetailToMainFile(processInteraction, 'IdathaApi', processInteraction.batchTypeName, processInteraction.batchTypeId);
                billing.AddCloseTranscriptionDetailToMainFile(processInteraction, processInteraction.batchTypeName, processInteraction.batchTypeId, 1, 'SUCCED');
                promiseArray.push(redis_service_1.Redis.DeleteQueue(`${this.queueBase}Execute`, currentQueueId));
            }
            else if (processInteraction?.status === 6
                &&
                    processInteraction?.statusDescription === 'pendingConvertAudio'
                &&
                    processInteraction?.retry < config_1.RETRY_SEMANTIC_ANALYSIS) {
                promiseArray.push(redis_service_1.Redis.InsertQueue(this.nextQueue, currentQueueId, config_1.RATE_RECURSIVE_SEMANTIC_ANALYSIS_DOWNLOAD * config_1.RATIO_SEMANTIC_ANALYSIS_DOWNLOAD * 2));
                promiseArray.push(redis_service_1.Redis.DeleteQueue(`${this.queueBase}Execute`, currentQueueId));
            }
            else if (processInteraction?.status === -6
                &&
                    processInteraction?.statusDescription === 'pendingAnalysis'
                &&
                    processInteraction.retry === (config_1.RETRY_SEMANTIC_ANALYSIS - 1)) {
                promiseArray.push(redis_service_1.Redis.InsertQueue(this.previousQueue, currentQueueId, config_1.RATE_RECURSIVE_SEMANTIC_ANALYSIS_DOWNLOAD * config_1.RATIO_SEMANTIC_ANALYSIS_DOWNLOAD * 2));
                promiseArray.push(redis_service_1.Redis.DeleteQueue(`${this.queueBase}Execute`, currentQueueId));
            }
            else if (processInteraction?.status === -6
                &&
                    (processInteraction?.statusDescription === 'pendingRemove'
                        ||
                            processInteraction?.retry >= config_1.RETRY_SEMANTIC_ANALYSIS)) {
                let resultMessage = processInteraction?.retry >= config_1.RETRY_SEMANTIC_ANALYSIS ? 'REMOVED_BY_RETRIES' : 'REMOVED_BY_INVALID_CONTENT';
                logger.error('[SemanticAnalysisDownloadImplementation::SemanticAnalysisDownloadExecute] executionCode:', executionCode, 'getting queueId:', currentQueueId, resultMessage);
                billing.AddCloseTranscriptionDetailToMainFile(processInteraction, processInteraction.batchTypeName, processInteraction.batchTypeId, 0, resultMessage);
                promiseArray.push(redis_service_1.Redis.DeleteQueue(`${this.queueBase}Execute`, currentQueueId));
            }
            else if (currentQueueId && !processInteraction) {
                logger.error('[SemanticAnalysisDownloadImplementation::SemanticAnalysisDownloadExecute] executionCode:', executionCode, 'getting queueId:', currentQueueId, 'INTERACTION_DO_NOT_EXIST');
                let interactionData = {
                    vcc: vcc,
                    processId: processId,
                    id: interactionId,
                    section: interactionSection,
                    part: interactionPart
                };
                billing.AddCloseTranscriptionDetailToMainFile(interactionData, processInteraction.batchTypeName, processInteraction.batchTypeId, 0, 'INTERACTION_DO_NOT_EXIST');
                promiseArray.push(redis_service_1.Redis.DeleteQueue(`${this.queueBase}Execute`, currentQueueId));
            }
            else {
                logger.warn('[SemanticAnalysisDownloadImplementation::SemanticAnalysisDownloadExecute] executionCode:', executionCode, 'getting queueId:', currentQueueId, 'QUEUE_WAS_NOT_REMOVED');
            }
            if (!promiseArray.length)
                promiseArray.push(Promise.resolve(true));
            return Promise.all(promiseArray);
        })
            .then(_ => {
            let promiseArray = [];
            if (processInteraction?.status === 6
                &&
                    processInteraction?.statusDescription === 'ready') {
                promiseArray.push(redis_service_1.Redis.UpdateProgress(storage));
                promiseArray.push(redis_service_1.Redis.DeleteProcessInteraction(storage, currentQueueId));
                promiseArray.push(redis_service_1.Redis.SaveReadyProcessInteraction(executionCode, currentQueueId, processInteraction));
            }
            else if (processInteraction?.retry >= config_1.RETRY_SEMANTIC_ANALYSIS) {
                processInteraction.isCancel = true;
                promiseArray.push(redis_service_1.Redis.UpdateProgress(storage));
                promiseArray.push(redis_service_1.Redis.UpdateErrorProgress(storage));
                promiseArray.push(redis_service_1.Redis.DeleteProcessInteraction(storage, currentQueueId));
                promiseArray.push(redis_service_1.Redis.SaveReadyProcessInteraction(executionCode, currentQueueId, processInteraction));
                promiseArray.push(mssql_service_1.MSSql.SaveBatchError(vcc, CURRENT_SYSTEM_SERVICE, processInteraction.batchTypeId, processInteraction.batchTypeId, CURRENT_IMPLEMENTATION, processInteraction.id, null, processInteraction.errorFunction, processInteraction.errorMessage, processInteraction.errorDetail));
            }
            if (!promiseArray.length)
                promiseArray.push(Promise.resolve(true));
            return Promise.all(promiseArray);
        })
            .then(_ => {
            return Promise.resolve(true);
        })
            .catch((err) => {
            let errorMessage = typeof err === 'string' ? err : (err.message || err.description);
            logger.error('[SemanticAnalysisDownloadImplementation::SemanticAnalysisDownloadExecute] executionCode:', executionCode, 'getting queueId:', currentQueueId, 'Catching error: ' + errorMessage);
            if (processInteraction)
                logger.error('[SemanticAnalysisDownloadImplementation::SemanticAnalysisDownloadExecute] executionCode:', executionCode, 'getting queueId:', currentQueueId, 'getting error:', errorMessage, ' vcc', processInteraction.vcc, 'processId:', processInteraction.processId, 'id:', processInteraction.id, 'section:', processInteraction.section, 'part:', processInteraction.part);
            return Promise.resolve(false);
        });
    }
    DownloadJSON(processInteraction, currentQueueId, executionCode) {
        let data;
        let silenceInTime;
        let silenceOutTime;
        let cognitiveService;
        return promises_shared_1.DummyPromise()
            .then(_ => {
            return cognitive_analysis_implementation_1.GetCognitiveService(processInteraction.vcc, null);
        })
            .then(result => {
            cognitiveService = result;
            let promiseArray = [];
            promiseArray.push(file_manager_1.CreateFolder(config_1.SEMANCTIC_ANALYSIS_FOLDER + '/' + processInteraction.vcc));
            promiseArray.push(file_manager_1.CreateFolder(config_1.SEMANCTIC_ANALYSIS_VIEW_FOLDER + '/' + processInteraction.vcc));
            promiseArray.push(file_manager_1.CreateFolder(config_1.COGNITIVE_ANALYSIS_FOLDER + '/' + processInteraction.vcc));
            promiseArray.push(file_manager_1.CreateFolder(config_1.GENERATIVE_ANALYSIS_FOLDER + '/' + processInteraction.vcc));
            return Promise.all(promiseArray);
        })
            .then(_ => {
            let regex = /\.(mp3|wav|ogg|json)$/;
            let jsonFileName = processInteraction.fileName.replace(regex, '.json');
            let requestFileName = processInteraction.fileName.replace(regex, '');
            return this.GetJsonFromSemanticEngine(processInteraction.vcc, requestFileName, true);
        })
            .then((result) => {
            data = result;
            if (result) {
                logger.info('[SemanticAnalysisDownloadImplementation::DownloadJSON] executionCode:', executionCode, 'getting queueId:', currentQueueId, 'got file');
                let jsonData = JSON.parse(data);
                let validChar = 0;
                let validWords = 0;
                if (jsonData?.words?.length) {
                    validWords = jsonData.words.length;
                    jsonData.words.forEach(r => {
                        validChar += r.name ? r.name.length + 1 : 0;
                    });
                }
                processInteraction['validChar'] = validChar;
                processInteraction['validWords'] = validWords;
                processInteraction['idathaApi'] = `${config_1.IDATHA_IP}:${config_1.IDATHA_HTTP_PORT}`;
                return Promise.resolve(true);
            }
            return Promise.resolve(null);
        })
            .then((result) => {
            if (result) {
                let transcription = this.SortJson(JSON.parse(data));
                silenceInTime = transcription.summarySpeechData.dataTotalIn.silence;
                silenceOutTime = transcription.summarySpeechData.dataTotalOut.silence;
                let promiseArray = [];
                promiseArray.push(this.SaveJsonForModelAnalysis(processInteraction, transcription, 'COGNITIVE', cognitiveService));
                promiseArray.push(this.SaveJsonForModelAnalysis(processInteraction, transcription, 'GENERATIVE'));
                return Promise.all(promiseArray);
            }
            return Promise.resolve([]);
        })
            .then((result) => {
            let saveResult = true;
            result.forEach(r => {
                saveResult = saveResult && r.status;
                if (r.status) {
                    processInteraction['charsPerPhrase'] = r.data.charsPerPhrase;
                    processInteraction['totalPhrases'] = r.data.totalPhrases;
                }
            });
            return Promise.resolve(saveResult);
        })
            .then((result) => {
            if (result) {
                logger.info('[SemanticAnalysisDownloadImplementation::DownloadJSON] executionCode:', executionCode, 'getting queueId:', currentQueueId, 'getting succes: ready');
                processInteraction['status'] = 6;
                processInteraction['statusDescription'] = processInteraction.interactionType !== 'CALL' ? 'ready' : 'pendingConvertAudio';
                processInteraction['isTranscribed'] = 1;
                processInteraction['retry'] = 0;
                processInteraction['uploadedDate'] = moment().format('x');
                processInteraction['errorFunction'] = null;
                processInteraction['errorMessage'] = null;
                processInteraction['silenceInTime'] = Number(silenceInTime);
                processInteraction['silenceOutTime'] = Number(silenceOutTime);
                processInteraction['silenceTime'] = Number(silenceInTime) + Number(silenceOutTime);
                this.errorData.CleanErrorData();
            }
            else {
                logger.error('[SemanticAnalysisDownloadImplementation::DownloadJSON] executionCode:', executionCode, 'getting queueId:', currentQueueId, 'getting error: JSON_IDATHA_IS_NOT_FOUND transcription content:', data);
                processInteraction['status'] = -6;
                processInteraction['retry'] = processInteraction['retry'] ? processInteraction['retry'] + 1 : 1;
                processInteraction['uploadedDate'] = moment().format('x');
                processInteraction['errorFunction'] = 'DownloadJSON';
                processInteraction['errorMessage'] = 'JSON_IDATHA_IS_NOT_FOUND';
                if (processInteraction['retry'] >= config_1.RETRY_SEMANTIC_ANALYSIS) {
                    logger.warn('[SemanticAnalysisDownloadImplementation::DownloadJSON] executionCode:', executionCode, 'getting queueId:', currentQueueId, 'returning to previous queue');
                    processInteraction['retry']--;
                    processInteraction['statusDescription'] = 'pendingAnalysis';
                }
                else
                    this.errorData.SetErrorData('DownloadJSON', processInteraction['errorMessage'], processInteraction['errorMessage']);
            }
            return Promise.resolve(processInteraction);
        })
            .catch((err) => {
            let errorMessage = typeof err === 'string' ? err : (err.message || err.description);
            logger.error('[SemanticAnalysisDownloadImplementation::DownloadJSON] executionCode:', executionCode, 'getting queueId:', currentQueueId, 'getting error:', errorMessage);
            processInteraction['status'] = -6;
            processInteraction['retry'] = processInteraction['retry'] ? processInteraction['retry'] + 1 : 1;
            processInteraction['uploadedDate'] = moment().format('x');
            processInteraction['errorFunction'] = 'DownloadJSON';
            processInteraction['errorMessage'] = errorMessage;
            this.errorData.SetErrorData('DownloadJSON', errorMessage, err);
            return Promise.resolve(processInteraction);
        });
    }
    GetJsonFromSemanticEngine(vcc, fileName, save) {
        let logLabel = `[${CURRENT_IMPLEMENTATION}::${this.GetJsonFromSemanticEngine.name}] [vcc::interactionId => ${vcc}::${fileName}]: `;
        let idatha = new idatha_model_1.IdathaConnection();
        let url = `http://${idatha.ip}:${idatha.httpPort}/api/query/document/${fileName}`;
        let options = {
            uri: url,
            retryProtocol: false,
            timeout: 180000,
            headers: {}
        };
        let data = null;
        return promises_shared_1.DummyPromise()
            .then(_ => {
            return idatha_service_1.IdathaLogin(vcc);
        })
            .then((userAccessToken) => {
            if (!userAccessToken)
                throw 'NOT_FOUND_ACCESS_TOKEN';
            options.headers = { 'Authorization': 'Bearer ' + userAccessToken.accessToken };
            return request_shared_1.SimpleGetRequest(options);
        })
            .then((response) => {
            try {
                let regex = /\bNaN\b/g;
                data = JSON.stringify(response);
                data = data.replace(regex, 'null');
                this.errorData.CleanErrorData();
                return Promise.resolve(data);
            }
            catch (err) {
                logger.error(`${logLabel}getting trycatch error`, err, 'options:', options);
                this.errorData.SetErrorData('GetJsonFromSemanticEngine', 'ERROR', err);
                return Promise.resolve(null);
            }
        })
            .then(_ => {
            if (save) {
                let path = `${config_1.SEMANCTIC_ANALYSIS_FOLDER}/${vcc}/${fileName}.json`;
                logger.info(`${logLabel}saving file ${path}`);
                return file_manager_1.WriteFileContent(path, data, 'utf-8');
            }
            else
                return Promise.resolve(true);
        })
            .then(result => {
            if (!result)
                throw 'SERVER_ERROR_WRITE_FILE_CONTENT_FAILED';
            return Promise.resolve(data);
        })
            .catch((err) => {
            let errorMessage = typeof err === 'string' ? err : (err.message || err.description);
            logger.error(`${logLabel}getting error:`, errorMessage, 'options:', options);
            this.errorData.SetErrorData('GetJsonFromSemanticEngine', errorMessage, err);
            return Promise.resolve(null);
        });
    }
    GetJsonForModelAnalysis(interaction, analysisType) {
        let logLabel = `[${CURRENT_IMPLEMENTATION}::${this.GetJsonForModelAnalysis.name}] [interactionId => ${interaction.id}]: `;
        let existsDocument = false;
        let regex = /\.(mp3|wav|ogg|json)$/;
        let jsonFileName = interaction.fileName.replace(regex, '.json');
        let modelFolder = null;
        switch (analysisType) {
            case 'COGNITIVE':
                modelFolder = config_1.COGNITIVE_ANALYSIS_FOLDER;
                break;
            case 'GENERATIVE':
                modelFolder = config_1.GENERATIVE_ANALYSIS_FOLDER;
                break;
        }
        return promises_shared_1.DummyPromise()
            .then(_ => {
            logger.info(`${logLabel}searching on cognitive folder.`);
            let path = `${analysisType}/${interaction.vcc}/${jsonFileName}`;
            return file_manager_1.ReadFileContent(path, 'utf-8', true);
        })
            .then((result) => {
            if (!result) {
                logger.info(`${logLabel}searching on semantic folder.`);
                let path = `${config_1.SEMANCTIC_ANALYSIS_FOLDER}/${interaction.vcc}/${jsonFileName}`;
                return file_manager_1.ReadFileContent(path, 'utf-8', true);
            }
            else
                existsDocument = true;
            return Promise.resolve(result);
        })
            .then((result) => {
            if (!result) {
                logger.info(`${logLabel}searching on idatha server.`);
                jsonFileName = interaction.fileName.replace(regex, '');
                return this.GetJsonFromSemanticEngine(interaction.vcc, jsonFileName, true);
            }
            return Promise.resolve(result);
        })
            .then((result) => {
            let jsonContent = null;
            try {
                jsonContent = typeof result === 'string' ? JSON.parse(result) : result;
                if (!existsDocument)
                    jsonContent = this.SortJson(jsonContent);
            }
            catch (ex) {
                jsonContent = null;
                this.errorData.SetErrorData('GetJsonForModelAnalysis', 'ERROR_FILE_JSON', ex);
                throw 'ERROR_FILE_JSON';
            }
            jsonContent.existsDocument = existsDocument;
            return Promise.resolve(jsonContent);
        })
            .catch((err) => {
            let errorMessage = typeof err === 'string' ? err : (err.message || err.description);
            logger.error(`${logLabel}getting error:`, errorMessage);
            throw err;
        });
    }
    SaveJsonForModelAnalysis(processInteraction, transcription, analysisType, cognitiveService) {
        let genericResult = new generic_result_1.GenericResult();
        let modelFolder = null;
        switch (analysisType) {
            case 'COGNITIVE':
                modelFolder = config_1.COGNITIVE_ANALYSIS_FOLDER;
                break;
            case 'GENERATIVE':
                modelFolder = config_1.GENERATIVE_ANALYSIS_FOLDER;
                break;
        }
        let charsPerPhrase = cognitiveService ? cognitiveService.GetCognitiveEngineCharsPerPhrase() : 1000;
        return promises_shared_1.DummyPromise()
            .then((result) => {
            if (result) {
                let phrases = this.SplitTranscriptionContentInPhrases(charsPerPhrase, transcription);
                return Promise.resolve(phrases);
            }
            return Promise.resolve(null);
        })
            .then((result) => {
            let phrases = null;
            if (result?.length) {
                genericResult.data = {
                    charsPerPhrase: charsPerPhrase,
                    totalPhrases: result.length
                };
                let regex = /\.(mp3|wav|ogg|json)$/;
                let jsonFileName = processInteraction.fileName.replace(regex, '.json');
                let path = `${modelFolder}/${processInteraction.vcc}/${jsonFileName}`;
                phrases = { phrases: result };
                let stringPhrases = JSON.stringify(phrases);
                return file_manager_1.WriteFileContent(path, stringPhrases, 'utf-8');
            }
            genericResult.status = false;
            genericResult.error = 'PHRASES_NOT_FOUND';
            return Promise.resolve(null);
        })
            .then(_ => {
            genericResult.status = true;
            return Promise.resolve(genericResult);
        })
            .catch((err) => {
            genericResult.status = false;
            genericResult.error ?? (genericResult.error = 'UNKNOWN_ERROR_CHECK_EXTRA_CONTENT');
            genericResult.extraContent = err;
            return Promise.resolve(genericResult);
        });
    }
    SplitTranscriptionContentInPhrases(charsPerPhrase, data) {
        let string_500 = [], phrases = '', len = data.transcription.length, item, phrasesResult = '', total = 0;
        string_500.push({ content: '', length: 0 });
        for (let i = 0; i < len; i++) {
            phrases = '';
            item = data.transcription[i];
            if (data.init === 'out') {
                phrases += item.out;
                phrases += ' ';
                phrases += item.in;
            }
            else {
                phrases += item.in;
                phrases += ' ';
                phrases += item.out;
            }
            total = phrasesResult.length + phrases.length;
            if (total > charsPerPhrase) {
                if (phrases.length < charsPerPhrase) {
                    string_500.push({ content: phrases, length: phrases.length });
                    phrasesResult = phrases;
                }
                else {
                    phrasesResult += phrases;
                    let phraseParts = phrasesResult.split('.');
                    phrasesResult = '';
                    phraseParts.map(pp => {
                        while (pp.length > 0) {
                            if ((phrasesResult.length + pp.length + 1) <= charsPerPhrase) {
                                phrasesResult += pp + '.';
                                pp = '';
                            }
                            else {
                                let arrPp = pp.split(' ');
                                if ((phrasesResult.length + 1 + arrPp[0].length + 1) > charsPerPhrase) {
                                    string_500.push({ content: phrasesResult, length: phrasesResult.length });
                                    phrasesResult = arrPp.shift();
                                }
                                else {
                                    phrasesResult += ' ' + arrPp.shift();
                                    if (pp.length === 0)
                                        phrasesResult += '.';
                                }
                                pp = arrPp.join(' ');
                            }
                            if (phrasesResult.length <= charsPerPhrase) {
                                string_500[string_500.length - 1].content = phrasesResult;
                                string_500[string_500.length - 1].length = phrasesResult.length;
                                if (phrasesResult.length === charsPerPhrase) {
                                    phrasesResult = '';
                                    string_500.push({ content: phrasesResult, length: phrasesResult.length });
                                }
                            }
                        }
                    });
                }
            }
            else {
                phrasesResult += phrases;
                string_500[string_500.length - 1].content += phrases;
                string_500[string_500.length - 1].length = phrasesResult.length;
            }
            phrases = '';
        }
        return string_500;
    }
    SortJson(jsonobj) {
        let outlength;
        let inlength;
        let i;
        let audioInit;
        let obj = {};
        obj.transcription = [];
        obj.summarySpeechData = {};
        obj.confidence = {};
        obj.init = '';
        let json = jsonobj;
        audioInit = json.words[0].speaker;
        obj.init = audioInit;
        obj.nlp = {};
        outlength = json.out_sentences.length;
        inlength = json.in_sentences.length;
        if (outlength >= inlength) {
            for (i in json.out_sentences) {
                if (i < inlength && inlength > 0) {
                    if (audioInit === 'out' && inlength > 0) {
                        obj.transcription.push({
                            out: json.out_sentences[i].text,
                            in: json.in_sentences[i] === 'undefined' ? '' : json.in_sentences[i].text
                        });
                    }
                    else if (audioInit === 'in' && inlength > 0) {
                        obj.transcription.push({
                            in: json.in_sentences[i].text,
                            out: json.out_sentences[i].text
                        });
                    }
                }
                else {
                    if (audioInit === 'out' && inlength >= 0) {
                        obj.transcription.push({
                            out: json.out_sentences[i] === undefined ? '' : json.out_sentences[i].text,
                            in: ''
                        });
                    }
                    else if (audioInit === 'in' && inlength >= 0) {
                        obj.transcription.push({
                            in: '',
                            out: json.out_sentences[i] === undefined ? '' : json.out_sentences[i].text
                        });
                    }
                }
            }
        }
        else if (outlength <= inlength) {
            for (i in json.in_sentences) {
                if (i < outlength && outlength > 0) {
                    if (audioInit === 'out' && outlength > 0) {
                        obj.transcription.push({
                            out: json.out_sentences[i].text,
                            in: json.in_sentences[i].text
                        });
                    }
                    else if (audioInit === 'in' && outlength > 0) {
                        obj.transcription.push({
                            in: json.in_sentences[i].text,
                            out: json.out_sentences[i].text
                        });
                    }
                }
                else {
                    if (audioInit === 'out' && inlength >= 0) {
                        obj.transcription.push({
                            out: '',
                            in: json.in_sentences[i] === undefined ? '' : json.in_sentences[i].text,
                        });
                    }
                    else if (audioInit === 'in' && inlength >= 0) {
                        obj.transcription.push({
                            in: json.in_sentences[i] === undefined ? '' : json.in_sentences[i].text,
                            out: ''
                        });
                    }
                }
            }
        }
        obj.nlp.in = json.pnl_scores_in ? json.pnl_scores_in : 0;
        obj.nlp.out = json.pnl_scores_out ? json.pnl_scores_out : 0;
        var dataTotalIn = {
            duration: 0,
            silence: 0
        }, dataTotalOut = {
            duration: 0,
            silence: 0
        }, data = json.words, len = data.length, word, inDuration = 0, inSilence = 0, outDuration = 0, outSilence = 0;
        for (i = 0; i < len; i++) {
            word = data[i];
            if (word.speaker === 'in') {
                inDuration += word.duration;
                inSilence += word.silence;
            }
            else {
                outDuration += word.duration;
                outSilence += word.silence;
            }
        }
        dataTotalIn.duration = inDuration.toFixed(2) * 1;
        dataTotalIn.silence = inSilence.toFixed(2) * 1;
        dataTotalOut.duration = outDuration.toFixed(2) * 1;
        dataTotalOut.silence = outSilence.toFixed(2) * 1;
        obj.summarySpeechData = {
            dataTotalIn: dataTotalIn,
            dataTotalOut: dataTotalOut
        };
        obj.duration = json.duration;
        obj.words = json.words;
        obj.topics = json.topics;
        obj.confidence.in_confidence = json.in_confidence ? json.in_confidence : 0;
        obj.confidence.out_confidence = json.out_confidence ? json.out_confidence : 0;
        return obj;
    }
}
exports.SemanticAnalysisDownloadImplementation = SemanticAnalysisDownloadImplementation;
