"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.SemanticAnalysisImplementation = void 0;
const log4js = require("log4js");
const moment = require("moment");
const mssql_service_1 = require("../db/mssql.service");
const config_1 = require("../config/config");
const redis_service_1 = require("../db/redis.service");
const promises_shared_1 = require("../shared/promises.shared");
const file_manager_1 = require("../shared/file-manager");
const idatha_model_1 = require("../models/idatha.model");
const SSH2Promise = require("ssh2-promise");
const billing_implementation_1 = require("../implementation/billing.implementation");
const idatha_service_1 = require("../services/idatha.service");
const cognitive_analysis_implementation_1 = require("./cognitive-analysis.implementation");
const request_shared_1 = require("../shared/request.shared");
const billing_batch_model_1 = require("../models/billing-batch.model");
const child_process_1 = require("child_process");
let logger = log4js.getLogger('Task');
const CURRENT_SYSTEM_SERVICE = 'TRANSCRIPTION';
const CURRENT_IMPLEMENTATION = 'SemanticAnalysisImplementation';
class SemanticAnalysisImplementation {
    constructor() {
        this.errorData = new billing_batch_model_1.BatchError();
        this.errorAudioConvert = false;
    }
    Execute(executionCode, rateRecursive) {
        logger.info(`[SemanticAnalysisImplementation::Execute] executionCode: ${executionCode} executing wave #${rateRecursive}`);
        let executeIndex = 1;
        let ratio = config_1.RATIO_SEMANTIC_ANALYSIS;
        let executeResult = true;
        return promises_shared_1.SleepPromise(config_1.SLEEP_SEMANTIC_ANALYSIS)
            .then(_ => {
            return redis_service_1.Redis.LRange('SemanticAnalysis', 0, ratio - 1);
        }).then((result) => {
            let promiseArray = [];
            ratio = result && result.length <= ratio ? result.length : 0;
            while (ratio >= executeIndex) {
                promiseArray.push(this.SemanticAnalysisExecute(executionCode));
                executeIndex++;
            }
            if (!promiseArray.length)
                promiseArray.push(Promise.resolve(true));
            return Promise.all(promiseArray);
        }).then((results) => {
            if (results.filter(item => item === false).length > 0) {
                executeResult = false;
            }
            return redis_service_1.Redis.LRange('SemanticAnalysis', 0, ratio - 1);
        }).then((result) => {
            if (result && result.length && rateRecursive >= 1)
                return this.Execute(executionCode, rateRecursive - 1);
            return Promise.resolve(executeResult);
        }).then(result => {
            if (rateRecursive === config_1.RATE_RECURSIVE_SEMANTIC_ANALYSIS)
                logger.info(`[SemanticAnalysisImplementation::Execute] executionCode: ${executionCode} Finishing execution wave #${rateRecursive}`);
            let currentResult = executeResult && result ? true : false;
            return Promise.resolve(currentResult);
        }).catch((error) => {
            let errorMessage = typeof error === 'string' ? error : (error.message || error.description);
            logger.error('[SemanticAnalysisImplementation::Execute] executionCode:', executionCode, 'getting error:', errorMessage);
            return Promise.resolve(false);
        });
    }
    SemanticAnalysisExecute(executionCode) {
        let storage = '';
        let currentQueueId = '';
        let processInteraction;
        let isCancel = false;
        let vcc = '';
        let processId = '';
        let interactionType = '';
        let interactionId = '';
        let interactionSection = 0;
        let interactionPart = 0;
        return redis_service_1.Redis.RPopLPush('SemanticAnalysis', 'SemanticAnalysisExecute')
            .then((queueId) => {
            if (queueId && queueId.length > 0) {
                currentQueueId = queueId;
                logger.info('[SemanticAnalysisImplementation::SemanticAnalysisExecute] executionCode:', executionCode, 'getting queueId:', currentQueueId, 'START');
                let item = queueId.split(':');
                if (item && item.length) {
                    vcc = item[0];
                    processId = item[1];
                    interactionType = item[2];
                    interactionId = item[3];
                    interactionSection = Number(item[4]);
                    interactionPart = Number(item[5]);
                    storage = vcc + ':' + processId;
                    logger.info('[SemanticAnalysisImplementation::SemanticAnalysisExecute] executionCode:', executionCode, 'getting queueId:', currentQueueId, 'storage :', storage);
                    return redis_service_1.Redis.GetProcessInteraction(storage, currentQueueId);
                }
                else {
                    logger.error('[SemanticAnalysisImplementation::SemanticAnalysisExecute] executionCode:', executionCode, 'getting queueId:', currentQueueId, 'QUEUE_FORMAT_ERROR');
                }
            }
            return Promise.resolve(null);
        })
            .then((processInteractionResult) => {
            if (processInteractionResult) {
                processInteraction = processInteractionResult;
                logger.info('[SemanticAnalysisImplementation::SemanticAnalysisExecute] executionCode:', executionCode, 'getting queueId:', currentQueueId, 'getting GetProcessInteraction: 1, status:', processInteraction.statusDescription);
                if (processInteraction.statusDescription === 'pendingAnalysis') {
                    return this.SendJSON(processInteraction, currentQueueId, executionCode);
                }
                else if (processInteraction.statusDescription === 'pendingDownloadJSON') {
                    let validTime = moment().format('x');
                    validTime = validTime * 1;
                    let nextExecutionTime = processInteraction.uploadedDate;
                    nextExecutionTime = nextExecutionTime * 1 + config_1.NEXT_EXECUTION_TIME_SEMANTIC_ANALYSIS;
                    if (validTime > nextExecutionTime) {
                        return this.DownloadJSON(processInteraction, currentQueueId, executionCode);
                    }
                    else {
                        logger.info('[SemanticAnalysisImplementation::SemanticAnalysisExecute] executionCode:', executionCode, 'getting queueId:', currentQueueId, 'NEXT_EXECUTION', nextExecutionTime, 'CURRENT_TIME', validTime);
                    }
                }
                else if (processInteraction.statusDescription === 'pendingConvertAudio') {
                    return this.ConvertAudio(processInteraction, currentQueueId, executionCode);
                }
            }
            if (!processInteraction && currentQueueId) {
                logger.warn('[SemanticAnalysisImplementation::SemanticAnalysisExecute] executionCode:', executionCode, 'getting queueId:', currentQueueId, 'getting GetProcessInteraction: 0');
                if (!processInteraction)
                    throw 'SERVER_ERROR_INTERACTION_DATA_NOT_FOUND_ON_STORAGE';
            }
            return Promise.resolve(null);
        })
            .then((result) => {
            if (result)
                processInteraction = result;
            if (processInteraction)
                return redis_service_1.Redis.SaveProcessInteraction(storage, currentQueueId, processInteraction);
            return Promise.resolve(null);
        })
            .then((result) => {
            let promiseArray = [];
            let billing = new billing_implementation_1.BillingImplementation();
            if (result && processInteraction?.status === 6 && processInteraction?.statusDescription === 'ready') {
                logger.info('[SemanticAnalysisImplementation::SemanticAnalysisExecute] executionCode:', executionCode, 'getting queueId:', currentQueueId, 'QUEUE_FINISHED');
                billing.AddUpdateTranscriptionDetailToMainFile(processInteraction, 'ValidChar', processInteraction.batchTypeName, processInteraction.batchTypeId);
                billing.AddUpdateTranscriptionDetailToMainFile(processInteraction, 'ValidWords', processInteraction.batchTypeName, processInteraction.batchTypeId);
                billing.AddUpdateTranscriptionDetailToMainFile(processInteraction, 'TotalPhrases', processInteraction.batchTypeName, processInteraction.batchTypeId);
                billing.AddUpdateTranscriptionDetailToMainFile(processInteraction, 'CharsPerPhrase', processInteraction.batchTypeName, processInteraction.batchTypeId);
                billing.AddUpdateTranscriptionDetailToMainFile(processInteraction, 'IdathaApi', processInteraction.batchTypeName, processInteraction.batchTypeId);
                billing.AddCloseTranscriptionDetailToMainFile(processInteraction, processInteraction.batchTypeName, processInteraction.batchTypeId, 1, 'SUCCED');
                promiseArray.push(redis_service_1.Redis.DeleteQueue('SemanticAnalysisExecute', currentQueueId));
            }
            else if (result
                &&
                    processInteraction?.status === 6
                &&
                    processInteraction?.statusDescription === 'pendingDownloadJSON'
                &&
                    processInteraction?.retry < config_1.RETRY_SEMANTIC_ANALYSIS) {
                promiseArray.push(redis_service_1.Redis.InsertQueue('SemanticAnalysis', currentQueueId, config_1.RATE_RECURSIVE_SEMANTIC_ANALYSIS * config_1.RATIO_SEMANTIC_ANALYSIS * 2));
                promiseArray.push(redis_service_1.Redis.DeleteQueue('SemanticAnalysisExecute', currentQueueId));
            }
            else if (result && processInteraction && processInteraction.status === -5 && processInteraction.statusDescription === 'pendingRemove') {
                logger.error('[SemanticAnalysisImplementation::SemanticAnalysisExecute] executionCode:', executionCode, 'getting queueId:', currentQueueId, 'REMOVED_BY_INVALID_CONTENT');
                billing.AddCloseTranscriptionDetailToMainFile(processInteraction, processInteraction.batchTypeName, processInteraction.batchTypeId, 0, 'REMOVED_BY_INVALID_CONTENT');
                promiseArray.push(redis_service_1.Redis.DeleteQueue('SemanticAnalysisExecute', currentQueueId));
            }
            else if (processInteraction?.retry >= config_1.RETRY_SEMANTIC_ANALYSIS) {
                logger.error('[SemanticAnalysisImplementation::SemanticAnalysisExecute] executionCode:', executionCode, 'getting queueId:', currentQueueId, 'REMOVED_BY_RETRIES');
                billing.AddCloseTranscriptionDetailToMainFile(processInteraction, processInteraction.batchTypeName, processInteraction.batchTypeId, 0, 'REMOVED_BY_RETRIES');
                promiseArray.push(redis_service_1.Redis.DeleteQueue('SemanticAnalysisExecute', currentQueueId));
            }
            else if (currentQueueId && !processInteraction) {
                logger.error('[SemanticAnalysisImplementation::SemanticAnalysisExecute] executionCode:', executionCode, 'getting queueId:', currentQueueId, 'INTERACTION_DO_NOT_EXIST');
                let interactionData = {
                    vcc: vcc,
                    processId: processId,
                    id: interactionId,
                    section: interactionSection,
                    part: interactionPart
                };
                billing.AddCloseTranscriptionDetailToMainFile(interactionData, processInteraction.batchTypeName, processInteraction.batchTypeId, 0, 'INTERACTION_DO_NOT_EXIST');
                promiseArray.push(redis_service_1.Redis.DeleteQueue('SemanticAnalysisExecute', currentQueueId));
            }
            else {
                logger.warn('[SemanticAnalysisImplementation::SemanticAnalysisExecute] executionCode:', executionCode, 'getting queueId:', currentQueueId, 'QUEUE_WAS_NOT_REMOVED');
            }
            if (!promiseArray.length)
                promiseArray.push(Promise.resolve(true));
            return Promise.all(promiseArray);
        }).then((result) => {
            let promiseArray = [];
            if (processInteraction
                && processInteraction.status === 6
                && processInteraction.statusDescription === 'ready') {
                promiseArray.push(redis_service_1.Redis.UpdateProgress(storage));
                promiseArray.push(redis_service_1.Redis.DeleteProcessInteraction(storage, currentQueueId));
                promiseArray.push(redis_service_1.Redis.SaveReadyProcessInteraction(executionCode, currentQueueId, processInteraction));
            }
            else if (processInteraction
                && (processInteraction.retry >= config_1.RETRY_SEMANTIC_ANALYSIS
                    ||
                        ['JSON_TRANSCRIPTION_CONTENT_IS_EMPTY', 'JSON_TRANSCRIPTION_CONTENT_HAS_NOT_WORD_TYPE_DATA'].includes(processInteraction.errorMessage))) {
                if (!this.errorAudioConvert)
                    processInteraction.isCancel = true;
                promiseArray.push(redis_service_1.Redis.UpdateProgress(storage));
                promiseArray.push(redis_service_1.Redis.UpdateErrorProgress(storage));
                promiseArray.push(redis_service_1.Redis.DeleteProcessInteraction(storage, currentQueueId));
                promiseArray.push(redis_service_1.Redis.SaveReadyProcessInteraction(executionCode, currentQueueId, processInteraction));
                promiseArray.push(mssql_service_1.MSSql.SaveBatchError(vcc, CURRENT_SYSTEM_SERVICE, processInteraction.batchTypeId, processInteraction.batchTypeId, CURRENT_IMPLEMENTATION, processInteraction.id, null, processInteraction.errorFunction, processInteraction.errorMessage, processInteraction.errorDetail));
            }
            if (!promiseArray.length)
                promiseArray.push(Promise.resolve(true));
            return Promise.all(promiseArray);
        }).then((result) => {
            return Promise.resolve(true);
        }).catch((err) => {
            let errorMessage = typeof err === 'string' ? err : (err.message || err.description);
            logger.error('[SemanticAnalysisImplementation::SemanticAnalysisExecute] executionCode:', executionCode, 'getting queueId:', currentQueueId, 'Catching error: ' + errorMessage);
            if (processInteraction)
                logger.error('[SemanticAnalysisImplementation::SemanticAnalysisExecute] executionCode:', executionCode, 'getting queueId:', currentQueueId, 'getting error:', errorMessage, ' vcc', processInteraction.vcc, 'processId:', processInteraction.processId, 'id:', processInteraction.id, 'section:', processInteraction.section, 'part:', processInteraction.part);
            return Promise.resolve(false);
        });
    }
    SendJSON(interaction, currentQueueId, executionCode) {
        let path = config_1.TRANSCRIPTION_FOLDER;
        let jsonContent = null;
        return promises_shared_1.DummyPromise()
            .then(_ => {
            return idatha_service_1.IdathaLogin(interaction.vcc);
        })
            .then((result) => {
            if (!result || !result.accessToken)
                throw 'SERVER_ERROR_CANNOT_GET_USER_ACCESS_TOKEN';
            let regex = /\.(mp3|wav|ogg)$/;
            let jsonFileName = interaction.fileName;
            if (!jsonFileName) {
                jsonFileName = `${interaction.id}_${interaction.section}_${interaction.part}.${interaction.format || 'wav'}`;
                interaction.fileName = jsonFileName;
            }
            jsonFileName = interaction.fileName.replace(regex, '.json');
            path += '/';
            path += interaction.vcc;
            path += '/';
            path += jsonFileName;
            logger.info('[SemanticAnalysisImplementation::SendJSON] executionCode:', executionCode, 'getting queueId:', currentQueueId, `Reading file ${path}`);
            return file_manager_1.ReadFileContent(path, 'utf-8');
        }).then((result) => {
            try {
                jsonContent = typeof result === 'string' ? JSON.parse(result) : result;
                jsonContent.filterdata = {
                    start_date: new Date(interaction.startDate),
                    end_date: new Date(interaction.endDate),
                    channel: interaction.interactionType,
                    campaign: interaction.campaign,
                    disposition: interaction.dispositionCode,
                    agent: interaction.agent,
                    origin: interaction.origin
                };
            }
            catch (ex) {
                jsonContent = '';
                throw 'ERROR_FILE_JSON';
            }
            logger.info('[SemanticAnalysisImplementation::SendJSON] executionCode:', executionCode, 'getting queueId:', currentQueueId, `Writing file ${path}`);
            return file_manager_1.WriteFileContent(path, JSON.stringify(jsonContent), 'utf-8', true);
        }).then((_) => {
            if (jsonContent?.results?.length) {
                let wordResults = jsonContent.results.filter(r => r.type === 'word').length;
                if (wordResults)
                    return this.IdathaSendJSONFiles(interaction, currentQueueId, executionCode);
                else
                    return Promise.resolve('JSON_TRANSCRIPTION_CONTENT_HAS_NOT_WORD_TYPE_DATA');
            }
            else {
                return Promise.resolve('JSON_TRANSCRIPTION_CONTENT_IS_EMPTY');
            }
        }).then((result) => {
            if (typeof result === 'boolean' && result === true) {
                interaction['status'] = 6;
                interaction['statusDescription'] = 'pendingDownloadJSON';
                interaction['retry'] = 0;
                interaction['uploadedDate'] = moment().format('x');
                interaction['errorMessage'] = null;
                this.errorData.CleanErrorData();
            }
            else {
                logger.error('[SemanticAnalysisImplementation::SendJSON] executionCode:', executionCode, 'getting queueId:', currentQueueId, 'JSON_TRANSCRIPTION_WAS_NOT_TRANSFERRED');
                interaction['status'] = -6;
                interaction['retry'] = interaction['retry'] ? interaction['retry'] + 1 : 1;
                if (typeof result === 'string' && ['JSON_TRANSCRIPTION_CONTENT_IS_EMPTY', 'JSON_TRANSCRIPTION_CONTENT_HAS_NOT_WORD_TYPE_DATA'].includes(result)) {
                    logger.error('[SemanticAnalysisImplementation::SendJSON] executionCode:', executionCode, 'getting queueId:', currentQueueId, result);
                    interaction['statusDescription'] = 'pendingRemove';
                    interaction['errorMessage'] = result;
                }
                else {
                    interaction['errorMessage'] = 'JSON_TRANSCRIPTION_WAS_NOT_TRANSFERRED';
                }
                this.errorData.SetErrorData('SendJSON', interaction['errorMessage'], interaction['errorMessage']);
            }
            logger.info('[SemanticAnalysisImplementation::SendJSON] executionCode:', executionCode, 'getting queueId:', currentQueueId, `Resolving for ${path}`);
            return Promise.resolve(interaction);
        }).catch((err) => {
            let errorMessage = typeof err === 'string' ? err : (err.message || err.description);
            logger.error('[SemanticAnalysisImplementation::SendJSON] executionCode:', executionCode, 'getting queueId:', currentQueueId, errorMessage);
            interaction['status'] = -6;
            interaction['retry'] = interaction['retry'] ? interaction['retry'] + 1 : 1;
            interaction['errorMessage'] = errorMessage;
            this.errorData.SetErrorData('SendJSON', errorMessage, err);
            return Promise.resolve(interaction);
        });
    }
    IdathaCreateDefaultDirIfNotExists(client, path) {
        return client.exec(`mkdir -p ${path}`)
            .then((_) => {
            return Promise.resolve(true);
        })
            .catch((err) => {
            logger.error('[SemanticAnalysisImplementation::IdathaCreateDefaultDirIfNotExists]', err);
            return Promise.reject(err);
        });
    }
    IdathaUploadFile(client, src, destination) {
        let sftp;
        sftp = client.sftp();
        logger.info(`[SemanticAnalysisImplementation::IdathaUploadFile] Uploading: "${src}"`);
        return promises_shared_1.DummyPromise()
            .then(_ => sftp.fastPut(src, destination))
            .then(_ => {
            logger.info(`[SemanticAnalysisImplementation::IdathaUploadFile] Uploaded corretly: "${src}"`);
            return Promise.resolve(true);
        })
            .catch(err => {
            logger.error('[SemanticAnalysisImplementation::IdathaUploadFile]', err);
            return Promise.reject(err);
        });
    }
    IdathaSendJSONFiles(processInteraction, currentQueueId, executionCode) {
        let idatha = new idatha_model_1.IdathaConnection();
        const client = {
            host: idatha.ip,
            port: idatha.sshPort,
            username: idatha.user,
            password: idatha.password,
            connTimeout: 50000,
            pasvTimeout: 50000
        };
        let ssh2 = new SSH2Promise(client);
        let regex = /\.(mp3|wav|ogg)$/;
        let src = `${config_1.TRANSCRIPTION_FOLDER}/${processInteraction.vcc.toLowerCase()}/${processInteraction.fileName.replace(regex, '.json')}`;
        let target = `/home/${idatha.user}/in_concert_speech_analytics/input/${processInteraction.vcc.toLowerCase()}/default/`;
        let destination = `${target}${processInteraction.fileName.replace(regex, '.json')}`;
        return ssh2.connect()
            .then(() => {
            return this.IdathaCreateDefaultDirIfNotExists(ssh2, target);
        })
            .then((result) => {
            if (result)
                return this.IdathaUploadFile(ssh2, src, destination);
            return Promise.resolve(result);
        })
            .then((result) => {
            if (result)
                this.errorData.CleanErrorData();
            ssh2.close();
            return Promise.resolve(result);
        })
            .catch(err => {
            ssh2.close();
            let errorMessage = typeof err === 'string' ? err : (err.message || err.description);
            logger.error('[SemanticAnalysisImplementation::IdathaSendJSONFiles] executionCode:', executionCode, 'getting queueId:', currentQueueId, 'getting error:', errorMessage);
            this.errorData.SetErrorData('IdathaSendJSONFiles', errorMessage, err);
            return Promise.resolve(false);
        });
    }
    DownloadJSON(interaction, currentQueueId, executionCode) {
        let path = '';
        let jsonContent = null;
        let data;
        let cognitiveService;
        return promises_shared_1.DummyPromise()
            .then(_ => {
            return cognitive_analysis_implementation_1.GetCognitiveService(interaction.vcc, null);
        }).then(result => {
            cognitiveService = result;
            let promiseArray = [];
            promiseArray.push(file_manager_1.CreateFolder(config_1.SEMANCTIC_ANALYSIS_FOLDER + '/' + interaction.vcc));
            promiseArray.push(file_manager_1.CreateFolder(config_1.SEMANCTIC_ANALYSIS_VIEW_FOLDER + '/' + interaction.vcc));
            promiseArray.push(file_manager_1.CreateFolder(config_1.COGNITIVE_ANALYSIS_FOLDER + '/' + interaction.vcc));
            return Promise.all(promiseArray);
        }).then((_) => {
            let regex = /\.(mp3|wav|ogg|json)$/;
            let jsonFileName = interaction.fileName.replace(regex, '.json');
            path = config_1.SEMANCTIC_ANALYSIS_FOLDER;
            path += '/';
            path += interaction.vcc;
            path += '/';
            path += jsonFileName;
            return idatha_service_1.IdathaLogin(interaction.vcc);
        }).then((userAccessToken) => {
            if (!userAccessToken)
                throw 'NOT_FOUND_ACCESS_TOKEN';
            let regex = /\.(mp3|wav|ogg|json)$/;
            let fileName = interaction.fileName.replace(regex, '');
            return this.IdathaDocumentWords(userAccessToken.accessToken, interaction.id, interaction.section, interaction.part, currentQueueId, executionCode, fileName);
        }).then((result) => {
            data = result;
            logger.info('[SemanticAnalysisImplementation::DownloadJSON] executionCode:', executionCode, 'getting queueId:', currentQueueId, 'getting path:', path);
            if (result) {
                let jsonData = JSON.parse(data);
                let validChar = 0;
                let validWords = 0;
                if (jsonData && jsonData.words && jsonData.words.length && jsonData.words.length > 0) {
                    validWords = jsonData.words.length;
                    jsonData.words.map(r => {
                        validChar += r.name ? r.name.length + 1 : 0;
                    });
                }
                interaction['validChar'] = validChar;
                interaction['validWords'] = validWords;
                interaction['idathaApi'] = config_1.IDATHA_IP + ':' + config_1.IDATHA_HTTP_PORT;
                return file_manager_1.WriteFileContent(path, data, 'utf-8');
            }
            return Promise.resolve(null);
        }).then((result) => {
            if (result) {
                let cognitiveAnalysis = new cognitive_analysis_implementation_1.CognitiveAnalysisImplementation();
                let phrases = [];
                logger.info('[SemanticAnalysisImplementation::DownloadJSON] executionCode:', executionCode, 'getting queueId:', currentQueueId, 'getting phrases:', phrases.length);
                return Promise.resolve(phrases);
            }
            return Promise.resolve(null);
        }).then((result) => {
            let phrases;
            if (result && result.length) {
                interaction['charsPerPhrase'] = cognitiveService ? cognitiveService.GetCognitiveEngineCharsPerPhrase() : 0;
                interaction['totalPhrases'] = result.length;
                let regex = /\.(mp3|wav|ogg|json)$/;
                let jsonFileName = interaction.fileName.replace(regex, '.json');
                let path = config_1.COGNITIVE_ANALYSIS_FOLDER;
                path += '/';
                path += interaction.vcc;
                path += '/';
                path += jsonFileName;
                phrases = { phrases: result };
                let stringPhrases = JSON.stringify(phrases);
                return file_manager_1.WriteFileContent(path, stringPhrases, 'utf-8');
            }
            return Promise.resolve(null);
        }).then((result) => {
            if (result) {
                logger.info('[SemanticAnalysisImplementation::DownloadJSON] executionCode:', executionCode, 'getting queueId:', currentQueueId, 'getting succes: ready');
                interaction['status'] = 6;
                interaction['statusDescription'] = interaction.interactionType !== 'CALL'
                    ? 'ready' : 'pendingConvertAudio';
                interaction['isTranscribed'] = 1;
                interaction['retry'] = 0;
                interaction['uploadedDate'] = moment().format('x');
                interaction['errorFunction'] = null;
                interaction['errorMessage'] = null;
                this.errorData.CleanErrorData();
            }
            else {
                logger.error('[SemanticAnalysisImplementation::DownloadJSON] executionCode:', executionCode, 'getting queueId:', currentQueueId, 'getting error: JSON_IDATHA_IS_NOT_FOUND transcription content:', data, 'path:', path);
                interaction['status'] = -6;
                interaction['retry'] = interaction['retry'] ? interaction['retry'] + 1 : 1;
                interaction['uploadedDate'] = moment().format('x');
                interaction['errorFunction'] = 'DownloadJSON';
                interaction['errorMessage'] = 'JSON_IDATHA_IS_NOT_FOUND';
                this.errorData.SetErrorData('DownloadJSON', interaction['errorMessage'], interaction['errorMessage']);
            }
            return Promise.resolve(interaction);
        }).catch((err) => {
            let errorMessage = typeof err === 'string' ? err : (err.message || err.description);
            logger.error('[SemanticAnalysisImplementation::DownloadJSON] executionCode:', executionCode, 'getting queueId:', currentQueueId, 'getting error:', errorMessage);
            interaction['status'] = -6;
            interaction['retry'] = interaction['retry'] ? interaction['retry'] + 1 : 1;
            interaction['uploadedDate'] = moment().format('x');
            interaction['errorFunction'] = 'DownloadJSON';
            interaction['errorMessage'] = errorMessage;
            this.errorData.SetErrorData('DownloadJSON', errorMessage, err);
            return Promise.resolve(interaction);
        });
    }
    ConvertAudio(interaction, currentQueueId, executionCode) {
        let newFormat = 'mp3';
        let newOldFileNamePath = '';
        let currentFolder = ['allegro', 'i6', 'i-ftp', 'i-sftp'].includes(interaction?.environmentType) ? config_1.PATH_FOLDER_TEMP_AUDIO : config_1.PATH_FOLDER_PROCESS_AUDIO;
        let oldFilePath = `${currentFolder}/${interaction.vcc}/${interaction.fileName}`;
        let newFilePath = `${currentFolder}/${interaction.vcc}/${interaction.fileName.replace('.wav', `.${newFormat}`).replace('.ogg', `.${newFormat}`)}`;
        let forceCoonvertion = false;
        if (oldFilePath === newFilePath) {
            forceCoonvertion = true;
            newOldFileNamePath = oldFilePath.replace('.mp3', '_changed.mp3');
        }
        return promises_shared_1.DummyPromise()
            .then(_ => {
            if (forceCoonvertion)
                return file_manager_1.RenameFile(oldFilePath, newOldFileNamePath);
            return Promise.resolve(true);
        })
            .then(_ => {
            if (forceCoonvertion)
                oldFilePath = newOldFileNamePath;
            return Promise.resolve(true);
        })
            .then(_ => {
            let channel = 1;
            if (interaction.channels !== 'mono') {
                channel = 2;
            }
            let cmd = `${config_1.FFMPEG_FOLDER_PATH}ffmpeg -y -i "${oldFilePath}" -vn -ar 22050 -ac ${channel} -b:a 16k "${newFilePath}"`;
            logger.debug('[SemanticAnalysisImplementation::ConvertAudio] executionCode:', executionCode, 'getting queueId:', currentQueueId, 'getting cmd:', cmd);
            return new Promise((resolve, reject) => {
                try {
                    child_process_1.exec(cmd, (err, stdout, stderr) => {
                        if (err) {
                            if (err.message?.includes('No such file or directory')
                                ||
                                    err.message?.includes('Command failed')) {
                                logger.error('[SemanticAnalysisImplementation::ConvertAudio] executionCode:', executionCode, 'getting queueId:', currentQueueId, 'the original file was not found.');
                                resolve(true);
                            }
                            else {
                                reject(err);
                            }
                        }
                        else {
                            interaction.fileName = interaction.fileName.replace('.wav', `.${newFormat}`).replace('.ogg', `.${newFormat}`);
                            resolve(stdout.media);
                        }
                    });
                }
                catch (e) {
                    if (e.message?.includes('No such file or directory')
                        ||
                            e.message?.includes('Command failed')) {
                        logger.error('[SemanticAnalysisImplementation::ConvertAudio] executionCode:', executionCode, 'getting queueId:', currentQueueId, 'the original file was not found.');
                        interaction.fileName = null;
                        resolve(true);
                    }
                    else {
                        reject(e);
                    }
                }
            });
        })
            .then(_ => {
            return file_manager_1.DeleteFile(oldFilePath, true);
        })
            .then(_ => {
            interaction['status'] = 6;
            interaction['statusDescription'] = 'ready';
            interaction['format'] = newFormat;
            interaction['fileName'] = interaction.fileName;
            interaction['retry'] = 0;
            this.errorAudioConvert = false;
            return Promise.resolve(interaction);
        })
            .catch((err) => {
            let errorMessage = typeof err === 'string' ? err : (err.message || err.description);
            logger.error('[SemanticAnalysisImplementation::ConvertAudio] executionCode:', executionCode, 'getting queueId:', currentQueueId, 'getting error:', errorMessage);
            interaction['status'] = -6;
            interaction['retry'] = interaction['retry'] ? interaction['retry'] + 1 : 1;
            interaction['uploadedDate'] = moment().format('x');
            interaction['errorMessage'] = errorMessage;
            this.errorAudioConvert = true;
            this.errorData.SetErrorData('ConvertAudio', errorMessage, err);
            return Promise.resolve(interaction);
        });
    }
    IdathaDocumentWords(accessToken, audioId, section, part, currentQueueId, executionCode, fileName) {
        let idatha = new idatha_model_1.IdathaConnection();
        let url = `http://${idatha.ip}:${idatha.httpPort}/api/query/document/${fileName}`;
        let options = {
            uri: url,
            retryProtocol: false,
            timeout: 180000,
            headers: {
                'Authorization': 'Bearer ' + accessToken
            }
        };
        let data = '';
        return promises_shared_1.DummyPromise()
            .then((result) => {
            return request_shared_1.SimpleGetRequest(options);
        }).then((response) => {
            try {
                let regex = /\bNaN\b/g;
                data = JSON.stringify(response);
                data = data.replace(regex, 'null');
                this.errorData.CleanErrorData();
                return Promise.resolve(data);
            }
            catch (err) {
                logger.error('[SemanticAnalysisImplementation::IdathaDocumentWords] executionCode:', executionCode, 'getting queueId:', currentQueueId, err, 'options:', options);
                this.errorData.SetErrorData('IdathaDocumentWords', 'ERROR', err);
                return Promise.resolve(null);
            }
        }).catch((err) => {
            let errorMessage = typeof err === 'string' ? err : (err.message || err.description);
            logger.error('[SemanticAnalysisImplementation::IdathaDocumentWords] executionCode:', executionCode, 'getting queueId:', currentQueueId, 'getting error:', errorMessage, 'options:', options);
            this.errorData.SetErrorData('IdathaDocumentWords', errorMessage, err);
            return Promise.resolve(null);
        });
    }
    SemanticAnalysisFileGenerate() {
        return null;
    }
    SemanticAnalysisViewFileGenerate(interaction) {
        let semanticAnalysisFolder = '';
        let semanticAnalysisFilePath = '';
        let semanticAnalysisViewFolder = '';
        let semanticAnalysisViewFileName = '';
        let semanticAnalysisViewFilePath = '';
        let semanticAnalysisViewFileContent = null;
        semanticAnalysisFolder = config_1.SEMANCTIC_ANALYSIS_FOLDER;
        semanticAnalysisFolder += '/';
        semanticAnalysisFolder += interaction.vcc;
        semanticAnalysisViewFolder = config_1.SEMANCTIC_ANALYSIS_VIEW_FOLDER;
        semanticAnalysisViewFolder += '/';
        semanticAnalysisViewFolder += interaction.vcc;
        semanticAnalysisViewFileName = interaction.jsonFileName;
        semanticAnalysisFilePath = semanticAnalysisFolder + '/' + semanticAnalysisViewFileName;
        semanticAnalysisViewFilePath = semanticAnalysisViewFolder + '/' + semanticAnalysisViewFileName;
        return promises_shared_1.DummyPromise()
            .then(result => {
            return file_manager_1.ExistInFileSystem(semanticAnalysisFilePath);
        })
            .then(result => {
            if (result) {
                return file_manager_1.CreateFolder(semanticAnalysisViewFolder);
            }
            else
                throw 'SERVER_ERROR_SEMANTIC_ANALYSIS_FILE_NOT_FOUND';
        }).then(result => {
            return file_manager_1.ReadFileContent(semanticAnalysisFilePath, 'utf-8');
        }).then(result => {
            if (result) {
                let semanticAnalysisFileContent = JSON.parse(result);
                return file_manager_1.WriteFileContent(semanticAnalysisViewFilePath, JSON.stringify(semanticAnalysisViewFileContent), 'utf-8');
            }
            else
                throw 'SERVER_ERROR_SEMANTIC_ANALYSIS_FILE_FAILED';
        })
            .then(result => {
            this.errorData.CleanErrorData();
            return {
                fileName: semanticAnalysisViewFileName,
                transcriptionResult: semanticAnalysisViewFileContent
            };
        }).catch((error) => {
            let errorMessage = typeof error === 'string' ? error : (error.message || error.description);
            logger.error('[SemanticAnalysisImplementation::SemanticAnalysisViewFileGenerate] Getting error:', errorMessage);
            this.errorData.SetErrorData('SemanticAnalysisViewFileGenerate', errorMessage, error);
            return Promise.resolve(false);
        });
    }
    CognitiveAnalysisViewFileGenerate() {
        return null;
    }
}
exports.SemanticAnalysisImplementation = SemanticAnalysisImplementation;
