"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.SentimentAnalysisImplementation = void 0;
const log4js = require("log4js");
const mssql_service_1 = require("../db/mssql.service");
const config_1 = require("../config/config");
const redis_service_1 = require("../db/redis.service");
const file_manager_1 = require("../shared/file-manager");
const promises_shared_1 = require("../shared/promises.shared");
const cognitive_analysis_implementation_1 = require("./cognitive-analysis.implementation");
const billing_implementation_1 = require("./billing.implementation");
const diarization_analysis_implementation_1 = require("../implementation/diarization-analysis.implementation");
let logger = log4js.getLogger('Task');
const CURRENT_SYSTEM_SERVICE = 'TRANSCRIPTION';
const CURRENT_IMPLEMENTATION = 'SentimentAnalysisImplementation';
class SentimentAnalysisImplementation {
    Execute(executionCode, rateRecursive) {
        let executeIndex = 1;
        let ratio = config_1.RATIO_SENTIMENT_ANALYSIS;
        let executeResult = true;
        return promises_shared_1.SleepPromise(config_1.SLEEP_SENTIMENT_ANALYSIS)
            .then(result => {
            return redis_service_1.Redis.LRange('SentimentAnalysis', 0, ratio - 1);
        }).then(result => {
            let promiseArray = [];
            ratio = result && result.length <= ratio ? result.length : 0;
            while (ratio >= executeIndex) {
                promiseArray.push(this.SentimentAnalysisExecute(executionCode));
                executeIndex++;
            }
            return Promise.all(promiseArray);
        }).then((results) => {
            if (results.filter(item => item === false).length > 0) {
                executeResult = false;
            }
            return redis_service_1.Redis.LRange('SentimentAnalysis', 0, ratio - 1);
        }).then((result) => {
            if (result && result.length && rateRecursive >= 1)
                return this.Execute(executionCode, rateRecursive - 1);
            return executeResult;
        }).then(result => {
            let currentResult = executeResult && result ? true : false;
            return Promise.resolve(currentResult);
        }).catch((error) => {
            let errorMessage = typeof error === 'string' ? error : (error.message || error.description);
            logger.error('[SentimentAnalysisImplementation::Execute] executionCode:', executionCode, 'getting error results:', errorMessage);
            return Promise.resolve(false);
        });
    }
    SentimentAnalysisExecute(executionCode) {
        let currentBlockPromise = 'BLOCK_BEFORE_PROMISE';
        let storage = '';
        let currentQueueId = '';
        let processInteraction;
        let isCancel = false;
        let vcc = '';
        let processId = '';
        let interactionType = '';
        let interactionId = '';
        let interactionSection = 0;
        let interactionPart = 0;
        let batchTypeName = '';
        let batchTypeId = '';
        return redis_service_1.Redis.RPopLPush('SentimentAnalysis', 'SentimentAnalysisExecute')
            .then((queueId) => {
            currentBlockPromise = 'BLOCK_TO_GET_PROCESS_INTERACTION';
            if (queueId && queueId.length > 0) {
                currentQueueId = queueId;
                logger.info('[SentimentAnalysisImplementation::SentimentAnalysisExecute] executionCode:', executionCode, 'getting queueId:', currentQueueId, 'START');
                let item = queueId.split(':');
                if (item && item.length) {
                    vcc = item[0];
                    processId = item[1];
                    interactionType = item[2];
                    interactionId = item[3];
                    interactionSection = Number(item[4]);
                    interactionPart = Number(item[5]);
                    storage = vcc + ':' + processId;
                    return redis_service_1.Redis.GetProcessInteraction(storage, currentQueueId);
                }
                else {
                    logger.error('[SentimentAnalysisImplementation::SentimentAnalysisExecute] executionCode:', executionCode, 'getting queueId:', currentQueueId, 'QUEUE_FORMAT_ERROR');
                }
            }
            return null;
        }).then((processInteractionResult) => {
            logger.debug('[SentimentAnalysisImplementation::SentimentAnalysisExecute] executionCode:', executionCode, 'getting queueId:', currentQueueId, 'Interaction: ', processInteractionResult);
            currentBlockPromise = 'BLOCK_TO_LAUNCH_MAIN_FUNCTION';
            if (processInteractionResult) {
                processInteraction = processInteractionResult;
                if (processInteraction.statusDescription === 'pendingSentimentAnalysis') {
                    return this.EvaluateSentiments(processInteraction, currentQueueId, executionCode);
                }
                return null;
            }
            else {
                if (currentQueueId)
                    logger.warn('[SentimentAnalysisImplementation::SentimentAnalysisExecute] executionCode:', executionCode, 'getting queueId:', currentQueueId, 'getting GetProcessInteraction: 0');
                return null;
            }
        }).then((result) => {
            currentBlockPromise = 'BLOCK_TO_UPDATE_INTERACTION';
            if (processInteraction)
                return redis_service_1.Redis.SaveProcessInteraction(storage, currentQueueId, processInteraction);
            return null;
        }).then((result) => {
            currentBlockPromise = 'BLOCK_TO_LAUNCH_FINISHING_FUNCTION';
            let promiseArray = [];
            let billing = new billing_implementation_1.BillingImplementation();
            if (result
                && processInteraction
                && processInteraction.status === 5
                && processInteraction.statusDescription === 'pendingAnalysis') {
                logger.info('[SentimentAnalysisImplementation::SentimentAnalysisExecute] executionCode:', executionCode, 'getting queueId:', currentQueueId, 'NEXT_QUEUE');
                promiseArray.push(redis_service_1.Redis.CreateQueueL('SemanticAnalysisUpload', currentQueueId));
                promiseArray.push(redis_service_1.Redis.DeleteQueue('SentimentAnalysisExecute', currentQueueId));
            }
            else if (result
                && processInteraction
                && (processInteraction.status === 4
                    ||
                        (processInteraction.status === 5 && processInteraction.statusDescription !== 'pendingSentimentAnalysis'))
                && processInteraction.retry < config_1.RETRY_SENTIMENT_ANALYSIS) {
                promiseArray.push(redis_service_1.Redis.InsertQueue('SentimentAnalysis', currentQueueId, config_1.RATE_RECURSIVE_SENTIMENT_ANALYSIS * config_1.RATIO_SENTIMENT_ANALYSIS * 2));
                promiseArray.push(redis_service_1.Redis.DeleteQueue('SentimentAnalysisExecute', currentQueueId));
            }
            else if (result
                && processInteraction
                && processInteraction.status === -5
                && processInteraction.statusDescription === 'REMOVED_BY_CANCELLED') {
                logger.error('[SentimentAnalysisImplementation::SentimentAnalysisExecute] executionCode:', executionCode, 'getting queueId:', currentQueueId, 'REMOVED_BY_CANCELLED');
                promiseArray.push(redis_service_1.Redis.DeleteQueue('SentimentAnalysisExecute', currentQueueId));
            }
            else if (processInteraction && processInteraction.retry >= config_1.RETRY_SENTIMENT_ANALYSIS) {
                logger.error('[SentimentAnalysisImplementation::SentimentAnalysisExecute] executionCode:', executionCode, 'getting queueId:', currentQueueId, 'REMOVED_BY_RETRIES');
            }
            else if (currentQueueId && !processInteraction) {
                logger.error('[SentimentAnalysisImplementation::SentimentAnalysisExecute] executionCode:', executionCode, 'getting queueId:', currentQueueId, 'INTERACTION_DO_NOT_EXIST');
                let interactionData = {
                    vcc: vcc,
                    processId: processId,
                    id: interactionId,
                    section: interactionSection,
                    part: interactionPart
                };
                billing.AddCloseTranscriptionDetailToMainFile(interactionData, batchTypeName, batchTypeId, 0, 'INTERACTION_DO_NOT_EXIST');
                promiseArray.push(redis_service_1.Redis.DeleteQueue('SentimentAnalysisExecute', currentQueueId));
            }
            else {
                logger.error('[SentimentAnalysisImplementation::SentimentAnalysisExecute] executionCode:', executionCode, 'getting queueId:', currentQueueId, 'QUEUE_WAS_NOT_REMOVED');
            }
            return Promise.resolve(true);
        }).then((result) => {
            currentBlockPromise = 'BLOCK_TO_UPDATE_REDIS';
            let promiseArray = [];
            if (processInteraction && processInteraction.retry >= config_1.RETRY_SENTIMENT_ANALYSIS) {
                promiseArray.push(mssql_service_1.MSSql.SaveBatchError(vcc, CURRENT_SYSTEM_SERVICE, processInteraction.batchTypeId, processInteraction.batchTypeId, CURRENT_IMPLEMENTATION, processInteraction.id, null, processInteraction.errorFunction, processInteraction.errorMessage, processInteraction.errorDetail));
            }
            return Promise.all(promiseArray);
        }).then((result) => {
            currentBlockPromise = 'BLOCK_TO_FINISH_EXECUTION';
            return Promise.resolve(true);
        }).catch((err) => {
            let errorMessage = typeof err === 'string' ? err : (err.message || err.description);
            logger.error('[SentimentAnalysisImplementation::SentimentAnalysisExecute] executionCode:', executionCode, 'getting queueId:', currentQueueId, ', Block:', currentBlockPromise, ', getting error:', errorMessage);
            return Promise.resolve(false);
        });
    }
    EvaluateSentiments(interaction, currentQueueId, executionCode) {
        let path = '';
        let jsonContent = null;
        let cognitiveService;
        let cognitiveAnalysis = new cognitive_analysis_implementation_1.CognitiveAnalysisImplementation();
        let phrases = null;
        let agentResults = [];
        let clientResults = [];
        let agentEvaluationResult = [];
        let clientEvaluationResult = [];
        let averageResults = [];
        let averageResultsDB = [];
        let sentimentAppId = '';
        let intentsDB;
        return promises_shared_1.DummyPromise()
            .then((result) => {
            let regex = /\.(mp3|wav|ogg)$/;
            let jsonFileName = interaction.fileName.replace(regex, '.json');
            path = config_1.TRANSCRIPTION_FOLDER;
            path += '/';
            path += interaction.vcc;
            path += '/';
            path += jsonFileName;
            return file_manager_1.ReadFileContent(path, 'utf-8');
        }).then(result => {
            try {
                jsonContent = typeof result === 'string' ? JSON.parse(result) : result;
                let diarizationAnalysis = new diarization_analysis_implementation_1.DiarizationAnalysisImplementation();
                phrases = diarizationAnalysis.GeneratePhrases(jsonContent.results, config_1.COGNITIVE_SERVICES_RASA_CHARS_PER_PHRASE);
                logger.debug('[SentimentAnalysisImplementation::EvaluateSentiments] executionCode:', executionCode, 'getting queueId:', currentQueueId, 'getting phrases:', phrases);
            }
            catch (ex) {
                logger.error('[SentimentAnalysisImplementation::EvaluateSentiments] executionCode:', executionCode, 'getting queueId:', currentQueueId, 'getting errorMessage:', ex);
                jsonContent = '';
                throw 'SERVER_ERROR_GET_JSON_FAILED';
            }
            return Promise.resolve(phrases);
        }).then(result => {
            if (!result)
                throw 'SERVER_ERROR_PHRASES_NOT_FOUND';
            return mssql_service_1.MSSql.GetNLU(interaction.vcc, 'sentiment-' + interaction.language);
        }).then(result => {
            if (!result)
                throw 'SERVER_ERROR_GETNLU_FAILED';
            sentimentAppId = result.appId;
            return cognitive_analysis_implementation_1.GetCognitiveService(null, 'RASA');
        }).then((result) => {
            if (!result)
                throw 'SERVER_ERROR_CANNOT_GET_COGNITIVE_SERVICE';
            cognitiveService = result;
            let promiseArray = [];
            phrases.agent.map((ap, i) => {
                let appId = sentimentAppId;
                promiseArray.push(new Promise(function (resolve, reject) {
                    setTimeout(() => {
                        return resolve(cognitiveAnalysis.ModelQuery(cognitiveService, null, null, appId, ap, appId, executionCode, currentQueueId));
                    }, i * 1000);
                }));
            });
            return Promise.all(promiseArray);
        }).then((result) => {
            if (!result)
                throw 'SERVER_ERROR_NLU_AGENT_EVALUATION_FAILED';
            logger.debug('[SentimentAnalysisImplementation::EvaluateSentiments] executionCode:', executionCode, 'getting queueId:', currentQueueId, 'getting agent evaluation result:', result);
            agentEvaluationResult = result;
            let promiseArray = [];
            phrases.client.map((ap, i) => {
                let appId = sentimentAppId;
                promiseArray.push(new Promise(function (resolve, reject) {
                    setTimeout(() => {
                        return resolve(cognitiveAnalysis.ModelQuery(cognitiveService, null, null, appId, ap, appId, executionCode, currentQueueId));
                    }, i * 1000);
                }));
            });
            return Promise.all(promiseArray);
        }).then((result) => {
            if (!result)
                throw 'SERVER_ERROR_NLU_CLIENT_EVALUATION_FAILED';
            logger.debug('[SentimentAnalysisImplementation::EvaluateSentiments] executionCode:', executionCode, 'getting queueId:', currentQueueId, 'getting client evaluation result:', result);
            clientEvaluationResult = result;
            agentEvaluationResult.map((aer, i) => {
                agentResults.push({
                    phrase: phrases.agent[i],
                    intentScores: aer.intents
                });
            });
            clientEvaluationResult.map((cer, i) => {
                clientResults.push({
                    phrase: phrases.client[i],
                    intentScores: cer.intents
                });
            });
            return mssql_service_1.MSSql.GetIntents(interaction.vcc, sentimentAppId, null, null, null);
        }).then(result => {
            intentsDB = result;
            let agentPhrasesNumber = agentResults.length;
            let clientPhrasesNumber = clientResults.length;
            let generalPhrasesNumber = agentResults.length + clientResults.length;
            let generalResults = agentResults.concat(clientResults);
            let agentAverageResults = [];
            let clientAverageResults = [];
            let generalAverageResults = [];
            intentsDB.forEach(intentDB => {
                let sumAgentResults = 0;
                let sumClientResults = 0;
                let sumGeneralResults = 0;
                let saveScore = 0;
                agentResults.forEach(agentr => {
                    let found = agentr.intentScores ? agentr.intentScores.find(intentScore => intentScore.intent === intentDB.intentId) : null;
                    if (found)
                        sumAgentResults = sumAgentResults + found.score;
                });
                agentAverageResults.push({
                    intentResult: intentDB.intentId,
                    averageResult: agentPhrasesNumber > 0 ? (sumAgentResults / agentPhrasesNumber).toFixed(5) : 0
                });
                clientResults.forEach(clientr => {
                    let found = clientr.intentScores ? clientr.intentScores.find(intentScore => intentScore.intent === intentDB.intentId) : null;
                    if (found)
                        sumClientResults = sumClientResults + found.score;
                });
                clientAverageResults.push({
                    intentResult: intentDB.intentId,
                    averageResult: clientPhrasesNumber > 0 ? (sumClientResults / clientPhrasesNumber).toFixed(5) : 0
                });
                generalResults.forEach(allr => {
                    let found = allr.intentScores ? allr.intentScores.find(intentScore => intentScore.intent === intentDB.intentId) : null;
                    if (found)
                        sumGeneralResults = sumGeneralResults + found.score;
                });
                generalAverageResults.push({
                    intentResult: intentDB.intentId,
                    averageResult: generalPhrasesNumber > 0 ? (sumGeneralResults / generalPhrasesNumber).toFixed(5) : 0
                });
            });
            averageResults.push({ analysisType: 'agent', averageResults: this.GetMax(agentAverageResults) });
            averageResults.push({ analysisType: 'client', averageResults: this.GetMax(clientAverageResults) });
            averageResults.push({ analysisType: 'general', averageResults: this.GetMax(generalAverageResults) });
            let totalScore = 0;
            averageResults.forEach(ar => {
                ar.averageResults.forEach(are => {
                    let saveScore = parseFloat(are.averageResult);
                    if (saveScore > 0) {
                        totalScore = totalScore + saveScore;
                        mssql_service_1.MSSql.SaveSentimentAnalysisResult(interaction.vcc, interaction.id, interaction.section ? interaction.section : 0, interaction.part ? interaction.part : 0, interaction.interactionType, 0, ar.analysisType, are.intentId, sentimentAppId, saveScore, null, 'admin', null, 'admin', interaction.endDate ? interaction.endDate.toString() : null, are.status);
                    }
                });
            });
            if (totalScore > 0)
                return true;
            return false;
        }).then((result) => {
            if (!result) {
                if (interaction.retry <= config_1.RETRY_SENTIMENT_ANALYSIS) {
                    return Promise.resolve('ERROR');
                }
                else {
                    return Promise.resolve('OK');
                }
            }
            return Promise.resolve('OK');
        })
            .then((result) => {
            if (result === 'OK') {
                interaction['status'] = 5;
                interaction['statusDescription'] = 'pendingAnalysis';
                interaction['retry'] = 0;
                interaction['errorFunction'] = null;
                interaction['errorMessage'] = null;
                interaction['errorDetail'] = null;
            }
            else {
                logger.error('[SentimentAnalysisImplementation::EvaluateSentiments] executionCode:', executionCode, 'getting queueId:', currentQueueId, 'getting errorMessage:', result);
                interaction['status'] = -5;
                interaction['retry'] = interaction['retry'] ? interaction['retry'] + 1 : 1;
                interaction['errorFunction'] = 'EvaluateSentiments';
                interaction['errorMessage'] = 'SENTIMENT_MODEL_WAS_NOT_TRAINED_CORRECTLY';
                interaction['errorDetail'] = 'SENTIMENT_MODEL_WAS_NOT_TRAINED_CORRECTLY';
            }
            return Promise.resolve(interaction);
        }).catch((err) => {
            let errorMessage = typeof err === 'string' ? err : (err.message || err.description);
            logger.error('[SentimentAnalysisImplementation::EvaluateSentiments] executionCode:', executionCode, 'getting queueId:', currentQueueId, 'getting errorMessage:', errorMessage);
            interaction['status'] = -5;
            interaction['retry'] = interaction['retry'] ? interaction['retry'] + 1 : 1;
            interaction['errorFunction'] = 'EvaluateSentiments';
            interaction['errorMessage'] = errorMessage;
            interaction['errorDetail'] = JSON.stringify(err);
            return Promise.resolve(interaction);
        });
    }
    SaveSentimentResults(interaction, jsonContent, currentQueueId, executionCode) {
        let sentimentFolder = config_1.SENTIMENT_FOLDER + '/' + interaction.vcc;
        let regex = /\.(mp3|wav|ogg|json)$/;
        let jsonFileName = interaction.fileName.replace(regex, '.json');
        let jsonFilePath = null;
        jsonFilePath = sentimentFolder;
        jsonFilePath += '/';
        jsonFilePath += jsonFileName;
        return promises_shared_1.DummyPromise()
            .then(result => {
            return file_manager_1.CreateFolder(sentimentFolder);
        })
            .then(result => {
            logger.info('[SentimentAnalysisImplementation::SaveSentimentResults] executionCode:', executionCode, 'getting queueId:', currentQueueId, 'saving results on:', jsonFilePath);
            return file_manager_1.WriteFileContent(jsonFilePath, JSON.stringify(jsonContent), 'utf-8');
        })
            .then((result) => {
            return Promise.resolve(true);
        })
            .catch((error) => {
            let errorMessage = typeof error === 'string' ? error : 'ERROR';
            logger.error('[SentimentAnalysisImplementation::SaveSentimentResults] executionCode:', executionCode, 'getting queueId:', currentQueueId, 'getting error:', errorMessage);
            return Promise.reject(errorMessage);
        });
    }
    GetSentimentResultsFromJsonFile(jsonFileName, vcc) {
        let path = '';
        let jsonContent = null;
        return promises_shared_1.DummyPromise()
            .then((result) => {
            path = config_1.SENTIMENT_FOLDER;
            path += '/';
            path += vcc;
            path += '/';
            path += jsonFileName;
            return (file_manager_1.ExistInFileSystem(path));
        }).then(result => {
            if (!result)
                return null;
            return file_manager_1.ReadFileContent(path, 'utf-8', true);
        })
            .then(result => {
            jsonContent = typeof result === 'string' && result !== null ? JSON.parse(result) : result;
            if (jsonContent !== null)
                return Promise.resolve(jsonContent.AverageResults);
            return Promise.resolve(null);
        })
            .catch((error) => {
            let errorMessage = typeof error === 'string' ? error : 'ERROR';
            logger.error('[SentimentAnalysisImplementation::GetSentimentResultsFromJsonFile] executionCode:', 'getting error:', errorMessage);
            return Promise.reject(errorMessage);
        });
    }
    GetSentimentResults(jsonFileName, vcc, interactionType) {
        jsonFileName = jsonFileName.replace('.json', '');
        let interactionId = '';
        let section = 0;
        let part = 0;
        let endDate = '';
        if (interactionType === 'CALL') {
            let lastFour = jsonFileName.substr(-4);
            section = Number(lastFour.split('_')[1]);
            part = Number(lastFour.split('_')[2]);
            interactionId = jsonFileName.replace(lastFour, '');
        }
        else {
            interactionId = jsonFileName.split('_')[3];
            endDate = jsonFileName.split('_')[2];
        }
        return promises_shared_1.DummyPromise()
            .then((result) => {
            return mssql_service_1.MSSql.GetSentimentAnalysisResult(vcc, interactionId, section, part, endDate);
        })
            .then(result => {
            if (result !== null)
                return Promise.resolve(result);
            return Promise.resolve(null);
        })
            .catch((error) => {
            let errorMessage = typeof error === 'string' ? error : 'ERROR';
            logger.error('[SentimentAnalysisImplementation::GetSentimentResults] executionCode:', 'getting error:', errorMessage);
            return Promise.reject(errorMessage);
        });
    }
    GetMax(resultsScores) {
        let maxValues = [];
        let highest = 0;
        let intentIdMax = '';
        let tmp = 0;
        let newResultScores = [];
        resultsScores.map(element => {
            tmp = Number(element.averageResult);
            if (tmp > highest) {
                highest = tmp;
                intentIdMax = element.intentResult;
            }
        });
        maxValues.push({
            intentIdMax: intentIdMax,
            averageMax: highest
        });
        resultsScores.map(score => {
            let found = maxValues.find(maxval => maxval.intentIdMax === score.intentResult);
            if (found) {
                newResultScores.push({
                    intentId: score.intentResult,
                    averageResult: score.averageResult,
                    status: "max"
                });
            }
            else {
                newResultScores.push({
                    intentId: score.intentResult,
                    averageResult: score.averageResult,
                    status: "normal"
                });
            }
        });
        return newResultScores;
    }
}
exports.SentimentAnalysisImplementation = SentimentAnalysisImplementation;
