"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.SpeechmaticsServiceImplementation = void 0;
const moment = require("moment");
const log4js = require("log4js");
const redis_service_1 = require("../db/redis.service");
var FormData = require('form-data');
const promises_shared_1 = require("../shared/promises.shared");
const task_config_1 = require("../config/task.config");
const server_config_1 = require("../config/server.config");
const request_shared_1 = require("../shared/request.shared");
const config_1 = require("../config/config");
const transcriptions_upload_implementation_1 = require("./transcriptions-upload.implementation");
let logger = log4js.getLogger('Task');
const CURRENT_SYSTEM_SERVICE = 'TRANSCRIPTION_SERVICE';
const CURRENT_IMPLEMENTATION = 'SpeechmaticsServiceImplementation';
class SpeechmaticsServiceImplementation {
    constructor() {
        this.executionCode = null;
        this.currentQueueId = null;
        this.availableServers = task_config_1.SPEECHMATICS_SERVERS;
        this.externalUserId = 'ext_user';
        this.externalUserVcc = 'system';
    }
    GetEngineFreeStorage(executionCode) {
        let logLabel = `[${CURRENT_IMPLEMENTATION}::GetEngineFreeStorage] `;
        return promises_shared_1.DummyPromise()
            .then(_ => {
            let promiseArray = [];
            this.availableServers.forEach(sa => {
                if (sa.ip) {
                    let getOptions = {
                        uri: `${sa.protocol}://${sa.ip}:8080/v1/management/freestorage`,
                        timeout: 30000
                    };
                    promiseArray.push(request_shared_1.SimpleGetRequest(getOptions));
                }
                else
                    promiseArray.push(Promise.resolve([true]));
            });
            return Promise.all(promiseArray);
        })
            .then(result => {
            result.forEach((r, i) => {
                if (typeof r !== 'boolean'
                    &&
                        r?.items?.length) {
                    let minimunStorage = (2 * 1024);
                    let freeStorage = r.items[0].bytes / (1024 * 1024);
                    this.availableServers[i].available = freeStorage > minimunStorage;
                    logger.info(`${logLabel}executionCode: ${executionCode} free storage for ${this.availableServers[i].ip} => ${freeStorage}mb => ${this.availableServers[i].available ? 'OK' : 'KO'}`);
                }
            });
            this.availableServers = this.availableServers.filter(as => as.available);
            return Promise.resolve(this.availableServers.length ? true : false);
        })
            .catch((err) => {
            let errorMessage = typeof err === 'string' ? err : (err.message || err.description);
            logger.error(`${logLabel}executionCode:`, executionCode, 'getting error:', errorMessage);
            return Promise.resolve(false);
        });
    }
    UploadAudio(interaction, stream, dictionaryContent, currentQueueId) {
        return promises_shared_1.DummyPromise()
            .then(_ => {
            let trancriptionConfig = {
                type: 'transcription',
                transcription_config: {
                    diarization: 'channel',
                    channel_diarization_labels: ['Agent', 'Caller'],
                    language: interaction.language,
                    additional_vocab: dictionaryContent
                }
            };
            if (task_config_1.SPEECHMATICS_CALLBACK_USAGE) {
                let transcriptionsUpload = new transcriptions_upload_implementation_1.TranscriptionsUploadImplementation();
                trancriptionConfig.notification_config = [{
                        url: `${server_config_1.WEBHOOK_URL}/speechmatics/webhook/`,
                        contents: ['transcript'],
                        auth_headers: [
                            `Authorization: Bearer ${transcriptionsUpload.GetSystemUserToken()}`,
                            `speech: ${server_config_1.EXTERNAL_URL}/api/public/task/update_transcription_queue/${this.currentQueueId}/`
                        ]
                    }];
            }
            const formData = new FormData();
            formData.append('data_file', stream, interaction.fileName);
            formData.append('config', JSON.stringify(trancriptionConfig));
            let data = {
                uri: `${interaction.transcriptionProtocol}://${interaction.transcriptionApi}/v2/jobs/`,
                fileName: interaction.fileName,
                validateErrorResponse: true,
                timeout: 60000
            };
            if (interaction.transcriptionToken)
                data['headers'] = {
                    Authorization: `Bearer ${interaction.transcriptionToken}`
                };
            return request_shared_1.AttachedFilePostRequest(data, formData);
        })
            .then((jobStatus) => {
            let jobId = jobStatus?.id ? jobStatus.id : null;
            if (jobId) {
                logger.info('[TranscriptionsUploadImplementation::UploadAudio] executionCode:', this.executionCode, 'getting queueId:', currentQueueId, 'UPLOADED');
                interaction['status'] = 3;
                interaction['statusDescription'] = 'pendingTranscription';
                interaction['retry'] = 0;
                interaction['jobId'] = jobId;
                interaction['uploadedDate'] = moment().format('x');
                interaction['errorFunction'] = null;
                interaction['errorMessage'] = null;
                interaction['errorDetail'] = null;
            }
            else {
                logger.error('[TranscriptionsUploadImplementation::UploadAudio] executionCode:', this.executionCode, 'getting queueId:', currentQueueId, 'JOBID_IS_NOT_FOUND');
                interaction['status'] = -3;
                interaction['retry'] = interaction['retry'] ? interaction['retry'] + 1 : 1;
                interaction['errorFunction'] = 'UploadAudio';
                interaction['errorMessage'] = 'JOBID_IS_NOT_FOUND';
                interaction['errorDetail'] = 'JOBID_IS_NOT_FOUND';
            }
            return Promise.resolve(interaction);
        })
            .catch((err) => {
            let errorMessage = typeof err === 'string' ? err : (err.message || err.description);
            logger.error('[TranscriptionsUploadImplementation::UploadAudio] executionCode:', this.executionCode, 'getting queueId:', currentQueueId, 'getting error:', errorMessage);
            if (typeof errorMessage === 'string'
                &&
                    errorMessage.includes('data_file is too small')) {
                interaction['isCancel'] = true;
                interaction['retry'] = config_1.RETRY_TRANSCRIPTION;
            }
            else
                interaction['retry'] = interaction['retry'] ? interaction['retry'] + 1 : 1;
            interaction['status'] = -3;
            interaction['errorFunction'] = 'UploadAudio';
            interaction['errorMessage'] = 'UPLOAD_ERROR';
            interaction['errorDetail'] = err.stack ? err.stack : errorMessage;
            return Promise.resolve(interaction);
        });
    }
    DownloadTranscription(interaction, currentQueueId) {
        let logLabel = `[${CURRENT_IMPLEMENTATION}::DownloadAudio] `;
        let options = {
            uri: `${interaction.transcriptionProtocol}://${interaction.transcriptionApi}/v2/jobs/${interaction.jobId}/transcript?format=json-v2`,
            retryProtocol: false,
            validateErrorResponse: true,
            timeout: 30000
        };
        if (interaction.transcriptionToken)
            options['headers'] = {
                Authorization: `Bearer ${interaction.transcriptionToken}`
            };
        let data;
        return request_shared_1.SimpleGetRequest(options)
            .then((result) => {
            data = result;
            return Promise.resolve(result);
        })
            .catch((err) => {
            let errorMessage = typeof err === 'string' ? err : (err.message || err.description);
            let pending = {};
            if (typeof errorMessage === 'string'
                &&
                    (errorMessage.indexOf('Job In Progress') > -1
                        ||
                            errorMessage.indexOf('Transcription not ready') > -1)) {
                logger.warn(`${logLabel}executionCode:`, this.executionCode, 'getting queueId:', currentQueueId, 'getting error:', errorMessage);
                pending['statusDescription'] = 'jobInProgress';
            }
            else if (typeof errorMessage === 'string'
                &&
                    (errorMessage.indexOf('Job not found') > -1
                        ||
                            errorMessage.includes('Job rejected'))) {
                logger.warn(`${logLabel}executionCode:`, this.executionCode, 'getting queueId:', currentQueueId, 'getting error:', errorMessage);
                pending['statusDescription'] = 'jobNotFound';
            }
            else {
                logger.error(`${logLabel}executionCode:`, this.executionCode, 'getting queueId:', currentQueueId, 'getting error:', errorMessage);
            }
            data = pending?.statusDescription ? pending : null;
            return Promise.resolve(data);
        })
            .catch((err) => {
            let errorMessage = typeof err === 'string' ? err : (err.message || err.description);
            logger.error(`${logLabel}executionCode:`, this.executionCode, 'getting queueId:', currentQueueId, 'getting error:', errorMessage);
            return Promise.resolve(null);
        });
    }
    StandardizeResponse(rawResponse) {
        return rawResponse;
    }
    GetTranscriptionApi(index) {
        let apiIndex = index % this.availableServers.length;
        let server = this.availableServers[apiIndex];
        return server;
    }
    ReinsertQueue(interaction, currentQueueId) {
        return redis_service_1.Redis.DeleteInsertOnList('TranscriptionsStandbyExecute', 'TranscriptionsDownload', currentQueueId);
    }
}
exports.SpeechmaticsServiceImplementation = SpeechmaticsServiceImplementation;
