"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.TipificationAnalysisImplementation = void 0;
const log4js = require("log4js");
const mssql_service_1 = require("../db/mssql.service");
const redis_service_1 = require("../db/redis.service");
const moment = require("moment");
const config_1 = require("../config/config");
const promises_shared_1 = require("../shared/promises.shared");
const file_manager_1 = require("../shared/file-manager");
const cognitive_analysis_implementation_1 = require("./cognitive-analysis.implementation");
const billing_batch_model_1 = require("../models/billing-batch.model");
const evaluation_result_model_1 = require("../models/evaluation-result.model");
let logger = log4js.getLogger('Task');
const CURRENT_SYSTEM_SERVICE = 'TIPIFICATION_ANALYSIS';
const CURRENT_IMPLEMENTATION = 'TipificationAnalysisImplementation';
class TipificationAnalysisImplementation {
    constructor() {
        this.errorData = new billing_batch_model_1.BatchError();
        this.cognitiveAnalysis = new cognitive_analysis_implementation_1.CognitiveAnalysisImplementation();
    }
    Execute(executionCode, rateRecursive, availableBatch, cancelledBatch) {
        let executeIndex = 1;
        let ratio = config_1.RATIO_TIPIFICATION_ANALYSIS;
        let alowedRatio = 0;
        let notAlowedRatio = 0;
        let executeResult = true;
        return promises_shared_1.SleepPromise(2)
            .then(_ => {
            return redis_service_1.Redis.LRange('TipificationAnalysis', 0, ratio - 1);
        })
            .then((result) => {
            if (!result)
                result = [];
            let promiseArray = [];
            ratio = result && result.length <= ratio ? result.length : 0;
            logger.info('[TipificationAnalysisImplementation::Execute] TipificationAnalysis lrange result: ' + result);
            result.forEach(r => {
                let arrQueue = r.split(':');
                if (availableBatch.find(b => b.VCC === arrQueue[0] && b.BatchTypeId.toLowerCase() === arrQueue[7].toLowerCase())) {
                    alowedRatio++;
                }
                else {
                    notAlowedRatio++;
                }
            });
            while (ratio >= executeIndex) {
                promiseArray.push(this.TipificationAnalysisExecute(executionCode, availableBatch, cancelledBatch));
                executeIndex++;
            }
            return Promise.all(promiseArray);
        })
            .then((results) => {
            if (results.filter(item => item === false).length > notAlowedRatio)
                executeResult = false;
            return redis_service_1.Redis.LRange('TipificationAnalysis', 0, ratio - 1);
        })
            .then((result) => {
            if (result && result.length && rateRecursive >= 1)
                return this.Execute(executionCode, rateRecursive - 1, availableBatch, cancelledBatch);
            return executeResult;
        })
            .then(result => {
            let currentResult = executeResult && result ? true : false;
            return Promise.resolve(currentResult);
        })
            .catch((err) => {
            let errorMessage = typeof err === 'string' ? err : (err.message || err.description);
            logger.error('[TipificationAnalysisImplementation::Execute] getting error:', errorMessage);
            return Promise.resolve(false);
        });
    }
    TipificationAnalysisExecute(executionCode, availableBatch, cancelledBatch) {
        let vcc = '';
        let processId = '';
        let interactionType = '';
        let type = '';
        let typeId = '';
        let currentQueueId = '';
        let interactionCognitive;
        let queueProgressStorage = '';
        let filePath = '';
        return redis_service_1.Redis.RemoveQueueL('TipificationAnalysis')
            .then((result) => {
            let arrQueue = result.split(':');
            let promiseArray = [];
            if (availableBatch.find(b => b.VCC === arrQueue[0] && b.BatchTypeId.toLowerCase() === arrQueue[8].toLowerCase())) {
                promiseArray.push(Promise.resolve('ALLOWED'));
                promiseArray.push(redis_service_1.Redis.CreateQueueL('TipificationAnalysisExecute', result));
                promiseArray.push(Promise.resolve(result));
            }
            else {
                promiseArray.push(Promise.resolve('NOT_ALLOWED'));
                promiseArray.push(redis_service_1.Redis.CreateQueueL('TipificationAnalysis', result));
                promiseArray.push(mssql_service_1.MSSql.SaveBatchStatus(arrQueue[0], arrQueue[8], 'COGNITIVE_ANALYSIS', 'PAUSED'));
            }
            return Promise.all(promiseArray);
        })
            .then((result) => {
            if (result && result[0] === 'NOT_ALLOWED')
                throw 'SERVER_ERROR_NOT_ALLOWED_QUEUE';
            let queueId = result[2];
            if (queueId?.length > 0) {
                currentQueueId = queueId;
                logger.info('[TipificationAnalysisImplementation::TipificationQueryExecute] executionCode:', executionCode, 'getting queueId:', currentQueueId, 'START');
                let item = queueId.split(':');
                if (item?.length) {
                    vcc = item[0];
                    processId = item[1];
                    interactionType = item[2];
                    type = item[7] ? item[7] : null;
                    typeId = item[8] ? item[8] : null;
                    if (type) {
                        queueProgressStorage = `${vcc}:${processId}:${type}`;
                        queueProgressStorage += typeId ? `:${typeId}` : ``;
                    }
                    logger.info('[TipificationAnalysisImplementation::TipificationAnalysisExecute] executionCode:', executionCode, 'getting queueId:', currentQueueId, 'storage :', queueProgressStorage);
                    return redis_service_1.Redis.GetTipificationInteraction(currentQueueId);
                }
                else {
                    logger.error('[TipificationAnalysisImplementation::TipificationAnalysisExecute] executionCode:', executionCode, 'getting queueId:', currentQueueId, 'QUEUE_FORMAT_ERROR');
                }
            }
            return Promise.resolve(null);
        })
            .then((result) => {
            if (!result)
                logger.warn('[TipificationAnalysisImplementation::TipificationAnalysisExecute] executionCode:', executionCode, 'getting queueId:', currentQueueId, 'getting error: NOT_FOUND_INTERACTION_COGNITIVE');
            if (result) {
                if (cancelledBatch.find(b => b.VCC === result.vcc && b.BatchTypeId.toLowerCase() === (result.studyId ? result.studyId : result.evaluationPlanId).toLowerCase())) {
                    result.status = -7;
                    result.statusDescription = 'REMOVED_BY_CANCELLED';
                    result.retry = config_1.RETRY_TIPIFICATION_ANALYSIS;
                    result.errorFunction = 'TipificationAnalysisExecute';
                    result.errorMessage = 'REMOVED_BY_CANCELLED';
                    result.errorDetail = 'REMOVED_BY_CANCELLED';
                }
                interactionCognitive = result;
            }
            if (interactionCognitive?.statusDescription === 'pendingAnalysis') {
                logger.info('[TipificationAnalysisImplementation::TipificationAnalysisExecute] executionCode:', executionCode, 'getting queueId:', currentQueueId, 'pendingAnalysis');
                return this.EvaluateTipification(interactionCognitive, currentQueueId, executionCode);
            }
            return Promise.resolve(null);
        })
            .then((result) => {
            interactionCognitive = result;
            return redis_service_1.Redis.SaveModelInteraction(currentQueueId, interactionCognitive);
        })
            .then((result) => {
            let promiseArray = [];
            if (result && interactionCognitive?.status === 7 && interactionCognitive?.statusDescription === 'ready') {
                logger.info('[TipificationAnalysisImplementation::TipificationAnalysisExecute] executionCode:', executionCode, 'getting queueId:', currentQueueId, 'READY');
                promiseArray.push(redis_service_1.Redis.DeleteQueue('TipificationAnalysisExecute', currentQueueId));
            }
            else if (result && interactionCognitive?.status === 7 && interactionCognitive?.statusDescription === 'pendingNLU' && interactionCognitive?.retry < config_1.RETRY_TIPIFICATION_ANALYSIS) {
                promiseArray.push(redis_service_1.Redis.CreateQueueL('TipificationAnalysis', currentQueueId));
                promiseArray.push(redis_service_1.Redis.DeleteQueue('TipificationAnalysisExecute', currentQueueId));
            }
            else if (result && interactionCognitive?.status === -7 && interactionCognitive?.statusDescription === 'REMOVED_BY_CANCELLED') {
                logger.error('[TipificationAnalysisImplementation::TipificationAnalysisExecute] executionCode:', executionCode, 'getting queueId:', currentQueueId, 'REMOVED_BY_CANCELLED');
                promiseArray.push(redis_service_1.Redis.DeleteQueue('TipificationAnalysisExecute', currentQueueId));
            }
            else if (result && interactionCognitive?.status === -7 && interactionCognitive?.statusDescription === 'ERROR_FILE_JSON') {
                promiseArray.push(redis_service_1.Redis.DeleteQueue('TipificationAnalysisExecute', currentQueueId));
            }
            else if (interactionCognitive?.retry >= config_1.RETRY_TIPIFICATION_ANALYSIS) {
                logger.error('[TipificationAnalysisImplementation::TipificationAnalysisExecute] executionCode:', executionCode, 'getting queueId:', currentQueueId, 'REMOVED_BY_RETRIES');
                promiseArray.push(redis_service_1.Redis.DeleteQueue('TipificationAnalysisExecute', currentQueueId));
            }
            else if (currentQueueId && !interactionCognitive) {
                logger.error('[TipificationAnalysisImplementation::TipificationAnalysisExecute] executionCode:', executionCode, 'getting queueId:', currentQueueId, 'INTERACTION_DO_NOT_EXIST');
                promiseArray.push(redis_service_1.Redis.DeleteQueue('TipificationAnalysisExecute', currentQueueId));
            }
            else {
                logger.error('[TipificationAnalysisImplementation::TipificationAnalysisExecute] executionCode:', executionCode, 'getting queueId:', currentQueueId, 'QUEUE_WAS_NOT_REMOVED');
            }
            return Promise.all(promiseArray);
        })
            .then((result) => {
            let promiseArray = [];
            if (interactionCognitive?.status === 7
                &&
                    interactionCognitive?.statusDescription === 'ready') {
                promiseArray.push(redis_service_1.Redis.UpdateProgress(queueProgressStorage));
                promiseArray.push(redis_service_1.Redis.DeleteTipificationInteraction(currentQueueId));
                if (type === 'tipification') {
                    logger.debug('[TipificationAnalysisImplementation::TipificationAnalysisExecute] executionCode:', executionCode, 'getting queueId:', currentQueueId, 'Saving cognitive result for evaluationplan:', interactionCognitive);
                    promiseArray.push(redis_service_1.Redis.SaveReadyTipificationInteraction(currentQueueId, interactionCognitive, executionCode));
                }
                else {
                    logger.error('[TipificationAnalysisImplementation::TipificationAnalysisExecute] executionCode:', executionCode, 'getting queueId:', currentQueueId, 'TYPE_IS_NOT_FOUND', type);
                }
            }
            else if (interactionCognitive?.retry >= config_1.RETRY_TIPIFICATION_ANALYSIS) {
                promiseArray.push(redis_service_1.Redis.UpdateProgress(queueProgressStorage));
                promiseArray.push(redis_service_1.Redis.UpdateErrorProgress(queueProgressStorage));
                promiseArray.push(redis_service_1.Redis.DeleteTipificationInteraction(currentQueueId));
                if (type === 'tipification') {
                    logger.error('[TipificationAnalysisImplementation::TipificationAnalysisExecute] executionCode:', executionCode, 'getting queueId:', currentQueueId, 'Saving cognitive result for evaluationplan:', interactionCognitive);
                    promiseArray.push(redis_service_1.Redis.SaveReadyTipificationInteraction(currentQueueId, interactionCognitive, executionCode));
                }
                else {
                    logger.error('[TipificationAnalysisImplementation::TipificationAnalysisExecute] executionCode:', executionCode, 'getting queueId:', currentQueueId, 'TYPE_IS_NOT_FOUND', type);
                }
                promiseArray.push(mssql_service_1.MSSql.SaveBatchError(vcc, CURRENT_SYSTEM_SERVICE, typeId, ['study', 'template'].includes(type) ? typeId : null, CURRENT_IMPLEMENTATION, interactionCognitive.id, interactionCognitive.modelId, this.errorData.errorFunction, this.errorData.errorMessage, this.errorData.errorDetail));
            }
            return Promise.all(promiseArray);
        })
            .then((result) => {
            return Promise.resolve(true);
        })
            .catch((err) => {
            let errorMessage = typeof err === 'string' ? err : (err.message || err.description);
            logger.error('[TipificationAnalysisImplementation::TipificationAnalysisExecute] executionCode:', executionCode, 'getting queueId:', currentQueueId, 'getting error:', errorMessage);
            return Promise.resolve(false);
        });
    }
    EvaluateTipification(interaction, currentQueueId, executionCode) {
        let phrases = [];
        let abilities = [];
        let evaluationTipificationResults = [];
        return promises_shared_1.DummyPromise()
            .then(_ => {
            let promiseArray = [];
            promiseArray.push(this.GetCognitiveResultsFromFile(interaction, currentQueueId, executionCode));
            promiseArray.push(redis_service_1.Redis.GetEvaluationPlanAbilities(interaction.vcc, interaction.evaluationPlanId));
            return Promise.all(promiseArray);
        })
            .then(results => {
            phrases = results[0].phrases;
            abilities = results[1];
            phrases.forEach(p => {
                abilities.filter(a => a.abilityCalificationType === 'QUALITATIVE').forEach(a => {
                    a.scores.filter(s => !s.value).forEach(s => {
                        s.tipifications.filter(t => t.tipificationQuery?.queryId?.length).forEach(t => {
                            let nluName = t.tipificationQuery.queryContent.split('|')[0];
                            let intentName = t.tipificationQuery.queryContent.split('|')[1];
                            let intentId = t.tipificationQuery.queryId;
                            let nluResult = p[nluName] ? p[nluName] : null;
                            if (nluResult?.intents?.length) {
                                let intentResult = nluResult.intents.find(i => [intentId, intentName].includes(i.intent));
                                if (intentResult?.score >= t.tipificationQuery.acceptancePercentage) {
                                    let evaluationTipificationResult = new evaluation_result_model_1.EvaluationTipificationResult();
                                    evaluationTipificationResult.abilityId = a.idAbility;
                                    evaluationTipificationResult.scoreId = t.scoreId;
                                    evaluationTipificationResult.id = t.id;
                                    evaluationTipificationResult.isChecked = true;
                                    evaluationTipificationResult.evaluationPlanId = interaction.evaluationPlanId;
                                    evaluationTipificationResult.interactionId = interaction.id;
                                    evaluationTipificationResult.auditorId = 'SPEECH';
                                    evaluationTipificationResults.push(evaluationTipificationResult);
                                    logger.info(`[TipificationAnalysisImplementation::EvaluateTipification] executionCode: ${executionCode}, getting queueId: ${currentQueueId}, Interaction ${interaction.id} have got OK for tipification ${t.description}(${t.id}) from ability ${a.descriptionAbility}(${a.idAbility}) on evaluationPlan ${interaction.evaluationPlanId} with score ${intentResult.score}`);
                                }
                            }
                        });
                    });
                });
            });
            interaction['status'] = 7;
            interaction['statusDescription'] = 'ready';
            interaction['retry'] = 0;
            interaction.succedTipifications = evaluationTipificationResults;
            return Promise.resolve(interaction);
        })
            .catch(err => {
            let errorMessage = typeof err === 'string' ? err : (err.message || err.description);
            logger.error(`[TipificationAnalysisImplementation::EvaluateTipification] executionCode: ${executionCode}, getting queueId: ${currentQueueId}, getting error: ${errorMessage}`);
            interaction['status'] = -7;
            interaction['retry'] = interaction['retry'] ? interaction['retry'] + 1 : 1;
            interaction['uploadedDate'] = moment().format('x');
            interaction['errorMessage'] = errorMessage;
            this.errorData.SetErrorData('EvaluateTipification', errorMessage, err);
            return Promise.resolve(interaction);
        });
    }
    GetCognitiveResultsFromFile(interaction, currentQueueId, executionCode) {
        let path = null;
        let regex = /\.(mp3|wav|ogg|json)$/;
        let jsonFileName = interaction.fileName.replace(regex, '.json');
        path = config_1.COGNITIVE_ANALYSIS_FOLDER;
        path += '/';
        path += interaction.vcc;
        path += '/';
        path += jsonFileName;
        return promises_shared_1.DummyPromise()
            .then(_ => {
            return file_manager_1.ReadFileContent(path, 'utf-8', true);
        })
            .then(result => {
            let jsonContent = null;
            try {
                jsonContent = typeof result === 'string' ? JSON.parse(result) : result;
            }
            catch (ex) {
                jsonContent = null;
                this.errorData.SetErrorData('GetCognitiveResultsFromFile', 'ERROR_FILE_JSON', ex);
                throw 'ERROR_FILE_JSON';
            }
            return Promise.resolve(jsonContent);
        })
            .then(result => {
            if (!result)
                throw 'ERROR_NOT_FOUND_FILE_JSON';
            return Promise.resolve(result);
        })
            .catch(err => {
            let errorMessage = typeof err === 'string' ? err : (err.message || err.description);
            logger.error(`[TipificationAnalysisImplementation::GetCognitiveResultsFromFile] executionCode: ${executionCode} getting queueId: ${currentQueueId} getting error: ${errorMessage} (json: ${path})`);
            return Promise.reject(err);
        });
    }
}
exports.TipificationAnalysisImplementation = TipificationAnalysisImplementation;
