"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.TranscriptionsUploadImplementation = void 0;
const jwt = require("jsonwebtoken");
const moment = require("moment");
const log4js = require("log4js");
const fs = require("fs");
const mssql_service_1 = require("../db/mssql.service");
const config_1 = require("../config/config");
const task_config_1 = require("../config/task.config");
const redis_service_1 = require("../db/redis.service");
const file_manager_1 = require("../shared/file-manager");
const promises_shared_1 = require("../shared/promises.shared");
const billing_implementation_1 = require("./billing.implementation");
const io_request_1 = require("../io/io.request");
const io_model_1 = require("../models/io.model");
const events_1 = require("../io/events");
const audio_transfer_implementation_1 = require("./audio-transfer.implementation");
const deepgram_service_implementation_1 = require("./deepgram-service.implementation");
const speechmatics_service_implementation_1 = require("./speechmatics-service.implementation");
const jwt_config_1 = require("../config/jwt.config");
const http = require('http');
const https = require('https');
let logger = log4js.getLogger('Task');
const CURRENT_SYSTEM_SERVICE = 'TRANSCRIPTION';
const CURRENT_IMPLEMENTATION = 'TranscriptionsUploadImplementation';
class TranscriptionsUploadImplementation {
    constructor() {
        this.externalUserId = 'ext_user';
        this.externalUserVcc = 'system';
        this.transcriptionService = null;
        this.GetTranscriptionEngine(null);
    }
    Execute(executionCode, rateRecursive, availableBatch, cancelledBatch) {
        logger.info(`[TranscriptionsUploadImplementation::Execute] executionCode: ${executionCode} executing wave #${rateRecursive}`);
        let startTime = moment();
        let executeIndex = 1;
        let ratio = config_1.RATIO_TRANSCRIPTION;
        let alowedRatio = 0;
        let notAlowedRatio = 0;
        let executeResult = true;
        let currentBlockPromise = 'BLOCK_00';
        this.transcriptionService.executionCode = executionCode;
        return promises_shared_1.SleepPromise(config_1.SLEEP_TRANSCRIPTION)
            .then(_ => {
            currentBlockPromise = 'BLOCK_01_GET_FROM_REDIS';
            return redis_service_1.Redis.GetLengthFromList('TranscriptionsUpload');
        })
            .then(result => {
            currentBlockPromise = 'BLOCK_02_GET_SPEECHMATICS_FREE_STORAGE';
            if (!result)
                throw 'SERVER_INFO_REDIS_KEYS_FOR_TRANSCRIPTIONS_UPLOAD_NOT_FOUND';
            return this.transcriptionService.GetEngineFreeStorage(executionCode);
        })
            .then(result => {
            currentBlockPromise = 'BLOCK_03_GET_QUEUE_ITEMS';
            if (!result)
                throw 'SERVER_INFO_NO_SPEECHMATICS_WITH_FREE_STORAGE';
            return redis_service_1.Redis.LRange('TranscriptionsUpload', 0, ratio - 1);
        })
            .then((result) => {
            let promiseArray = [];
            ratio = result?.length <= ratio ? result.length : 0;
            result.forEach(r => {
                let arrQueue = r.split(':');
                if (availableBatch.find(b => b.VCC === arrQueue[0] && b.BatchTypeId.toLowerCase() === arrQueue[1].toLowerCase())) {
                    alowedRatio++;
                }
                else {
                    notAlowedRatio++;
                }
            });
            while (ratio >= executeIndex) {
                promiseArray.push(this.TranscribeAudioExecute(executionCode, availableBatch, cancelledBatch, executeIndex));
                executeIndex++;
            }
            if (!promiseArray.length)
                promiseArray.push(Promise.resolve(true));
            return Promise.all(promiseArray);
        })
            .then((results) => {
            if (results.filter(item => item === false).length > notAlowedRatio)
                executeResult = false;
            return redis_service_1.Redis.LRange('TranscriptionsUpload', 0, ratio - 1);
        })
            .then((result) => {
            if (result?.length && rateRecursive >= 1)
                return this.Execute(executionCode, rateRecursive - 1, availableBatch, cancelledBatch);
            return Promise.resolve(executeResult);
        })
            .then(result => {
            if (rateRecursive === config_1.RATE_RECURSIVE_TRANSCRIPTION)
                logger.info(`[TranscriptionsUploadImplementation::Execute] executionCode: ${executionCode} Finishing execution wave #${rateRecursive} after ${moment().diff(startTime, 'minutes')} minutes`);
            let currentResult = executeResult && result ? true : false;
            return Promise.resolve(currentResult);
        })
            .catch((error) => {
            let errorMessage = typeof error === 'string' ? error : (error.message || error.description);
            if (errorMessage.startsWith('SERVER_INFO_'))
                logger.info('[TranscriptionsUploadImplementation::Execute] executionCode:', executionCode, ', Block:', currentBlockPromise, ', getting info:', errorMessage);
            else
                logger.error('[TranscriptionsUploadImplementation::Execute] executionCode:', executionCode, ', Block:', currentBlockPromise, ', getting error:', errorMessage);
            return Promise.resolve(false);
        });
    }
    TranscribeAudioExecute(executionCode, availableBatch, cancelledBatch, index) {
        let storage = '';
        let currentQueueId = '';
        let processInteraction;
        let vcc = '';
        let processId = '';
        let interactionType = '';
        let interactionId = '';
        let interactionSection = 0;
        let interactionPart = 0;
        let currentBlockPromise = null;
        return redis_service_1.Redis.RemoveQueueL('TranscriptionsUpload')
            .then((queueId) => {
            currentBlockPromise = 'BLOCK_01_GETTING_INTERACTION_DATA_FROM_REDIS_STORAGE';
            currentQueueId = queueId;
            let item = currentQueueId.split(':');
            if (item?.length) {
                vcc = item[0];
                processId = item[1];
                interactionType = item[2];
                interactionId = item[3];
                interactionSection = Number(item[4]);
                interactionPart = Number(item[5]);
                storage = vcc + ':' + processId;
                logger.info('[TranscriptionsUploadImplementation::TranscribeAudioExecute] executionCode:', executionCode, 'getting queueId:', currentQueueId, 'storage :', storage);
                return redis_service_1.Redis.GetProcessInteraction(storage, currentQueueId);
            }
            else {
                logger.error('[TranscriptionsUploadImplementation::TranscribeAudioExecute] executionCode:', executionCode, 'getting queueId:', currentQueueId, 'QUEUE_FORMAT_ERROR');
                return Promise.resolve(null);
            }
        })
            .then((result) => {
            currentBlockPromise = 'BLOCK_02_SETTING_INTERACTION_DATA';
            if (result) {
                processInteraction = result;
                logger.info('[TranscriptionsUploadImplementation::TranscribeAudioExecute] executionCode:', executionCode, 'getting queueId:', currentQueueId, 'getting GetProcessInteraction: 1, status:', processInteraction.statusDescription);
                return Promise.resolve(true);
            }
            else {
                if (currentQueueId)
                    logger.warn('[TranscriptionsUploadImplementation::TranscribeAudioExecute] executionCode:', executionCode, 'getting queueId:', currentQueueId, 'getting GetProcessInteraction: 0');
                return Promise.resolve(null);
            }
        })
            .then(result => {
            currentBlockPromise = 'BLOCK_03_VALIDATE_ALLOWABLE';
            let arrQueue = currentQueueId.split(':');
            let promiseArray = [];
            if (result) {
                if (availableBatch.find(b => b.VCC === arrQueue[0] && b.BatchTypeId.toLowerCase() === processInteraction.batchTypeId.toLowerCase())) {
                    promiseArray.push(Promise.resolve('ALLOWED'));
                    promiseArray.push(redis_service_1.Redis.CreateQueueL('TranscriptionsUploadExecute', currentQueueId));
                    promiseArray.push(Promise.resolve(currentQueueId));
                }
                else {
                    promiseArray.push(Promise.resolve('NOT_ALLOWED'));
                    promiseArray.push(redis_service_1.Redis.CreateQueueR('TranscriptionsUpload', currentQueueId));
                    promiseArray.push(mssql_service_1.MSSql.SaveBatchStatus(arrQueue[0], arrQueue[1], 'TRANSCRIPTION', 'PAUSED'));
                    promiseArray.push(io_request_1.SendIOEvents([
                        new io_model_1.IOEvent(io_model_1.VCC_EVENT, events_1.UPDATE_TRANSCRIPTION, null, arrQueue[0])
                    ]));
                }
            }
            if (!promiseArray.length)
                promiseArray.push(Promise.resolve(true));
            return Promise.all(promiseArray);
        })
            .then((result) => {
            currentBlockPromise = 'BLOCK_04_EXECUTE_MAIN_FUNCTION';
            if (result && result[0] === 'NOT_ALLOWED')
                throw 'SERVER_ERROR_NOT_ALLOWED_QUEUE';
            if (processInteraction) {
                if (processInteraction.statusDescription === 'pendingUpload') {
                    if (processInteraction.interactionType === 'CALL') {
                        if (!processInteraction.transcriptionApi
                            ||
                                processInteraction.status === -3) {
                            let server = this.transcriptionService.GetTranscriptionApi(index - 1);
                            processInteraction.transcriptionApi = server.url;
                            processInteraction.transcriptionToken = server.token;
                            processInteraction.transcriptionProtocol = server.protocol;
                        }
                        return this.UploadAudio(processInteraction, currentQueueId, executionCode);
                    }
                    else {
                        processInteraction.status = 3;
                        processInteraction.statusDescription = config_1.USAGE_SENTIMENT_ANALYSIS ? 'pendingSentimentAnalysis' : 'pendingAnalysis';
                    }
                }
                else
                    logger.error(`[TranscriptionsUploadImplementation::TranscribeAudioExecute] executionCode: ${executionCode} getting queueId: ${currentQueueId}, Received invalid status: ${processInteraction.statusDescription}`);
            }
            return Promise.resolve(null);
        })
            .then((result) => {
            currentBlockPromise = 'BLOCK_05_SAVING_UPDATED_DATA_ON_REDIS';
            if (processInteraction)
                return redis_service_1.Redis.SaveProcessInteraction(storage, currentQueueId, processInteraction);
            return Promise.resolve(null);
        })
            .then(result => {
            currentBlockPromise = 'BLOCK_06_SENDING_TO_NEXT_QUEUE';
            let promiseArray = [];
            let billing = new billing_implementation_1.BillingImplementation();
            if (result
                &&
                    processInteraction?.status === 3
                &&
                    ['pendingTranscription', 'pendingSentimentAnalysis', 'pendingAnalysis'].includes(processInteraction?.statusDescription)
                &&
                    processInteraction?.retry < config_1.RETRY_TRANSCRIPTION) {
                if (processInteraction.interactionType === 'CALL' && task_config_1.SPEECHMATICS_CALLBACK_USAGE) {
                    if (processInteraction.statusDescription === 'pendingAnalysis')
                        promiseArray.push(redis_service_1.Redis.InsertQueue('TranscriptionsDownload', currentQueueId, config_1.RATE_RECURSIVE_TRANSCRIPTION * config_1.RATIO_TRANSCRIPTION * 2));
                    else
                        promiseArray.push(redis_service_1.Redis.InsertQueue('TranscriptionsStandby', currentQueueId));
                }
                else
                    promiseArray.push(redis_service_1.Redis.InsertQueue('TranscriptionsDownload', currentQueueId, config_1.RATE_RECURSIVE_TRANSCRIPTION * config_1.RATIO_TRANSCRIPTION * 2));
                promiseArray.push(redis_service_1.Redis.DeleteQueue('TranscriptionsUploadExecute', currentQueueId));
            }
            else if (result
                &&
                    processInteraction?.status === -3
                &&
                    processInteraction?.statusDescription === 'REMOVED_BY_CANCELLED') {
                logger.error('[TranscriptionsUploadImplementation::TranscribeAudioExecute] executionCode:', executionCode, 'getting queueId:', currentQueueId, 'REMOVED_BY_CANCELLED');
                billing.AddCloseTranscriptionDetailToMainFile(processInteraction, processInteraction.batchTypeName, processInteraction.batchTypeId, 0, 'REMOVED_BY_CANCELLED');
                promiseArray.push(redis_service_1.Redis.DeleteQueue('TranscriptionsUploadExecute', currentQueueId));
            }
            else if (processInteraction?.status === -3
                &&
                    processInteraction?.errorMessage === 'UPLOAD_ERROR'
                &&
                    (processInteraction?.retry >= 500
                        ||
                            processInteraction?.errorDetail === 'SERVER_ERROR_AUDIO_FILE_NOT_FOUND')) {
                logger.error('[TranscriptionsUploadImplementation::TranscribeAudioExecute] executionCode:', executionCode, 'getting queueId:', currentQueueId, 'UPLOAD_ERROR');
                if (processInteraction.retry >= 500) {
                    billing.AddCloseTranscriptionDetailToMainFile(processInteraction, processInteraction.batchTypeName, processInteraction.batchTypeId, 0, 'UPLOAD_ERROR');
                    processInteraction.retry = config_1.RETRY_TRANSCRIPTION;
                }
                else if (processInteraction.errorDetail === 'SERVER_ERROR_AUDIO_FILE_NOT_FOUND') {
                    logger.info('[TranscriptionsUploadImplementation::TranscribeAudioExecute] executionCode:', executionCode, 'getting queueId:', currentQueueId, 'REENQUEUEING_TO_AUDIO_TRANSFER');
                    promiseArray.push(redis_service_1.Redis.InsertQueue('Audios', currentQueueId, 1));
                }
                promiseArray.push(redis_service_1.Redis.DeleteQueue('TranscriptionsUploadExecute', currentQueueId));
            }
            else if (processInteraction?.retry >= config_1.RETRY_TRANSCRIPTION) {
                logger.error('[TranscriptionsUploadImplementation::TranscribeAudioExecute] executionCode:', executionCode, 'getting queueId:', currentQueueId, 'REMOVED_BY_RETRIES');
                billing.AddCloseTranscriptionDetailToMainFile(processInteraction, processInteraction.batchTypeName, processInteraction.batchTypeId, 0, 'REMOVED_BY_RETRIES');
                promiseArray.push(redis_service_1.Redis.DeleteQueue('TranscriptionsUploadExecute', currentQueueId));
                processInteraction.retry = config_1.RETRY_TRANSCRIPTION;
            }
            else if (currentQueueId && !processInteraction) {
                logger.error('[TranscriptionsUploadImplementation::TranscribeAudioExecute] executionCode:', executionCode, 'getting queueId:', currentQueueId, 'INTERACTION_DO_NOT_EXIST');
                let interactionData = {
                    vcc: vcc,
                    processId: processId,
                    id: interactionId,
                    section: interactionSection,
                    part: interactionPart,
                    retry: config_1.RETRY_TRANSCRIPTION
                };
                let audioTransfer = new audio_transfer_implementation_1.AudioTransferImplementation();
                audioTransfer.CloseTranscriptionDetailWithMissingData(billing, interactionData, 'TranscriptionsUploadImplementation');
                promiseArray.push(redis_service_1.Redis.DeleteQueue('TranscriptionsUploadExecute', currentQueueId));
            }
            else {
                logger.error('[TranscriptionsUploadImplementation::TranscribeAudioExecute] executionCode:', executionCode, 'getting queueId:', currentQueueId, 'QUEUE_WAS_NOT_REMOVED');
            }
            if (!promiseArray.length)
                promiseArray.push(Promise.resolve(true));
            return Promise.all(promiseArray);
        })
            .then((result) => {
            currentBlockPromise = 'BLOCK_07_UPDATING_PROGRESS';
            let promiseArray = [];
            if (processInteraction && processInteraction.retry >= config_1.RETRY_TRANSCRIPTION) {
                processInteraction.isCancel = true;
                promiseArray.push(redis_service_1.Redis.UpdateProgress(storage));
                promiseArray.push(redis_service_1.Redis.UpdateErrorProgress(storage));
                promiseArray.push(redis_service_1.Redis.DeleteProcessInteraction(storage, currentQueueId));
                promiseArray.push(redis_service_1.Redis.SaveReadyProcessInteraction(executionCode, currentQueueId, processInteraction));
                promiseArray.push(mssql_service_1.MSSql.SaveBatchError(vcc, CURRENT_SYSTEM_SERVICE, processInteraction.batchTypeId, processInteraction.batchTypeId, CURRENT_IMPLEMENTATION, processInteraction.id, null, processInteraction.errorFunction, processInteraction.errorMessage, processInteraction.errorDetail));
            }
            if (!promiseArray.length)
                promiseArray.push(Promise.resolve(true));
            return Promise.all(promiseArray);
        })
            .then((result) => {
            currentBlockPromise = 'BLOCK_08_FINISHING';
            return Promise.resolve(true);
        })
            .catch((err) => {
            let errorMessage = typeof err === 'string' ? err : (err.message || err.description);
            logger.error('[TranscriptionsUploadImplementation::TranscribeAudioExecute] executionCode:', executionCode, 'getting queueId:', currentQueueId, ', Block:', currentBlockPromise, ', getting error:', errorMessage);
            return Promise.resolve(false);
        });
    }
    UploadAudio(interaction, currentQueueId, executionCode) {
        let fullPath = '';
        let dictionaryContent = [];
        return promises_shared_1.DummyPromise()
            .then(_ => {
            return this.GetDictionaryContent(interaction, currentQueueId, executionCode);
        })
            .then(result => {
            dictionaryContent = result;
            return new Promise((resolve, reject) => {
                let baseFolder = (['ftp', 'sftp', 's3'].includes(interaction.environmentType)) ? config_1.PATH_FOLDER_PROCESS_AUDIO : config_1.PATH_FOLDER_TEMP_AUDIO;
                fullPath = `${baseFolder}/${interaction.vcc}/${interaction.fileName}`;
                logger.info('[TranscriptionsUploadImplementation::UploadAudio] executionCode:', executionCode, 'getting queueId:', currentQueueId, 'audio:', fullPath);
                fs.stat(fullPath, (err, stat) => {
                    resolve({ fullPath: fullPath, stat: stat });
                });
            });
        })
            .then((result) => {
            if (result?.stat?.size > 0) {
                return file_manager_1.BufferFile(result.fullPath);
            }
            else {
                logger.error('[TranscriptionsUploadImplementation::UploadAudio] executionCode:', executionCode, 'getting queueId:', currentQueueId, 'Audio file not found on fullPath:', fullPath);
                throw 'SERVER_ERROR_AUDIO_FILE_NOT_FOUND';
            }
        })
            .then((stream) => {
            if (stream)
                return this.transcriptionService.UploadAudio(interaction, stream, dictionaryContent, currentQueueId);
            return Promise.resolve(null);
        })
            .then(result => {
            if (result)
                return Promise.resolve(interaction);
            else
                throw 'UPLOAD_FAILED';
        })
            .catch((err) => {
            let errorMessage = typeof err === 'string' ? err : (err.message || err.description);
            logger.error('[TranscriptionsUploadImplementation::UploadAudio] executionCode:', executionCode, 'getting queueId:', currentQueueId, 'getting error:', errorMessage);
            interaction['status'] = -3;
            interaction['errorFunction'] = 'UploadAudio';
            interaction['errorMessage'] = 'UPLOAD_ERROR';
            interaction['errorDetail'] = err.stack ? err.stack : errorMessage;
            interaction['retry'] = interaction['retry'] ? interaction['retry'] + 1 : 1;
            return Promise.resolve(interaction);
        });
    }
    GetDictionaryContent(interaction, currentQueueId, executionCode) {
        let dictionaryWords = [];
        let dictionaryContent = [];
        return promises_shared_1.DummyPromise()
            .then(_ => {
            if (interaction.dictionaryId)
                return mssql_service_1.MSSql.GetDictionaryWords(interaction.vcc, null, null, interaction.dictionaryId);
            else
                return Promise.resolve([]);
        })
            .then(result => {
            if (!result)
                throw 'SERVER_ERROR_CANNOT_GET_DICTIONARY_WORDS';
            dictionaryWords = result;
            if (dictionaryWords.length) {
                let dictionaryWordIds = [];
                dictionaryWords.forEach(dw => dictionaryWordIds.push(dw.id));
                return mssql_service_1.MSSql.GetDictionaryWordSoundLikes(interaction.vcc, dictionaryWordIds.join('\',\''));
            }
            else
                return Promise.resolve([]);
        })
            .then((result) => {
            logger.info('[TranscriptionsUploadImplementation::GetDictionaryContent] executionCode:', executionCode, 'getting queueId:', currentQueueId, 'found words:', dictionaryWords.length);
            dictionaryWords.forEach(dw => {
                dw.soundLikes = [];
                dw.soundLikes = result.filter(r => r.dictionaryWordId === dw.id);
            });
            dictionaryWords.forEach(dw => {
                if (dw.soundLikes && dw.soundLikes.length) {
                    let soundLikeWords = [];
                    dw.soundLikes.forEach(sl => soundLikeWords.push(sl.soundLike));
                    dictionaryContent.push({
                        content: dw.word,
                        sounds_like: soundLikeWords
                    });
                }
                else {
                    dictionaryContent.push(dw.word);
                }
            });
            return Promise.resolve(dictionaryContent);
        })
            .catch((err) => {
            let errorMessage = typeof err === 'string' ? err : (err.message || err.description);
            logger.error('[TranscriptionsUploadImplementation::GetDictionaryContent] executionCode:', executionCode, 'getting queueId:', currentQueueId, 'getting error:', errorMessage);
            return Promise.resolve([]);
        });
    }
    GetTranscriptionEngine(executionCode) {
        let transcriptionService = null;
        try {
            switch (task_config_1.TRANSCRIPTION_ENGINE) {
                case 'DEEPGRAM':
                    transcriptionService = new deepgram_service_implementation_1.DeepgramServiceImplementation();
                    break;
                case 'SPEECHMATICS':
                    transcriptionService = new speechmatics_service_implementation_1.SpeechmaticsServiceImplementation();
                    break;
            }
        }
        catch (e) {
            logger.error('[TranscriptionsUploadImplementation::GetTranscriptionEngine] executionCode:', executionCode, 'getting error:', e);
        }
        this.transcriptionService = transcriptionService;
    }
    GetSystemUserToken() {
        let token = jwt.sign({ userId: this.externalUserId, vcc: this.externalUserVcc }, jwt_config_1.JWT_SECRET, { expiresIn: jwt_config_1.JWT_EXPIRATION_TIME });
        return token;
    }
}
exports.TranscriptionsUploadImplementation = TranscriptionsUploadImplementation;
