"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.init = void 0;
const http = require("http");
const https = require("https");
const express = require("express");
const bodyParser = require("body-parser");
const cookieParser = require("cookie-parser");
const compression = require("compression");
const path = require("path");
const fs = require("fs");
const log4js = require("log4js");
const services = require("./services");
const permissions = require("./shared/permission.shared");
const httpShared = require("./shared/http.shared");
const sessionShared = require("./shared/session.shared");
const securityShared = require("./shared/security.shared");
const logger_config_1 = require("./config/logger.config");
const mssql_init_1 = require("./db/mssql.init");
const redis_init_1 = require("./db/redis.init");
const promises_shared_1 = require("./shared/promises.shared");
const server_config_1 = require("./config/server.config");
const app_config_1 = require("./config/app.config");
const permissions_config_1 = require("./config/permissions.config");
const io_init_1 = require("./io/io.init");
const crypto_shared_1 = require("./shared/crypto.shared");
let app = express();
let httpServer;
let httpsServer;
function init() {
    let clientPath = '../client';
    let logsPath = path.resolve(__dirname, 'logs');
    if (!fs.existsSync(logsPath))
        fs.mkdirSync(logsPath);
    log4js.configure(logger_config_1.LOGGER_CONFIG);
    let logger = log4js.getLogger('ServerScripts');
    logger.info('********************************************************');
    logger.info('* Inicializando servicio                               *');
    logger.info('********************************************************');
    app.use(log4js.connectLogger(log4js.getLogger('AccessLog'), { level: 'auto' }));
    app.use(bodyParser.urlencoded({ extended: false }));
    app.use(bodyParser.json({ limit: '5mb' }));
    app.use(bodyParser.text({ limit: '5mb' }));
    app.use(bodyParser.raw({ limit: '5mb' }));
    app.use(cookieParser());
    app.use(compression());
    app.use(function (err, req, res, next) {
        if (err && err instanceof SyntaxError && err.status >= 400 && err.status < 500 && err.message.indexOf('JSON')) {
            logger.warn('No se pudo parsear contenido JSON de request ' + req.method + ' para \'' + req.path + '\'');
            res.status(400).json({ status: false, description: 'Invalid JSON', error: 'Invalid JSON' });
        }
    });
    securityShared.InitHeaders(app);
    logger.info('Servicio ejecutando en modo \'' + server_config_1.SERVER_MODE + '\'');
    app.enable('trust proxy');
    app.set('trust proxy', true);
    app.use(function (req, res, next) {
        if (req.headers.host.slice(0, 4) !== 'www.' && req.secure) {
            next();
        }
        else if (server_config_1.SERVER_MODE === 'prod') {
            let newHost = req.headers.host.slice(0, 4) === 'www.' ? req.headers.host.slice(4) : req.headers.host;
            return res.redirect(301, 'https://' + newHost + req.originalUrl);
        }
        else {
            next();
        }
    });
    if (server_config_1.SERVER_MODE === 'prod') {
        app.use('/client/assets', express.static(path.resolve(__dirname, clientPath + '/assets')));
        app.use('/client/css', express.static(path.resolve(__dirname, clientPath + '/css')));
        app.use('/client/flags', express.static(path.resolve(__dirname, clientPath + '/flags')));
        app.use('/client/fonts', express.static(path.resolve(__dirname, clientPath + '/fonts')));
        app.use('/client/js', express.static(path.resolve(__dirname, clientPath + '/js')));
        crypto_shared_1.UpdateClientPublicKey(path.resolve(__dirname, clientPath + '/index.html'), path.resolve(__dirname, clientPath + '/index-form.html'));
    }
    else {
        app.use('/client/assets', express.static(path.resolve(__dirname, clientPath + '/assets')));
        app.use('/client/css', express.static(path.resolve(__dirname, clientPath + '/css')));
        app.use('/client/flags', express.static(path.resolve(__dirname, clientPath + '/flags')));
        app.use('/client/fonts', express.static(path.resolve(__dirname, clientPath + '/fonts')));
        app.use('/assets', express.static(path.resolve(__dirname, clientPath + '/assets')));
        app.use('/app', express.static(path.resolve(__dirname, clientPath + '/app')));
        app.use('/app-form', express.static(path.resolve(__dirname, clientPath + '/app-form')));
        app.use('/components', express.static(path.resolve(__dirname, clientPath + '/components')));
        app.use('/languages', express.static(path.resolve(__dirname, clientPath + '/languages')));
        app.use('/layout', express.static(path.resolve(__dirname, clientPath + '/layout')));
        app.use('/models', express.static(path.resolve(__dirname, clientPath + '/models')));
        app.use('/pipes', express.static(path.resolve(__dirname, clientPath + '/pipes')));
        app.use('/services', express.static(path.resolve(__dirname, clientPath + '/services')));
        app.use('/shared', express.static(path.resolve(__dirname, clientPath + '/shared')));
        app.use('/validators', express.static(path.resolve(__dirname, clientPath + '/validators')));
        app.use('/dist/dev/client/assets', express.static(path.resolve(__dirname, clientPath + '/assets')));
        app.use('/dist/dev/client/css', express.static(path.resolve(__dirname, clientPath + '/css')));
        app.use('/src/server/static', express.static(path.resolve(__dirname, 'static')));
        app.use('/node_modules', express.static(path.resolve(__dirname, '../../../node_modules')));
        app.use(express.static(path.resolve(process.cwd())));
        app.use(express.static(path.resolve(process.cwd(), path.resolve(__dirname, clientPath + '/**/'))));
        crypto_shared_1.UpdateClientPublicKey(path.resolve(__dirname, clientPath + '/index.html'), path.resolve(__dirname, clientPath + '/index-form.html'));
    }
    app.use('/static', express.static(path.resolve(__dirname, 'static')));
    if (!app_config_1.IS_DISTRIBUTED_SERVICE) {
        app.use(httpShared.GetRequestDataByVCC);
        app.use(securityShared.HstsGuard);
        app.use(sessionShared.ValidateUserSession);
        if (permissions_config_1.CHECK_API_PERMISSIONS)
            app.use(permissions.GrantAccess);
        app.disable('etag');
        services.init(app);
        app.use(httpShared.ValidateDomainRedirect);
        app.get('/*', function (req, res, next) {
            if (['socket.io', 'io'].includes(req.url)) {
                next();
                return;
            }
            if (req.url.match('^\/(api|auth|public|internal|popup|builder|payments|distributed|static|template|client|assets|favicon)(.*)$')) {
                res.status(404).type('txt').send('Page not found');
            }
            else if (req.url.match('^\/(form-login|form\/)(.*)$')) {
                res.sendFile(path.resolve(__dirname, clientPath + '/index-form.html'));
            }
            else {
                res.sendFile(path.resolve(__dirname, clientPath + '/index.html'));
            }
        });
    }
    else {
        app.get('/*', function (req, res, next) {
            res.status(404).type('txt').send('Page not found');
        });
    }
    return promises_shared_1.DummyPromise().then(result => {
        return mssql_init_1.MsSqlInit(app);
    }).then(result => {
        return redis_init_1.RedisInit(app);
    })
        .then(result => {
        return new Promise((resolve, reject) => {
            if (!server_config_1.HTTP_ENABLED)
                return resolve(true);
            logger.info('Iniciando servicio HTTP');
            console.log('Iniciando servicio HTTP');
            httpServer = http.createServer(app);
            httpServer.listen(server_config_1.HTTP_BINDING_PORT, server_config_1.HTTP_BINDING_HOST)
                .on('listening', () => {
                logger.info('Servicio HTTP escuchando en: ' + httpServer.address().toString() + ':' + server_config_1.HTTP_BINDING_PORT.toString());
                console.log('Servicio HTTP escuchando en: ' + httpServer.address().toString() + ':' + server_config_1.HTTP_BINDING_PORT.toString());
                resolve(true);
            });
        });
    }).then(result => {
        return new Promise((resolve, reject) => {
            if (!server_config_1.HTTPS_ENABLED)
                return resolve(true);
            logger.info('Iniciando servicio HTTPS');
            console.log('Iniciando servicio HTTPS');
            httpsServer = https.createServer({
                key: fs.readFileSync(server_config_1.HTTPS_KEY),
                cert: fs.readFileSync(server_config_1.HTTPS_CERT)
            }, app);
            httpsServer.listen(server_config_1.HTTPS_BINDING_PORT, server_config_1.HTTP_BINDING_HOST)
                .on('listening', () => {
                logger.info('Servicio HTTPS escuchando en: ' + httpsServer.address().toString() + ':' + server_config_1.HTTPS_BINDING_PORT.toString());
                console.log('Servicio HTTPS escuchando en: ' + httpsServer.address().toString() + ':' + server_config_1.HTTPS_BINDING_PORT.toString());
                resolve(true);
            });
        });
    })
        .then(_ => {
        return io_init_1.IOServerInit(app, httpServer, httpsServer);
    })
        .then(result => {
        if (!result)
            logger.error('No se pudo establecer conexión con el servicio de mail.');
        logger.info('Servicio inicializado correctamente');
        console.log('Servicio inicializado correctamente');
    }).catch(err => {
        let errorMsg = (typeof err === 'string' ? err : err.message || err.description || '');
        logger.info('Error al inicializar el servicio: ' + errorMsg);
        console.error('Error al inicializar el servicio: ' + errorMsg);
    });
}
exports.init = init;
