"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.EvaluationService = void 0;
const moment = require("moment");
const log4js = require("log4js");
const mssql_service_1 = require("../db/mssql.service");
const config_1 = require("../config/config");
const promises_shared_1 = require("../shared/promises.shared");
const redis_service_1 = require("../db/redis.service");
const file_manager_1 = require("../shared/file-manager");
const utilities_1 = require("../shared/utilities");
const role_service_1 = require("./role.service");
const evaluation_model_1 = require("../models/evaluation.model");
const evaluation_plan_model_1 = require("../models/evaluation-plan.model");
const evaluation_result_model_1 = require("../models/evaluation-result.model");
const interaction_model_1 = require("../models/interaction.model");
const interaction_evaluation_plan_model_1 = require("../models/interaction-evaluation-plan.model");
const process_implementation_1 = require("../implementation/process.implementation");
const arrays_shared_1 = require("../shared/arrays.shared");
const generative_service_implementation_1 = require("../implementation/generative-service.implementation");
const generative_analysis_implementation_1 = require("../implementation/generative-analysis.implementation");
let logger = log4js.getLogger('ServerScripts');
function EvaluationService(app) {
    app.post('/api/evaluation/get_evaluation_results/', (req, res, next) => {
        logger.info('[EvaluationService::get_evaluation_results] Received ' + req.method + ' method on \'' + req.path + '\'');
        let vcc = res.locals.instance;
        let userId = res.locals.userId;
        let evaluationPlanId = req.body.evaluationPlanId || null;
        let interactionId = req.body.interactionId || null;
        let auditorId = req.body.auditorId || userId || null;
        let efectivityPlanId = req.body.efectivityPlanId || null;
        promises_shared_1.DummyPromise()
            .then(_ => {
            return mssql_service_1.MSSql.GetEvaluationResults(vcc, evaluationPlanId, interactionId, auditorId, efectivityPlanId);
        })
            .then(result => {
            if (res.headersSent)
                return;
            res.json({ status: true, description: 'SERVER_SUCCESS_MESSAGE', data: result });
        })
            .catch(err => {
            let msg = (typeof err === 'string' ? err : 'CANNOT_PROCESS_REQUEST');
            let errorMsg = (typeof err === 'string' ? err : err.message || err.description || 'SERVER_ERROR_UNKNOWN_MESSAGE');
            logger.error('[EvaluationService::get_evaluation_results] Error on \'' + req.path + '\': ' + errorMsg);
            if (res.headersSent)
                return;
            res.json({ status: false, description: msg, error: errorMsg });
        });
    });
    app.post('/api/evaluation/save_evaluation_status', (req, res, next) => {
        logger.info('[EvaluationService::save_evaluation_status] Received ' + req.method + ' method on \'' + req.path + '\'');
        let vcc = res.locals.instance;
        let userId = res.locals.userId;
        let evaluationToUpdate = (req.body.evaluation || {});
        evaluationToUpdate.auditorId = userId;
        promises_shared_1.DummyPromise()
            .then(result => {
            return redis_service_1.Redis.SaveEvaluationStatus(vcc, evaluationToUpdate);
        })
            .then(result => {
            if (result && result.status === false)
                throw result.error;
            if (res.headersSent)
                return;
            logger.info('[EvaluationService::save_evaluation_status] The evaluation update has been finished.');
            res.json({ status: true, description: (result ? 'SERVER_SUCCESS_MESSAGE' : 'SERVER_ERROR_UNKNOWN_MESSAGE'), data: true });
        })
            .catch(err => {
            let msg = (typeof err === 'string' ? err : 'CANNOT_PROCESS_REQUEST');
            let errorMsg = (typeof err === 'string' ? err : err.message || err.description || 'Executing request error');
            logger.error('[EvaluationService::save_evaluation_status] Error on \'' + req.path + '\': ' + errorMsg);
            if (res.headersSent)
                return;
            res.json({ status: false, description: msg, error: errorMsg });
        });
    });
    app.post('/api/evaluation/discard_evaluation_status', (req, res, next) => {
        logger.info('[EvaluationService::save_evaluation_status] Received ' + req.method + ' method on \'' + req.path + '\'');
        let vcc = res.locals.instance;
        let userId = res.locals.userId;
        let evaluationToUpdate = (req.body.evaluation || {});
        evaluationToUpdate.auditorId = userId;
        promises_shared_1.DummyPromise()
            .then(result => {
            return redis_service_1.Redis.DeletEvaluationStatus(vcc, evaluationToUpdate);
        })
            .then(result => {
            if (result && result.status === false)
                throw result.error;
            if (res.headersSent)
                return;
            logger.info('[EvaluationService::discard_evaluation_status] The evaluation update has been finished.');
            res.json({ status: true, description: (result ? 'SERVER_SUCCESS_MESSAGE' : 'SERVER_ERROR_UNKNOWN_MESSAGE'), data: true });
        })
            .catch(err => {
            let msg = (typeof err === 'string' ? err : 'CANNOT_PROCESS_REQUEST');
            let errorMsg = (typeof err === 'string' ? err : err.message || err.description || 'Executing request error');
            logger.error('[EvaluationService::discard_evaluation_status] Error on \'' + req.path + '\': ' + errorMsg);
            if (res.headersSent)
                return;
            res.json({ status: false, description: msg, error: errorMsg });
        });
    });
    app.post('/api/evaluation/save_evaluationresult', (req, res) => {
        logger.info('[EvaluationService::save_evaluationresult] Received ' + req.method + ' method on \'' + req.path + '\'');
        let vcc = res.locals.instance;
        let userId = res.locals.userId;
        let evaluations = (req.body || {});
        evaluations.interactionData.vcc = vcc;
        let currentAuditor = evaluations.executionType === 'REALTIME' ? 'SPEECH' : userId;
        if (evaluations.isDraft) {
            evaluations.evaluationResult = evaluations.evaluationResult.filter(er => er.hasOwnProperty('qualification') && er.qualification !== null);
        }
        let saveResult = null;
        promises_shared_1.DummyPromise()
            .then(_ => {
            if (evaluations.evaluationResult.length === 0)
                throw 'SERVER_ERROR_EVALUATION_RESULTS_NOT_RECEIVED';
            if (evaluations.executionType === 'REALTIME')
                return CreateDataForRealtime(evaluations.interactionData.evaluationPlanId, evaluations.interactionData, userId);
            else
                return Promise.resolve(true);
        })
            .then(_ => {
            return mssql_service_1.MSSql.GetAuditorEvaluationPlanEvaluation(vcc, currentAuditor, evaluations.evaluationPlanId, evaluations.interactionData.id, evaluations.interactionData.type);
        })
            .then(result => {
            if (!result)
                throw 'SERVER_ERROR_EVALUATION_NOT_FOUND';
            if (result.completed || !result.status) {
                throw 'SERVER_ERROR_EVALUATION_IS_NOT_AVAILABLE';
            }
            else {
                return SaveEvaluationResults(vcc, currentAuditor, result.evaluation_mode, evaluations);
            }
        })
            .then(result => {
            saveResult = result;
            let evaluationToRemove = new evaluation_model_1.EvaluationToUpdate();
            evaluationToRemove.evaluationPlanId = evaluations.interactionData.evaluationPlanId;
            evaluationToRemove.interactionId = evaluations.interactionData.id;
            evaluationToRemove.auditorId = currentAuditor;
            return redis_service_1.Redis.DeletEvaluationStatus(vcc, evaluationToRemove);
        })
            .then(_ => {
            if (evaluations.executionType === 'REALTIME' && !evaluations.isDraft)
                return SaveGenerativeUsage(vcc, evaluations.evaluationPlanId, evaluations.interactionData.id);
            else
                return Promise.resolve(true);
        })
            .then(_ => {
            res.json({ status: true, description: (saveResult ? 'SERVER_SUCCESS_MESSAGE' : 'Evaluation Results save failed'), data: saveResult });
        })
            .catch(err => {
            let clientMsg = (typeof err === 'string' ? err : 'CANNOT_PROCESS_REQUEST');
            let errorMsg = (typeof err === 'string' ? err : err.message || err.description || 'SERVER_ERROR_UNKNOWN_MESSAGE');
            logger.error('[EvaluationService::save_evaluationresult] Error on \'' + req.path + '\': ' + errorMsg);
            if (res.headersSent)
                return;
            res.json({ status: false, description: clientMsg, error: errorMsg });
        });
    });
    app.post('/api/evaluation/save_evaluationresult_efectivity', (req, res) => {
        logger.info('[EvaluationService::save_evaluationresult_efectivity] Received ' + req.method + ' method on \'' + req.path + '\'');
        let evaluationResultsEfectivityPromises = [];
        let vcc = res.locals.instance;
        let evaluationResultEfectivityArray = req.body;
        let saveResult = null;
        promises_shared_1.DummyPromise()
            .then(_ => {
            evaluationResultEfectivityArray.forEach(evaluationResultEfectivity => {
                evaluationResultsEfectivityPromises.push(mssql_service_1.MSSql.SaveEvaluationResultEfectivity(vcc, evaluationResultEfectivity));
            });
            return Promise.all(evaluationResultsEfectivityPromises);
        }).then(result => {
            res.json({ status: true, description: (saveResult ? 'SERVER_SUCCESS_MESSAGE' : 'Evaluation Results efectivity save failed'), data: saveResult });
        })
            .catch(err => {
            let clientMsg = (typeof err === 'string' ? err : 'CANNOT_PROCESS_REQUEST');
            let errorMsg = (typeof err === 'string' ? err : err.message || err.description || 'SERVER_ERROR_UNKNOWN_MESSAGE');
            logger.error('[EvaluationService::save_evaluationresult_efectivity] Error on \'' + req.path + '\': ' + errorMsg);
            if (res.headersSent)
                return;
            res.json({ status: false, description: clientMsg, error: errorMsg });
        });
    });
    app.post('/api/evaluation/discard_evaluation_draft', (req, res, next) => {
        logger.info('[EvaluationService::discard_evaluation_draft] Received ' + req.method + ' method on \'' + req.path + '\'');
        let vcc = res.locals.instance;
        let userId = res.locals.userId;
        let evaluationPlanId = req.body.evaluation.evaluationPlanId || null;
        let interactionId = req.body.evaluation.interactionId || null;
        logger.info(`[EvaluationPlanService::discard_evaluation_draft] DiscardDraf evaluationPlan ${evaluationPlanId} by the user ${res.locals.userId}`);
        promises_shared_1.DummyPromise()
            .then(_ => {
            return mssql_service_1.MSSql.RestoreEvaluationResultDraftBackup(vcc, evaluationPlanId, interactionId, userId);
        })
            .then(result => {
            if (result && result.status === false)
                throw result.error;
            if (res.headersSent)
                return;
            logger.info('[EvaluationService::discard_evaluation_draft] The evaluation draft have been deleted.');
            res.json({ status: true, description: (result ? 'SERVER_SUCCESS_MESSAGE' : 'SERVER_ERROR_UNKNOWN_MESSAGE'), data: true });
        })
            .catch(err => {
            let msg = (typeof err === 'string' ? err : 'CANNOT_PROCESS_REQUEST');
            let errorMsg = (typeof err === 'string' ? err : err.message || err.description || 'Executing request error');
            logger.error('[EvaluationService::discard_evaluation_draft] Error on \'' + req.path + '\': ' + errorMsg);
            if (res.headersSent)
                return;
            res.json({ status: false, description: msg, error: errorMsg });
        });
    });
    app.get('/api/evaluation/get_interaction_evaluation/:interaction?', (req, res, next) => {
        logger.info('Se recibió ' + req.method + ' para \'' + req.path + '\'');
        let instance = res.locals.instance;
        let interaction = req.params.interaction || '';
        let interactionType = req.query.interactionType ? req.query.interactionType.toString() : '';
        let evaluationDateFrom = req.query.evaluationDateFrom ? req.query.evaluationDateFrom.toString() : '1980-01-01';
        let evaluationDateTo = req.query.evaluationDateTo ? req.query.evaluationDateTo.toString() : '2050-01-01';
        promises_shared_1.DummyPromise()
            .then(result => {
            return mssql_service_1.MSSql.GetEvaluationsOfInteraction(instance, interaction, interactionType, evaluationDateFrom, evaluationDateTo);
        }).then(result => {
            if (res.headersSent)
                return;
            res.json({ status: true, description: 'SERVER_SUCCESS_MESSAGE', data: result });
        }).catch(err => {
            let msg = (typeof err === 'string' ? err : 'CANNOT_PROCESS_REQUEST');
            let errorMsg = (typeof err === 'string' ? err : err.message || err.description || 'SERVER_ERROR_UNKNOWN_MESSAGE');
            logger.error('Error al procesar \'' + req.path + '\': ' + errorMsg);
            if (res.headersSent)
                return;
            res.json({ status: false, description: msg, error: errorMsg });
        });
    });
    app.post('/api/evaluation/get_evaluation_agent/', (req, res, next) => {
        logger.info('[EvaluationService::get_evaluation_agent] Received ' + req.method + ' method on \'' + req.path + '\'');
        let instance = res.locals.instance;
        let idAgent = res.locals.userId;
        logger.info('[EvaluationService::get_evaluation_agent] Filling the filter\'s object');
        let params = new evaluation_result_model_1.EvaluationResultFilters;
        params.dateIni = req.body.dateIni || '';
        params.dateFin = req.body.dateFin || '';
        params.campaigns = req.body.campaigns || [];
        params.evaluationPlans = req.body.evaluationPlans || [];
        params.origins = req.body.origins || [];
        params.minDuration = req.body.minDuration || null;
        params.maxDuration = req.body.maxDuration || null;
        params.evaluationTypes = req.body.evaluationTypes || [];
        params.tags = req.body.tags || [];
        params.hasVideoOptions = req.body.hasVideoOptions || [];
        params.results = req.body.results ? req.body.results.map(r => {
            let dispositionCode = null;
            if (typeof r === 'string') {
                dispositionCode = r;
            }
            else {
                dispositionCode = r.campaignId;
                if (r.description)
                    dispositionCode += '_' + r.description;
                if (r.description2)
                    dispositionCode += '_' + r.description2;
                if (r.description3)
                    dispositionCode += '_' + r.description3;
                if (r.description4)
                    dispositionCode += '_' + r.description4;
            }
            return dispositionCode;
        }) : [];
        params.hourIni = req.body.hourIni || '';
        params.hourFin = req.body.hourFin || '';
        params.multimediaFormats = ['mp3', 'wav', 'ogg'];
        if (params.hasVideoOptions.includes(true)) {
            params.multimediaFormats.push('flv');
        }
        params.origins = params.origins.map(o => {
            if (o == "E")
                return "IN";
            else if (o == "S")
                return "OUT";
            else
                return null;
        });
        params.interactionIds = req.body.interactionIds || [];
        params.contactAddress = req.body.contactAddress || '';
        params.sentimentIds = req.body.sentimentIds || '';
        logger.debug('[EvaluationService::get_evaluation_agent] Filter\'s object: ', JSON.stringify(params));
        promises_shared_1.DummyPromise()
            .then(result => {
            logger.info('[EvaluationService::get_evaluation_agent] Calling GetEvaluations SP for VCC|idAgent: ' + instance + '|' + idAgent);
            return mssql_service_1.MSSql.GetEvaluations(instance, null, idAgent, params.campaigns.toString(), params.evaluationPlans.toString(), params.dateIni, params.dateFin, params.hourIni, params.hourFin, params.minDuration, params.maxDuration, params.origins.toString(), params.results.toString(), params.evaluationTypes.toString(), params.tags.toString(), params.multimediaFormats.toString(), null, params.interactionIds.toString(), params.contactAddress.toString(), params.sentimentIds.toString());
        })
            .then(result => {
            logger.debug('[EvaluationService::get_evaluation_agent] Received result from GetEvaluations SP: ' + JSON.stringify(result));
            if (res.headersSent)
                return;
            logger.info('[EvaluationService::get_evaluation_agent] Sending results: ' + result.length);
            res.json({ status: true, description: 'SERVER_SUCCESS_MESSAGE', data: result });
        })
            .catch(err => {
            let msg = (typeof err === 'string' ? err : 'CANNOT_PROCESS_REQUEST');
            let errorMsg = (typeof err === 'string' ? err : err.message || err.description || 'Requesting error');
            logger.error('[EvaluationService::get_evaluation_agent] Error on \'' + req.path + '\': ' + errorMsg);
            if (res.headersSent)
                return;
            res.json({ status: false, description: msg, error: errorMsg });
        });
    });
    app.post('/api/evaluation/get_evaluation_auditor/', (req, res, next) => {
        logger.info('[EvaluationService::get_evaluation_auditor] Received ' + req.method + ' method on \'' + req.path + '\'');
        let vcc = res.locals.instance;
        let idAuditor = res.locals.userId;
        let currentUserId = res.locals.userId;
        let currentUserRole = res.locals.user.role;
        logger.info('[EvaluationService::get_evaluation_auditor] Filling the filter\'s object');
        let params = new evaluation_result_model_1.EvaluationResultFilters;
        params.dateIni = req.body.dateIni || '';
        params.dateFin = req.body.dateFin || '';
        params.campaigns = req.body.campaigns || [];
        params.evaluationPlans = req.body.evaluationPlans || [];
        params.agents = req.body.agents || [];
        params.origins = req.body.origins || [];
        params.minDuration = req.body.minDuration || null;
        params.maxDuration = req.body.maxDuration || null;
        params.evaluationTypes = req.body.evaluationTypes && req.body.evaluationTypes.length ? [] : ['M', 'X'];
        params.tags = req.body.tags || [];
        params.hasVideoOptions = req.body.hasVideoOptions || [];
        params.coached = req.body.coached || [];
        params.results = req.body.results ? req.body.results.map(r => {
            let dispositionCode = null;
            if (typeof r === 'string') {
                dispositionCode = r;
            }
            else {
                dispositionCode = r.campaignId;
                if (r.description)
                    dispositionCode += '_' + r.description;
                if (r.description2)
                    dispositionCode += '_' + r.description2;
                if (r.description3)
                    dispositionCode += '_' + r.description3;
                if (r.description4)
                    dispositionCode += '_' + r.description4;
            }
            return dispositionCode;
        }) : [];
        params.hourIni = req.body.hourIni || '';
        params.hourFin = req.body.hourFin || '';
        params.multimediaFormats = ['mp3', 'wav', 'ogg'];
        if (params.hasVideoOptions.includes(true)) {
            params.multimediaFormats.push('flv');
        }
        params.origins = params.origins.map(o => {
            if (o == "E")
                return "IN";
            else if (o == "S")
                return "OUT";
            else
                return null;
        });
        params.interactionIds = req.body.interactionIds || [];
        params.contactAddress = req.body.contactAddress || '';
        params.sentimentIds = req.body.sentimentIds || [];
        logger.debug('[EvaluationService::get_evaluation_auditor] Filter\'s object: ', JSON.stringify(params));
        promises_shared_1.DummyPromise()
            .then(_ => {
            return role_service_1.GetRoleFunctions(vcc, currentUserId, currentUserRole);
        })
            .then(result => {
            logger.info('[EvaluationService::get_evaluation_auditor] Calling GetEvaluations SP for VCC|IdAuditor: ' + vcc + '|' + currentUserId);
            if (result.data.hasSupervisorFunctions)
                idAuditor = null;
            return mssql_service_1.MSSql.GetEvaluations(vcc, idAuditor, params.agents.toString(), params.campaigns.toString(), params.evaluationPlans.toString(), params.dateIni, params.dateFin, params.hourIni, params.hourFin, params.minDuration, params.maxDuration, params.origins.toString(), params.results.toString(), params.evaluationTypes.toString(), params.tags.toString(), params.multimediaFormats.toString(), params.coached.toString(), params.interactionIds.toString(), params.contactAddress.toString(), params.sentimentIds.toString());
        })
            .then(result => {
            logger.debug('[EvaluationService::get_evaluation_auditor] Received result from GetEvaluations SP: ' + JSON.stringify(result));
            if (res.headersSent)
                return;
            logger.info('[EvaluationService::get_evaluation_auditor] Sending results: ' + result.length);
            res.json({ status: true, description: 'SERVER_SUCCESS_MESSAGE', data: result });
        })
            .catch(err => {
            let msg = (typeof err === 'string' ? err : 'CANNOT_PROCESS_REQUEST');
            let errorMsg = (typeof err === 'string' ? err : err.message || err.description || 'Error requesting');
            logger.error('[EvaluationService::get_evaluation_auditor] Error on \'' + req.path + '\': ' + errorMsg);
            if (res.headersSent)
                return;
            res.json({ status: false, description: msg, error: errorMsg });
        });
    });
    app.post('/api/evaluation/get_evaluation_admin/', (req, res, next) => {
        logger.info('[EvaluationService::get_evaluation_admin] Received ' + req.method + ' method on \'' + req.path + '\'');
        let instance = res.locals.instance;
        logger.info('[EvaluationService::get_evaluation_admin] Filling the filter\'s object');
        let params = new evaluation_result_model_1.EvaluationResultFilters;
        params.dateIni = req.body.dateIni || '';
        params.dateFin = req.body.dateFin || '';
        params.campaigns = req.body.campaigns || [];
        params.evaluationPlans = req.body.evaluationPlans || [];
        params.auditors = req.body.auditors || [];
        params.agents = req.body.agents || [];
        params.origins = req.body.origins || [];
        params.minDuration = req.body.minDuration || null;
        params.maxDuration = req.body.maxDuration || null;
        params.evaluationTypes = req.body.evaluationTypes || [];
        params.tags = req.body.tags || [];
        params.hasVideoOptions = req.body.hasVideoOptions || [];
        params.coached = req.body.coached || [];
        params.results = req.body.results ? req.body.results.map(r => {
            let dispositionCode = null;
            if (typeof r === 'string') {
                dispositionCode = r;
            }
            else {
                dispositionCode = r.campaignId;
                if (r.description)
                    dispositionCode += '_' + r.description;
                if (r.description2)
                    dispositionCode += '_' + r.description2;
                if (r.description3)
                    dispositionCode += '_' + r.description3;
                if (r.description4)
                    dispositionCode += '_' + r.description4;
            }
            return dispositionCode;
        }) : [];
        params.hourIni = req.body.hourIni || '';
        params.hourFin = req.body.hourFin || '';
        params.multimediaFormats = ['mp3', 'wav', 'ogg'];
        if (params.hasVideoOptions.includes(true)) {
            params.multimediaFormats.push('flv');
        }
        params.origins = params.origins.map(o => {
            if (o == "E")
                return "IN";
            else if (o == "S")
                return "OUT";
            else
                return null;
        });
        params.interactionIds = req.body.interactionIds || [];
        params.contactAddress = req.body.contactAddress || '';
        params.sentimentIds = req.body.sentimentIds || '';
        logger.debug('[EvaluationService::get_evaluation_admin] Filter\'s object: ', JSON.stringify(params));
        promises_shared_1.DummyPromise()
            .then(result => {
            logger.info('[EvaluationService::get_evaluation_admin] Calling GetEvaluations SP for VCC|IdAuditor: ' + instance);
            return mssql_service_1.MSSql.GetEvaluations(instance, params.auditors.toString(), params.agents.toString(), params.campaigns.toString(), params.evaluationPlans.toString(), params.dateIni, params.dateFin, params.hourIni, params.hourFin, params.minDuration, params.maxDuration, params.origins.toString(), params.results.toString(), params.evaluationTypes.toString(), params.tags.toString(), params.multimediaFormats.toString(), params.coached.toString(), params.interactionIds.toString(), params.contactAddress.toString(), params.sentimentIds.toString());
        })
            .then(result => {
            logger.debug('[EvaluationService::get_evaluation_admin] Received result from GetEvaluations SP: ' + JSON.stringify(result));
            if (res.headersSent)
                return;
            logger.info('[EvaluationService::get_evaluation_admin] Sending results: ' + result.length);
            res.json({ status: true, description: 'SERVER_SUCCESS_MESSAGE', data: result });
        })
            .catch(err => {
            let msg = (typeof err === 'string' ? err : 'CANNOT_PROCESS_REQUEST');
            let errorMsg = (typeof err === 'string' ? err : err.message || err.description || 'SERVER_ERROR_UNKNOWN_MESSAGE');
            logger.error('Error al procesar \'' + req.path + '\': ' + errorMsg);
            if (res.headersSent)
                return;
            res.json({ status: false, description: msg, error: errorMsg });
        });
    });
    app.post('/api/evaluation/get_evaluation_result', (req, res, next) => {
        logger.info('[EvaluationService::get_evaluation_result] Received ' + req.method + ' method on \'' + req.path + '\'');
        let vcc = res.locals.vcc;
        let evaluationPlanId = req.body.evaluationPlanId || '';
        let interactionId = req.body.interactionId || '';
        let interactionType = req.body.interactionType || '';
        let auditorId = req.body.auditorId || '';
        let agentId = req.body.agentId || '';
        let efectivityPlanId = req.body.efectivityPlanId || '';
        let results = {};
        promises_shared_1.DummyPromise()
            .then(_ => {
            return mssql_service_1.MSSql.GetEvaluationResults(vcc, evaluationPlanId, interactionId, auditorId, efectivityPlanId);
        })
            .then(result => {
            if (!result.evaluation)
                throw 'SERVER_ERROR_EVALUATION_RESULT_NOT_FOUND';
            results.evaluationResultData = result.evaluation;
            logger.info('[EvaluationService::get_evaluation_result] Calling GetEvaluationCustomFieldResultsFromAuditorEvaluationPlan SP for VCC|EP: ' + vcc + '|' + evaluationPlanId);
            return mssql_service_1.MSSql.GetEvaluationCustomFieldResultsFromAuditorEvaluationPlan(vcc, evaluationPlanId, interactionId, auditorId, agentId);
        })
            .then(result => {
            results.evaluationCustomFieldResultData = result;
            return mssql_service_1.MSSql.GetEvaluationFormatAttachedSignatures(vcc, null, evaluationPlanId);
        })
            .then(result => {
            results.attachedSignatures = result;
            if (res.headersSent)
                return;
            logger.info('[EvaluationService::get_evaluation_result] Received result from GetEvaluationResultsFromAuditorEvaluationPlan SP');
            res.json({ status: true, description: 'SERVER_SUCCESS_MESSAGE', data: results });
        })
            .catch(err => {
            let msg = (typeof err === 'string' ? err : 'CANNOT_PROCESS_REQUEST');
            let errorMsg = (typeof err === 'string' ? err : err.message || err.description || 'Error requesting');
            logger.error('[EvaluationService::get_evaluation_result] Error on \'' + req.path + '\': ' + errorMsg);
            if (res.headersSent)
                return;
            res.json({ status: false, description: msg, error: errorMsg });
        });
    });
    app.post('/api/evaluation/get_evaluation_results_from_auditor_evaluation_plan/:evaluationPlanId', (req, res, next) => {
        logger.info('[EvaluationService::get_evaluation_results_from_auditor_evaluation_plan] Received ' + req.method + ' method on \'' + req.path + '\'');
        let instance = res.locals.instance;
        let auditorId = res.locals.userId;
        logger.info('[EvaluationService::get_evaluation_results_from_auditor_evaluation_plan] Filling the filter\'s object');
        let params = new evaluation_result_model_1.EvaluationResultInteractionFilters;
        params.evaluationPlanId = req.params.evaluationPlanId || '';
        params.interactionId = req.body.interactionId || '';
        params.auditorId = req.body.auditorId || '';
        params.agentId = req.body.agentId || '';
        params.interactionType = req.body.interactionType || '';
        let results = {};
        logger.debug('[EvaluationService::get_evaluation_results_from_auditor_evaluation_plan] Filter\'s object: ', JSON.stringify(params));
        promises_shared_1.DummyPromise()
            .then(result => {
            logger.info('[EvaluationService::get_evaluation_results_from_auditor_evaluation_plan] Calling GetEvaluationResultsFromAuditorEvaluationPlan SP for VCC|IdEP: ' + instance + '|' + params.evaluationPlanId);
            return mssql_service_1.MSSql.GetEvaluationResultsFromAuditorEvaluationPlan(instance, params.evaluationPlanId, params.interactionId, params.interactionType, params.auditorId, params.agentId);
        })
            .then(result => {
            results.evaluationResultData = result;
            logger.info('[EvaluationService::get_evaluation_results_from_auditor_evaluation_plan] Calling GetEvaluationCustomFieldResultsFromAuditorEvaluationPlan SP for VCC|IdEP: ' + instance + '|' + params.evaluationPlanId);
            return mssql_service_1.MSSql.GetEvaluationCustomFieldResultsFromAuditorEvaluationPlan(instance, params.evaluationPlanId, params.interactionId, params.auditorId, params.agentId);
        })
            .then(result => {
            results.evaluationCustomFieldResultData = result;
            return mssql_service_1.MSSql.GetEvaluationFormatAttachedSignatures(instance, results.evaluationResultData[0].EvaluationFormatId, params.evaluationPlanId);
        })
            .then(result => {
            results.attachedSignatures = result;
            if (res.headersSent)
                return;
            logger.info('[EvaluationService::get_evaluation_results_from_auditor_evaluation_plan] Received result from GetEvaluationResultsFromAuditorEvaluationPlan SP');
            res.json({ status: true, description: 'SERVER_SUCCESS_MESSAGE', data: results });
        })
            .catch(err => {
            let msg = (typeof err === 'string' ? err : 'CANNOT_PROCESS_REQUEST');
            let errorMsg = (typeof err === 'string' ? err : err.message || err.description || 'Error requesting');
            logger.error('[EvaluationService::get_evaluation_results_from_auditor_evaluation_plan] Error on \'' + req.path + '\': ' + errorMsg);
            if (res.headersSent)
                return;
            res.json({ status: false, description: msg, error: errorMsg });
        });
    });
    app.get('/api/evaluation/get_evaluation_reports/:evformatid?', (req, res, next) => {
        logger.info('Se recibió ' + req.method + ' para \'' + req.path + '\'');
        let instance = res.locals.instance;
        let idEvaluationFormat = req.params.evformatid || '';
        let idsCampanias = req.query.campaigns || null, idAgente = req.query.agent || null, fechaDesde = req.query.dateFrom || null, fechaHasta = req.query.dateTo || null, idAuditor = req.query.auditor || null, planAuditoria = req.query.evaluationPlanId || null;
        promises_shared_1.DummyPromise()
            .then(result => {
            return mssql_service_1.MSSql.ReportAuditsByAgent(instance, idEvaluationFormat, idsCampanias, idAgente, fechaDesde, fechaHasta, idAuditor, planAuditoria);
        }).then(result => {
            if (res.headersSent)
                return;
            res.json({ status: true, description: 'SERVER_SUCCESS_MESSAGE', data: result });
        }).catch(err => {
            let msg = (typeof err === 'string' ? err : 'CANNOT_PROCESS_REQUEST');
            let errorMsg = (typeof err === 'string' ? err : err.message || err.description || 'SERVER_ERROR_UNKNOWN_MESSAGE');
            logger.error('Error al procesar \'' + req.path + '\': ' + errorMsg);
            if (res.headersSent)
                return;
            res.json({ status: false, description: msg, error: errorMsg });
        });
    });
    app.post('/api/evaluation/get_evaluation_reviews', (req, res, next) => {
        logger.info('[EvaluationService::get_evaluation_reviews] Received ' + req.method + ' method on \'' + req.path + '\'');
        let vcc = res.locals.vcc;
        let currentUserId = res.locals.userId;
        let currentUserRole = res.locals.user.role;
        let evaluationPlanId = req.body.evaluationPlanId || null;
        let interactionId = req.body.interactionId || null;
        let supervisorId = req.body.supervisorId || null;
        let agentId = req.body.agentId || null;
        let userHasOnlyAgentFunctions = false;
        promises_shared_1.DummyPromise()
            .then(_ => {
            return role_service_1.GetRoleFunctions(vcc, currentUserId, currentUserRole);
        })
            .then(result => {
            if (result
                &&
                    result.data
                &&
                    (!result.data.hasAdminFunctions
                        &&
                            !result.data.hasAuditorFunctions
                        &&
                            result.data.hasAgentFunctions))
                userHasOnlyAgentFunctions = true;
            if (userHasOnlyAgentFunctions) {
                logger.info('[EvaluationService::get_evaluation_reviews] Current user has only agent function.');
                agentId = currentUserId;
            }
            logger.info(`[EvaluationService::get_evaluation_reviews] Getting review for VCC|EvaluationPlan|InteractionId|IdAuditor|Agent: ${vcc}|${evaluationPlanId}|${interactionId}|${supervisorId}|${agentId}`);
            return mssql_service_1.MSSql.GetEvaluationReview(vcc, evaluationPlanId, interactionId, supervisorId, agentId);
        })
            .then(result => {
            logger.debug('[EvaluationService::get_evaluation_reviews] Received result from GetEvaluationReview SP: ' + JSON.stringify(result));
            if (res.headersSent)
                return;
            res.json({ status: true, description: 'SERVER_SUCCESS_MESSAGE', data: result });
        })
            .catch(err => {
            let msg = (typeof err === 'string' ? err : 'CANNOT_PROCESS_REQUEST');
            let errorMsg = (typeof err === 'string' ? err : err.message || err.description || 'SERVER_ERROR_UNKNOWN_MESSAGE');
            logger.error('[EvaluationService::get_evaluation_reviews] Error al procesar \'' + req.path + '\': ' + errorMsg);
            if (res.headersSent)
                return;
            res.json({ status: false, description: msg, error: errorMsg });
        });
    });
    app.post('/api/evaluation/save_evaluation_review', (req, res, next) => {
        logger.info('[EvaluationService::save_evaluation_review] Received ' + req.method + ' method on \'' + req.path + '\'');
        let vcc = res.locals.vcc;
        let currentUserId = res.locals.userId;
        let evaluationReview = req.body.evaluationReview || null;
        promises_shared_1.DummyPromise()
            .then(_ => {
            logger.info(`[EvaluationService::save_evaluation_review] Saving review for VCC|EvaluationPlan|InteractionId|IdAuditor: ${vcc}|${evaluationReview.evaluationPlanId}|${evaluationReview.interactionId}|${evaluationReview.supervisorId}`);
            return mssql_service_1.MSSql.SaveEvaluationReview(vcc, evaluationReview, currentUserId);
        })
            .then(result => {
            if (res.headersSent)
                return;
            logger.info('[EvaluationService::save_evaluation_review] The evaluation update has been finished.');
            res.json({ status: true, description: (result ? 'SERVER_SUCCESS_MESSAGE' : 'SERVER_ERROR_UNKNOWN_MESSAGE'), data: true });
        })
            .catch(err => {
            let msg = (typeof err === 'string' ? err : 'CANNOT_PROCESS_REQUEST');
            let errorMsg = (typeof err === 'string' ? err : err.message || err.description || 'Executing request error');
            logger.error('[EvaluationService::save_evaluation_review] Error on \'' + req.path + '\': ' + errorMsg);
            if (res.headersSent)
                return;
            res.json({ status: false, description: msg, error: errorMsg });
        });
    });
    app.post('/api/evaluation/get_evaluation_review_commitments', (req, res, next) => {
        logger.info('[EvaluationService::get_evaluation_review_commitments] Received ' + req.method + ' method on \'' + req.path + '\'');
        let vcc = res.locals.vcc;
        let currentUserId = res.locals.userId;
        let currentUserRole = res.locals.user.role;
        let agentId = req.body.agentId || null;
        let userHasOnlyAgentFunctions = false;
        promises_shared_1.DummyPromise()
            .then(_ => {
            return role_service_1.GetRoleFunctions(vcc, currentUserId, currentUserRole);
        })
            .then(result => {
            if (result
                &&
                    result.data
                &&
                    (!result.data.hasAdminFunctions
                        &&
                            !result.data.hasAuditorFunctions
                        &&
                            result.data.hasAgentFunctions))
                userHasOnlyAgentFunctions = true;
            if (userHasOnlyAgentFunctions) {
                logger.info('[EvaluationService::get_evaluation_review_commitments] Current user has only agent function.');
                agentId = currentUserId;
            }
            logger.info(`[EvaluationService::get_evaluation_review_commitments] Getting review for VCC|Agent: ${vcc}|${agentId}`);
            return mssql_service_1.MSSql.GetEvaluationReviewCommitments(vcc, null, null, null, agentId);
        })
            .then(result => {
            logger.debug('[EvaluationService::get_evaluation_review_commitments] Received result from GetEvaluationReview SP: ' + JSON.stringify(result));
            if (res.headersSent)
                return;
            res.json({ status: true, description: 'SERVER_SUCCESS_MESSAGE', data: result });
        })
            .catch(err => {
            let msg = (typeof err === 'string' ? err : 'CANNOT_PROCESS_REQUEST');
            let errorMsg = (typeof err === 'string' ? err : err.message || err.description || 'SERVER_ERROR_UNKNOWN_MESSAGE');
            logger.error('[EvaluationService::get_evaluation_review_commitments] Error al procesar \'' + req.path + '\': ' + errorMsg);
            if (res.headersSent)
                return;
            res.json({ status: false, description: msg, error: errorMsg });
        });
    });
    app.post('/api/evaluation/save_evaluation_review_commitment', (req, res, next) => {
        logger.info('[EvaluationService::save_evaluation_review_commitment] Received ' + req.method + ' method on \'' + req.path + '\'');
        let vcc = res.locals.vcc;
        let currentUserId = res.locals.userId;
        let evaluationReviewCommitment = req.body.evaluationReviewCommitment || null;
        evaluationReviewCommitment.supervisorId = currentUserId;
        promises_shared_1.DummyPromise()
            .then(_ => {
            return mssql_service_1.MSSql.SaveEvaluationReviewCommitment(vcc, evaluationReviewCommitment, currentUserId);
        })
            .then(result => {
            if (res.headersSent)
                return;
            logger.info('[EvaluationService::save_evaluation_review_commitment] The evaluation update has been finished.');
            res.json({ status: true, description: (result ? 'SERVER_SUCCESS_MESSAGE' : 'SERVER_ERROR_UNKNOWN_MESSAGE'), data: true });
        })
            .catch(err => {
            let msg = (typeof err === 'string' ? err : 'CANNOT_PROCESS_REQUEST');
            let errorMsg = (typeof err === 'string' ? err : err.message || err.description || 'Executing request error');
            logger.error('[EvaluationService::save_evaluation_review_commitment] Error on \'' + req.path + '\': ' + errorMsg);
            if (res.headersSent)
                return;
            res.json({ status: false, description: msg, error: errorMsg });
        });
    });
    app.post('/api/evaluation/get_cognitive_results', (req, res) => {
        let cognitiveInteraction = req.body.cognitiveInteraction;
        logger.debug('cognitiveInteraction', cognitiveInteraction);
        let interactionId = cognitiveInteraction.interactionId;
        let section = cognitiveInteraction.section;
        let part = cognitiveInteraction.part;
        let vcc = cognitiveInteraction.vcc;
        let jsonName = interactionId + '_' + section + '_' + part + '.json';
        promises_shared_1.DummyPromise().then(_ => {
            var path = config_1.COGNITIVE_ANALYSIS_FOLDER;
            path += '/';
            path += vcc;
            path += '/';
            path += jsonName;
            return file_manager_1.ReadFileContent(path, 'utf-8', true);
        }).then(result => {
            let rawData = JSON.parse(JSON.stringify(result));
            var jsonData = JSON.parse(result);
            return FormatWithIntensDB(cognitiveInteraction, jsonData);
        }).then(result => {
            res.json({ status: true, data: result, description: 'SERVER_SUCCESS_MESSAGE' });
        }).catch(err => {
            let clientMsg = (typeof err === 'string' ? err : 'CANNOT_PROCESS_REQUEST');
            let errorMsg = (typeof err === 'string' ? err : err.message || err.description || 'SERVER_ERROR_UNKNOWN_MESSAGE');
            logger.error('[EvaluationService::get-cognitive-results] Getting errorMsg:', errorMsg);
            if (res.headersSent)
                return;
            res.json({ status: false, description: clientMsg, error: errorMsg });
        });
    });
    app.post('/api/evaluation/evaluate', (req, res) => {
        let vcc = res.locals.vcc;
        let evaluation = (req.body || {});
        let abilityEvaluationFormats = [];
        let appIds = [];
        let generativeIntents = [];
        let summaries = [];
        let generativeUsages = null;
        let currentBlockPromise = null;
        let logLabel = `[EvaluationService::evaluate] [vcc => ${vcc}]: `;
        logger.info(`${logLabel}Received ${req.method} method on '${req.path}'`);
        promises_shared_1.DummyPromise()
            .then(_ => {
            return redis_service_1.Redis.GetGenerativeUsages(vcc, evaluation.evaluationPlanId, evaluation.interactionId);
        })
            .then(result => {
            generativeUsages = result || {};
            currentBlockPromise = 'BLOCK_CALLING_REALTIME_AEF';
            return GetRealtimeAbilityEvaluationFormat(vcc, evaluation.evaluationPlanId);
        })
            .then(result => {
            currentBlockPromise = 'BLOCK_SEARCHING_GENERATIVE_INTENTS';
            if (!result)
                throw 'SERVER_ERROR_GET_REALTIME_AEF_FAILED';
            abilityEvaluationFormats = result.filter(r => evaluation.abilityIds.includes(r.idAbility));
            let intentIds = [];
            abilityEvaluationFormats.forEach(aef => aef.queries.forEach(q => intentIds.push(q.queryId)));
            intentIds = arrays_shared_1.GetUniqueFromArray(intentIds);
            return mssql_service_1.MSSql.GetGenerativeIntents(vcc, null, null, intentIds.toString());
        })
            .then((result) => {
            currentBlockPromise = 'BLOCK_EVALUATING_ABILITIES';
            if (!result)
                throw 'SERVER_ERROR_GET_GENERATIVE_INTENTS_FAILED';
            generativeIntents = result;
            generativeIntents.forEach(gi => appIds.push(gi.appId));
            appIds = arrays_shared_1.GetUniqueFromArray(appIds);
            let generativeService = new generative_service_implementation_1.GenerativeServiceImplementation(vcc);
            let generativieAnalysis = new generative_analysis_implementation_1.GenerativeAnalysisImplementation();
            let promiseArray = [];
            appIds.forEach(ai => {
                promiseArray.push(generativeService.Query(ai, evaluation.text));
                redis_service_1.Redis.SaveGenerativeUsage(vcc, evaluation.evaluationPlanId, evaluation.interactionId, ai, (generativeUsages[ai] || 0) + generativieAnalysis.CalculateTokens(evaluation.text));
            });
            return Promise.all(promiseArray);
        })
            .then(result => {
            currentBlockPromise = 'BLOCK_MOUNTING_SUMMARIES';
            if (!result)
                throw 'SERVER_ERROR_QUERY_TEXT_FAILED';
            appIds.forEach((a, i) => {
                let appResult = result[i];
                appResult.intents.forEach(i => {
                    let intent = generativeIntents.find(gi => gi.appId === a && gi.name === i.intent);
                    if (intent)
                        summaries.push({
                            appId: a,
                            intentId: intent.id,
                            intentName: intent.name,
                            score: i.score
                        });
                });
            });
            return Promise.resolve(summaries);
        })
            .then(result => {
            currentBlockPromise = 'BLOCK_CONSOLIDATING_RESULTS';
            if (!result)
                throw 'SERVER_ERROR_GET_SUMMARIES_FAILED';
            let abilityResults = [];
            abilityEvaluationFormats.forEach(aef => {
                let abilitySuccess = true;
                let queryResults = [];
                aef.queries.forEach(q => {
                    let summary = summaries.find(s => s.intentId === q.queryId);
                    if (summary) {
                        if (summary.score < q.acceptancePercentage)
                            abilitySuccess = false;
                        if (aef.isNegative)
                            abilitySuccess = !abilitySuccess;
                        queryResults.push({
                            queryContent: q.queryContent,
                            appId: summary.appId,
                            appName: q.queryContent.split('|')[0],
                            intentId: summary.intentId,
                            intentName: summary.intentName,
                            score: summary.score,
                            umbral: q.acceptancePercentage
                        });
                    }
                });
                if (queryResults.length)
                    abilityResults.push({
                        abilityId: aef.idAbility,
                        success: abilitySuccess,
                        queryResults
                    });
            });
            if (res.headersSent)
                return;
            res.json({ status: true, description: 'SERVER_SUCCESS_MESSAGE', data: abilityResults });
        })
            .catch(err => {
            let clientMsg = (typeof err === 'string' ? err : 'SERVER_ERROR_GET_GENERATIVE_INTENT_ITEMS_FAILED');
            let errorMsg = (typeof err === 'string' ? err : err.message || err.description || 'SERVER_ERROR_UNKNOWN_MESSAGE');
            logger.error(`${logLabel}Error on '${req.path}': ${errorMsg}`);
            logger.error(`${logLabel}Block: ${currentBlockPromise}`);
            if (res.headersSent)
                return;
            res.json({ status: false, description: clientMsg, error: clientMsg });
        });
    });
}
exports.EvaluationService = EvaluationService;
function CreateDataForRealtime(evaluationPlanId, id, currentUser) {
    id.endDate ?? (id.endDate = moment().utc().toDate());
    id.duration ?? (id.duration = moment(id.endDate).diff(moment(id.startDate), 'seconds'));
    return promises_shared_1.DummyPromise()
        .then(_ => {
        let iep = new interaction_evaluation_plan_model_1.InteractionEvaluationPlan();
        iep.vcc = id.vcc;
        iep.evaluationPlanId = evaluationPlanId;
        iep.interactionId = id.id;
        iep.interactionSection = 0;
        iep.interactionPart = 0;
        iep.interactionType = id.type;
        iep.campaignId = id.campaignId;
        iep.agentId = id.agentId;
        iep.agentName = id.agentDescription;
        iep.interactionStartDate = id.startDate;
        iep.interactionEndDate = id.endDate;
        iep.interactionDuration = id.duration;
        iep.interactionOrigin = id.origin;
        iep.contactId = id.contactId;
        iep.needI6Update = true;
        return mssql_service_1.MSSql.SaveInteractionEvaluationPlan(iep, currentUser);
    })
        .then(result => {
        if (!result)
            throw 'SERVER_ERROR_SAVE_INTERACTION_EVALUATION_PLAN_FAILED';
        let i = new interaction_model_1.Interaction(id.id, id.campaignId, id.agentId, id.startDate, id.endDate, id.duration, id.type, null, id.contactId, id.origin, false, id.campaignId, id.agentId, evaluationPlanId, evaluationPlanId);
        let aep = new evaluation_plan_model_1.AuditorEvaluationPlan(evaluationPlanId, 'SPEECH', 'SPEECH ANALYTIC', null, null, null, null, null);
        aep.vcc = id.vcc;
        aep.campaignId = id.campaignId;
        aep.agentId = id.agentId;
        aep.interactions = [i];
        return mssql_service_1.MSSql.SaveAuditorEvaluationPlan(aep, currentUser);
    })
        .then(result => {
        if (!result)
            throw 'SERVER_ERROR_SAVE_AUDITOR_EVALUATION_PLAN_FAILED';
        return Promise.resolve(true);
    })
        .catch((error) => {
        let errorMessage = typeof error === 'string' ? error : 'ERROR';
        logger.error('[EvaluationService::CreateDataForRealtime] Error: ', error);
        return Promise.reject(errorMessage);
    });
}
function FormatWithIntensDB(cognitiveInteraction, jsonData) {
    return promises_shared_1.DummyPromise()
        .then(_ => {
        return mssql_service_1.MSSql.GetCognitiveInteractionIntents(cognitiveInteraction);
        ;
    })
        .then(result => {
        if (result.length === 0)
            return Promise.reject("NOT_FOUND_COGNITIVE_RESULTS");
        let resDB = result ? result : [];
        let models = [] = resDB.map(function (item) { return item.modelName; });
        let sorted = models.sort();
        let uniquemodels = sorted.filter(function (value, index) {
            return value !== sorted[index + 1] && value !== "";
        });
        uniquemodels.map((current => Object.assign({ modelName: current })));
        logger.debug("uniquemodels", uniquemodels.length);
        let phrases = [];
        let intentsNew = [];
        let arrPhrases = [];
        for (let i = 0; i < jsonData.phrases.length; i++) {
            let arrModels = [];
            for (let j = 0; j < uniquemodels.length; j++) {
                let currentModel = uniquemodels[j];
                let currentModelLenght = resDB.filter(element => element.modelName === currentModel).length;
                let intentsCurrentModel = [] = resDB.filter(element => element.modelName === currentModel);
                let arrIntents = [];
                for (let k = 0; k < currentModelLenght; k++) {
                    let foundIntent = [];
                    if (intentsCurrentModel[k].cognitiveEngine === 'RASA') {
                        foundIntent = jsonData.phrases[i][currentModel].intents.find(element => element.intent === intentsCurrentModel[k].intentId);
                    }
                    else {
                        foundIntent = jsonData.phrases[i][currentModel].intents.find(element => element.intent === intentsCurrentModel[k].intentName);
                    }
                    if (foundIntent) {
                        arrIntents.push({
                            intentId: foundIntent.intent,
                            intentName: intentsCurrentModel[k].intentName,
                            intentScore: (parseFloat(foundIntent.score) * 100).toFixed(0)
                        });
                    }
                }
                if (arrIntents.length > 0) {
                    arrIntents.sort((a, b) => a.intentScore > b.intentScore ? -1 : 1);
                    arrModels.push({
                        modelName: currentModel,
                        intents: arrIntents
                    });
                }
            }
            if (arrModels.length > 0)
                arrPhrases.push({
                    phrase: jsonData.phrases[i].content,
                    models: arrModels
                });
        }
        return arrPhrases;
    }).catch((error) => {
        let errorMessage = typeof error === 'string' ? error : 'ERROR';
        logger.error('[EvaluationService::FormatWithIntensDB] Error: ', error);
        return Promise.reject(errorMessage);
    });
}
function SaveEvaluationResults(vcc, userId, evaluationPlanMode, interactionEvaluationPlanResult) {
    let nowUtc = moment().utc().toDate();
    let evaluationResultPromises = [];
    let evaluationCustomFieldResultPromises = [];
    let ok;
    let evaluationPlanId;
    let interactionId;
    let interactionType;
    return promises_shared_1.DummyPromise()
        .then(_ => {
        if (interactionEvaluationPlanResult?.evaluationResult?.length > 0) {
            ok = true;
            evaluationPlanId = interactionEvaluationPlanResult.interactionData.evaluationPlanId;
            interactionId = interactionEvaluationPlanResult.interactionData.id;
            interactionType = interactionEvaluationPlanResult.interactionData.type;
            if (evaluationPlanMode === 'X' && interactionEvaluationPlanResult.isDraft)
                return mssql_service_1.MSSql.SaveEvaluationResultDraftBackup(vcc, evaluationPlanId, interactionType, interactionId, userId);
            else
                return Promise.resolve(true);
        }
        else {
            throw 'SERVER_ERROR_EVALUATION_RESULTS_NOT_FOUND';
        }
    })
        .then(result => {
        interactionEvaluationPlanResult.evaluationResult.forEach(newEvaluationResult => {
            if (!newEvaluationResult.id) {
                newEvaluationResult.id = utilities_1.guid();
                newEvaluationResult.createdDate = nowUtc;
                newEvaluationResult.createdByUserId = userId;
            }
            else {
                newEvaluationResult.createdDate = null;
                newEvaluationResult.createdByUserId = null;
            }
            newEvaluationResult.lastModifiedDate = nowUtc;
            newEvaluationResult.lastModifiedByUserId = userId;
            newEvaluationResult.idAuditor = userId;
            newEvaluationResult.evaluationDate = nowUtc;
            evaluationResultPromises.push(mssql_service_1.MSSql.SaveEvaluationResult(vcc, newEvaluationResult.id, interactionEvaluationPlanResult.interactionData.campaignId, interactionEvaluationPlanResult.interactionData.id, interactionEvaluationPlanResult.interactionData.agentId, interactionEvaluationPlanResult.interactionData.evaluationFormatId, interactionEvaluationPlanResult.interactionData.evaluationFormatMinQualificationAcceptable, newEvaluationResult.groupId, newEvaluationResult.subGroupId, newEvaluationResult.abilityId, newEvaluationResult.qualification, newEvaluationResult.ownScore, newEvaluationResult.inheritedScore, newEvaluationResult.idAuditor, newEvaluationResult.evaluationDate, newEvaluationResult.maxAbilityScore, newEvaluationResult.abilityInnerScore, interactionEvaluationPlanResult.interactionData.evaluationPlanId, newEvaluationResult.comments, newEvaluationResult.notApplies, newEvaluationResult.scoreDescription, interactionEvaluationPlanResult.interactionData.campaignDescription, interactionEvaluationPlanResult.interactionData.agentDescription, new Date(interactionEvaluationPlanResult.interactionData.startDate), new Date(interactionEvaluationPlanResult.interactionData.endDate), interactionEvaluationPlanResult.interactionData.address, interactionEvaluationPlanResult.interactionData.type, newEvaluationResult.abilityOrder, newEvaluationResult.createdDate, newEvaluationResult.createdByUserId, newEvaluationResult.lastModifiedDate, newEvaluationResult.lastModifiedByUserId, newEvaluationResult.conditionalSelected));
            if (!newEvaluationResult.qualification && newEvaluationResult?.tipifications?.length) {
                newEvaluationResult.tipifications.forEach(t => {
                    evaluationResultPromises.push(mssql_service_1.MSSql.SaveEvaluationTipification(vcc, newEvaluationResult.abilityId, t.scoreId, t.id, t.isChecked, t.comment, interactionEvaluationPlanResult.interactionData.evaluationPlanId, interactionEvaluationPlanResult.interactionData.id, newEvaluationResult.idAuditor, userId));
                    if (t?.items?.length) {
                        t.items.forEach(ti => {
                            evaluationResultPromises.push(mssql_service_1.MSSql.SaveEvaluationTipificationItem(vcc, t.scoreId, ti.tipificationId, ti.id, ti.isChecked, interactionEvaluationPlanResult.interactionData.evaluationPlanId, interactionEvaluationPlanResult.interactionData.id, newEvaluationResult.idAuditor, userId));
                        });
                    }
                });
            }
        });
        return Promise.all(evaluationResultPromises);
    })
        .then(result => {
        if (!result || (result && result.includes(false)))
            throw 'SERVER_ERROR_EVALUATION_RESULTS_SAVE_FAILED';
        if (interactionEvaluationPlanResult && interactionEvaluationPlanResult.evaluationCustomFieldResults.length > 0) {
            interactionEvaluationPlanResult.evaluationCustomFieldResults.map(ecfr => {
                ecfr.results.map(r => {
                    evaluationCustomFieldResultPromises.push(mssql_service_1.MSSql.SaveEvaluationCustomFieldResult(vcc, evaluationPlanId, interactionId, userId, ecfr.agentId, ecfr.customFieldId, r, userId, userId));
                });
            });
            return Promise.all(evaluationCustomFieldResultPromises);
        }
        else {
            return Promise.resolve([true]);
        }
    })
        .then(data => {
        return mssql_service_1.MSSql.SaveInteractionEvaluationPlanResult(vcc, evaluationPlanId, userId, interactionId, interactionType, interactionEvaluationPlanResult.interactionScore, interactionEvaluationPlanResult.interactionAutomaticScore, interactionEvaluationPlanResult.interactionManualScore, interactionEvaluationPlanResult.isDraft, interactionEvaluationPlanResult.interactionData.evaluationTime);
    })
        .then(data => {
        return mssql_service_1.MSSql.DeleteAuditorEvaluationPlanByInteraction(vcc, evaluationPlanId, interactionId, interactionType, interactionEvaluationPlanResult.isDraft);
    })
        .then(data => {
        return mssql_service_1.MSSql.SaveEvaluationPlanCompleted(vcc, evaluationPlanId);
    })
        .then(data => {
        if (evaluationPlanMode === 'X' && !interactionEvaluationPlanResult.isDraft)
            return mssql_service_1.MSSql.DeleteEvaluationResultDraftBackup(vcc, evaluationPlanId, interactionId, userId);
        else
            return Promise.resolve(true);
    })
        .then(data => {
        if (ok)
            return Promise.resolve(data);
        else
            return Promise.reject(data);
    })
        .catch(err => {
        return Promise.reject(err);
    });
}
function GetRealtimeAbilityEvaluationFormat(vcc, evaluationPlanId) {
    let aefs = [];
    let needSaveOnRedis = false;
    let currentBlockPromise = null;
    let logLabel = `[EvaluationService::GetRealtimeAbilityEvaluationFormat] [vcc::evaluationPlanId => ${vcc}::${evaluationPlanId}]: `;
    return promises_shared_1.DummyPromise()
        .then(_ => {
        currentBlockPromise = 'BLOCK_SEARCHING_DATA_ON_REDIS';
        return redis_service_1.Redis.GetEvaluationPlanAbilities(vcc, evaluationPlanId);
    })
        .then(result => {
        if (result) {
            logger.info(`${logLabel}Data found on redis.`);
            aefs = result;
            return Promise.resolve(null);
        }
        else {
            currentBlockPromise = 'BLOCK_SEARCHING_DATA_ON_DB';
            logger.info(`${logLabel}Data not found on redis, searching in db.`);
            needSaveOnRedis = true;
            let process = new process_implementation_1.ProcessImplementation();
            return process.GetEvaluationPlanAbilities(vcc, evaluationPlanId);
        }
    })
        .then(result => {
        if (needSaveOnRedis) {
            currentBlockPromise = 'BLOCK_SAVING_DATA_ON_REDIS';
            logger.info(`${logLabel}Data found on db.`);
            aefs = result;
            return redis_service_1.Redis.SaveEvaluationPlanAbilities(vcc, evaluationPlanId, aefs);
        }
        return Promise.resolve(true);
    })
        .then(_ => {
        currentBlockPromise = 'BLOCK_RETURNING_RESULT';
        logger.info(`${logLabel}Returning ${aefs.length} items.`);
        return Promise.resolve(aefs);
    })
        .catch((err) => {
        let errorMsg = (typeof err === 'string' ? err : err.message || err.description || 'SERVER_ERROR_UNKNOWN_MESSAGE');
        logger.error(`${logLabel}Block: ${currentBlockPromise}, getting error: ${errorMsg}`);
        return Promise.reject(errorMsg);
    });
}
function SaveGenerativeUsage(vcc, evaluationPlanId, interactionId) {
    let currentBlockPromise = null;
    let logLabel = `[EvaluationService::SaveGenerativeUsage] [vcc::evaluationPlanId::interactionId => ${vcc}::${evaluationPlanId}::${interactionId}]: `;
    let appIds = [];
    let generativeUsages = null;
    let pengingAppIds = [];
    return promises_shared_1.DummyPromise()
        .then(_ => {
        currentBlockPromise = 'BLOCK_GETTING_USAGES_FROM_REDIS';
        return redis_service_1.Redis.GetGenerativeUsages(vcc, evaluationPlanId, interactionId);
    })
        .then(result => {
        currentBlockPromise = 'BLOCK_SAVING_ON_DATABASE';
        generativeUsages = result || {};
        let promiseArray = [];
        appIds = Object.keys(generativeUsages);
        appIds.forEach(ai => {
            promiseArray.push(mssql_service_1.MSSql.SaveGenerativeEngineUsage(vcc, evaluationPlanId, interactionId, ai, generativeUsages[ai]));
        });
        return Promise.all(promiseArray);
    })
        .then(result => {
        currentBlockPromise = 'BLOCK_DELETING_FROM_REDIS';
        let promiseArray = [];
        result.forEach((r, i) => {
            if (r)
                promiseArray.push(redis_service_1.Redis.DeleteGenerativeUsage(vcc, evaluationPlanId, interactionId, appIds[i]));
            else
                pengingAppIds.push(appIds[i]);
        });
        return Promise.all(promiseArray);
    })
        .then(_ => {
        currentBlockPromise = 'BLOCK_RELAUNCH_FUNCTION';
        if (pengingAppIds.length)
            return SaveGenerativeUsage(vcc, evaluationPlanId, interactionId);
        return Promise.resolve(true);
    })
        .catch((err) => {
        let errorMsg = (typeof err === 'string' ? err : err.message || err.description || 'SERVER_ERROR_UNKNOWN_MESSAGE');
        logger.error(`${logLabel}Block: ${currentBlockPromise}, getting error: ${errorMsg}`);
        return Promise.resolve(false);
    });
}
