"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.GenerativeService = void 0;
const moment = require("moment");
const log4js = require("log4js");
const mssql_service_1 = require("../db/mssql.service");
const promises_shared_1 = require("../shared/promises.shared");
const generative_service_implementation_1 = require("../implementation/generative-service.implementation");
const generative_model_1 = require("../models/generative.model");
const arrays_shared_1 = require("../shared/arrays.shared");
let logger = log4js.getLogger('ServerScripts');
function GenerativeService(app) {
    app.post('/api/generative/get_generatives/', (req, res, next) => {
        let vcc = res.locals.vcc;
        let logLabel = `[GenerativeService::get_generatives] [vcc => ${vcc}]: `;
        logger.info(`${logLabel}Received ${req.method} method on '${req.path}'`);
        let search_criteria = req.body.searchCriteria || '';
        let search_expression = req.body.searchExpression || '';
        let generatives = [];
        promises_shared_1.DummyPromise()
            .then(_ => {
            let generativeService = new generative_service_implementation_1.GenerativeServiceImplementation(vcc);
            return generativeService.GetAppsList();
        })
            .then(result => {
            if (!result)
                throw 'SERVER_ERROR_GET_APPS_LIST_FAILED';
            logger.info(`${logLabel}getting generatives from service:`, result.length);
            generatives = result;
            return mssql_service_1.MSSql.GetGeneratives(vcc, search_criteria, search_expression, null);
        })
            .then(result => {
            if (!result)
                throw 'SERVER_ERROR_GET_GENERATIVES_FAILED';
            logger.info(`${logLabel}getting generatives from database:`, result.length);
            generatives = result;
            if (res.headersSent)
                return;
            res.json({ status: true, description: 'SERVER_SUCCESS_MESSAGE', data: generatives });
        })
            .catch(err => {
            let clientMsg = (typeof err === 'string' ? err : 'SERVER_ERROR_GET_GENERATIVES_FAILED');
            let errorMsg = (typeof err === 'string' ? err : err.message || err.description || 'SERVER_ERROR_UNKNOWN_MESSAGE');
            logger.error(`${logLabel}Error on '${req.path}': ${errorMsg}`);
            logger.error(`${logLabel}getting errorMsg:`, errorMsg);
            if (res.headersSent)
                return;
            res.json({ status: false, description: clientMsg, error: clientMsg });
        });
    });
    app.get('/api/generative/get/:id', (req, res, next) => {
        let vcc = res.locals.vcc;
        let generative = new generative_model_1.Generative();
        generative.id = req.params.id;
        let logLabel = `[GenerativeService::get] [vcc::modelId => ${vcc}::${generative.id}]: `;
        logger.info(`${logLabel}Received ${req.method} method on '${req.path}'`);
        promises_shared_1.DummyPromise()
            .then(_ => {
            return mssql_service_1.MSSql.GetGenerative(vcc, 'id', generative.id);
        })
            .then(result => {
            if (!result)
                throw 'SERVER_ERROR_GET_GENERATIVE_FAILED';
            generative = result;
            if (res.headersSent)
                return;
            res.json({ status: true, description: 'SERVER_SUCCESS_MESSAGE', data: generative });
        })
            .catch(err => {
            let clientMsg = (typeof err === 'string' ? err : 'SERVER_ERROR_GET_FAILED');
            let errorMsg = (typeof err === 'string' ? err : err.message || err.description || 'SERVER_ERROR_UNKNOWN_MESSAGE');
            logger.error(`${logLabel}Error on '${req.path}': ${errorMsg}`);
            logger.error(`${logLabel}getting errorMsg:`, errorMsg);
            if (res.headersSent)
                return;
            res.json({ status: false, description: clientMsg, error: clientMsg });
        });
    });
    app.get('/api/generative/get_generative_by_name/:name', (req, res, next) => {
        let vcc = res.locals.vcc;
        let generative = new generative_model_1.Generative();
        generative.name = req.params.name;
        let logLabel = `[GenerativeService::get_generative_by_name] [vcc::modelName => ${vcc}::${generative.name}]: `;
        logger.info(`${logLabel}Received ${req.method} method on '${req.path}'`);
        promises_shared_1.DummyPromise()
            .then(_ => {
            return mssql_service_1.MSSql.GetGenerative(vcc, 'name', generative.name);
        })
            .then(result => {
            if (res.headersSent)
                return;
            res.json({ status: true, description: 'SERVER_SUCCESS_MESSAGE', data: result });
        })
            .catch(err => {
            let clientMsg = (typeof err === 'string' ? err : 'SERVER_ERROR_GET_BY_NAME_FAILED');
            let errorMsg = (typeof err === 'string' ? err : err.message || err.description || 'SERVER_ERROR_UNKNOWN_MESSAGE');
            logger.error(`${logLabel}Error on '${req.path}': ${errorMsg}`);
            logger.error(`${logLabel}getting errorMsg:`, errorMsg);
            if (res.headersSent)
                return;
            res.json({ status: false, description: clientMsg, error: clientMsg });
        });
    });
    app.post('/api/generative/create', (req, res, next) => {
        let vcc = res.locals.vcc;
        let userId = res.locals.userId;
        let generative = (req.body || {});
        generative.status = 'DRAFT';
        let logLabel = `[GenerativeService::create] [vcc::modelName => ${vcc}::${generative.name}]: `;
        logger.info(`${logLabel}Received ${req.method} method on '${req.path}'`);
        promises_shared_1.DummyPromise()
            .then(_ => {
            return mssql_service_1.MSSql.GetGenerative(vcc, 'name', generative.name);
        })
            .then(result => {
            if (result)
                throw 'SERVER_ERROR_GENERATIVE_NAME_ALREADY_EXISTS';
            return SaveGenerative(vcc, generative, userId);
        })
            .then(result => {
            if (!result)
                throw 'SERVER_ERROR_SAVE_GENERATIVE_FAILED';
            if (res.headersSent)
                return;
            res.json({ status: true, description: 'SERVER_SUCCESS_MESSAGE', data: generative });
        })
            .catch(err => {
            let clientMsg = (typeof err === 'string' ? err : 'SERVER_ERROR_CREATE_FAILED');
            let errorMsg = (typeof err === 'string' ? err : err.message || err.description || 'SERVER_ERROR_UNKNOWN_MESSAGE');
            logger.error(`${logLabel}Error on '${req.path}': ${errorMsg}`);
            logger.error(`${logLabel}getting errorMsg:`, errorMsg);
            if (res.headersSent)
                return;
            res.json({ status: false, description: clientMsg, error: clientMsg });
        });
    });
    app.post('/api/generative/save', (req, res, next) => {
        let vcc = res.locals.vcc;
        let userId = res.locals.userId;
        let generative = (req.body.generative || {});
        let publish = (req.body.publish || false);
        let generativeService = new generative_service_implementation_1.GenerativeServiceImplementation(vcc);
        let logLabel = `[GenerativeService::save] [vcc::modelId => ${vcc}::${generative.id}]: `;
        logger.info(`${logLabel}Received ${req.method} method on '${req.path}'`);
        promises_shared_1.DummyPromise()
            .then(_ => {
            return mssql_service_1.MSSql.GetGenerative(vcc, 'id', generative.id);
        })
            .then(result => {
            if (!result)
                throw 'SERVER_ERROR_GENERATIVE_NOT_FOUND';
            return SaveGenerative(vcc, generative, userId, result);
        })
            .then(result => {
            if (!result)
                throw 'SERVER_ERROR_SAVE_GENERATIVE_FAILED';
            if (['DRAFT'].includes(generative.status))
                return generativeService.AddApp(generative);
            else
                return generativeService.SaveApp(generative);
        })
            .then(result => {
            if (!result)
                throw 'SERVER_ERROR_ADD_APP_FAILED';
            let requestId = result;
            return generativeService.GetRequestById(requestId);
        })
            .then(result => {
            if (!result)
                throw 'SERVER_ERROR_GET_REQUEST_BY_ID_FAILED';
            let newGenerativeId = result;
            if (['DRAFT'].includes(generative.status))
                return SaveGenerativeId(vcc, generative, newGenerativeId, userId, generativeService);
            return Promise.resolve(generative);
        })
            .then(result => {
            if (!result)
                throw 'SERVER_ERROR_SAVE_GENERATIVE_ID_FAILED';
            if (publish)
                return PublishGenerative(vcc, generative, userId, generativeService);
            return Promise.resolve(true);
        })
            .then(result => {
            if (!result)
                throw 'SERVER_ERROR_PUBLISH_GENERATIVE_FAILED';
            if (res.headersSent)
                return;
            res.json({ status: true, description: 'SERVER_SUCCESS_MESSAGE', data: true });
        })
            .catch(err => {
            let clientMsg = (typeof err === 'string' ? err : 'SERVER_ERROR_CREATE_FAILED');
            let errorMsg = (typeof err === 'string' ? err : err.message || err.description || 'SERVER_ERROR_UNKNOWN_MESSAGE');
            logger.error(`${logLabel}Error on '${req.path}': ${errorMsg}`);
            logger.error(`${logLabel}getting errorMsg:`, errorMsg);
            if (res.headersSent)
                return;
            res.json({ status: false, description: clientMsg, error: clientMsg });
        });
    });
    app.get('/api/generative/delete/:id', (req, res, next) => {
        let vcc = res.locals.vcc;
        let userId = res.locals.userId;
        let generativeId = req.params.id;
        let currentBlockPromise = null;
        let logLabel = `[GenerativeService::delete] [vcc => ${vcc}]: `;
        logger.info(`${logLabel}Received ${req.method} method on '${req.path}'`);
        promises_shared_1.DummyPromise()
            .then(_ => {
            currentBlockPromise = 'BLOCK_SEARCHING_APP_ON_DB';
            return mssql_service_1.MSSql.GetGenerative(vcc, 'id', generativeId);
        })
            .then(result => {
            currentBlockPromise = 'BLOCK_DELETING_APP_ON_API';
            if (!result)
                throw 'SERVER_ERROR_GENERATIVE_NOT_FOUND';
            if (result.lastPublishedDateTime) {
                let generativeService = new generative_service_implementation_1.GenerativeServiceImplementation(vcc);
                return generativeService.DeleteAppById(generativeId);
            }
            else
                return Promise.resolve(true);
        })
            .then(_ => {
            currentBlockPromise = 'BLOCK_DELETING_APP_ON_DB';
            return mssql_service_1.MSSql.DeleteGenerative(vcc, generativeId, userId);
        })
            .then(result => {
            if (!result)
                throw 'SERVER_ERROR_DELETE_APP_ON_BD_FAILED';
            if (res.headersSent)
                return;
            res.json({ status: true, description: 'SERVER_SUCCESS_MESSAGE', data: true });
        })
            .catch(err => {
            let clientMsg = (typeof err === 'string' ? err : 'SERVER_ERROR_SERVICE_FAILED');
            let errorMsg = (typeof err === 'string' ? err : err.message || err.description || 'SERVER_ERROR_UNKNOWN_MESSAGE');
            logger.error(`${logLabel}Error on '${req.path}': ${errorMsg}`);
            logger.error(`${logLabel}Block: ${currentBlockPromise}`);
            if (res.headersSent)
                return;
            res.json({ status: false, description: clientMsg, error: clientMsg });
        });
    });
    app.get('/api/generative/get_generative_intent_items', (req, res, next) => {
        let vcc = res.locals.vcc;
        let logLabel = `[GenerativeService::get_generative_intent_items] [vcc => ${vcc}]: `;
        logger.info(`${logLabel}Received ${req.method} method on '${req.path}'`);
        promises_shared_1.DummyPromise()
            .then(_ => {
            return mssql_service_1.MSSql.GetGenerativeIntentItems(vcc);
        })
            .then(result => {
            if (!result)
                throw 'SERVER_ERROR_GET_GENERATIVE_INTENT_ITEMS_FAILED';
            logger.info(`${logLabel}getting generatives-itent-items from database:`, result.length);
            if (res.headersSent)
                return;
            res.json({ status: true, description: 'SERVER_SUCCESS_MESSAGE', data: result });
        })
            .catch(err => {
            let clientMsg = (typeof err === 'string' ? err : 'SERVER_ERROR_GET_GENERATIVE_INTENT_ITEMS_FAILED');
            let errorMsg = (typeof err === 'string' ? err : err.message || err.description || 'SERVER_ERROR_UNKNOWN_MESSAGE');
            logger.error(`${logLabel}Error on '${req.path}': ${errorMsg}`);
            logger.error(`${logLabel}getting errorMsg:`, errorMsg);
            if (res.headersSent)
                return;
            res.json({ status: false, description: clientMsg, error: clientMsg });
        });
    });
    app.post('/api/generative/get_unique_models_by_intents', (req, res, nex) => {
        let vcc = res.locals.vcc;
        let logLabel = `[GenerativeService::get_unique_models_by_intents] [vcc => ${vcc}]: `;
        logger.info(`${logLabel}Received ${req.method} method on '${req.path}'`);
        let templateId = (req.body?.templateId ? req.body.templateId : null);
        let analysisType = (req.body?.analysisType ? req.body.analysisType : null);
        let intents = (req.body?.intents ? req.body.intents : []);
        promises_shared_1.DummyPromise()
            .then(_ => {
            function getIntentNames(intents) {
                let intentNames = [];
                intents.forEach(i => {
                    if (i.includes('|'))
                        intentNames.push(i.split('|')[1]);
                    else
                        intentNames.push(i);
                });
                return intentNames.join(',');
            }
            return mssql_service_1.MSSql.GetUniqueModelsByIntents(vcc, templateId, analysisType, getIntentNames(intents));
        })
            .then((results) => {
            if (res.headersSent)
                return;
            res.json({ status: true, description: 'SERVER_SUCCESS_MESSAGE', data: results });
        })
            .catch((err) => {
            let clientMsg = (typeof err === 'string' ? err : 'CANNOT_PROCESS_REQUEST');
            let errorMsg = (typeof err === 'string' ? err : err.message || err.description || 'SERVER_ERROR_UNKNOWN_MESSAGE');
            logger.error(`${logLabel}Getting errorMsg:`, errorMsg, 'intents:', intents);
            if (res.headersSent)
                return;
            res.json({ status: false, description: clientMsg, error: errorMsg });
        });
    });
}
exports.GenerativeService = GenerativeService;
function SaveGenerative(vcc, generative, currentUser, oldGenerative) {
    let logLabel = `[GenerativeService::SaveGenerative] [vcc::modelName::user => ${vcc}::${generative.name}::${currentUser}]: `;
    logger.info(`${logLabel}appId: ${generative.id}`);
    oldGenerative ?? (oldGenerative = new generative_model_1.GenerativeExtended());
    let intentsSummary = null;
    return promises_shared_1.DummyPromise()
        .then(_ => {
        if (generative.status === 'PUBLISHED'
            &&
                generative.intents.length) {
            generative.needPublish = true;
            generative.status = 'NEED_PUBLISH';
        }
        return mssql_service_1.MSSql.SaveGenerative(vcc, generative, currentUser);
    })
        .then(result => {
        if (!result)
            throw 'SERVER_ERROR_SAVE_GENERATIVE_FAILED';
        let promiseArray = [];
        intentsSummary = arrays_shared_1.GetCompareItemsFromObjectArray(oldGenerative.intents, generative.intents, 'id');
        intentsSummary.old.forEach(i => {
            i.appId ?? (i.appId = generative.id);
            promiseArray.push(mssql_service_1.MSSql.SaveGenerativeIntent(vcc, i, currentUser));
        });
        intentsSummary.new.forEach(i => {
            i.appId ?? (i.appId = generative.id);
            promiseArray.push(mssql_service_1.MSSql.SaveGenerativeIntent(vcc, i, currentUser));
        });
        intentsSummary.removed.forEach(i => {
            i.appId ?? (i.appId = generative.id);
            promiseArray.push(mssql_service_1.MSSql.DeleteGenerativeIntent(vcc, i, currentUser));
        });
        return Promise.all(promiseArray);
    })
        .then(result => {
        if (result.filter(r => !r).length)
            throw 'SERVER_ERROR_AT_LEAST_ONE_SAVE_INTENT_FAILED';
        let promiseArray = [];
        generative.intents.forEach(i => {
            let intent = oldGenerative.intents.find(oi => oi.id === i.id);
            if (intent) {
                let examplesSummary = arrays_shared_1.GetCompareItemsFromObjectArray(intent.examples, i.examples, 'id');
                examplesSummary.new.forEach(e => {
                    e.appId ?? (e.appId = generative.id);
                    e.intentId ?? (e.intentId = i.id);
                    promiseArray.push(mssql_service_1.MSSql.SaveGenerativeExample(vcc, e, currentUser));
                });
                examplesSummary.removed.forEach(e => {
                    e.appId ?? (e.appId = generative.id);
                    e.intentId ?? (e.intentId = i.id);
                    promiseArray.push(mssql_service_1.MSSql.DeleteGenerativeExample(vcc, e, currentUser));
                });
            }
            else
                i.examples.forEach(e => {
                    e.appId ?? (e.appId = generative.id);
                    e.intentId ?? (e.intentId = i.id);
                    promiseArray.push(mssql_service_1.MSSql.SaveGenerativeExample(vcc, e, currentUser));
                });
        });
        return Promise.all(promiseArray);
    })
        .then(result => {
        if (result.filter(r => !r).length)
            throw 'SERVER_ERROR_AT_LEAST_ONE_SAVE_EXAMPLE_FAILED';
        return Promise.resolve(true);
    })
        .catch((err) => {
        let errorMsg = (typeof err === 'string' ? err : err.message || err.description || 'SERVER_ERROR_UNKNOWN_MESSAGE');
        logger.error(`${logLabel}getting errorMsg:`, errorMsg);
        return Promise.resolve(false);
    });
}
function SaveGenerativeId(vcc, generative, id, currentUser, generativeService) {
    let logLabel = `[GenerativeService::SaveGenerativeId] [vcc::modelName::user => ${vcc}::${generative.name}::${currentUser}]: `;
    logger.info(`${logLabel} oldAppId::newAppId => ${generative.id}::${id}`);
    generativeService ?? (generativeService = new generative_service_implementation_1.GenerativeServiceImplementation(vcc));
    let generativeServiceModel = null;
    return promises_shared_1.DummyPromise()
        .then(_ => {
        return generativeService.GetAppById(id);
    })
        .then(result => {
        if (!result)
            throw 'SERVER_ERROR_GET_APP_BY_ID_FAILED';
        generativeServiceModel = result;
        return mssql_service_1.MSSql.SaveGenerativeId(vcc, generative.id, id, currentUser);
    })
        .then(result => {
        if (!result)
            throw 'SERVER_ERROR_SAVE_GENERATIVE_ID_FAILED';
        generative.id = id;
        return Promise.resolve(generative);
    })
        .catch((err) => {
        let errorMsg = (typeof err === 'string' ? err : err.message || err.description || 'SERVER_ERROR_UNKNOWN_MESSAGE');
        logger.error(`${logLabel}getting errorMsg:`, errorMsg);
        return Promise.resolve(null);
    });
}
function PublishGenerative(vcc, generative, currentUser, generativeService) {
    let logLabel = `[GenerativeService::PublishGenerative] [vcc::modelName::user => ${vcc}::${generative.name}::${currentUser}]: `;
    generativeService ?? (generativeService = new generative_service_implementation_1.GenerativeServiceImplementation(vcc));
    return promises_shared_1.DummyPromise()
        .then(_ => {
        return generativeService.PublishVersion(generative.id);
    })
        .then(result => {
        if (!result)
            throw 'SERVER_ERROR_PUBLISH_VERSION_FAILED';
        generative.status = 'PUBLISHED';
        generative.isPublished = true;
        generative.needPublish = false;
        generative.lastPublishedDate = moment().utc().toDate();
        return mssql_service_1.MSSql.SaveGenerative(vcc, generative, currentUser);
    })
        .then(result => {
        if (!result)
            throw 'SERVER_ERROR_SAVE_GENERATIVE_FAILED';
        return Promise.resolve(true);
    })
        .catch((err) => {
        let errorMsg = (typeof err === 'string' ? err : err.message || err.description || 'SERVER_ERROR_UNKNOWN_MESSAGE');
        logger.error(`${logLabel}getting errorMsg:`, errorMsg);
        return Promise.resolve(false);
    });
}
