"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.SaveProcess = exports.CreateScheduleForProcess = exports.CreateProcess = exports.GetProcessChannelInteractions = exports.GetProcess = exports.ProcessService = void 0;
const moment = require("moment");
const path = require("path");
const log4js = require("log4js");
const redis_service_1 = require("../db/redis.service");
let alasql = require('alasql');
let csv = require('csvtojson');
const mssql_service_1 = require("../db/mssql.service");
const file_manager_1 = require("../shared/file-manager");
const promises_shared_1 = require("../shared/promises.shared");
const config_1 = require("../config/config");
const linguistic_service_1 = require("./linguistic.service");
const audio_mssql_1 = require("../db/mssql/audio.mssql");
const billing_batch_implementation_1 = require("../implementation/billing-batch.implementation");
const interaction_service_1 = require("./interaction.service");
const linguistic_service_2 = require("./linguistic.service");
const platform_service_1 = require("./platform.service");
const arrays_shared_1 = require("../shared/arrays.shared");
const batch_schedule_service_1 = require("./batch-schedule.service");
const utilities_1 = require("../shared/utilities");
let logger = log4js.getLogger('ServerScripts');
function ProcessService(app) {
    app.get('/api/process/get_process_available_languages', (req, res, next) => {
        logger.info('[ProcessService::get_process_available_languages] Received ' + req.method + ' method on \'' + req.path + '\'');
        let vcc = res.locals.vcc;
        logger.info('[ProcessService::get_process_available_languages] getting vcc:', vcc);
        promises_shared_1.DummyPromise()
            .then(result => {
            let cultures = [];
            cultures.push({ name: "English", code: "en" });
            cultures.push({ name: "German", code: "de" });
            cultures.push({ name: "Spanish", code: "es" });
            cultures.push({ name: "French", code: "fr" });
            cultures.push({ name: "Japanese", code: "ja" });
            cultures.push({ name: "Korean", code: "ko" });
            cultures.push({ name: "Dutch", code: "nl" });
            cultures.push({ name: "Portuguese", code: "pt" });
            res.json({
                status: true,
                description: 'SERVER_SUCCESS_MESSAGE',
                data: cultures.filter(c => config_1.TRANSCRIPTION_AVAILABLE_LANGUAGES.includes(c.code))
            });
        }).catch(err => {
            let clientMsg = (typeof err === 'string' ? err : 'CANNOT_PROCESS_REQUEST');
            let errorMsg = (typeof err === 'string' ? err : err.message || err.description || 'SERVER_ERROR_UNKNOWN_MESSAGE');
            logger.error('Error al procesar \'' + req.path + '\': ' + errorMsg);
            logger.error('[ProcessService::get_process_available_languages] getting errorMsg:', errorMsg);
            if (res.headersSent)
                return;
            res.json({ status: false, description: clientMsg, error: errorMsg });
        });
    });
    app.post('/api/process/get_processes', (req, res, next) => {
        logger.info('[ProcessService::get_processes] Received ' + req.method + ' on \'' + req.path + '\'');
        let vcc = res.locals.vcc;
        let process = (req.body || {});
        promises_shared_1.DummyPromise()
            .then(result => {
            return mssql_service_1.MSSql.GetProcesses(vcc, process.searchCriteria, process.searchExpression, process.startDate, process.endDate, process.type, process.enviroment, process.language);
        })
            .then((processes) => {
            logger.info('[ProcessService::get_processes] Getting processes:', processes.length, 'after MSSql.GetProcesses');
            if (res.headersSent)
                return;
            res.json({ status: true, description: 'SERVER_SUCCESS_MESSAGE', data: processes });
        })
            .catch(err => {
            let processMsg = (typeof err === 'string' ? err : 'CANNOT_PROCESS_REQUEST');
            let errorMsg = (typeof err === 'string' ? err : err.message || err.description || 'SERVER_ERROR_UNKNOWN_MESSAGE');
            logger.error('Error al procesar \'' + req.path + '\': ' + errorMsg);
            logger.error('[ProcessService::get_processes] Getting errorMsg:', errorMsg);
            if (res.headersSent)
                return;
            res.json({ status: false, description: processMsg, error: errorMsg });
        });
    });
    app.get('/api/process/get_process/:process_id', (req, res, next) => {
        logger.info('[ProcessService::get_process] Received ' + req.method + ' on \'' + req.path + '\'');
        let vcc = res.locals.vcc;
        logger.info('[ProcessService::get_process] Getting vcc:', vcc);
        let processId = req.params.process_id;
        logger.info('[ProcessService::get_process] Getting processId:', processId);
        promises_shared_1.DummyPromise()
            .then(result => {
            if (!processId)
                throw 'MISSING_PROCESS_INPUT_PARAMTER';
            return mssql_service_1.MSSql.GetProcess(vcc, processId);
        })
            .then((resultProcess) => {
            logger.info('[ProcessService::get_process] Getting resultProcess after MSSql.GetProcess');
            if (res.headersSent)
                return;
            res.json({ status: true, description: 'SERVER_SUCCESS_MESSAGE', data: resultProcess });
        })
            .catch(err => {
            let processMsg = (typeof err === 'string' ? err : 'CANNOT_PROCESS_REQUEST');
            let errorMsg = (typeof err === 'string' ? err : err.message || err.description || 'SERVER_ERROR_UNKNOWN_MESSAGE');
            logger.error('Error al procesar \'' + req.path + '\': ' + errorMsg);
            logger.error('[ProcessService::get_process] Getting errorMsg:', errorMsg);
            if (res.headersSent)
                return;
            res.json({ status: false, description: processMsg, error: errorMsg });
        });
    });
    app.get('/api/process/get_process_filters_data/:processId/:evaluationFormatId', (req, res, next) => {
        logger.info(`[ProcessService::get_process_filters_data] Received ${req.method} on '${req.path}'`);
        let vcc = res.locals.vcc;
        let processId = req.params.processId;
        let evaluationFormatId = req.params.evaluationFormatId !== 'null' ? req.params.evaluationFormatId : null;
        promises_shared_1.DummyPromise()
            .then(_ => {
            if (!processId)
                throw 'MISSING_PROCESS_INPUT_PARAMTER';
            return mssql_service_1.MSSql.GetProcessFiltersData(vcc, processId, evaluationFormatId);
        })
            .then(result => {
            logger.info(`[ProcessService::get_process_filters_data] Getting resultProcess after MSSql.GetProcessFiltersData`);
            if (res.headersSent)
                return;
            res.json({ status: true, description: 'SERVER_SUCCESS_MESSAGE', data: result });
        })
            .catch(err => {
            let processMsg = (typeof err === 'string' ? err : 'CANNOT_PROCESS_REQUEST');
            let errorMsg = (typeof err === 'string' ? err : err.message || err.description || 'SERVER_ERROR_UNKNOWN_MESSAGE');
            logger.error('[ProcessService::get_process] Getting errorMsg:', errorMsg);
            if (res.headersSent)
                return;
            res.json({ status: false, description: processMsg, error: errorMsg });
        });
    });
    app.get('/api/process/get_interactions_result/:process_id', (req, res, next) => {
        logger.info('[ProcessService::get_interactions_result] Received ' + req.method + ' method on \'' + req.path + '\'');
        let vcc = res.locals.vcc;
        let processId = req.params.process_id;
        promises_shared_1.DummyPromise()
            .then(_ => {
            return GetProcessChannelInteractions(vcc, processId);
        })
            .then((result) => {
            logger.info('[ProcessService::get_interactions_result] Getting result:', result);
            if (res.headersSent)
                return;
            res.json({ status: true, description: 'SERVER_SUCCESS_MESSAGE', data: result });
        })
            .catch((err) => {
            let processMsg = (typeof err === 'string' ? err : 'CANNOT_PROCESS_REQUEST');
            let errorMsg = (typeof err === 'string' ? err : err.message || err.description || 'SERVER_ERROR_UNKNOWN_MESSAGE');
            logger.error('[ProcessService::get_interactions_result] Getting errorMsg:', errorMsg);
            if (res.headersSent)
                return;
            res.json({ status: false, description: processMsg, error: errorMsg });
        });
    });
    app.post('/api/process/create_process', (req, res, next) => {
        logger.info('[ProcessService::create_process] Received ' + req.method + ' method on \'' + req.path + '\'');
        let vcc = res.locals.vcc;
        let vccType = res.locals.type;
        let userId = res.locals.userId;
        let newProcess = (req.body || {});
        if (newProcess.percentage > 100)
            newProcess.percentage = newProcess.percentage % 100;
        if (newProcess.percentage > 0)
            newProcess.percentage = newProcess.percentage / 100;
        newProcess.name = (newProcess.name || '').toLowerCase();
        logger.info('[ProcessService::create_process] Getting vcc::userId::process => ', vcc, '::', userId, '::', newProcess.name);
        newProcess.queryDataIdDate = (newProcess.queryDataIdDate ? new Date(newProcess.queryDataIdDate) : null);
        newProcess.startDate = (newProcess.startDate ? new Date(newProcess.startDate) : null);
        newProcess.endDate = (newProcess.endDate ? new Date(newProcess.endDate) : null);
        newProcess.minDurationTime = (newProcess.minDurationTime ? newProcess.minDurationTime * 1 : 0);
        newProcess.maxDurationTime = (newProcess.maxDurationTime ? newProcess.maxDurationTime * 1 : 0);
        let params = {};
        params = interaction_service_1.GetInteractionSearchFromRequest(null, req.body.interactionSearch, false);
        params.searchType = 'TO_OPERATE';
        promises_shared_1.DummyPromise()
            .then(_ => {
            if (newProcess.scheduleData) {
                return CreateScheduleForProcess(vcc, vccType, newProcess, params, userId);
            }
            else {
                return CreateProcess(vcc, vccType, newProcess, params, userId);
            }
        })
            .then(result => {
            logger.info('[ProcessService::create_process] Getting result:', result, 'after delete Ftp of Redis');
            if (res.headersSent)
                return;
            res.json({ status: true, description: (result.status ? 'SERVER_SUCCESS_MESSAGE' : result.saveResult ? result.saveResult : 'PROCESS_SAVE_FAILED'), data: result.status });
        })
            .catch(err => {
            let processMsg = (typeof err === 'string' ? err : 'CANNOT_PROCESS_REQUEST');
            let errorMsg = (typeof err === 'string' ? err : err.message || err.description || 'SERVER_ERROR_UNKNOWN_MESSAGE');
            logger.error('Error al procesar \'' + req.path + '\': ' + errorMsg);
            logger.error('[ProcessService::create_process] Getting errorMsg:', errorMsg);
            if (res.headersSent)
                return;
            res.json({
                status: false,
                description: processMsg,
                error: errorMsg,
                detail: {
                    type: 'ERROR',
                    datetime: moment().utc().toDate(),
                    pid: process.pid,
                    source: __filename,
                    stack: err.stack
                }
            });
        });
    });
    app.post('/api/process/save_process', (req, res, next) => {
        logger.info('Se recibió ' + req.method + ' para \'' + req.path + '\'');
        let vcc = res.locals.vcc;
        logger.info('[ProcessService::/api/process/save_process] Getting vcc:', vcc);
        let userId = res.locals.userId;
        logger.info('[ProcessService::/api/process/save_process] Getting userId:', userId);
        let nowUtc = moment().utc().toDate();
        logger.info('[ProcessService::/api/process/save_process] Getting nowUtc:', nowUtc);
        let newProcess = (req.body.process || {});
        logger.info('[ProcessService::/api/process/save_process] Getting newProcess:', newProcess);
        let tempFolder = (req.body.tempFolder || '');
        logger.info('[ProcessService::/api/process/save_process] Getting tempFolder:', tempFolder);
        newProcess.createdDate = null;
        newProcess.createdByUserId = null;
        newProcess.lastModifiedDate = nowUtc;
        newProcess.lastModifiedByUserId = userId;
        newProcess.startDate = (newProcess.startDate ? new Date(newProcess.startDate) : null);
        newProcess.endDate = (newProcess.endDate ? new Date(newProcess.endDate) : null);
        newProcess.queryDataIdDate = (newProcess.queryDataIdDate ? new Date(newProcess.queryDataIdDate) : null);
        newProcess.minDurationTime = (newProcess.minDurationTime ? newProcess.minDurationTime * 1 : 0);
        newProcess.maxDurationTime = (newProcess.maxDurationTime ? newProcess.maxDurationTime * 1 : 0);
        logger.info('[ProcessService::/api/process/save_process] Getting newProcess:', newProcess);
        promises_shared_1.DummyPromise()
            .then(result => {
            if (newProcess.status === 'COMPLETE' && newProcess.updateDurationTimeRate && newProcess.maxDurationTime > newProcess.minDurationTime) {
                return SaveDurationTimeRate(vcc, newProcess, userId);
            }
            return Promise.resolve(true);
        })
            .then(resultSaveDurationTimeRate => {
            logger.info('[ProcessService::/api/process/save_process] Getting resultSaveDurationTimeRate:', resultSaveDurationTimeRate, 'after SaveDurationTimeRate');
            return Promise.resolve(true);
        })
            .then(resultDeleteProcess => {
            if (!resultDeleteProcess)
                throw 'ERROR_SAVING_PROCESS';
            return SaveProcess(vcc, newProcess);
        })
            .then((resultSaveProcess) => {
            logger.info('[ProcessService::/api/process/save_process] Getting resultSaveProcess:', resultSaveProcess, 'after SaveProcess');
            if (!resultSaveProcess)
                throw 'ERROR_SAVING_PROCESS';
            return Promise.resolve(true);
        })
            .then(resultSaveProcess => {
            if (res.headersSent)
                return;
            res.json({ status: resultSaveProcess, description: (resultSaveProcess ? 'SERVER_SUCCESS_MESSAGE' : 'PROCESS_SAVE_FAILED'), data: resultSaveProcess });
        })
            .catch(err => {
            let processMsg = (typeof err === 'string' ? err : 'CANNOT_PROCESS_REQUEST');
            let errorMsg = (typeof err === 'string' ? err : err.message || err.description || 'SERVER_ERROR_UNKNOWN_MESSAGE');
            logger.error('Error al procesar \'' + req.path + '\': ' + errorMsg);
            logger.error('[ProcessService::/api/process/save_process] Getting errorMsg:', errorMsg);
            if (res.headersSent)
                return;
            res.json({ status: false, description: processMsg, error: errorMsg, detail: { type: 'ERROR', datetime: moment().utc().toDate(), pid: process.pid, source: __filename, stack: err.stack } });
        });
    });
    app.post('/api/process/delete_process', (req, res, next) => {
        logger.info('[ProcessService::delete_process] Received ' + req.method + ' method on \'' + req.path + '\'');
        let vcc = res.locals.vcc;
        logger.info('[ProcessService::delete_process] Getting vcc:', vcc);
        let process = req.body.process;
        logger.info('[ProcessService::delete_process] Getting process:', process);
        let processId = process.name;
        logger.info(`[ProcessService::delete_process] Deleting process ${processId} by the user ${res.locals.userId}`);
        promises_shared_1.DummyPromise()
            .then(result => {
            if (!process)
                throw 'MISSING_PROCESS_INPUT_PARAMTER';
            return mssql_service_1.MSSql.DeleteProcess(vcc, processId);
        })
            .then((resultDeleteProcess) => {
            logger.info('[ProcessService::/api/process/delete_process] Getting resultDeleteProcess:', resultDeleteProcess, 'MSSql.DeleteProcess');
            if (!resultDeleteProcess)
                throw 'ERROR_DELETING_PROCESS';
            return Promise.resolve(true);
        })
            .then(resultDeleteProcess => {
            if (res.headersSent)
                return;
            res.json({ status: resultDeleteProcess, description: (resultDeleteProcess ? 'SERVER_SUCCESS_MESSAGE' : 'PROCESS_DELETE_FAILED'), data: resultDeleteProcess });
        })
            .catch(err => {
            let processMsg = (typeof err === 'string' ? err : 'CANNOT_PROCESS_REQUEST');
            let errorMsg = (typeof err === 'string' ? err : err.message || err.description || 'SERVER_ERROR_UNKNOWN_MESSAGE');
            logger.error('Error al procesar \'' + req.path + '\': ' + errorMsg);
            logger.error('[ProcessService::/api/process/delete_process] Getting errorMsg:', errorMsg);
            if (res.headersSent)
                return;
            res.json({ status: false, description: processMsg, error: errorMsg });
        });
    });
    app.post('/api/process/delete_processes', (req, res, next) => {
        logger.info('Se recibió ' + req.method + ' para \'' + req.path + '\'');
        let vcc = res.locals.vcc;
        logger.info('[ProcessService::/api/process/delete_processes] Getting vcc:', vcc);
        let processes = req.body.processes || [];
        logger.info(`[ProcessService::delete_processes] Deleting processes ${processes.map(x => x.name).join(',')} by the user ${res.locals.userId}`);
        promises_shared_1.DummyPromise()
            .then(result => {
            if (!processes)
                throw 'MISSING_PROCESSES_INPUT_PARAMETER';
            let promiseArray = [];
            processes.forEach(process => {
                promiseArray.push(mssql_service_1.MSSql.DeleteProcess(vcc, process.name));
            });
            return Promise.all(promiseArray);
        })
            .then((resultDeleteProcesses) => {
            logger.info('[ProcessService::/api/process/delete_processes] Getting resultDeleteProcesses:', resultDeleteProcesses);
            let result;
            if (resultDeleteProcesses.filter(result => { return (result !== true); }).length > 0) {
                result = false;
                logger.error('[ProcessService::/api/process/delete_processes] could not delete all requested items');
            }
            else {
                result = true;
                logger.info('[ProcessService::/api/process/delete_processes] Processes successfully deleted');
            }
            return result;
        })
            .then(resultDeleteProcesses => {
            if (res.headersSent)
                return;
            res.json({ status: resultDeleteProcesses, description: (resultDeleteProcesses ? 'SERVER_SUCCESS_MESSAGE' : 'SERVER_ERROR_UNKNOWN_MESSAGE'), data: resultDeleteProcesses });
        })
            .catch(err => {
            let processMsg = (typeof err === 'string' ? err : 'CANNOT_PROCESS_REQUEST');
            let errorMsg = (typeof err === 'string' ? err : err.message || err.description || 'SERVER_ERROR_UNKNOWN_MESSAGE');
            logger.error('Error al procesar \'' + req.path + '\': ' + errorMsg);
            logger.error('[ProcessService::/api/process/delete_processes] Getting errorMsg:', errorMsg);
            if (res.headersSent)
                return;
            res.json({ status: false, description: processMsg, error: errorMsg });
        });
    });
    app.get('/api/process/get_process_usage/:name', (req, res, next) => {
        logger.info('[ProcessService::/api/process/get_process_usage/] Received ' + req.method + ' method on \'' + req.path + '\'');
        let vcc = res.locals.vcc;
        let name = req.params.name;
        logger.info('[ProcessService::/api/process/get_process_usage/] Received vcc: ' + vcc, 'name:', name);
        promises_shared_1.DummyPromise()
            .then(result => {
            if (!name)
                throw 'MISSING_NAME_PARAMETER';
            logger.info('[ProcessService::/api/process/get_process_usage/] Calling GetProcessUsage SP');
            return mssql_service_1.MSSql.GetProcessUsage(vcc, name);
        }).then((result) => {
            if (res.headersSent)
                return;
            logger.info('[ProcessService::/api/process/get_process_usage/] Received result from GetProcessUsage SP: ' + result);
            res.json({ status: true, description: 'SERVER_SUCCESS_MESSAGE', data: result });
        }).catch(err => {
            let clientMsg = (typeof err === 'string' ? err : 'CANNOT_PROCESS_REQUEST');
            let errorMsg = (typeof err === 'string' ? err : err.message || err.description || 'Requesting error');
            logger.error('[ProcessService::/api/process/get_process_usage/] Error on \'' + req.path + '\': ' + errorMsg);
            if (res.headersSent)
                return;
            res.json({ status: false, description: clientMsg, error: errorMsg });
        });
    });
    app.post('/api/process/get_process_duration', (req, res, next) => {
        logger.info('[ProcessService::get_process_duration] Received ' + req.method + ' method on \'' + req.path + '\'');
        let vcc = res.locals.vcc;
        let processId = req.body.processId === '' ? null : req.body.processId;
        logger.info('[ProcessService::get_process_duration] Received vcc: ' + vcc, 'processId:', name);
        promises_shared_1.DummyPromise()
            .then(result => {
            if (!processId)
                throw 'MISSING_NAME_PARAMETER';
            logger.info('[ProcessService::get_process_duration] Calling GetProcessUsage SP');
            return mssql_service_1.MSSql.GetProcessValidDurationTime(vcc, processId);
        })
            .then((result) => {
            if (res.headersSent)
                return;
            logger.info('[ProcessService::get_process_duration] Received result from GetProcessUsage SP: ' + result);
            res.json({ status: true, description: 'SERVER_SUCCESS_MESSAGE', data: result });
        })
            .catch(err => {
            let clientMsg = (typeof err === 'string' ? err : 'CANNOT_PROCESS_REQUEST');
            let errorMsg = (typeof err === 'string' ? err : err.message || err.description || 'Requesting error');
            logger.error('[ProcessService::get_process_duration] Error on \'' + req.path + '\': ' + errorMsg);
            if (res.headersSent)
                return;
            res.json({ status: false, description: clientMsg, error: errorMsg });
        });
    });
    app.get('/api/process/audios/get_audios_statistic/:processId', (req, res, next) => {
        logger.info('Se recibió ' + req.method + ' para \'' + req.path + '\'');
        let vcc = res.locals.vcc;
        logger.info('[ProcessService::/api/process/audios/get_audios_statistic/] Getting vcc:', vcc);
        let processId = req.params.processId;
        logger.info('[ProcessService::/api/process/audios/get_audios_statistic/] Getting processId:', processId);
        promises_shared_1.DummyPromise()
            .then(result => {
            return mssql_service_1.MSSql.GetProcessSummary(vcc, processId);
        })
            .then((result) => {
            logger.info('[ProcessService::/api/process/audios/get_audios_statistic/] Getting result:', result, 'after MSSql.GetProcessSummary');
            if (res.headersSent)
                return;
            res.json({ status: true, description: 'SERVER_SUCCESS_MESSAGE', data: result });
        })
            .catch(err => {
            let clientMsg = (typeof err === 'string' ? err : 'CANNOT_PROCESS_REQUEST');
            let errorMsg = (typeof err === 'string' ? err : err.message || err.description || 'SERVER_ERROR_UNKNOWN_MESSAGE');
            logger.error('Error al procesar \'' + req.path + '\': ' + errorMsg);
            logger.error('[ProcessService::/api/process/audios/get_audios_statistic/] Getting errorMsg:', errorMsg);
            if (res.headersSent)
                return;
            res.json({ status: false, description: clientMsg, error: errorMsg });
        });
    });
    app.post('/api/process/audios/process', (req, res, next) => {
        logger.info('Se recibió ' + req.method + ' para \'' + req.path + '\'');
        let vcc = res.locals.vcc;
        logger.info('[ProcessService::/api/process/audios/process] Getting vcc:', vcc);
        let userId = res.locals.userId;
        logger.info('[ProcessService::/api/process/audios/process] Getting userId:', userId);
        let newProcessAudio = (req.body || {});
        logger.info('[ProcessService::/api/process/audios/process] Getting newProcessAudio:', newProcessAudio);
        promises_shared_1.DummyPromise()
            .then(result => {
            return mssql_service_1.MSSql.SaveProcessStatus(vcc, newProcessAudio.id, 'COMPLETE', moment().utc().toDate(), userId);
        })
            .then((result) => {
            logger.info('[ProcessService::/api/process/audios/process] Getting result:', result, 'after MSSql.SaveProcessStatus');
            if (res.headersSent)
                return;
            result = 'COMPLETE';
            res.json({ status: true, description: 'SERVER_SUCCESS_MESSAGE', data: result });
        })
            .catch(err => {
            let clientMsg = (typeof err === 'string' ? err : 'CANNOT_PROCESS_REQUEST');
            let errorMsg = (typeof err === 'string' ? err : err.message || err.description || 'SERVER_ERROR_UNKNOWN_MESSAGE');
            logger.error('Error al procesar \'' + req.path + '\': ' + errorMsg);
            logger.error('[ProcessService::/api/process/audios/process] Getting errorMsg:', errorMsg);
            if (res.headersSent)
                return;
            res.json({ status: false, description: clientMsg, error: errorMsg });
        });
    });
    app.post('/api/process/upload_file/:batch_type_name?/:upload_dest?', (req, res, next) => {
        logger.info('[ProcessService::upload_file] Received ' + req.method + ' method on \'' + req.path + '\'');
        let isReprocess = req.path.search("RE_") === -1 ? false : true;
        let vcc = res.locals.vcc;
        let userId = res.locals.userId;
        let nowUtc = moment().utc().toDate();
        let uploadToFolder = req.params.upload_dest || '';
        let ftpCSVRedisKey = req.params.batch_type_name ? ('FTP_' + req.params.batch_type_name + ':') : null;
        let tempFolderName;
        let tempZipFileName;
        let tempUploadPath;
        let resultUpload = false;
        tempFolderName = (uploadToFolder ? uploadToFolder : (config_1.PROCESS_TEMP_PREFIX + '-' + vcc + '-' + userId).toLowerCase() + '-' + nowUtc.valueOf());
        tempZipFileName = tempFolderName + '.csv';
        tempUploadPath = (uploadToFolder ? uploadToFolder : config_1.PROCESS_TEMP_PATH);
        promises_shared_1.DummyPromise()
            .then(result => {
            if (!ftpCSVRedisKey)
                throw 'SERVER_ERROR_FTP_TARGET_NOT_FOUND';
            logger.info('[ProcessService::upload_file] Getting tempFolderName:', tempFolderName, 'with tempZipFileName:', tempZipFileName, 'with tempUploadPath:', tempUploadPath);
            return file_manager_1.ExistInFileSystem(tempUploadPath);
        })
            .then((result) => {
            if (result === false) {
                return file_manager_1.CreateFolder(tempUploadPath);
            }
            else {
                return true;
            }
        })
            .then((result) => {
            if (result) {
                return file_manager_1.UploadFileToServer(tempZipFileName, tempUploadPath, req, res, 'csv');
            }
            else {
                return false;
            }
        })
            .then((resultUpload) => {
            logger.info('[ProcessService::upload_file] Getting resultUpload:', resultUpload, 'after UploadFileToServer');
            if (!resultUpload)
                throw 'COULD_NOT_UPLOAD_FILE';
            let fileName = `${tempUploadPath}/${tempZipFileName}`;
            logger.info('[ProcessService::upload_file] Getting fileName:', fileName);
            return file_manager_1.ReadFileContent(fileName, 'utf-8');
        })
            .then((result) => {
            logger.info('[ProcessService::upload_file] Getting result from file');
            let csvText = result;
            if (csvText && csvText.length > 0) {
                let json = csvJSON(csvText, isReprocess);
                logger.info('[ProcessService::upload_file] Getting json after csvJSON');
                ftpCSVRedisKey = (isReprocess) ? 'FTP_' + req.params.batch_type_name.replace("RE_", "") + ':' : ftpCSVRedisKey;
                let response = false;
                if (json && json.length > 0) {
                    json = JSON.parse(json);
                    response = redis_service_1.Redis.SaveCsvProcess(vcc, (ftpCSVRedisKey + userId), JSON.stringify(json).toString());
                }
                return Promise.resolve(response);
            }
            else {
                throw 'EMPTY_FILE';
            }
        })
            .then((result) => {
            logger.info('[ProcessService::upload_file] Getting result:', result, 'after ReadFileContent');
            if (result === true) {
                let tempCSVFile = `${tempUploadPath}/${tempZipFileName}`;
                logger.info('[ProcessService::upload_file] Getting tempCSVFile');
                return file_manager_1.DeleteFile(tempCSVFile);
            }
            else {
                return Promise.resolve(false);
            }
        })
            .then(result => {
            logger.info('[ProcessService::upload_file] Getting resultUpload:', resultUpload, 'after DeleteFile');
            resultUpload = result;
            return Promise.resolve(result);
        })
            .then(result => {
            if (result === true) {
                return redis_service_1.Redis.GetCsvProcess(vcc, (ftpCSVRedisKey + userId));
            }
            else {
                return Promise.resolve([]);
            }
        })
            .then(result => {
            if (res.headersSent)
                return;
            res.json({ status: resultUpload, description: (resultUpload ? 'FILE_STORED_SUCCESSFULLY' : 'FILE_UPLOAD_FAILED'), data: { tempFolderName: tempFolderName, content: result.data } });
        })
            .catch(err => {
            let processMsg = (typeof err === 'string' ? err : 'CANNOT_PROCESS_REQUEST');
            let errorMsg = (typeof err === 'string' ? err : err.message || err.description || 'SERVER_ERROR_UNKNOWN_MESSAGE');
            logger.error('Error al procesar \'' + req.path + '\': ' + errorMsg);
            logger.error('[ProcessService::upload_file] Getting errorMsg:', errorMsg);
            if (res.headersSent)
                return;
            res.json({ status: false, description: processMsg, error: errorMsg });
        });
    });
    app.post('/api/process/get_csv_filters', (req, res, next) => {
        logger.info('[ProcessService::get_csv_filters] Received ' + req.method + ' method on \'' + req.path + '\'');
        let vcc = res.locals.vcc;
        let userId = res.locals.userId;
        let ftpCSVRedisKey = req.body.batchTypeName ? ('FTP_' + req.body.batchTypeName + ':') : null;
        promises_shared_1.DummyPromise()
            .then(result => {
            if (!ftpCSVRedisKey)
                throw 'SERVER_ERROR_FTP_TARGET_NOT_FOUND';
            return redis_service_1.Redis.GetCsvProcess(vcc, ftpCSVRedisKey + userId);
        }).then(result => {
            if (result === null)
                throw 'SERVER_ERROR_CSV_DATA_EXPIRED';
            return mssql_service_1.MSSql.BulkCSVFiltersInteractions(result.data);
        }).then(result => {
            logger.info('[ProcessService::get_csv_filters] Getting csvFilters after BulkCSVFiltersInteractions');
            if (res.headersSent)
                return;
            res.json({ status: true, description: 'SERVER_SUCCESS_MESSAGE', data: result });
        }).catch(err => {
            let processMsg = (typeof err === 'string' ? err : 'CANNOT_PROCESS_REQUEST');
            let errorMsg = (typeof err === 'string' ? err : err.message || err.description || 'SERVER_ERROR_UNKNOWN_MESSAGE');
            logger.error('Error al procesar \'' + req.path + '\': ' + errorMsg);
            logger.error('[ProcessService::get_csv_filters] Getting errorMsg:', errorMsg);
            if (res.headersSent)
                return;
            res.json({ status: false, description: processMsg, error: errorMsg });
        });
    });
    app.get('/api/process/download_file/:process_id', (req, res, next) => {
        logger.info('Se recibio ' + req.method + ' para \'' + req.path + '\'');
        let processId = req.params.process_id;
        logger.info('[ProcessService::/api/process/download_file/] Getting processId:', processId);
        let zipFileName = ('dataprocess' + '.csv');
        logger.info('[ProcessService::/api/process/download_file/] Getting zipFileName:', zipFileName);
        let zipFilePath = path.join(config_1.PATH_FOLDER_PROCESS_AUDIO, processId, zipFileName);
        logger.info('[ProcessService::/api/process/download_file/] Getting zipFilePath:', zipFilePath);
        promises_shared_1.DummyPromise()
            .then(result => {
            if (!result)
                throw 'PROCESS_NOT_EXIST \'' + processId + '\'';
            return file_manager_1.ExistInFileSystem(zipFilePath);
        })
            .then((resultExistInFileSystem) => {
            logger.info('[ProcessService::/api/process/download_file/] Getting resultExistInFileSystem:', resultExistInFileSystem, 'after ExistInFileSystem');
            if (!resultExistInFileSystem)
                throw 'NO_CSV_FILE_ASSOCIATED';
            logger.info('Descargando archivo \'' + processId + '\'');
            res.download(zipFilePath);
        })
            .catch(err => {
            let clientMsg = (typeof err === 'string' ? err : 'CANNOT_PROCESS_REQUEST');
            let errorMsg = (typeof err === 'string' ? err : err.message || err.description || 'SERVER_ERROR_UNKNOWN_MESSAGE');
            logger.error('Error al procesar \'' + req.path + '\': ' + errorMsg);
            logger.error('[ProcessService::/api/process/download_file] Getting errorMsg:', errorMsg);
            if (res.headersSent)
                return;
            res.json({ status: false, description: clientMsg, error: errorMsg });
        });
    });
    app.get('/api/process/download_template_audio_csv/', (req, res, next) => {
        logger.info('Se recibio ' + req.method + ' para \'' + req.path + '\'');
        promises_shared_1.DummyPromise()
            .then((result) => {
            let content = generateCSVFileContent();
            res.setHeader('Content-type', 'application/octet-stream');
            res.setHeader('Content-disposition', 'attachment; filename=');
            res.end(content);
        })
            .catch(err => {
            let clientMsg = (typeof err === 'string' ? err : 'CANNOT_PROCESS_REQUEST');
            let errorMsg = (typeof err === 'string' ? err : err.message || err.description || 'SERVER_ERROR_UNKNOWN_MESSAGE');
            logger.error('[ProcessService::/api/process/download_template_audio_csv] Getting errorMsg:', errorMsg);
            if (res.headersSent)
                return;
            res.json({ status: false, description: clientMsg, error: errorMsg });
        });
    });
    app.get('/api/process/download_errors_template_audio_csv/', (req, res, next) => {
        logger.info('Se recibio ' + req.method + ' para \'' + req.path + '\'');
        promises_shared_1.DummyPromise()
            .then((result) => {
            let data = [{ ID: "audio1", errorMessage: "en blanco" }, { ID: "audio2", errorMessage: "longitud mayor a la permitida" }];
            let content = generateErrorsCSVFileContent(data);
            res.setHeader('Content-type', 'application/octet-stream');
            res.setHeader('Content-disposition', 'attachment; filename=');
            res.end(content);
        })
            .catch(err => {
            let clientMsg = (typeof err === 'string' ? err : 'CANNOT_PROCESS_REQUEST');
            let errorMsg = (typeof err === 'string' ? err : err.message || err.description || 'SERVER_ERROR_UNKNOWN_MESSAGE');
            logger.error('[ProcessService::/api/process/download_template_audio_csv] Getting errorMsg:', errorMsg);
            if (res.headersSent)
                return;
            res.json({ status: false, description: clientMsg, error: errorMsg });
        });
    });
    app.post('/api/process/delete_csv_temporal/', (req, res, next) => {
        logger.info('[ProcessService::delete_csv_temporal] Received ' + req.method + ' method on \'' + req.path + '\'');
        let vcc = res.locals.vcc;
        let userId = res.locals.userId;
        let batchTypeName = req.body.batchTypeName || null;
        let redisKey;
        logger.info('[ProcessService::delete_csv_temporal] Getting vcc:userId : ', vcc, ':', userId);
        logger.info(`[ProcessService::delete_csv_temporal] Deleting delete_csv_temporal ${batchTypeName} by the user ${res.locals.userId}`);
        promises_shared_1.DummyPromise()
            .then(result => {
            if (!batchTypeName)
                throw 'SERVER_ERROR_MISSIGN_CSV_TARGET';
            redisKey = 'FTP_' + batchTypeName + ':' + userId;
            return redis_service_1.Redis.GetCsvProcess(vcc, redisKey);
        })
            .then(result => {
            if (result && result.data.length > 0) {
                return redis_service_1.Redis.DeleteCsvProcess(vcc, redisKey);
            }
            else {
                return Promise.resolve(true);
            }
        })
            .then(result => {
            if (res.headersSent)
                return;
            res.json({ status: result, description: (result ? 'CLEAR SUCCESS' : 'CLEAR ERROR '), data: result });
        })
            .catch(err => {
            let processMsg = (typeof err === 'string' ? err : 'CANNOT_PROCESS_REQUEST');
            let errorMsg = (typeof err === 'string' ? err : err.message || err.description || 'SERVER_ERROR_UNKNOWN_MESSAGE');
            logger.error('Error al procesar \'' + req.path + '\': ' + errorMsg);
            logger.error('[ProcessService::delete_csv_temporal] Getting errorMsg:', errorMsg);
            if (res.headersSent)
                return;
            res.json({ status: false, description: processMsg, error: errorMsg });
        });
    });
}
exports.ProcessService = ProcessService;
function GetProcess(vcc, id) {
    logger.info('[ProcessService::GetProcess] Getting vcc:', vcc, 'id:', id);
    let cacheUpdateNeeded = false;
    return Promise.resolve()
        .then(() => {
        return mssql_service_1.MSSql.GetProcess(vcc, id);
    })
        .catch((err) => {
        let errorMsg = (typeof err === 'string' ? err : err.message || err.description || 'SERVER_ERROR_UNKNOWN_MESSAGE');
        logger.error('[ProcessService::GetProcess] Getting errorMsg:', errorMsg);
    });
}
exports.GetProcess = GetProcess;
function GetProcessChannelInteractions(vcc, processId) {
    return promises_shared_1.DummyPromise()
        .then(_ => {
        if (!processId)
            throw 'MISSING_PROCESS_INPUT_PARAMETER';
        return mssql_service_1.MSSql.GetProcessChannelByProcessId(vcc, processId);
    })
        .then(result => {
        if (result === null)
            throw 'SERVER_ERROR_GET_PROCESS_CHANNELS_FAILED';
        let promiseArray = [];
        result.map(r => promiseArray.push(mssql_service_1.MSSql.GetInteractionsResult(vcc, processId, r.Value)));
        return Promise.all(promiseArray);
    })
        .catch((err) => {
        let processMsg = (typeof err === 'string' ? err : 'CANNOT_PROCESS_REQUEST');
        let errorMsg = (typeof err === 'string' ? err : err.message || err.description || 'SERVER_ERROR_UNKNOWN_MESSAGE');
        logger.error('[ProcessService::GetProcessChannelInteractions] Getting errorMsg:', errorMsg);
        return Promise.reject(errorMsg);
    });
}
exports.GetProcessChannelInteractions = GetProcessChannelInteractions;
function CreateProcess(vcc, vccType, newProcess, params, userId) {
    let nowUtc = moment().utc().toDate();
    newProcess.createdByUserId = userId;
    newProcess.createdDate = nowUtc;
    let channelInteractions = [];
    let totalSyncInteraction = 0;
    let audios = [];
    let asyncInteractions = [];
    let totalAsyncInteraction = 0;
    let totalAsyncDetail = 0;
    let totalInteractionsToSave = 0;
    let status = false;
    let saveResult = null;
    let platformVersion = null;
    return promises_shared_1.DummyPromise()
        .then(_ => {
        return mssql_service_1.MSSql.GetProcessByName(vcc, newProcess.name);
    })
        .then(result => {
        if (result)
            throw 'PROCESS_ALREADY_EXIST \'' + newProcess.name + '\'';
        logger.info(`[ProcessService::CreateProcess(${newProcess.name})] Getting resultProcess:`, result, 'after MSSql.GetProcessByName');
        return platform_service_1.GetPlatformVersion();
    })
        .then(result => {
        if (result === null)
            throw 'INSTALLATIONID_MISSING';
        platformVersion = result;
        let batch = {};
        batch.installationId = platformVersion.installationId;
        batch.vcc = vcc;
        batch.batchTypeName = 'TRANSCRIPTION';
        batch.batchTypeId = newProcess.name;
        let billing = new billing_batch_implementation_1.BillingBatchImplementation();
        return billing.FindBatchId(batch);
    })
        .then(result => {
        if (result && result.result && result.result.length > 0)
            throw 'ERROR_PROCESS_ID_INVALID';
        logger.info(`[ProcessService::CreateProcess(${newProcess.name})] Getting platformType::environmenType => `, vccType, '::', newProcess.environmentType);
        if (newProcess.environmentType
            &&
                ['allegro', 'i6', 'i-ftp', 'i-sftp', 'i-s3'].includes(newProcess.environmentType)) {
            return Promise.resolve({ data: [] });
        }
        else {
            let ftpCSVRedisKey;
            if (params.evaluationFormatId)
                ftpCSVRedisKey = 'FTP_EVALUATION_PLAN:';
            else
                ftpCSVRedisKey = 'FTP_PROCESS:';
            return redis_service_1.Redis.GetCsvProcess(vcc, ftpCSVRedisKey + userId);
        }
    })
        .then(result => {
        if (['ftp', 'sftp', 's3'].includes(newProcess.environmentType)) {
            if (result === null)
                throw '7';
            vccType = 'OTHER';
        }
        let promiseArray = [];
        promiseArray = interaction_service_1.GetChannelNewInteractionSummaryPromises(vcc, vccType, false, 'DETAILED', params, result.data);
        return Promise.all(promiseArray);
    })
        .then(result => {
        logger.info(`[ProcessService::CreateProcess(${newProcess.name})] Getting result for ` + result.length + ' channels');
        if (!result || result.length === 0)
            throw 'SERVER_ERROR_NO_INTEGRATED_DATA_FOUND';
        let processAgentIds = [];
        channelInteractions = result;
        channelInteractions.forEach(ci => {
            if (ci) {
                if (ci.interactionType === 'CALL') {
                    totalInteractionsToSave += ci.totalInteractionDetails;
                    ci.interactions.map(a => {
                        if (a.Agent)
                            processAgentIds.push(a.Agent);
                    });
                }
                else {
                    totalInteractionsToSave += ci.totalInteractions;
                    ci.interactions.map(a => {
                        if (a.firstTakedAgent)
                            processAgentIds.push(a.firstTakedAgent);
                        if (a.lastTakedAgent)
                            processAgentIds.push(a.lastTakedAgent);
                    });
                }
            }
        });
        processAgentIds = arrays_shared_1.GetUniqueFromArray(processAgentIds);
        totalInteractionsToSave = Math.round(totalInteractionsToSave * newProcess.percentage);
        if (vccType !== 'OTHER' && processAgentIds.length)
            return linguistic_service_1.ImportAgents(vcc, vccType, processAgentIds);
        else
            return Promise.resolve(true);
    })
        .then(result => {
        logger.info(`[ProcessService::CreateProcess(${newProcess.name})] Getting result:`, result, 'after bulk agents');
        let interactionBulkPromiseArray = [];
        let max = null;
        let min = null;
        let hasInteractionIds = params.interactionIds.length > 0;
        if (hasInteractionIds) {
            params.hourIni = "00:00:00";
            params.hourFin = "23:59:59";
        }
        channelInteractions.map(ci => {
            if (ci?.totalInteractions) {
                if (ci.interactionType === 'CALL') {
                    let iv = {};
                    iv = ci;
                    let idTitle = 'InteractionId';
                    let sampleResult = linguistic_service_2.GetInteractionsSample(iv.interactions, newProcess.percentage, idTitle, totalInteractionsToSave);
                    totalSyncInteraction = sampleResult.sampleInteractionIds.length;
                    audios = sampleResult.sampleInteractionDetails;
                    totalInteractionsToSave -= sampleResult.sampleInteractionIds.length;
                    interactionBulkPromiseArray.push(mssql_service_1.MSSql.BulkAudioTemp(vcc, newProcess, audios, userId));
                    if (hasInteractionIds) {
                        if (!max) {
                            max = new Date(audios[0].EndDate.substring(0, 10));
                            min = max;
                        }
                        for (let a of audios) {
                            let d = new Date(a.EndDate.substring(0, 10));
                            if (d > max)
                                max = d;
                            if (d < min)
                                min = d;
                        }
                    }
                }
                else {
                    let aiv = {};
                    aiv = ci;
                    asyncInteractions = [];
                    let sampleResult = linguistic_service_2.GetInteractionsSample(aiv.interactions, newProcess.percentage, 'id', totalInteractionsToSave);
                    totalAsyncInteraction += sampleResult.sampleInteractionIds.length;
                    totalAsyncDetail += sampleResult.sampleInteractionDetails.length;
                    asyncInteractions = sampleResult.sampleInteractionDetails;
                    totalInteractionsToSave -= sampleResult.sampleInteractionIds.length;
                    interactionBulkPromiseArray.push(mssql_service_1.MSSql.BulkAsyncInteractionTemp(vcc, newProcess.name, asyncInteractions, userId));
                    if (hasInteractionIds) {
                        if (!max) {
                            if (typeof asyncInteractions[0].endDate === 'string')
                                asyncInteractions[0].endDate = new Date(asyncInteractions[0].endDate);
                            max = new Date(asyncInteractions[0].endDate.toISOString().substring(0, 10));
                            min = max;
                        }
                        for (let a of asyncInteractions) {
                            if (typeof a.endDate === 'string')
                                a.endDate = new Date(a.endDate);
                            let d = new Date(a.endDate.toISOString().substring(0, 10));
                            if (d > max)
                                max = d;
                            if (d < min)
                                min = d;
                        }
                    }
                }
            }
        });
        if (hasInteractionIds) {
            params.dateFin = moment(`${max.toISOString().substring(0, 10)} ${params.hourFin}`).utc().toDate();
            params.dateIni = moment(`${min.toISOString().substring(0, 10)} ${params.hourIni}`).utc().toDate();
        }
        return Promise.all(interactionBulkPromiseArray);
    })
        .then(result => {
        logger.info(`[ProcessService::CreateProcess(${newProcess.name})] Getting result:`, result, 'after BulkAudioTemp and BulkAsyncInteractionTemp');
        let summary = [];
        let audioSummary = alasql('SELECT Category1, Category2, Category3, Description, COUNT(*) AS Total FROM ? WHERE Description <> \'\' GROUP BY Category1, Category2, Category3, Description', [audios]);
        return mssql_service_1.MSSql.BulkAudioSummaryTemp(vcc, newProcess, audioSummary, userId);
    })
        .then(result => {
        logger.info(`[ProcessService::CreateProcess(${newProcess.name})] Getting result:`, result, 'after MSSql.BulkAudioSummaryTemp');
        newProcess.processFilters = interaction_service_1.GetProcessFilterFromInteractionSearch(newProcess.name, params, userId);
        return SaveProcess(vcc, newProcess);
    })
        .then(result => {
        logger.info(`[ProcessService::CreateProcess(${newProcess.name})] Getting result:`, result, 'after SaveProcess');
        if (newProcess && newProcess.start === true) {
            let promiseArray = [];
            promiseArray.push(mssql_service_1.MSSql.GetInteractionsResult(vcc, newProcess.name, 'CALL'));
            promiseArray.push(mssql_service_1.MSSql.GetProcessInteractionsHistory(vcc, newProcess.name, 1));
            return Promise.all(promiseArray);
        }
        else
            return Promise.resolve([result]);
    })
        .then(result => {
        if (newProcess?.start === true) {
            if (result === null)
                throw 'SERVER_ERROR_GET_PROCESS_INFORMATION_FAILED';
            logger.info(`[ProcessService::CreateProcess(${newProcess.name})] Getting results from GetInteractionsResult: ` + result.length);
            let batch = {};
            batch.vcc = vcc;
            batch.installationId = platformVersion.installationId;
            batch.totalInteraction = totalSyncInteraction + totalAsyncInteraction;
            batch.totalAsync = totalAsyncInteraction;
            batch.totalAsyncDetail = totalAsyncDetail;
            batch.alreadyTextPhrases = 0;
            batch.totalSync = totalSyncInteraction;
            batch.totalAudio = result[0].quantity;
            batch.alreadyCallPhrases = 0;
            batch.durationTime = result[0].timeQuantity;
            batch.roundedTime = 0;
            batch.inProcessAudio = 0;
            batch.inProcessDurationTime = 0;
            batch.alreadyTranscribedAudio = 0;
            batch.alreadyDurationTime = 0;
            batch.alreadyTranscribedDurationTime = 0;
            batch.alreadyTranscribedRoundedDurationTime = 0;
            batch.alreadyCallPhrases = 0;
            batch.alreadyTextPhrases = 0;
            result[1].map(pih => {
                if (pih.interactionType === 'CALL') {
                    batch.alreadyCallPhrases = pih.totalCallPhrases || 0;
                    batch.inProcessAudio = pih.inProcessAudio || 0;
                    batch.inProcessDurationTime = pih.inProcessDurationTime || 0;
                    batch.alreadyTranscribedAudio = pih.alreadyTranscribedAudio || 0;
                    batch.alreadyDurationTime = pih.alreadyTranscribedDurationTime || 0;
                    batch.alreadyTranscribedDurationTime = pih.alreadyTranscribedValidDurationTime || 0;
                    batch.alreadyTranscribedRoundedDurationTime = pih.alreadyTranscribedRoundedDurationTime || 0;
                }
                else {
                    batch.alreadyTextPhrases += pih.totalTextPhrases || 0;
                }
            });
            batch.batchScheduleId = newProcess.batchScheduleId;
            batch.batchTypeName = 'TRANSCRIPTION';
            batch.batchTypeId = newProcess.name;
            batch.processId = newProcess.name;
            batch.userApi = userId;
            batch.startDate = nowUtc;
            let billing = new billing_batch_implementation_1.BillingBatchImplementation();
            return billing.SaveBatchSummary(batch);
        }
        else
            return Promise.resolve(true);
    })
        .then(result => {
        if (typeof result !== 'boolean') {
            if (result) {
                if (result.status === true) {
                    if (newProcess && newProcess.start === true) {
                        status = true;
                        return linguistic_service_1.BeginProcess(vcc, newProcess.name);
                    }
                }
                else {
                    status = false;
                    saveResult = result.error ? result.error : 'SERVER_ERROR_CAN_NOT_SAVE_BATCH';
                }
            }
            else {
                return Promise.resolve(false);
            }
        }
        else {
            status = result;
        }
        return Promise.resolve(true);
    })
        .then(result => {
        logger.info(`[ProcessService::CreateProcess(${newProcess.name})] Getting result:`, result, 'after SaveProcess');
        return redis_service_1.Redis.DeleteCsvProcess(vcc, "FTP_PROCESS:" + userId);
    })
        .then(_ => {
        logger.info(`[ProcessService::CreateProcess(${newProcess.name})] Finishing function`);
        return Promise.resolve({ status, saveResult });
    })
        .catch(err => {
        logger.error('[ProcessService::CreateProcess] Function failed');
        return Promise.reject(err);
    });
}
exports.CreateProcess = CreateProcess;
function CreateScheduleForProcess(vcc, vccType, process, params, userId) {
    let newId = utilities_1.guid();
    process.processFilters = interaction_service_1.GetProcessFilterFromInteractionSearch(newId, params, userId);
    process.scheduleData.batchType = 'TRANSCRIPTION';
    process.scheduleData.id = process.name;
    process.scheduleData.processFiltersId = newId;
    return promises_shared_1.DummyPromise()
        .then(_ => {
        return batch_schedule_service_1.SaveProcessBatchSchedule(vcc, vccType, process, userId);
    })
        .then(result => {
        logger.info('[ProcessService::CreateScheduleForProcess] Result of SaveProcessBatchSchedule:', result);
        return Promise.resolve({ status: result, saveResult: null });
    })
        .catch(err => {
        logger.error('[ProcessService::CreateScheduleForProcess] Function failed');
        return Promise.reject(err);
    });
}
exports.CreateScheduleForProcess = CreateScheduleForProcess;
function SaveProcess(vcc, process) {
    logger.info('[ProcessService::GetProcess] Getting vcc:', vcc, 'processId:', process.name);
    let promiseArray = [];
    promiseArray.push(mssql_service_1.MSSql.SaveProcess(vcc, process.name, process.description, process.clientId, process.status, process.type, process.startDate, process.endDate, process.minDurationTime, process.maxDurationTime, process.language, process.dictionaryId, process.environment, process.queryId, process.queryDataId, process.queryDataIdDate, process.batchScheduleId, process.createdDate, process.createdByUserId, process.lastModifiedDate, process.lastModifiedByUserId, process.retranscription ? 1 : 0, process.diarization ? 1 : 0, process.channelchange ? 1 : 0));
    if (process.processFilters) {
        process.processFilters.forEach(processFilter => {
            promiseArray.push(mssql_service_1.MSSql.SaveProcessFilter(vcc, processFilter.processId, processFilter.key, processFilter.value, moment().utc().toDate(), processFilter.createdByUserId, moment().utc().toDate(), processFilter.lastModifiedByUserId));
        });
    }
    return Promise.all(promiseArray)
        .then(results => {
        logger.info('[ProcessService::GetProcess] Getting results after Promise.all MSSql.SaveProcess and MSSql.SaveProcessFilter');
        if (results.filter(result => { return (result !== true); }).length > 0) {
            return Promise.resolve(false);
        }
        else {
            return Promise.resolve(true);
        }
    });
}
exports.SaveProcess = SaveProcess;
function SaveDurationTimeRate(vcc, process, userId) {
    logger.info('[ProcessService::SaveDurationTimeRate] Getting vcc:', vcc, 'process:', process);
    let durationTimeRate;
    return file_manager_1.ReadFileContent(config_1.PATH_FOLDER_PROCESS_AUDIO + '/' + process.name + '/' + 'dataprocess' + '.csv', 'utf-8')
        .then(resultReadFileContent => {
        logger.info('[ProcessService::SaveDurationTimeRate] Getting resultReadFileContent:', resultReadFileContent, 'after ReadFileContent');
        return csv().fromString(resultReadFileContent);
    })
        .then(resultConvertCsvToJson => {
        logger.info('[ProcessService::SaveDurationTimeRate] Getting resultConvertCsvToJson:', resultConvertCsvToJson, 'after csv().fromString');
        durationTimeRate = resultConvertCsvToJson.filter((x) => x.DurationTime >= process.minDurationTime && x.DurationTime <= process.maxDurationTime);
        durationTimeRate = (durationTimeRate.length / resultConvertCsvToJson.length) * 100;
        durationTimeRate = durationTimeRate.toFixed(2) + '%';
        return Promise.resolve(durationTimeRate);
    })
        .then(resultdurationTimeRate => {
        logger.info('[ProcessService::SaveDurationTimeRate] Getting resultdurationTimeRate:', resultdurationTimeRate);
        return mssql_service_1.MSSql.GetProcessSummary(vcc, process.name);
    })
        .then(resultProcessSummary => {
        logger.info('[ProcessService::SaveDurationTimeRate] Getting resultProcessSummary:', resultProcessSummary, 'after MSSql.GetProcessSummary');
        return mssql_service_1.MSSql.SaveProcessSummaryUpdate(vcc, process.name, resultProcessSummary.silenceTime, resultProcessSummary.averageSilenceTime, resultProcessSummary.durationTime, resultProcessSummary.averageDurationTime, durationTimeRate, null, null, moment().utc().toDate(), userId);
    })
        .catch((err) => {
        let errorMsg = (typeof err === 'string' ? err : err.message || err.description || 'SERVER_ERROR_UNKNOWN_MESSAGE');
        logger.error('Error al ejecutar SaveDurationTimeRate: ' + errorMsg);
        logger.error('[ProcessService::SaveDurationTimeRates] Getting errorMsg:', errorMsg);
    });
}
function csvJSON(csv, isReprocess) {
    logger.info('[ProcessService::csvJSON] Getting csv lines: ', csv.length);
    let lines = null;
    if (csv.includes('\r\n'))
        lines = csv.split('\r\n');
    else if (csv.includes('\r'))
        lines = csv.split('\r');
    else if (csv.includes('\n'))
        lines = csv.split('\n');
    else
        lines = [];
    if (lines.length === 0)
        throw `SERVER_ERROR_CSV_CONTENT_EMPTY`;
    let linesLen = lines.length;
    let result = [];
    let headers = lines[0].split(',');
    let headersLen = headers.length;
    let i = null;
    let j = null;
    let obj = {};
    let currentline = [];
    logger.info('[ProcessService::csvJSON] Getting lines:', linesLen, 'headers:', headers, 'headersLength:', headers.length);
    ValidateCSVHeaders(headers);
    for (i = 1; i < linesLen; i++) {
        obj = {};
        currentline = lines[i].split(',');
        if (currentline && currentline.length > 1) {
            for (j = 0; j <= headersLen; j++) {
                if (j === headersLen)
                    obj["LN"] = i + 1;
                else
                    obj[headers[j]] = currentline[j];
            }
            result.push(obj);
        }
    }
    logger.info('[ProcessService::csvJSON] Getting result lines: ', result.length);
    let newResutl = ValidateCSVContent(result, isReprocess);
    return JSON.stringify(newResutl);
}
function ValidateCSVHeaders(csvHeaders) {
    let csvTemplateHeaders = audio_mssql_1.headerTemplateCSV;
    csvTemplateHeaders.map(cth => {
        if (!csvHeaders.includes(cth)) {
            throw `SERVER_ERROR_CSV_HEADER_${cth.toUpperCase()}_NOT_FOUND`;
        }
    });
}
function ValidateCSVContent(csvContent, isReprocess) {
    let csvDuplicates = GetDuplicatesIDs(csvContent);
    let csvErrors = GetErrorsCSVContent(csvContent, csvDuplicates);
    if (csvErrors.length === 0) {
        return csvContent;
    }
    else {
        if (csvErrors.length / csvContent.length > 0.4) {
            throw "csv_error_limit_exceededcsvErrors" + generateErrorsCSVFileContent(csvErrors);
        }
        else {
            if (isReprocess) {
                let csvContentWithoutErrors = csvContent.filter(function (objFromCsvContent) {
                    return !csvErrors.find(function (objFromCsvErrors) {
                        return objFromCsvContent.ID === objFromCsvErrors.ID;
                    });
                });
                return csvContentWithoutErrors;
            }
            else {
                throw "csv_error_limit_acceptablecsvErrors" + csvErrors.length + "csvErrors" + csvContent.length + "csvErrors" + generateErrorsCSVFileContent(csvErrors);
            }
        }
    }
}
function GetDuplicatesIDs(csvContent) {
    let IDs = csvContent.map(function (interaction) { return interaction.ID; });
    let sorted = IDs.sort();
    let duplicates = sorted.filter(function (value, index) {
        return value === sorted[index + 1] && value !== "";
    });
    return duplicates.map((current => Object.assign({ ID: current })));
}
function GetErrorsCSVContent(csvContent, csvDuplicates) {
    let csvErrorsArray = [];
    for (let cc of csvContent) {
        let errorMessage = "";
        let isDuplicate = csvDuplicates.findIndex(x => x.ID === cc.ID);
        if (isDuplicate >= 0) {
            errorMessage += 'Duplicate value' + '/';
        }
        ;
        if (cc.ID.length > 100) {
            errorMessage += `The character length for the ID exceeds the set limit of 100` + '/';
        }
        if (cc.ID === '') {
            errorMessage += `The ID can not be empty` + '/';
        }
        let hasWhiteSpace = (s) => (/\s/).test(s);
        if (hasWhiteSpace(cc.ID)) {
            errorMessage += `The ID does not accept white spaces` + '/';
        }
        if (cc.ID.startsWith(".")) {
            errorMessage += `The ID cannot start with the period character (.)` + '/';
        }
        if (cc.ID.includes("#")) {
            errorMessage += `The caracter # is not allowed into ID column` + '/';
        }
        if (cc.Number.length > 50)
            errorMessage += `The character length for Number exceeds the set limit of 50` + '/';
        let isNotIntegerValue = (s) => !(/^[0-9]+$/).test(s);
        if (cc.DurationTime === '') {
            errorMessage += `The Duration can not be empty` + '/';
        }
        if (isNotIntegerValue(cc.DurationTime)) {
            errorMessage += 'The Duration must be integer' + '/';
        }
        ;
        if (cc.Gender.length > 50)
            errorMessage += `The character length for Gender exceeds the set limit of 50.` + '/';
        if (cc.Gender !== '') {
            if (!["F", "M"].includes(cc.Gender))
                errorMessage += `You have entered an incorrect value for the Gender field (Valid F and M)` + '/';
        }
        if (cc.Category1.length > 100)
            errorMessage += `The character length for Category1 exceeds the set limit of 100.` + '/';
        if (cc.Category2.length > 100)
            errorMessage += `The character length for Category2 exceeds the set limit of 100.` + '/';
        if (cc.Category3.length > 100)
            errorMessage += `The character length for Category3 exceeds the set limit of 100.` + '/';
        if (cc.Description.length > 100)
            errorMessage += `The Character length for Description exceeds the set limit of 100` + '/';
        if (cc.Campaign.length > 100) {
            errorMessage += `The character length for Campaign exceeds the set limit of 100` + '/';
        }
        ;
        if (cc.Campaign === '') {
            errorMessage += `The Campaign can not be empty` + '/';
        }
        if (cc.Agent.length > 50) {
            errorMessage += `The character length for the Agent exceeds the set limit of 50` + '/';
        }
        if (cc.Agent === '') {
            errorMessage += `The Agent can not be empty` + '/';
        }
        if (hasWhiteSpace(cc.Agent)) {
            errorMessage += `The Agent does not accept white spaces` + '/';
        }
        if (cc.Direction.length > 1)
            errorMessage += `The character length for Direction exceeds the set limit of 1` + '/';
        if (!["E", "S"].includes(cc.Direction))
            errorMessage += `You have entered an incorrect value for the Direction field (Valid E and S)` + '/';
        if (cc.Direction === '')
            errorMessage += `The Direction can not be empty` + '/';
        if (cc.StartDate === '') {
            errorMessage += `The StartDate can not be empty` + '/';
        }
        let invalidCsvFormatDate = (s) => !((/^\d{4}[\-\/\s]?((((0[13578])|(1[02]))[\-\/\s]?(([0-2][0-9])|(3[01])))|(((0[469])|(11))[\-\/\s]?(([0-2][0-9])|(30)))|(02[\-\/\s]?[0-2][0-9]))\s(0[0-9]|1[0-9]|2[0-3]):([0-5][0-9]):([0-5][0-9])/).test(s) || (/^\d{4}[\-\/\s]?((((0[13578])|(1[02]))[\-\/\s]?(([0-2][0-9])|(3[01])))|(((0[469])|(11))[\-\/\s]?(([0-2][0-9])|(30)))|(02[\-\/\s]?[0-2][0-9]))\s(0[0-9]|1[0-9]|2[0-3]):([0-5][0-9])/).test(s));
        if (invalidCsvFormatDate(cc.StartDate) && cc.StartDate !== '') {
            errorMessage += `Invalid date format for StartDate` + '/';
        }
        if (cc.EndDate === '') {
            errorMessage += `The EndDate can not be empty` + '/';
        }
        if (invalidCsvFormatDate(cc.EndDate) && cc.EndDate !== '') {
            errorMessage += `Invalid date format for EndDate` + '/';
        }
        if (cc.DispositionCode.length > 50)
            errorMessage += `The character length for DispositionCode exceeds the set limit of 50` + '/';
        if (cc.RelativePath.length > 500)
            errorMessage += `The character length for Relative path exceeds the set limit of 500` + '/';
        let invalidRelativePath = (s) => !(/^[A-Za-z0-9_{}.-\s|\/-]*$/).test(s);
        if (invalidRelativePath(cc.RelativePath))
            errorMessage += `RelativePath only accept letters,numbers and "/" , "-" , "_"` + '/';
        if (errorMessage !== '') {
            let csvError = { LN: 0, ID: "", errorMessage: "" };
            csvError.ID = cc.ID;
            csvError.errorMessage = errorMessage;
            csvError.LN = cc.LN;
            csvErrorsArray.push(csvError);
        }
    }
    return csvErrorsArray;
}
function generateCSVFileContent() {
    let headers = audio_mssql_1.headerTemplateCSV;
    let result = '';
    if (headers && headers.length) {
        result = headers.join(',');
        result += '\r\n';
    }
    return result;
}
function generateErrorsCSVFileContent(data) {
    let headers = ["LN", "ID", "errorMessage"];
    let result = '';
    const replacer = (key, value) => (value === null ? '' : value);
    const csv = data.map((row) => headers
        .map((ID) => JSON.stringify(row[ID], replacer))
        .join(','));
    csv.unshift(headers.join(','));
    const csvArray = csv.join('\r\n');
    return csvArray;
}
