"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.GetItemFromQueueId = exports.TaskService = void 0;
const jwt = require("jsonwebtoken");
const log4js = require("log4js");
const mssql_service_1 = require("../db/mssql.service");
const redis_service_1 = require("../db/redis.service");
const md5_1 = require("ts-md5/dist/md5");
const promises_shared_1 = require("../shared/promises.shared");
const jwt_config_1 = require("../config/jwt.config");
const transcriptions_download_implementation_1 = require("../implementation/transcriptions-download.implementation");
const integration_config_1 = require("../config/integration.config");
const transcriptions_upload_implementation_1 = require("../implementation/transcriptions-upload.implementation");
let logger = log4js.getLogger('ServerScripts');
function TaskService(app) {
    app.post('/api/public/task/update_transcription_queue/:queue_id', (req, res, next) => {
        logger.info('[TaskService::update_transcription_queue] Received ' + req.method + ' on \'' + req.path + '\'');
        let queueId = req.params.queue_id;
        let callbackData = (req.body || {});
        let storage = null;
        let executionCode = Date.now().toString();
        let processInteraction = null;
        const { vcc, processId } = GetItemFromQueueId(queueId);
        logger.info(`[TaskService::update_transcription_queue] Received callback for ${queueId} queue`);
        logger.info(`[TaskService::update_transcription_queue] Received:`, callbackData);
        promises_shared_1.DummyPromise()
            .then(_ => {
            let decodedToken = null;
            try {
                decodedToken = jwt.verify((req.headers['authorization'] || '').replace('Bearer ', ''), jwt_config_1.JWT_SECRET);
            }
            catch (e) {
                decodedToken = null;
                logger.warn(`[TaskService::update_transcription_queue] Received invalid token: ${req.headers['authorization']}`);
                throw 'SERVER_ERROR_CANNOT_DECODE_ACCESS_TOKEN';
            }
            let transcriptionsUpload = new transcriptions_upload_implementation_1.TranscriptionsUploadImplementation();
            if (decodedToken.userId === transcriptionsUpload.externalUserId
                &&
                    decodedToken.vcc === transcriptionsUpload.externalUserVcc)
                return Promise.resolve(true);
            else
                throw 'SERVER_ERROR_ACCESS_TOKEN_VALIDATION_FAILED';
        })
            .then(_ => {
            return redis_service_1.Redis.DeleteInsertOnList('TranscriptionsStandby', 'TranscriptionsStandbyTask', queueId);
        })
            .then(result => {
            if (!result)
                throw 'SERVER_ERROR_QUEUE_EXCHANGE_FAILED';
            storage = `${vcc}:${processId}`;
            return redis_service_1.Redis.GetProcessInteraction(storage, queueId);
        })
            .then(result => {
            if (!result) {
                logger.warn(`[TaskService::update_transcription_queue] ProcessInteraction for "${queueId}" was not found on storage "${vcc}:${processId}"`);
                throw 'SERVER_ERROR_INTERACTION_DATA_WAS_NOT_FOUND';
            }
            processInteraction = result;
            if (!processInteraction)
                throw 'SERVER_ERROR_INTERACTION_DATA_NOT_FOUND';
            logger.info(`[TaskService::update_transcription_queue] Saving content from callback for ${queueId} queue`);
            let transcriptionsDownload = new transcriptions_download_implementation_1.TranscriptionsDownloadImplementation();
            return transcriptionsDownload.DownloadTranscription(processInteraction, callbackData, queueId, executionCode);
        })
            .then(result => {
            return redis_service_1.Redis.SaveProcessInteraction(storage, queueId, result);
        })
            .then(_ => {
            logger.info(`[TaskService::update_transcription_queue] Inserting the ${queueId} queue to download`);
            return redis_service_1.Redis.DeleteInsertOnList('TranscriptionsStandbyTask', 'TranscriptionsDownload', queueId);
        })
            .then(result => {
            if (res.headersSent)
                return;
            res.json({ status: true, description: 'SERVER_SUCCESS_MESSAGE', data: result });
        })
            .catch((err) => {
            let clientMsg = (typeof err === 'string' ? err : 'CANNOT_PROCESS_REQUEST');
            let errorMsg = (typeof err === 'string' ? err : err.message || err.description || 'SERVER_ERROR_UNKNOWN_MESSAGE');
            logger.error('[TaskService::update_transcription_queue] Error on \'' + req.path + '\': ' + errorMsg);
            if (res.headersSent)
                return;
            redis_service_1.Redis.lremAsync('TranscriptionsStandbyTask', 1, queueId);
            res.json({ status: false, description: clientMsg, error: errorMsg });
        });
    });
    app.post('/api/public/task/update_semantic_queue/:queue_id', (req, res, next) => {
        logger.info('[TaskService::update_semantic_queue] Received ' + req.method + ' on \'' + req.path + '\'');
        let queueId = req.params.queue_id;
        let callbackData = (req.body || {});
        let storage = null;
        let executionCode = Date.now().toString();
        let processInteraction = null;
        const { vcc, processId } = GetItemFromQueueId(queueId);
        logger.info(`[TaskService::update_semantic_queue] Received speechmatics callback for ${queueId} queue`);
        logger.info(`[TaskService::update_semantic_queue] Received:`, callbackData);
        promises_shared_1.DummyPromise()
            .then(_ => {
            res.json({ status: true, description: 'SERVER_SUCCESS_MESSAGE', data: 'OK' });
        })
            .catch((err) => {
            let clientMsg = (typeof err === 'string' ? err : 'CANNOT_PROCESS_REQUEST');
            let errorMsg = (typeof err === 'string' ? err : err.message || err.description || 'SERVER_ERROR_UNKNOWN_MESSAGE');
            logger.error('[TaskService::update_semantic_queue] Error on \'' + req.path + '\': ' + errorMsg);
            if (res.headersSent)
                return;
            res.json({ status: false, description: clientMsg, error: errorMsg });
        });
    });
    app.get('/api/public/occ/get_session_token/:vcc', (req, res, next) => {
        logger.info('[TaskService::get_session_token] Received ' + req.method + ' on \'' + req.path + '\'');
        let vcc = req.params.vcc || 'vcci6';
        let userId = integration_config_1.USER_REALTIME;
        let password = integration_config_1.PASS_REALTIME;
        let session = null;
        promises_shared_1.DummyPromise()
            .then(_ => {
            return redis_service_1.Redis.GetI6Realtime(vcc);
        })
            .then(result => {
            if (result) {
                session = result;
                return Promise.resolve(true);
            }
            else {
                if (!userId || !password)
                    throw 'SERVER_ERROR_MISSING_USER_OR_PASSWORD';
                return mssql_service_1.MSSql.ValidateUserLogin(vcc, userId, md5_1.Md5.hashStr(userId + password));
            }
        })
            .then(result => {
            if (session)
                return Promise.resolve(true);
            else {
                if (!result)
                    throw 'SERVER_ERROR_WRONG_USER_OR_PASSWORD';
                session = jwt.sign({ userId: userId, vcc: vcc }, jwt_config_1.JWT_SECRET);
                redis_service_1.Redis.SaveI6Realtime(vcc, session);
                return redis_service_1.Redis.SaveSession(vcc, userId, session);
            }
        })
            .then(result => {
            if (!result)
                throw 'SERVER_ERROR_CANNOT_SAVE_SESSION';
            if (res.headersSent)
                return;
            res.json({ status: true, description: 'OK', data: session });
        })
            .catch((err) => {
            let clientMsg = (typeof err === 'string' ? err : 'CANNOT_PROCESS_REQUEST');
            let errorMsg = (typeof err === 'string' ? err : err.message || err.description || 'SERVER_ERROR_UNKNOWN_MESSAGE');
            logger.error('[TaskService::get_session_token] Error on \'' + req.path + '\': ' + errorMsg);
            if (res.headersSent)
                return;
            res.json({ status: false, description: clientMsg, error: errorMsg });
        });
    });
}
exports.TaskService = TaskService;
function GetItemFromQueueId(queueId) {
    let vcc = '';
    let processId = '';
    let interactionType = '';
    let interactionId = '';
    let interactionSection = 0;
    let interactionPart = 0;
    let item = queueId.split(':');
    if (item?.length === 6) {
        vcc = item[0];
        processId = item[1];
        interactionType = item[2];
        interactionId = item[3];
        interactionSection = Number(item[4]);
        interactionPart = Number(item[5]);
    }
    else {
        logger.error('[TaskService::GetItemFromQueueId] QueueId has not conrrect format');
        throw 'SERVER_ERROR_QUEUEID_INVALID_FORMAT';
    }
    return { vcc, processId, interactionType, interactionId, interactionSection, interactionPart };
}
exports.GetItemFromQueueId = GetItemFromQueueId;
