"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.TipificationService = void 0;
const log4js = require("log4js");
const mssql_service_1 = require("../db/mssql.service");
const promises_shared_1 = require("../shared/promises.shared");
const utilities_1 = require("../shared/utilities");
const arrays_shared_1 = require("../shared/arrays.shared");
let logger = log4js.getLogger('ServerScripts');
function TipificationService(app) {
    app.post('/api/tipification/create_tipification', (req, res, next) => {
        logger.info(`[TipificationService::create_tipification] Received ${req.method} method on '${req.path}'`);
        let vcc = res.locals.vcc;
        let userId = res.locals.userId;
        let newTipification = (req.body || {});
        let wasCreated = false;
        newTipification.description = newTipification.description.trim();
        promises_shared_1.DummyPromise()
            .then(_ => {
            return mssql_service_1.MSSql.GetTipifications(vcc, 'description', newTipification.description);
        })
            .then(result => {
            if (result?.length)
                throw 'SERVER_ERROR_TIPIFICATION_ALREADY_EXISTS';
            newTipification.id = newTipification.id ? newTipification.id : utilities_1.guid();
            return mssql_service_1.MSSql.SaveTipification(vcc, newTipification.id, newTipification.description, newTipification.detail, userId);
        })
            .then(result => {
            if (!result)
                throw 'SERVER_ERROR_TIPIFICATION_SAVE_FAILED';
            wasCreated = true;
            return SaveTipificationItems(vcc, newTipification, userId);
        })
            .then(result => {
            if (!result)
                throw 'SERVER_ERROR_TIPIFICATION_ITEMS_SAVE_FAILED';
            if (res.headersSent)
                return;
            res.json({ status: true, description: 'SERVER_SUCCESS_MESSAGE', data: result });
        })
            .catch(err => {
            let clientMsg = (typeof err === 'string' ? err : 'CANNOT_PROCESS_REQUEST');
            let errorMsg = (typeof err === 'string' ? err : err.message || err.description || 'SERVER_ERROR_UNKNOWN_MESSAGE');
            logger.error(`[TipificationService::create_tipification] Error on '${req.path}': ${errorMsg}`);
            if (wasCreated)
                mssql_service_1.MSSql.DeleteTipification(vcc, newTipification.id);
            if (res.headersSent)
                return;
            res.json({ status: false, description: clientMsg, error: errorMsg });
        });
    });
    app.post('/api/tipification/save_tipification', (req, res, next) => {
        logger.info(`[TipificationService::save_tipification] Received ${req.method} method on '${req.path}'`);
        let vcc = res.locals.vcc;
        let userId = res.locals.userId;
        let tipification = (req.body || {});
        tipification.description = tipification.description.trim();
        promises_shared_1.DummyPromise()
            .then(_ => {
            return mssql_service_1.MSSql.SaveTipification(vcc, tipification.id, tipification.description, tipification.detail, userId);
        })
            .then(result => {
            if (!result)
                throw 'SERVER_ERROR_TIPIFICATION_SAVE_FAILED';
            return SaveTipificationItems(vcc, tipification, userId);
        })
            .then(resultSave => {
            if (!resultSave)
                throw 'SERVER_ERROR_TIPIFICATION_ITEMS_SAVE_FAILED';
            if (res.headersSent)
                return;
            res.json({ status: resultSave, description: 'SERVER_SUCCESS_MESSAGE', data: resultSave });
        })
            .catch(err => {
            let clientMsg = (typeof err === 'string' ? err : 'CANNOT_PROCESS_REQUEST');
            let errorMsg = (typeof err === 'string' ? err : err.message || err.description || 'SERVER_ERROR_UNKNOWN_MESSAGE');
            logger.error(`[TipificationService::create_tipification] Error on '${req.path}': ${errorMsg}`);
            if (res.headersSent)
                return;
            res.json({ status: false, description: clientMsg, error: errorMsg });
        });
    });
    app.get('/api/tipification/get_tipifications/:search_criteria/:search_expression?', (req, res, next) => {
        logger.info(`[TipificationService::get_tipifications] Received ${req.method} method on '${req.path}'`);
        let vcc = res.locals.vcc;
        let searchCriteria = req.params.search_criteria || '';
        let searchExpression = req.params.search_expression || '';
        promises_shared_1.DummyPromise()
            .then(_ => {
            return mssql_service_1.MSSql.GetTipifications(vcc, searchCriteria, searchExpression);
        })
            .then(tipifications => {
            if (res.headersSent)
                return;
            res.json({ status: true, description: 'SERVER_SUCCESS_MESSAGE', data: tipifications });
        })
            .catch(err => {
            let clientMsg = (typeof err === 'string' ? err : 'CANNOT_PROCESS_REQUEST');
            let errorMsg = (typeof err === 'string' ? err : err.message || err.description || 'SERVER_ERROR_UNKNOWN_MESSAGE');
            logger.error(`[TipificationService::get_tipifications] Error on '${req.path}': ${errorMsg}`);
            if (res.headersSent)
                return;
            res.json({ status: false, description: clientMsg, error: errorMsg });
        });
    });
    app.get('/api/tipification/get_tipification/:tipificationId', (req, res, next) => {
        logger.info(`[TipificationService::get_tipification] Received ${req.method} method on '${req.path}'`);
        let vcc = res.locals.vcc;
        let tipification = {};
        let tipificationId = req.params.tipificationId;
        promises_shared_1.DummyPromise()
            .then(_ => {
            if (!tipificationId)
                throw 'SERVER_ERROR_MISSING_TIPIFICATION_ID';
            return mssql_service_1.MSSql.GetTipification(vcc, tipificationId);
        })
            .then(result => {
            if (!result)
                throw 'SERVER_ERROR_TIPIFICATION_NOT_FOUND';
            tipification = result;
            return mssql_service_1.MSSql.GetTipificationItems(vcc, tipification.id);
        })
            .then(result => {
            tipification.items = result ? result : [];
            res.json({ status: true, description: 'SERVER_SUCCESS_MESSAGE', data: tipification });
        })
            .catch(err => {
            let clientMsg = (typeof err === 'string' ? err : 'CANNOT_PROCESS_REQUEST');
            let errorMsg = (typeof err === 'string' ? err : err.message || err.description || 'SERVER_ERROR_UNKNOWN_MESSAGE');
            logger.error(`[TipificationService::get_tipification] Error on '${req.path}': ${errorMsg}`);
            if (res.headersSent)
                return;
            res.json({ status: false, description: clientMsg, error: errorMsg });
        });
    });
    app.post('/api/tipification/delete_tipification', (req, res, next) => {
        logger.info(`[TipificationService::delete_tipification] Received ${req.method} method on '${req.path}'`);
        let vcc = res.locals.vcc;
        let tipification = (req.body || {});
        logger.info(`[TipificationService::delete_tipifications] Deleting tipifications ${tipification.id} by the user ${res.locals.userId}`);
        promises_shared_1.DummyPromise()
            .then(_ => {
            if (!tipification.id)
                throw 'SERVER_ERROR_MISSING_TIPIFICATION_ID';
            return mssql_service_1.MSSql.DeleteTipification(vcc, tipification.id);
        })
            .then(result => {
            if (!result)
                throw 'SERVER_ERROR_TIPIFICATION_DELETE_FAILED';
            if (res.headersSent)
                return;
            res.json({ status: result, description: 'SERVER_SUCCESS_MESSAGE', data: result });
        })
            .catch(err => {
            let clientMsg = (typeof err === 'string' ? err : 'CANNOT_PROCESS_REQUEST');
            let errorMsg = (typeof err === 'string' ? err : err.message || err.description || 'SERVER_ERROR_UNKNOWN_MESSAGE');
            logger.error(`[TipificationService::get_tipification] Error on '${req.path}': ${errorMsg}`);
            if (res.headersSent)
                return;
            res.json({ status: false, description: clientMsg, error: errorMsg });
        });
    });
    app.post('/api/tipification/delete_tipifications', (req, res, next) => {
        logger.info(`[TipificationService::delete_tipifications] Received ${req.method} method on '${req.path}'`);
        let vcc = res.locals.vcc;
        let tipifications = req.body.tipifications || [];
        logger.info(`[TipificationService::delete_tipifications] Deleting tipifications ${tipifications.map(x => x.id).join(',')} by the user ${res.locals.userId}`);
        promises_shared_1.DummyPromise()
            .then(_ => {
            if (!tipifications)
                throw 'SERVER_ERROR_MISING_TIPIFICATIONS';
            let promiseArray = [];
            tipifications.forEach(t => promiseArray.push(mssql_service_1.MSSql.DeleteTipification(vcc, t.id)));
            return Promise.all(promiseArray);
        })
            .then(results => {
            let result;
            if (results.filter(r => r !== true).length) {
                result = false;
                logger.error('No se pudo eliminar todos los elementos solicitados.');
            }
            else {
                result = true;
                logger.info('Vccs eliminados correctamente');
            }
            return result;
        })
            .then(result => {
            if (res.headersSent)
                return;
            res.json({ status: result, description: 'SERVER_SUCCESS_MESSAGE', data: result });
        })
            .catch(err => {
            let clientMsg = (typeof err === 'string' ? err : 'CANNOT_PROCESS_REQUEST');
            let errorMsg = (typeof err === 'string' ? err : err.message || err.description || 'SERVER_ERROR_UNKNOWN_MESSAGE');
            logger.error(`[TipificationService::get_tipification] Error on '${req.path}': ${errorMsg}`);
            if (res.headersSent)
                return;
            res.json({ status: false, description: clientMsg, error: errorMsg });
        });
    });
}
exports.TipificationService = TipificationService;
function SaveTipificationItems(vcc, tipification, userId) {
    return promises_shared_1.DummyPromise()
        .then(_ => {
        return mssql_service_1.MSSql.GetTipificationItems(vcc, tipification.id);
    })
        .then(result => {
        let tipificationItems = result ? result : [];
        let classifiedItems = arrays_shared_1.GetCompareItemsFromObjectArray(tipificationItems, tipification.items, 'id');
        let promiseArray = [];
        if (tipification.tipificationQuery?.queryId?.length) {
            promiseArray.push(mssql_service_1.MSSql.SaveTipificationQuery(vcc, tipification.id, tipification.tipificationQuery.queryId, tipification.tipificationQuery.queryContent, tipification.tipificationQuery.acceptancePercentage, userId));
        }
        else
            promiseArray.push(mssql_service_1.MSSql.DeleteTipificationQuery(vcc, tipification.id));
        if (tipification?.items?.length) {
            tipification.items.map(i => {
                i.tipificationId = tipification.id;
                promiseArray.push(mssql_service_1.MSSql.SaveTipificationItem(vcc, tipification.id, i.id, i.secondLevelItemId, i.description, i.order, userId));
            });
        }
        if (classifiedItems?.removed?.length)
            classifiedItems.removed.forEach(i => promiseArray.push(mssql_service_1.MSSql.DeleteTipificationItem(vcc, i.id)));
        if (!promiseArray.length)
            promiseArray.push(Promise.resolve(true));
        return Promise.all(promiseArray);
    })
        .then(result => {
        let saveResult = false;
        if (result.filter(r => { return (r !== true); }).length)
            saveResult = false;
        else
            saveResult = true;
        return Promise.resolve(true);
    })
        .catch(err => {
        let errorMsg = (typeof err === 'string' ? err : err.message || err.description || 'Executing request error');
        logger.error(`[TipificationService::get_tipifications] Error: ${errorMsg}`);
        return Promise.resolve(false);
    });
}
