"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.SimpleDeleteRequest = exports.SimplePutRequest = exports.SimplePostRequest = exports.SimpleGetRequest = exports.AttachedBinaryFilePostRequest = exports.AttachedFilePostRequest = void 0;
const log4js = require("log4js");
const axios_1 = require("axios");
const http = require('http');
const https = require('https');
let logger = log4js.getLogger('ServerScripts');
function SimpleResult(rawResponse) {
    let requestResponse = null;
    if (typeof rawResponse === 'string') {
        try {
            requestResponse = JSON.parse(rawResponse);
        }
        catch (err) {
            requestResponse = null;
        }
    }
    else {
        requestResponse = rawResponse;
    }
    return requestResponse;
}
function AttachedFilePostRequest(data, formData, formDataBackup) {
    let requestConfig = {
        method: 'POST',
        timeout: 300000,
        proxy: false,
        httpAgent: new http.Agent({ keepAlive: true, keepAliveMsecs: 300000 }),
        httpsAgent: new https.Agent({ keepAlive: true, keepAliveMsecs: 300000 }),
        baseURL: data.uri,
        headers: {
            'Content-Type': `multipart/form-data; boundary=${formData._boundary}`,
            'Connection': 'keep-alive',
            'FileName': data.fileName
        },
        maxContentLength: Infinity,
        maxBodyLength: Infinity,
        data: formData,
        onUploadProgress: (progressEvent) => {
            let percentCompleted = Math.round((progressEvent.loaded * 100) / progressEvent.total);
            let mbRate = 1024 * 1024;
            let loaded = progressEvent.loaded / mbRate;
            let total = progressEvent.total / mbRate;
            let now = new Date().getTime();
            logger.info(`[request.shared::AttachedFilePostRequest] File: ${data.fileName} uploaded: ${loaded.toFixed(2)}MB/${total.toFixed(2)}MB (${percentCompleted}%) at ${now}`);
        }
    };
    if (data.headers) {
        requestConfig.headers['Authorization'] = data.headers['Authorization'];
    }
    logger.info(`[request.shared::AttachedFilePostRequest] Uploading file: ${data.fileName}`);
    if (data.timeout)
        requestConfig.timeout = data.timeout;
    return axios_1.default(requestConfig)
        .catch((err) => {
        if (data?.validateErrorResponse && err?.response?.data)
            err = JSON.stringify(err.response.data);
        let errorMessage = typeof err === 'string' ? err : (err.message || err.description);
        logger.error('[request.shared::AttachedFilePostRequest] Getting first try request error:', err);
        logger.error(`[request.shared::AttachedFilePostRequest] Getting first try request error: ${errorMessage} from "${requestConfig.baseURL}"`);
        if ((requestConfig.baseURL.includes('http:')
            ||
                requestConfig.baseURL.includes('https:'))
            &&
                data.retryProtocol) {
            if (requestConfig.baseURL.includes('http:'))
                requestConfig.baseURL = requestConfig.baseURL.replace('http:', 'https:');
            else if (requestConfig.baseURL.includes('https:'))
                requestConfig.baseURL = requestConfig.baseURL.replace('https', 'http');
            logger.warn(`[request.shared::AttachedFilePostRequest] Failed "${data.uri}" retrying "${requestConfig.baseURL}"`);
            return axios_1.default(requestConfig);
        }
        else if (data.retryUri) {
            logger.warn(`[request.shared::AttachedFilePostRequest] Failed "${data.uri}", retrying "${data.retryUri}"`);
            requestConfig = {
                method: 'POST',
                timeout: 300000,
                proxy: false,
                httpAgent: new http.Agent({ keepAlive: true, keepAliveMsecs: 300000 }),
                httpsAgent: new https.Agent({ keepAlive: true, keepAliveMsecs: 300000 }),
                baseURL: data.retryUri,
                headers: {
                    'Content-Type': `multipart/form-data; boundary=${formDataBackup._boundary}`,
                    'Connection': 'keep-alive',
                    'FileName': data.fileName
                },
                maxContentLength: Infinity,
                maxBodyLength: Infinity,
                data: formDataBackup,
                onUploadProgress: (progressEvent) => {
                    let percentCompleted = Math.round((progressEvent.loaded * 100) / progressEvent.total);
                    let mbRate = 1024 * 1024;
                    let loaded = progressEvent.loaded / mbRate;
                    let total = progressEvent.total / mbRate;
                    let now = new Date().getTime();
                    logger.info(`[request.shared::AttachedFilePostRequest] File: ${data.fileName} uploaded: ${loaded.toFixed(2)}MB/${total.toFixed(2)}MB (${percentCompleted}%) at ${now}`);
                }
            };
            return axios_1.default(requestConfig);
        }
        else
            throw err;
    })
        .then(result => {
        logger.info(`[request.shared::AttachedFilePostRequest] Receiving response for file: ${data.fileName}`);
        let requestResponse = SimpleResult(result);
        if (requestResponse?.data) {
            return Promise.resolve(requestResponse.data);
        }
        else {
            throw 'SERVER_ERROR_ATTACHED_FILE_POST_REQUEST_FAILED';
        }
    })
        .catch((err) => {
        if (data?.validateErrorResponse && err?.response?.data)
            err = JSON.stringify(err.response.data);
        let errorMessage = typeof err === 'string' ? err : (err.message || err.description);
        logger.error('[request.shared::AttachedFilePostRequest] Getting error:', errorMessage);
        return Promise.reject(err);
    });
}
exports.AttachedFilePostRequest = AttachedFilePostRequest;
function AttachedBinaryFilePostRequest(data, stream) {
    let requestConfig = {
        method: 'POST',
        timeout: 300000,
        proxy: false,
        httpAgent: new http.Agent({ keepAlive: true, keepAliveMsecs: 300000 }),
        httpsAgent: new https.Agent({ keepAlive: true, keepAliveMsecs: 300000 }),
        headers: {
            'Content-Type': `audio/wav`,
            'Connection': 'keep-alive'
        },
        maxContentLength: Infinity,
        maxBodyLength: Infinity,
        onUploadProgress: (progressEvent) => {
            let percentCompleted = Math.round((progressEvent.loaded * 100) / progressEvent.total);
            let mbRate = 1024 * 1024;
            let loaded = progressEvent.loaded / mbRate;
            let total = progressEvent.total / mbRate;
            let now = new Date().getTime();
            logger.info(`[request.shared::AttachedBinaryFilePostRequest] File: ${data.fileName} uploaded: ${loaded.toFixed(2)}MB/${total.toFixed(2)}MB (${percentCompleted}%) at ${now}`);
        }
    };
    if (data.headers) {
        requestConfig.headers['Authorization'] = data.headers['Authorization'];
    }
    return axios_1.default.post(data.uri, stream, requestConfig)
        .then(result => {
        logger.info(`[request.shared::AttachedBinaryFilePostRequest] Receiving response for file: ${data.fileName}`);
        let requestResponse = SimpleResult(result);
        if (requestResponse?.data) {
            return Promise.resolve(requestResponse.data);
        }
        else {
            throw 'SERVER_ERROR_ATTACHED_FILE_POST_REQUEST_FAILED';
        }
    })
        .catch((err) => {
        if (data?.validateErrorResponse && err?.response?.data)
            err = JSON.stringify(err.response.data);
        let errorMessage = typeof err === 'string' ? err : (err.message || err.description);
        logger.error('[request.shared::AttachedBinaryFilePostRequest] Getting error:', errorMessage);
        return Promise.reject(err);
    });
}
exports.AttachedBinaryFilePostRequest = AttachedBinaryFilePostRequest;
function SimpleGetRequest(data) {
    let requestConfig = {
        method: 'GET',
        timeout: 30000,
        proxy: false,
        httpAgent: new http.Agent({ keepAlive: true, keepAliveMsecs: 300000 }),
        httpsAgent: new https.Agent({ keepAlive: true, keepAliveMsecs: 300000 }),
        baseURL: data.uri,
        headers: data.headers
    };
    if (data.responseType)
        requestConfig.responseType = data.responseType;
    if (data.timeout)
        requestConfig.timeout = data.timeout;
    return axios_1.default(requestConfig)
        .catch((err) => {
        if (data?.validateErrorResponse && err?.response?.data)
            err = JSON.stringify(err.response.data);
        let errorMessage = typeof err === 'string' ? err : (err.message || err.description || JSON.stringify(err));
        logger.error('[request.shared::SimpleGetRequest] Getting first try request error:', err);
        logger.error(`[request.shared::SimpleGetRequest] Getting first try request error: ${errorMessage} from "${requestConfig.baseURL}"`);
        if ((requestConfig.baseURL.includes('http:')
            ||
                requestConfig.baseURL.includes('https:'))
            &&
                data.retryProtocol) {
            if (requestConfig.baseURL.includes('http:'))
                requestConfig.baseURL = requestConfig.baseURL.replace('http:', 'https:');
            else if (requestConfig.baseURL.includes('https:'))
                requestConfig.baseURL = requestConfig.baseURL.replace('https', 'http');
            logger.warn(`[request.shared::SimpleGetRequest] Failed "${data.uri}" retrying "${requestConfig.baseURL}"`);
            return axios_1.default(requestConfig);
        }
        else if (data.retryUri) {
            logger.warn(`[request.shared::SimpleGetRequest] Failed "${data.uri}", retrying "${data.retryUri}"`);
            requestConfig.baseURL = data.retryUri;
            return axios_1.default(requestConfig);
        }
        else
            throw err;
    })
        .then(result => {
        let requestResponse = SimpleResult(result);
        if (requestResponse && requestResponse.data) {
            return Promise.resolve(requestResponse.data);
        }
        else {
            throw 'SERVER_ERROR_SIMPLE_GET_REQUEST_FAILED';
        }
    })
        .catch((err) => {
        if (data?.validateErrorResponse && err?.response?.data)
            err = JSON.stringify(err.response.data);
        let errorMessage = typeof err === 'string' ? err : (err.message || err.description || JSON.stringify(err));
        logger.error('[request.shared::SimpleGetRequest] Getting second error:', errorMessage, 'for:', data.uri);
        if (data?.uri)
            logger.error('[request.shared::SimpleGetRequest] Failed for:', data.uri);
        return Promise.reject(err);
    });
}
exports.SimpleGetRequest = SimpleGetRequest;
function SimplePostRequest(data) {
    const httpAgentDefault = { keepAlive: true, keepAliveMsecs: 300000 };
    let requestConfig = {
        method: 'POST',
        timeout: 30000,
        proxy: false,
        httpAgent: new http.Agent(data.httpAgent || httpAgentDefault),
        httpsAgent: new https.Agent(data.httpsAgent || httpAgentDefault),
        baseURL: data.uri,
        headers: data.headers,
        data: data.body
    };
    if (data.responseType)
        requestConfig.responseType = data.responseType;
    if (data.timeout)
        requestConfig.timeout = data.timeout;
    return axios_1.default(requestConfig)
        .catch((err) => {
        let errorMessage = typeof err === 'string' ? err : (err.message || err.description);
        logger.error(`[request.shared::SimplePostRequest] Getting first try request error: ${errorMessage} from "${requestConfig.baseURL}"`);
        if ((requestConfig.baseURL.includes('http:')
            ||
                requestConfig.baseURL.includes('https:'))
            &&
                data.retryProtocol) {
            if (requestConfig.baseURL.includes('http:'))
                requestConfig.baseURL = requestConfig.baseURL.replace('http:', 'https:');
            else if (requestConfig.baseURL.includes('https:'))
                requestConfig.baseURL = requestConfig.baseURL.replace('https', 'http');
            logger.warn(`[request.shared::SimplePostRequest] Failed "${data.uri}" retrying "${requestConfig.baseURL}"`);
            return axios_1.default(requestConfig);
        }
        else if (data.retryUri) {
            logger.warn(`[request.shared::SimplePostRequest] Failed "${data.uri}", retrying "${data.retryUri}"`);
            requestConfig.baseURL = data.retryUri;
            return axios_1.default(requestConfig);
        }
        else
            throw err;
    })
        .then(result => {
        let requestResponse = SimpleResult(result);
        if (requestResponse && requestResponse.data) {
            return Promise.resolve(requestResponse.data);
        }
        else {
            throw 'SERVER_ERROR_SIMPLE_POST_REQUEST_FAILED';
        }
    })
        .catch((err) => {
        if (data?.validateErrorResponse && err?.response?.data)
            err = JSON.stringify(err.response.data);
        let errorMessage = typeof err === 'string' ? err : (err.message || err.description);
        logger.error('[request.shared::SimplePostRequest] Getting second error:', errorMessage);
        if (data?.uri)
            logger.error('[request.shared::SimpleGetRequest] Failed for:', data.uri);
        return Promise.reject(err);
    });
}
exports.SimplePostRequest = SimplePostRequest;
function SimplePutRequest(data) {
    let requestConfig = {
        method: 'PUT',
        timeout: 30000,
        proxy: false,
        httpAgent: new http.Agent({ keepAlive: true, keepAliveMsecs: 300000 }),
        httpsAgent: new https.Agent({ keepAlive: true, keepAliveMsecs: 300000 }),
        baseURL: data.uri,
        headers: data.headers,
        data: data.body
    };
    if (data.timeout)
        requestConfig.timeout = data.timeout;
    return axios_1.default(requestConfig)
        .then(result => {
        let requestResponse = SimpleResult(result);
        if (requestResponse && requestResponse.data) {
            return Promise.resolve(requestResponse.data);
        }
        else {
            throw 'SERVER_ERROR_SIMPLE_DELETE_PUT_FAILED';
        }
    })
        .catch((err) => {
        let errorMessage = typeof err === 'string' ? err : (err.message || err.description);
        logger.error('[request.shared::SimplePutRequest] Getting error:', errorMessage);
        return Promise.reject(err);
    });
}
exports.SimplePutRequest = SimplePutRequest;
function SimpleDeleteRequest(data) {
    let requestConfig = {
        method: 'DELETE',
        timeout: 30000,
        proxy: false,
        httpAgent: new http.Agent({ keepAlive: true, keepAliveMsecs: 300000 }),
        httpsAgent: new https.Agent({ keepAlive: true, keepAliveMsecs: 300000 }),
        baseURL: data.uri,
        headers: data.headers,
        data: data.body
    };
    if (data.timeout)
        requestConfig.timeout = data.timeout;
    return axios_1.default(requestConfig)
        .then(result => {
        let requestResponse = SimpleResult(result);
        if (requestResponse) {
            if (requestResponse.data)
                return Promise.resolve(requestResponse.data);
            else
                return Promise.resolve(requestResponse);
        }
        else {
            throw 'SERVER_ERROR_SIMPLE_DELETE_REQUEST_FAILED';
        }
    })
        .catch((err) => {
        let errorMessage = typeof err === 'string' ? err : (err.message || err.description);
        logger.error('[request.shared::SimpleDeleteRequest] Getting error:', errorMessage);
        return Promise.reject(err);
    });
}
exports.SimpleDeleteRequest = SimpleDeleteRequest;
axios_1.default.interceptors.request.use(x => {
    if (x.headers.FileName) {
        x.data = x.data || {};
        x.data.requestStartedAt = new Date().getTime();
    }
    return x;
});
axios_1.default.interceptors.response.use(x => {
    if (x.config.headers.FileName) {
        let now = new Date().getTime();
        logger.info(`[request.shared::interceptors.response] Total execution time for: ${x.config.headers.FileName} => ${now - x.config.data.requestStartedAt} ms at ${now}`);
        x.headers['request-duration'] = now - x.config.data.requestStartedAt;
    }
    return x;
}, x => {
    if (x.config.headers.FileName)
        logger.error(`[request.shared::interceptors.response] Execution time for: ${x.config.baseURL}>${x.config.headers.FileName} => ${new Date().getTime() - x.config.data.requestStartedAt} ms`);
    throw x;
});
